@@ -1,3 +1,21 @@
+/*
+ *   mem-debug.h
+ *
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
+ *
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+#ifndef MOO_MEM_DEBUG_H
+#define MOO_MEM_DEBUG_H
+
 #undef DEBUG_MEMORY
 
 #ifndef DEBUG_MEMORY
@@ -75,3 +93,5 @@ init_mem_stuff (void)
     }
 }
 #endif
+
+#endif /* MOO_MEM_DEBUG_H */
