@@ -1,13 +1,16 @@
 /*
  *   mooapp/mooappabout.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_APP_ABOUT_H
@@ -26,7 +29,6 @@
 #endif
 
 #ifdef __WIN32__
-#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #endif
 
@@ -34,45 +36,18 @@
 #include <errno.h>
 #include <gtk/gtk.h>
 
-#if defined(MOO_OS_MINGW)
-#define MOO_OS_NAME "Win32"
-#elif defined(MOO_OS_DARWIN)
-#define MOO_OS_NAME "Darwin"
-#elif defined(MOO_OS_BSD)
-#define MOO_OS_NAME "BSD"
-#elif defined(MOO_OS_UNIX)
-#define MOO_OS_NAME "UNIX"
-#else
-#define MOO_OS_NAME "Unknown???"
-#endif
 
+#ifdef __WIN32__
 
-#ifdef HAVE_SYS_UTSNAME_H
 static char *
-get_uname (void)
+get_system_name (void)
 {
-    struct utsname name;
-
-    if (uname (&name))
-    {
-        int err = errno;
-        g_critical ("%s: %s", G_STRLOC, g_strerror (err));
-        return NULL;
-    }
-
-    return g_strdup_printf ("%s %s (%s), %s", name.sysname,
-                            name.release, name.version, name.machine);
-}
-#elif defined __WIN32__
-static char *
-get_windows_name (void)
-{
-    OSVERSIONINFOEXA ver;
+    OSVERSIONINFOEXW ver;
 
     memset (&ver, 0, sizeof (ver));
-    ver.dwOSVersionInfoSize = sizeof (OSVERSIONINFOA);
+    ver.dwOSVersionInfoSize = sizeof (OSVERSIONINFOW);
 
-    if (!GetVersionExA ((OSVERSIONINFOA*) &ver))
+    if (!GetVersionExW ((OSVERSIONINFOW*) &ver))
         return g_strdup ("Win32");
 
     switch (ver.dwMajorVersion)
@@ -109,32 +84,30 @@ get_windows_name (void)
             break;
 
         case 6:
+        case 7:
             memset (&ver, 0, sizeof (ver));
-            ver.dwOSVersionInfoSize = sizeof (OSVERSIONINFOEXA);
+            ver.dwOSVersionInfoSize = sizeof (OSVERSIONINFOEXW);
 
-            if (!GetVersionExA ((OSVERSIONINFOA*) &ver))
-                return g_strdup ("Windows Vista");
-
-            if (ver.wProductType == VER_NT_WORKSTATION)
-                return g_strdup ("Windows Vista");
+            if (!GetVersionExW ((OSVERSIONINFOW*) &ver) || ver.wProductType == VER_NT_WORKSTATION)
+                return ver.dwMajorVersion == 6 ? g_strdup ("Windows Vista") : g_strdup ("Windows 7");
             else
-                return g_strdup ("Windows Server \"Longhorn\"");
+                return ver.dwMajorVersion == 6 ? g_strdup ("Windows Server 2008") : g_strdup ("Windows Server 2008 R2");
+
+            break;
     }
 
     return g_strdup ("Win32");
 }
-#else
-static char *
-get_uname (void)
-{
-    char *string;
 
-    if (g_spawn_command_line_sync ("uname -a", &string, NULL, NULL, NULL))
-        return string;
-    else
-        return NULL;
+#else /* __WIN32__ */
+
+static char *
+get_system_name (void)
+{
+    return g_strdup (MOO_OS_NAME);
 }
-#endif
+
+#endif /* __WIN32__ */
 
 
 static char *
