@@ -1,18 +1,21 @@
 /*
  *   moohtml.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
-#include "mooapp/moohtml.h"
-#include "mooutils/moomarshals.h"
+#include "moohtml.h"
+#include "marshals.h"
 #include "mooutils/mooutils-misc.h"
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
@@ -1007,8 +1010,8 @@ moo_html_attr_copy (const MooHtmlAttr *src)
     g_return_val_if_fail (src != NULL, NULL);
 
     attr = g_new (MooHtmlAttr, 1);
-    memcpy (attr, src, sizeof (MooHtmlAttr));
 
+    *attr = *src;
     attr->fg = g_strdup (src->fg);
     attr->bg = g_strdup (src->bg);
     attr->link = g_strdup (src->link);
