@@ -1302,10 +1302,9 @@ process_ice_messages (IceConn ice_conn)
 
     case IceProcessMessagesConnectionClosed:
       return FALSE;
-
-    default:
-      g_assert_not_reached ();
     }
+
+  g_return_val_if_reached (FALSE);
 }
 
 static gboolean
