@@ -1,12 +1,12 @@
-#ifndef __GTK_SOURCE_VIEW_API_H__
-#define __GTK_SOURCE_VIEW_API_H__
+#ifndef GTK_SOURCE_VIEW_API_H
+#define GTK_SOURCE_VIEW_API_H
 
 
-#include "mooedit/gtksourceview/gtksourcelanguage-private-mangled.h"
-#include "mooedit/gtksourceview/gtksourcelanguagemanager-mangled.h"
-#include "mooedit/gtksourceview/gtksourcestyleschememanager-mangled.h"
-#include "mooedit/gtksourceview/gtksourceiter-mangled.h"
-#include "mooedit/gtksourceview/gtksourcestyle-private-mangled.h"
+#include "gtksourceview/gtksourcelanguage-private-mangled.h"
+#include "gtksourceview/gtksourcelanguagemanager-mangled.h"
+#include "gtksourceview/gtksourcestyleschememanager-mangled.h"
+#include "gtksourceview/gtksourceiter-mangled.h"
+#include "gtksourceview/gtksourcestyle-private-mangled.h"
 
 #define GtkSourceLanguage			MooGtkSourceLanguage
 #define GtkSourceLanguageClass			MooGtkSourceLanguageClass
@@ -26,6 +26,7 @@
 #define gtk_source_style_scheme_manager_set_search_path		_moo_gtk_source_style_scheme_manager_set_search_path
 #define gtk_source_style_scheme_manager_get_scheme_ids		_moo_gtk_source_style_scheme_manager_get_scheme_ids
 #define gtk_source_style_scheme_manager_get_scheme		_moo_gtk_source_style_scheme_manager_get_scheme
+#define gtk_source_language_manager_new				_moo_gtk_source_language_manager_new
 #define gtk_source_language_manager_get_language_ids		_moo_gtk_source_language_manager_get_language_ids
 #define gtk_source_language_manager_get_language		_moo_gtk_source_language_manager_get_language
 #define _gtk_source_engine_attach_buffer			_moo_gtk_source_engine_attach_buffer
@@ -49,4 +50,4 @@
 #define _gtk_source_style_apply					_moo_gtk_source_style_apply
 
 
-#endif /* __GTK_SOURCE_VIEW_API_H__ */
+#endif /* GTK_SOURCE_VIEW_API_H */
