@@ -24,6 +24,7 @@
 #include <config.h>
 #endif
 
+#undef DEBUG
 #undef ENABLE_DEBUG
 
 #ifdef ENABLE_DEBUG
@@ -1528,6 +1529,7 @@ file_parse (gchar                     *filename,
 {
 	ParserState *parser_state;
 	xmlTextReader *reader = NULL;
+	int ret;
 	int fd = -1;
 	GError *tmp_error = NULL;
 	GtkSourceLanguageManager *lm;
@@ -1585,7 +1587,7 @@ file_parse (gchar                     *filename,
 						parser_state);
 
 	while (parser_state->error == NULL &&
-	       xmlTextReaderRead (parser_state->reader) == 1)
+	       (ret = xmlTextReaderRead (parser_state->reader)) == 1)
 	{
 		int type;
 
