@@ -1,13 +1,16 @@
 /*
  *   moocmdview.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __MOO_CMD_VIEW__
@@ -67,6 +70,8 @@ GtkWidget  *moo_cmd_view_new                (void);
 
 void        moo_cmd_view_set_filter         (MooCmdView         *view,
                                              MooOutputFilter    *filter);
+void        moo_cmd_view_add_filter_dirs    (MooCmdView         *view,
+                                             char              **dirs);
 
 gboolean    moo_cmd_view_run_command        (MooCmdView         *view,
                                              const char         *cmd,
