@@ -1,24 +1,50 @@
 /*
  *   mooedit-lua.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
 #include "mooedit/mooedit-lua.h"
-#include "mooedit/mooedit-private.h"
-#include "mooedit/moocommand-exe.h"
+#include "mooedit/mooeditor.h"
 #include "mooutils/moohistorycombo.h"
 #include <string.h>
 #include <glib/gprintf.h>
 #include <gtk/gtk.h>
 
+#ifndef __WIN32__
+G_BEGIN_DECLS
+void    _moo_edit_run_in_pane       (const char     *cmd_line,
+                                     const char     *working_dir,
+                                     char          **envp,
+                                     MooEditWindow  *window,
+                                     MooEdit        *doc);
+void    _moo_edit_run_async         (const char     *cmd_line,
+                                     const char     *working_dir,
+                                     char          **envp,
+                                     MooEditWindow  *window,
+                                     MooEdit        *doc);
+void    _moo_edit_run_sync          (const char     *cmd_line,
+                                     const char     *working_dir,
+                                     char          **envp,
+                                     MooEditWindow  *window,
+                                     MooEdit        *doc,
+                                     const char     *input,
+                                     int            *exit_status,
+                                     char          **output,
+                                     char          **output_err);
+G_END_DECLS
+#endif
+
 #define VAR_WINDOW       "window"
 #define VAR_DOC          "doc"
 #define VAR_FILE         "file"
@@ -486,7 +512,7 @@ insert_and_scroll (GtkTextView *view,
 static int
 cfunc_new_line (lua_State *L)
 {
-    int n;
+    int n = 1;
     GtkTextView *doc;
     char *freeme = NULL;
     const char *insert;
@@ -666,8 +692,9 @@ cfunc_insert (lua_State *L)
 
     for (i = 1; i <= lua_gettop (L); ++i)
     {
-        const char *s = lua_tostring (L, i);
-        gtk_text_buffer_insert (buffer, &start, s, -1);
+        size_t len;
+        const char *s = lua_check_utf8string (L, i, &len);
+        gtk_text_buffer_insert (buffer, &start, s, len);
     }
 
     scroll_to_cursor (doc);
@@ -695,8 +722,9 @@ cfunc_insert_text (lua_State *L)
 
     for (i = 2; i <= n_args; ++i)
     {
-        const char *s = lua_tostring (L, i);
-        gtk_text_buffer_insert (buffer, &iter, s, -1);
+        size_t len;
+        const char *s = lua_check_utf8string (L, i, &len);
+        gtk_text_buffer_insert (buffer, &iter, s, len);
     }
 
     return 0;
@@ -738,12 +766,16 @@ cfunc_selection (lua_State *L)
     CHECK_DOC (L, doc);
     parse_args (L, "Selection", "");
 
-    text = moo_text_view_get_selection (doc);
-    if (text && text[0])
-        lua_pushstring (L, text);
+    if ((text = moo_text_view_get_selection (doc)) && text[0])
+    {
+        lua_take_utf8string (L, text);
+    }
     else
+    {
         lua_pushnil (L);
-    g_free (text);
+        g_free (text);
+    }
+
     return 1;
 }
 
@@ -963,8 +995,7 @@ cfunc_get_text (lua_State *L)
     gtk_text_iter_order (&start, &end);
 
     text = gtk_text_buffer_get_slice (buffer, &start, &end, TRUE);
-    lua_pushstring (L, text);
-    g_free (text);
+    lua_take_utf8string (L, text);
 
     return 1;
 }
@@ -1060,7 +1091,7 @@ cfunc_open (lua_State *L)
 
     parse_args (L, "_medit.open", "s", &filename);
 
-    editor = MOO_IS_EDIT (data->doc) ? MOO_EDIT(data->doc)->priv->editor : moo_editor_instance ();
+    editor = MOO_IS_EDIT (data->doc) ? moo_edit_get_editor (data->doc) : moo_editor_instance ();
     g_return_val_if_fail (MOO_IS_EDITOR (editor), 0);
 
     window = MOO_IS_EDIT_WINDOW (data->window) ? MOO_EDIT_WINDOW (data->window) : NULL;
@@ -1125,6 +1156,8 @@ zfunc_history_entry (lua_State *L)
 }
 #endif
 
+#ifndef __WIN32__
+
 static int
 cfunc_run_in_pane (lua_State *L)
 {
@@ -1178,14 +1211,18 @@ cfunc_run_sync (lua_State *L)
     return 2;
 }
 
+#endif /* !__WIN32__ */
+
 static void
 add_medit_api (lua_State *L)
 {
     static const struct luaL_reg meditlib[] = {
         {"open", cfunc_open},
+#ifndef __WIN32__
         {"run_in_pane", cfunc_run_in_pane},
         {"run_async", cfunc_run_async},
         {"run_sync", cfunc_run_sync},
+#endif
         {NULL, NULL}
     };
 
