@@ -3,11 +3,14 @@
  *
  *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
@@ -18,19 +21,25 @@
 #include "mooedit/mooeditdialogs.h"
 #include "mooedit/mooeditprefs.h"
 #include "mooedit/mootextbuffer.h"
-#include "mooedit/mooeditprogress-glade.h"
 #include "mooedit/mooeditfiltersettings.h"
 #include "mooedit/mooeditor-private.h"
-#include "mooutils/moomarshals.h"
-#include "mooutils/mooglade.h"
+#include "mooedit/moolangmgr.h"
+#include "marshals.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mootype-macros.h"
+#include "mooutils/mooatom.h"
+#include "glade/mooeditprogress-gxml.h"
 #include <string.h>
+#include <stdlib.h>
 
 
-GSList *_moo_edit_instances = NULL;
+#define KEY_ENCODING "encoding"
+#define KEY_LINE "line"
 
 
+MooEditList *_moo_edit_instances = NULL;
+
 static GObject *moo_edit_constructor        (GType                  type,
                                              guint                  n_construct_properties,
                                              GObjectConstructParam *construct_param);
@@ -102,10 +111,11 @@ enum {
     PROP_EDITOR,
     PROP_ENABLE_BOOKMARKS,
     PROP_HAS_COMMENTS,
+    PROP_LINE_END_TYPE,
     PROP_ENCODING
 };
 
-/* MOO_TYPE_EDIT */
+MOO_DEFINE_BOXED_TYPE_C (MooEditFileInfo, moo_edit_file_info)
 G_DEFINE_TYPE (MooEdit, moo_edit, MOO_TYPE_TEXT_VIEW)
 
 
@@ -142,7 +152,7 @@ moo_edit_class_init (MooEditClass *klass)
                                              "editor",
                                              "editor",
                                              MOO_TYPE_EDITOR,
-                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                                             (GParamFlags) (G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_ENABLE_BOOKMARKS,
@@ -150,7 +160,7 @@ moo_edit_class_init (MooEditClass *klass)
                                              "enable-bookmarks",
                                              "enable-bookmarks",
                                              TRUE,
-                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                             (GParamFlags) (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_HAS_COMMENTS,
@@ -160,18 +170,23 @@ moo_edit_class_init (MooEditClass *klass)
                                              FALSE,
                                              G_PARAM_READABLE));
 
+    g_object_class_install_property (gobject_class, PROP_LINE_END_TYPE,
+        g_param_spec_enum ("line-end-type", "line-end-type", "line-end-type",
+                           MOO_TYPE_LINE_END_TYPE, MOO_LE_NONE,
+                           (GParamFlags) G_PARAM_READWRITE));
+
     g_object_class_install_property (gobject_class,
                                      PROP_ENCODING,
                                      g_param_spec_string ("encoding",
                                              "encoding",
                                              "encoding",
                                              NULL,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     signals[CONFIG_NOTIFY] =
             g_signal_new ("config-notify",
                           G_OBJECT_CLASS_TYPE (klass),
-                          G_SIGNAL_RUN_FIRST | G_SIGNAL_DETAILED,
+                          (GSignalFlags) (G_SIGNAL_RUN_FIRST | G_SIGNAL_DETAILED),
                           G_STRUCT_OFFSET (MooEditClass, config_notify),
                           NULL, NULL,
                           _moo_marshal_VOID__UINT_POINTER,
@@ -200,7 +215,7 @@ moo_edit_class_init (MooEditClass *klass)
     signals[COMMENT] =
             _moo_signal_new_cb ("comment",
                                 G_OBJECT_CLASS_TYPE (klass),
-                                G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                                (GSignalFlags) (G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
                                 G_CALLBACK (moo_edit_comment),
                                 NULL, NULL,
                                 _moo_marshal_VOID__VOID,
@@ -209,7 +224,7 @@ moo_edit_class_init (MooEditClass *klass)
     signals[UNCOMMENT] =
             _moo_signal_new_cb ("uncomment",
                                 G_OBJECT_CLASS_TYPE (klass),
-                                G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                                (GSignalFlags) (G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
                                 G_CALLBACK (moo_edit_uncomment),
                                 NULL, NULL,
                                 _moo_marshal_VOID__VOID,
@@ -260,7 +275,9 @@ moo_edit_init (MooEdit *edit)
 
     edit->priv->actions = moo_action_collection_new ("MooEdit", "MooEdit");
 
-    indent = moo_indenter_new (edit, NULL);
+    edit->priv->line_end_type = MOO_LE_NONE;
+
+    indent = moo_indenter_new (edit);
     moo_text_view_set_indenter (MOO_TEXT_VIEW (edit), indent);
     g_object_unref (indent);
 }
@@ -281,7 +298,7 @@ moo_edit_constructor (GType                  type,
     edit = MOO_EDIT (object);
 
     _moo_edit_add_class_actions (edit);
-    _moo_edit_instances = g_slist_prepend (_moo_edit_instances, edit);
+    _moo_edit_instances = moo_edit_list_prepend (_moo_edit_instances, edit);
 
     edit->priv->modified_changed_handler_id =
             g_signal_connect (get_buffer (edit),
@@ -289,7 +306,7 @@ moo_edit_constructor (GType                  type,
                               G_CALLBACK (modified_changed_cb),
                               edit);
 
-    _moo_edit_set_filename (edit, NULL, NULL);
+    _moo_edit_set_file (edit, NULL, NULL);
 
     buffer = get_buffer (edit);
     g_signal_connect_swapped (buffer, "line-mark-moved",
@@ -308,7 +325,10 @@ moo_edit_finalize (GObject *object)
 {
     MooEdit *edit = MOO_EDIT (object);
 
+    if (edit->priv->file)
+        g_object_unref (edit->priv->file);
     g_free (edit->priv->filename);
+    g_free (edit->priv->norm_filename);
     g_free (edit->priv->display_filename);
     g_free (edit->priv->display_basename);
     g_free (edit->priv->encoding);
@@ -323,7 +343,7 @@ moo_edit_dispose (GObject *object)
 {
     MooEdit *edit = MOO_EDIT (object);
 
-    _moo_edit_instances = g_slist_remove (_moo_edit_instances, edit);
+    _moo_edit_instances = moo_edit_list_remove (_moo_edit_instances, edit);
 
     if (edit->config)
     {
@@ -388,6 +408,17 @@ modified_changed_cb (GtkTextBuffer      *buffer,
 }
 
 
+static void
+modify_status (MooEdit       *edit,
+               MooEditStatus  status,
+               gboolean       add)
+{
+    if (add)
+        edit->priv->status = (MooEditStatus) (edit->priv->status | status);
+    else
+        edit->priv->status = (MooEditStatus) (edit->priv->status & ~status);
+}
+
 void
 moo_edit_set_modified (MooEdit            *edit,
                        gboolean            modified)
@@ -411,10 +442,7 @@ moo_edit_set_modified (MooEdit            *edit,
                                   edit->priv->modified_changed_handler_id);
     }
 
-    if (modified)
-        edit->priv->status |= MOO_EDIT_MODIFIED;
-    else
-        edit->priv->status &= ~MOO_EDIT_MODIFIED;
+    modify_status (edit, MOO_EDIT_MODIFIED, modified);
 
     moo_edit_status_changed (edit);
 }
@@ -425,10 +453,7 @@ moo_edit_set_clean (MooEdit            *edit,
                     gboolean            clean)
 {
     g_return_if_fail (MOO_IS_EDIT (edit));
-    if (clean)
-        edit->priv->status |= MOO_EDIT_CLEAN;
-    else
-        edit->priv->status &= ~MOO_EDIT_CLEAN;
+    modify_status (edit, MOO_EDIT_CLEAN, clean);
     moo_edit_status_changed (edit);
 }
 
@@ -449,7 +474,6 @@ moo_edit_status_changed (MooEdit *edit)
 }
 
 
-#if 0
 void
 _moo_edit_set_status (MooEdit        *edit,
                       MooEditStatus   status)
@@ -462,41 +486,6 @@ _moo_edit_set_status (MooEdit        *edit,
         moo_edit_status_changed (edit);
     }
 }
-#endif
-
-
-MooEditFileInfo*
-moo_edit_file_info_new (const char *filename,
-                        const char *encoding)
-{
-    MooEditFileInfo *info = g_new0 (MooEditFileInfo, 1);
-    info->filename = g_strdup (filename);
-    info->encoding = g_strdup (encoding);
-    return info;
-}
-
-
-MooEditFileInfo*
-moo_edit_file_info_copy (const MooEditFileInfo *info)
-{
-    MooEditFileInfo *copy;
-    g_return_val_if_fail (info != NULL, NULL);
-    copy = g_new (MooEditFileInfo, 1);
-    copy->encoding = g_strdup (info->encoding);
-    copy->filename = g_strdup (info->filename);
-    return copy;
-}
-
-void
-moo_edit_file_info_free (MooEditFileInfo *info)
-{
-    if (info)
-    {
-        g_free (info->encoding);
-        g_free (info->filename);
-        g_free (info);
-    }
-}
 
 
 gboolean
@@ -525,7 +514,7 @@ moo_edit_is_untitled (MooEdit *edit)
 MooEditStatus
 moo_edit_get_status (MooEdit *edit)
 {
-    g_return_val_if_fail (MOO_IS_EDIT (edit), 0);
+    g_return_val_if_fail (MOO_IS_EDIT (edit), (MooEditStatus) 0);
     return edit->priv->status;
 }
 
@@ -541,7 +530,7 @@ moo_edit_set_property (GObject        *object,
     switch (prop_id)
     {
         case PROP_EDITOR:
-            edit->priv->editor = g_value_get_object (value);
+            edit->priv->editor = (MooEditor*) g_value_get_object (value);
             break;
 
         case PROP_ENABLE_BOOKMARKS:
@@ -549,7 +538,11 @@ moo_edit_set_property (GObject        *object,
             break;
 
         case PROP_ENCODING:
-            _moo_edit_set_encoding (edit, g_value_get_string (value));
+            moo_edit_set_encoding (edit, g_value_get_string (value));
+            break;
+
+        case PROP_LINE_END_TYPE:
+            moo_edit_set_line_end_type (edit, (MooLineEndType) g_value_get_enum (value));
             break;
 
         default:
@@ -585,6 +578,10 @@ moo_edit_get_property (GObject        *object,
             g_value_set_string (value, edit->priv->encoding);
             break;
 
+        case PROP_LINE_END_TYPE:
+            g_value_set_enum (value, edit->priv->line_end_type);
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
             break;
@@ -633,17 +630,56 @@ moo_edit_focus_out (GtkWidget     *widget,
 }
 
 
-GType
-moo_edit_file_info_get_type (void)
+static MooEditFileInfo *
+moo_edit_file_info_new (GFile      *file,
+                        const char *encoding)
 {
-    static GType type = 0;
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooEditFileInfo",
-                                             (GBoxedCopyFunc) moo_edit_file_info_copy,
-                                             (GBoxedFreeFunc) moo_edit_file_info_free);
-    return type;
+    MooEditFileInfo *info = moo_new (MooEditFileInfo);
+    info->file = file;
+    info->encoding = g_strdup (encoding);
+    return info;
 }
 
+MooEditFileInfo *
+moo_edit_file_info_new_path (const char *path,
+                             const char *encoding)
+{
+    g_return_val_if_fail (path != NULL, NULL);
+    return moo_edit_file_info_new (g_file_new_for_path (path), encoding);
+}
+
+MooEditFileInfo *
+moo_edit_file_info_new_uri (const char *uri,
+                            const char *encoding)
+{
+    g_return_val_if_fail (uri != NULL, NULL);
+    return moo_edit_file_info_new (g_file_new_for_uri (uri), encoding);
+}
+
+MooEditFileInfo *
+moo_edit_file_info_copy (MooEditFileInfo *info)
+{
+    return info ? moo_edit_file_info_new ((GFile*) g_object_ref (info->file), info->encoding) : NULL;
+}
+
+void
+moo_edit_file_info_free (MooEditFileInfo *info)
+{
+    if (info)
+    {
+        g_free (info->encoding);
+        g_object_unref (info->file);
+        moo_free (MooEditFileInfo, info);
+    }
+}
+
+
+GFile *
+_moo_edit_get_file (MooEdit *edit)
+{
+    g_return_val_if_fail (MOO_IS_EDIT (edit), NULL);
+    return edit->priv->file ? g_file_dup (edit->priv->file) : NULL;
+}
 
 char *
 moo_edit_get_filename (MooEdit *edit)
@@ -652,6 +688,13 @@ moo_edit_get_filename (MooEdit *edit)
     return g_strdup (edit->priv->filename);
 }
 
+char *
+moo_edit_get_norm_filename (MooEdit *edit)
+{
+    g_return_val_if_fail (MOO_IS_EDIT (edit), NULL);
+    return g_strdup (edit->priv->norm_filename);
+}
+
 const char *
 moo_edit_get_display_name (MooEdit *edit)
 {
@@ -670,11 +713,7 @@ char *
 moo_edit_get_uri (MooEdit *edit)
 {
     g_return_val_if_fail (MOO_IS_EDIT (edit), NULL);
-
-    if (edit->priv->filename)
-        return g_filename_to_uri (edit->priv->filename, NULL, NULL);
-    else
-        return NULL;
+    return edit->priv->file ? g_file_get_uri (edit->priv->file) : NULL;
 }
 
 const char *
@@ -685,8 +724,8 @@ moo_edit_get_encoding (MooEdit *edit)
 }
 
 void
-_moo_edit_set_encoding (MooEdit    *edit,
-                        const char *encoding)
+moo_edit_set_encoding (MooEdit    *edit,
+                       const char *encoding)
 {
     g_return_if_fail (MOO_IS_EDIT (edit));
     g_return_if_fail (encoding != NULL);
@@ -714,6 +753,52 @@ moo_edit_get_editor (MooEdit *doc)
 }
 
 
+void
+_moo_edit_history_item_set_encoding (MdHistoryItem *item,
+                                     const char    *encoding)
+{
+    g_return_if_fail (item != NULL);
+    md_history_item_set (item, KEY_ENCODING, encoding);
+}
+
+void
+_moo_edit_history_item_set_line (MdHistoryItem *item,
+                                 int            line)
+{
+    char *value = NULL;
+
+    g_return_if_fail (item != NULL);
+
+    if (line >= 0)
+        value = g_strdup_printf ("%d", line + 1);
+
+    md_history_item_set (item, KEY_LINE, value);
+    g_free (value);
+}
+
+const char *
+_moo_edit_history_item_get_encoding (MdHistoryItem *item)
+{
+    g_return_val_if_fail (item != NULL, NULL);
+    return md_history_item_get (item, KEY_ENCODING);
+}
+
+int
+_moo_edit_history_item_get_line (MdHistoryItem *item)
+{
+    const char *strval;
+
+    g_return_val_if_fail (item != NULL, -1);
+
+    strval = md_history_item_get (item, KEY_LINE);
+
+    if (strval && strval[0])
+        return strtol (strval, NULL, 10) - 1;
+    else
+        return -1;
+}
+
+
 typedef void (*SetVarFunc) (MooEdit    *edit,
                             const char *name,
                             char       *val);
@@ -841,7 +926,7 @@ parse_emacs_mode_string (MooEdit *edit,
 
     g_strstrip (string);
 
-    mgr = moo_editor_get_lang_mgr (edit->priv->editor);
+    mgr = moo_lang_mgr_default ();
 
     if (_moo_lang_mgr_find_lang (mgr, string))
         set_emacs_var (edit, "mode", string);
@@ -979,8 +1064,9 @@ moo_edit_set_lang (MooEdit *edit,
     if (old_lang != lang)
     {
         moo_text_view_set_lang (MOO_TEXT_VIEW (edit), lang);
-        _moo_lang_mgr_update_config (moo_editor_get_lang_mgr (edit->priv->editor),
-                                     edit->config, _moo_lang_id (lang));
+        _moo_lang_mgr_update_config (moo_lang_mgr_default (),
+                                     edit->config,
+                                     _moo_lang_id (lang));
         _moo_edit_update_config_from_global (edit);
         g_object_notify (G_OBJECT (edit), "has-comments");
     }
@@ -991,7 +1077,7 @@ static void
 moo_edit_apply_lang_config (MooEdit *edit)
 {
     const char *lang_id = moo_edit_config_get_string (edit->config, "lang");
-    MooLangMgr *mgr = moo_editor_get_lang_mgr (edit->priv->editor);
+    MooLangMgr *mgr = moo_lang_mgr_default ();
     MooLang *lang = lang_id ? _moo_lang_mgr_find_lang (mgr, lang_id) : NULL;
     moo_edit_set_lang (edit, lang);
 }
@@ -1011,7 +1097,7 @@ moo_edit_apply_config (MooEdit *edit)
                          "show-line-numbers", &line_numbers,
                          "tab-width", &tab_width,
                          "word-chars", &word_chars,
-                         NULL);
+                         (char*) 0);
 
     gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (edit), wrap_mode);
     moo_text_view_set_show_line_numbers (MOO_TEXT_VIEW (edit), line_numbers);
@@ -1035,9 +1121,9 @@ moo_edit_queue_apply_config (MooEdit *edit)
 {
     if (!edit->priv->apply_config_idle)
         edit->priv->apply_config_idle =
-                _moo_idle_add_full (G_PRIORITY_HIGH,
-                                    (GSourceFunc) do_apply_config,
-                                    edit, NULL);
+                moo_idle_add_full (G_PRIORITY_HIGH,
+                                   (GSourceFunc) do_apply_config,
+                                   edit, NULL);
 }
 
 
@@ -1066,12 +1152,12 @@ _moo_edit_update_config_from_global (MooEdit *edit)
 void
 _moo_edit_update_lang_config (void)
 {
-    GSList *l;
+    MooEditList *l;
 
     for (l = _moo_edit_instances; l != NULL; l = l->next)
     {
         MooEdit *edit = l->data;
-        _moo_lang_mgr_update_config (moo_editor_get_lang_mgr (edit->priv->editor), edit->config,
+        _moo_lang_mgr_update_config (moo_lang_mgr_default (), edit->config,
                                      _moo_lang_id (moo_text_view_get_lang (MOO_TEXT_VIEW (edit))));
     }
 }
@@ -1095,14 +1181,14 @@ moo_edit_filename_changed (MooEdit    *edit,
 
     if (filename)
     {
-        MooLangMgr *mgr = moo_editor_get_lang_mgr (edit->priv->editor);
+        MooLangMgr *mgr = moo_lang_mgr_default ();
         lang = moo_lang_mgr_get_lang_for_file (mgr, filename);
         lang_id = lang ? _moo_lang_id (lang) : NULL;
         filter_config = _moo_edit_filter_settings_get_for_file (filename);
     }
 
     moo_edit_config_set (edit->config, MOO_EDIT_CONFIG_SOURCE_FILENAME,
-                         "lang", lang_id, "indent", NULL, NULL);
+                         "lang", lang_id, "indent", (void*) NULL, (char*) NULL);
 
     if (filter_config)
         moo_edit_config_parse (edit->config, filter_config,
@@ -1119,8 +1205,9 @@ moo_edit_filename_changed (MooEdit    *edit,
 
     if (!lang_changed)
     {
-        _moo_lang_mgr_update_config (moo_editor_get_lang_mgr (edit->priv->editor),
-                                     edit->config, _moo_lang_id (lang));
+        _moo_lang_mgr_update_config (moo_lang_mgr_default (),
+                                     edit->config,
+                                     _moo_lang_id (lang));
         _moo_edit_update_config_from_global (edit);
     }
 
@@ -1192,7 +1279,7 @@ find_uri_atom (GdkDragContext *context)
     GList *targets;
     GdkAtom atom;
 
-    atom = gdk_atom_intern ("text/uri-list", FALSE);
+    atom = moo_atom_uri_list ();
     targets = context->targets;
 
     while (targets)
@@ -1402,14 +1489,14 @@ block_uncomment (GtkTextBuffer *buffer,
     }
 
     limit = *end;
-    found = moo_text_search_forward (start, comment_start, 0,
+    found = moo_text_search_forward (start, comment_start, (MooTextSearchFlags) 0,
                                      &start1, &start2,
                                      &limit);
 
     if (!found)
     {
         gtk_text_iter_set_line_offset (&limit, 0);
-        found = gtk_text_iter_backward_search (start, comment_start, 0,
+        found = gtk_text_iter_backward_search (start, comment_start, (GtkTextSearchFlags) 0,
                                                &start1, &start2,
                                                &limit);
     }
@@ -1418,14 +1505,14 @@ block_uncomment (GtkTextBuffer *buffer,
         return;
 
     limit = start2;
-    found = gtk_text_iter_backward_search (end, comment_end, 0,
+    found = gtk_text_iter_backward_search (end, comment_end, (GtkTextSearchFlags) 0,
                                            &end1, &end2, &limit);
 
     if (!found)
     {
         limit = *end;
         iter_to_line_end (&limit);
-        found = moo_text_search_forward (end, comment_end, 0,
+        found = moo_text_search_forward (end, comment_end, (MooTextSearchFlags) 0,
                                          &end1, &end2, &limit);
     }
 
@@ -1545,7 +1632,7 @@ _moo_edit_ensure_newline (MooEdit *edit)
     buffer = get_buffer (edit);
     gtk_text_buffer_get_end_iter (buffer, &iter);
 
-    if (!gtk_text_iter_starts_line (&iter) || gtk_text_iter_is_start (&iter))
+    if (!gtk_text_iter_starts_line (&iter))
         gtk_text_buffer_insert (buffer, &iter, "\n", -1);
 }
 
@@ -1633,7 +1720,7 @@ void
 _moo_edit_do_popup (MooEdit        *edit,
                     GdkEventButton *event)
 {
-    MooUIXML *xml;
+    MooUiXml *xml;
     MooEditWindow *window;
     GtkMenu *menu;
 
@@ -1641,8 +1728,8 @@ _moo_edit_do_popup (MooEdit        *edit,
     xml = moo_editor_get_doc_ui_xml (edit->priv->editor);
     g_return_if_fail (xml != NULL);
 
-    menu = moo_ui_xml_create_widget (xml, MOO_UI_MENU, "Editor/Popup", edit->priv->actions,
-                                     window ? MOO_WINDOW(window)->accel_group : NULL);
+    menu = (GtkMenu*) moo_ui_xml_create_widget (xml, MOO_UI_MENU, "Editor/Popup", edit->priv->actions,
+                                                window ? MOO_WINDOW(window)->accel_group : NULL);
     g_return_if_fail (menu != NULL);
     MOO_OBJECT_REF_SINK (menu);
 
@@ -1753,23 +1840,19 @@ progress_cancel_clicked (MooEdit *doc)
 static gboolean
 show_progress (MooEdit *edit)
 {
-    MooGladeXML *xml;
-    GtkButton *cancel;
+    ProgressDialogXml *xml;
 
     edit->priv->progress_timeout = 0;
 
     g_return_val_if_fail (!edit->priv->progress, FALSE);
 
-    xml = moo_glade_xml_new_from_buf (mooeditprogress_glade_xml, -1,
-                                      "eventbox", GETTEXT_PACKAGE, NULL);
-    g_return_val_if_fail (xml != NULL, FALSE);
+    xml = progress_dialog_xml_new ();
 
-    edit->priv->progress = moo_glade_xml_get_widget (xml, "eventbox");
-    edit->priv->progressbar = moo_glade_xml_get_widget (xml, "progressbar");
+    edit->priv->progress = GTK_WIDGET (xml->ProgressDialog);
+    edit->priv->progressbar = GTK_WIDGET (xml->progressbar);
     g_assert (GTK_IS_WIDGET (edit->priv->progressbar));
 
-    cancel = moo_glade_xml_get_widget (xml, "cancel");
-    g_signal_connect_swapped (cancel, "clicked",
+    g_signal_connect_swapped (xml->cancel, "clicked",
                               G_CALLBACK (progress_cancel_clicked),
                               MOO_EDIT (edit));
 
@@ -1785,7 +1868,6 @@ show_progress (MooEdit *edit)
                               (GSourceFunc) pulse_progress,
                               edit);
 
-    g_object_unref (xml);
     return FALSE;
 }
 
@@ -1836,3 +1918,279 @@ _moo_edit_set_state (MooEdit        *edit,
         edit->priv->progress_text = g_strdup (text);
     }
 }
+
+
+#ifdef MOO_ENABLE_UNIT_TESTS
+
+#include <mooedit/mooedit-tests.h>
+#include <mooutils/mooutils-fs.h>
+#include <mooutils/moocompat.h>
+
+static struct {
+    char *working_dir;
+    char *encodings_dir;
+} test_data;
+
+#ifdef __WIN32__
+#define LE "\r\n"
+#else
+#define LE "\n"
+#endif
+
+#define TT1 "blah blah blah"
+#define TT2 "blah blah blah" LE "blah blah blah"
+#define TT3 LE LE LE LE
+#define TT4 "lala\nlala\n"
+#define TT5 "lala\r\nlala\r\n"
+#define TT6 "lala\rlala\r"
+
+static void
+check_contents (const char *filename,
+                const char *expected)
+{
+    char *contents;
+    GError *error = NULL;
+
+    if (!g_file_get_contents (filename, &contents, NULL, &error))
+    {
+        TEST_FAILED_MSG ("could not load file '%s': %s",
+                         filename, error->message);
+        g_error_free (error);
+        return;
+    }
+
+    TEST_ASSERT_STR_EQ (contents, expected);
+
+    g_free (contents);
+}
+
+static void
+test_basic (void)
+{
+    MooEditor *editor;
+    MooEdit *doc, *doc2;
+    GtkTextBuffer *buffer;
+    char *filename;
+
+    editor = moo_editor_instance ();
+    filename = g_build_filename (test_data.working_dir, "test.txt", NULL);
+    doc = moo_editor_new_file (editor, NULL, NULL, filename, NULL);
+    TEST_ASSERT (doc != NULL);
+    TEST_ASSERT (moo_edit_save (doc, NULL));
+    check_contents (filename, "");
+
+    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (doc));
+
+    gtk_text_buffer_set_text (buffer, TT1, -1);
+    TEST_ASSERT (moo_edit_save (doc, NULL));
+    check_contents (filename, TT1);
+
+    gtk_text_buffer_set_text (buffer, TT2, -1);
+    TEST_ASSERT (moo_edit_save (doc, NULL));
+    check_contents (filename, TT2);
+
+    gtk_text_buffer_set_text (buffer, TT2 LE, -1);
+    TEST_ASSERT (moo_edit_save (doc, NULL));
+    check_contents (filename, TT2 LE);
+
+    gtk_text_buffer_set_text (buffer, TT3, -1);
+    TEST_ASSERT (moo_edit_save (doc, NULL));
+    check_contents (filename, TT3);
+
+    doc2 = moo_editor_open_file (editor, NULL, NULL, filename, NULL);
+    TEST_ASSERT (doc2 == doc);
+
+    TEST_ASSERT (moo_edit_close (doc, TRUE));
+    TEST_ASSERT (moo_editor_get_doc (editor, filename) == NULL);
+
+    g_file_set_contents (filename, TT4, -1, NULL);
+    doc = moo_editor_open_file (editor, NULL, NULL, filename, NULL);
+    TEST_ASSERT (doc != NULL);
+    TEST_ASSERT (moo_edit_save (doc, NULL));
+    check_contents (filename, TT4);
+    TEST_ASSERT (moo_edit_close (doc, TRUE));
+
+    g_file_set_contents (filename, TT5, -1, NULL);
+    doc = moo_editor_open_file (editor, NULL, NULL, filename, NULL);
+    TEST_ASSERT (doc != NULL);
+    TEST_ASSERT (moo_edit_save (doc, NULL));
+    check_contents (filename, TT5);
+    TEST_ASSERT (moo_edit_close (doc, TRUE));
+
+    g_file_set_contents (filename, TT6, -1, NULL);
+    doc = moo_editor_open_file (editor, NULL, NULL, filename, NULL);
+    TEST_ASSERT (doc != NULL);
+    TEST_ASSERT (moo_edit_save (doc, NULL));
+    check_contents (filename, TT6);
+    TEST_ASSERT (moo_edit_close (doc, TRUE));
+
+    g_free (filename);
+}
+
+#define TEST_ASSERT_SAME_FILE_CONTENT(filename1, filename2)             \
+{                                                                       \
+    char *contents1__ = NULL;                                           \
+    char *contents2__ = NULL;                                           \
+    g_file_get_contents (filename1, &contents1__, NULL, NULL);          \
+    g_file_get_contents (filename2, &contents2__, NULL, NULL);          \
+    if (!contents1__)                                                   \
+        moo_test_assert_msg (FALSE, __FILE__, __LINE__,                 \
+                             "could not open file %s",                  \
+                             filename1);                                \
+    if (!contents2__)                                                   \
+        moo_test_assert_msg (FALSE, __FILE__, __LINE__,                 \
+                             "could not open file %s",                  \
+                             filename2);                                \
+    if (contents1__ && contents2__)                                     \
+    {                                                                   \
+        gboolean equal = strcmp (contents1__, contents2__) == 0;        \
+        TEST_ASSERT_MSG (equal, "contents of %s and %s differ",         \
+                         filename1, filename2);                         \
+    }                                                                   \
+    g_free (contents2__);                                               \
+    g_free (contents1__);                                               \
+}
+
+static void
+test_encodings_1 (const char *name,
+                  const char *working_dir)
+{
+    char *filename;
+    char *filename2;
+    char *encoding;
+    const char *dot;
+    MooEditor *editor;
+    MooEdit *doc;
+
+    if ((dot = strchr (name, '.')))
+        encoding = g_strndup (name, dot - name);
+    else
+        encoding = g_strdup (name);
+
+    filename = g_build_filename (test_data.encodings_dir, name, (char*)0);
+    filename2 = g_build_filename (working_dir, name, (char*)0);
+
+    editor = moo_editor_instance ();
+    doc = moo_editor_open_file (editor, NULL, NULL, filename, encoding);
+    TEST_ASSERT (doc != NULL);
+
+    if (doc)
+    {
+        TEST_ASSERT (moo_edit_save_as (doc, filename2, NULL, NULL));
+        TEST_ASSERT_SAME_FILE_CONTENT (filename2, filename);
+        TEST_ASSERT (moo_edit_close (doc, TRUE));
+    }
+
+    g_free (encoding);
+    g_free (filename2);
+    g_free (filename);
+}
+
+static void
+test_encodings (void)
+{
+    GDir *dir;
+    const char *name;
+    char *working_dir;
+
+    dir = g_dir_open (test_data.encodings_dir, 0, NULL);
+
+    if (!dir)
+    {
+        g_critical ("could not open encodings dir");
+        TEST_ASSERT (FALSE);
+        return;
+    }
+
+    working_dir = g_build_filename (test_data.working_dir, "encodings", (char*)0);
+    _moo_mkdir_with_parents (working_dir);
+
+    while ((name = g_dir_read_name (dir)))
+        test_encodings_1 (name, working_dir);
+
+    g_free (working_dir);
+    g_dir_close (dir);
+}
+
+static gboolean
+test_suite_init (G_GNUC_UNUSED gpointer data)
+{
+    /* try to avoid accidental deletion of good stuff */
+    if (strcmp (g_get_prgname(), "run-tests") != 0)
+    {
+        g_critical ("fix the program name!");
+        return FALSE;
+    }
+
+    test_data.working_dir = g_build_filename (moo_test_get_working_dir (),
+                                              "editor-work", (char*)0);
+    test_data.encodings_dir = g_build_filename (moo_test_get_data_dir (),
+                                                "encodings", (char*)0);
+
+    if (_moo_mkdir_with_parents (test_data.working_dir) != 0)
+    {
+        g_critical ("could not create directory '%s'",
+                    test_data.working_dir);
+        g_free (test_data.working_dir);
+        test_data.working_dir = NULL;
+        return FALSE;
+    }
+
+    moo_editor_create_instance (FALSE);
+    return TRUE;
+}
+
+static void
+test_suite_cleanup (G_GNUC_UNUSED gpointer data)
+{
+    GError *error = NULL;
+    char *recent_file;
+    char *cache_dir;
+    MooEditor *editor;
+
+//     if (!_moo_remove_dir (test_data.working_dir, TRUE, &error))
+//     {
+//         g_critical ("could not remove directory '%s': %s",
+//                     test_data.working_dir, error->message);
+//         g_error_free (error);
+//         error = NULL;
+//     }
+
+    g_free (test_data.working_dir);
+    g_free (test_data.encodings_dir);
+    test_data.working_dir = NULL;
+    test_data.encodings_dir = NULL;
+
+    editor = moo_editor_instance ();
+    moo_editor_close_all (editor, FALSE, FALSE);
+    recent_file = _md_history_mgr_get_filename (_moo_editor_get_history_mgr (editor));
+    g_object_unref (moo_editor_instance ());
+
+    if (!g_file_test (recent_file, G_FILE_TEST_EXISTS))
+        g_critical ("recent file %s does not exist", recent_file);
+
+    cache_dir = moo_get_user_cache_dir ();
+    if (!_moo_remove_dir (cache_dir, TRUE, &error))
+    {
+        g_critical ("could not remove directory '%s': %s",
+                    cache_dir, error->message);
+        g_error_free (error);
+        error = NULL;
+    }
+
+    g_free (cache_dir);
+    g_free (recent_file);
+}
+
+void
+moo_test_editor (void)
+{
+    MooTestSuite *suite = moo_test_suite_new ("Editor",
+                                              test_suite_init,
+                                              test_suite_cleanup,
+                                              NULL);
+    moo_test_suite_add_test (suite, "basic", (MooTestFunc) test_basic, NULL);
+    moo_test_suite_add_test (suite, "encodings", (MooTestFunc) test_encodings, NULL);
+}
+
+#endif
