@@ -1,13 +1,16 @@
 /*
  *   mooedit.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_EDIT_H
@@ -15,6 +18,7 @@
 
 #include <mooedit/mootextview.h>
 #include <mooedit/mooeditconfig.h>
+#include <mooedit/mooedit-enums.h>
 #include <mooutils/mooprefs.h>
 
 G_BEGIN_DECLS
@@ -29,42 +33,22 @@ G_BEGIN_DECLS
 #define MOO_IS_EDIT_CLASS(klass)            (G_TYPE_CHECK_CLASS_TYPE ((klass), MOO_TYPE_EDIT))
 #define MOO_EDIT_GET_CLASS(obj)             (G_TYPE_INSTANCE_GET_CLASS ((obj), MOO_TYPE_EDIT, MooEditClass))
 
-typedef enum /*< flags >*/
-{
-    MOO_EDIT_MODIFIED_ON_DISK   = 1 << 0,
-    MOO_EDIT_DELETED            = 1 << 1,
-    MOO_EDIT_CHANGED_ON_DISK    = MOO_EDIT_MODIFIED_ON_DISK | MOO_EDIT_DELETED,
-    MOO_EDIT_MODIFIED           = 1 << 2,
-    MOO_EDIT_NEW                = 1 << 3,
-    MOO_EDIT_CLEAN              = 1 << 4  /* doesn't prompt if it's closed, even if it's modified*/
-} MooEditStatus;
-
-typedef enum
-{
-    MOO_EDIT_STATE_NORMAL,
-    MOO_EDIT_STATE_LOADING,
-    MOO_EDIT_STATE_SAVING,
-    MOO_EDIT_STATE_PRINTING
-} MooEditState;
-
 #define MOO_EDIT_IS_MODIFIED(edit)  (moo_edit_get_status (edit) & MOO_EDIT_MODIFIED)
 #define MOO_EDIT_IS_CLEAN(edit)     (moo_edit_get_status (edit) & MOO_EDIT_CLEAN)
 #define MOO_EDIT_IS_BUSY(edit)      (moo_edit_get_state (edit) != MOO_EDIT_STATE_NORMAL)
 
-typedef struct _MooEditFileInfo MooEditFileInfo;
-typedef struct _MooEdit         MooEdit;
-typedef struct _MooEditPrivate  MooEditPrivate;
-typedef struct _MooEditClass    MooEditClass;
+typedef struct MooEdit         MooEdit;
+typedef struct MooEditPrivate  MooEditPrivate;
+typedef struct MooEditClass    MooEditClass;
 
-
-struct _MooEdit
+struct MooEdit
 {
     MooTextView parent;
     MooEditConfig *config;
     MooEditPrivate *priv;
 };
 
-struct _MooEditClass
+struct MooEditClass
 {
     MooTextViewClass parent_class;
 
@@ -91,10 +75,23 @@ GType            moo_edit_file_info_get_type    (void) G_GNUC_CONST;
 
 char            *moo_edit_get_uri               (MooEdit        *edit);
 char            *moo_edit_get_filename          (MooEdit        *edit);
+char            *moo_edit_get_norm_filename     (MooEdit        *edit);
 const char      *moo_edit_get_display_name      (MooEdit        *edit);
 const char      *moo_edit_get_display_basename  (MooEdit        *edit);
 
 const char      *moo_edit_get_encoding          (MooEdit        *edit);
+void             moo_edit_set_encoding          (MooEdit        *edit,
+                                                 const char     *encoding);
+
+#ifdef __WIN32__
+#define MOO_LE_DEFAULT MOO_LE_WIN32
+#else
+#define MOO_LE_DEFAULT MOO_LE_UNIX
+#endif
+
+MooLineEndType   moo_edit_get_line_end_type     (MooEdit        *edit);
+void             moo_edit_set_line_end_type     (MooEdit        *edit,
+                                                 MooLineEndType  le);
 
 gboolean         moo_edit_is_empty              (MooEdit        *edit);
 gboolean         moo_edit_is_untitled           (MooEdit        *edit);
@@ -107,11 +104,6 @@ MooEditStatus    moo_edit_get_status            (MooEdit        *edit);
 void             moo_edit_status_changed        (MooEdit        *edit);
 MooEditState     moo_edit_get_state             (MooEdit        *edit);
 
-MooEditFileInfo *moo_edit_file_info_new         (const char     *filename,
-                                                 const char     *encoding);
-MooEditFileInfo *moo_edit_file_info_copy        (const MooEditFileInfo *info);
-void             moo_edit_file_info_free        (MooEditFileInfo       *info);
-
 gboolean         moo_edit_close                 (MooEdit        *edit,
                                                  gboolean        ask_confirm);
 gboolean         moo_edit_save                  (MooEdit        *edit,
@@ -128,6 +120,14 @@ gboolean         moo_edit_save_copy             (MooEdit        *edit,
 void             moo_edit_comment               (MooEdit        *edit);
 void             moo_edit_uncomment             (MooEdit        *edit);
 
+typedef struct MooEditFileInfo MooEditFileInfo;
+MooEditFileInfo *moo_edit_file_info_new_path    (const char         *path,
+                                                 const char         *encoding);
+MooEditFileInfo *moo_edit_file_info_new_uri     (const char         *uri,
+                                                 const char         *encoding);
+MooEditFileInfo *moo_edit_file_info_copy        (MooEditFileInfo    *info);
+void             moo_edit_file_info_free        (MooEditFileInfo    *info);
+
 
 G_END_DECLS
 
