@@ -1,19 +1,21 @@
 /*
  *   mooeditaction.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
 #include "mooedit/mooeditaction.h"
 #include "mooedit/mooeditaction-factory.h"
-#include "mooedit/mooedit-private.h"
 #include "mooedit/mooeditfiltersettings.h"
 #include <glib/gregex.h>
 #include <string.h>
@@ -121,7 +123,12 @@ moo_edit_action_set_file_filter (MooEditAction *action,
                                  const char    *string)
 {
     _moo_edit_filter_free (action->priv->file_filter);
-    action->priv->file_filter = _moo_edit_filter_new (string);
+
+    if (string && string[0])
+        action->priv->file_filter = _moo_edit_filter_new (string);
+    else
+        action->priv->file_filter = NULL;
+
     g_object_notify (G_OBJECT (action), "file-filter");
 }
 
