@@ -1,19 +1,23 @@
 /*
  *   mooeditconfig.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_EDIT_CONFIG_H
 #define MOO_EDIT_CONFIG_H
 
 #include <glib-object.h>
+#include <mooedit/mooedit-enums.h>
 
 #ifndef G_GNUC_NULL_TERMINATED
 #if __GNUC__ >= 4
@@ -37,16 +41,6 @@ typedef struct MooEditConfig         MooEditConfig;
 typedef struct MooEditConfigPrivate  MooEditConfigPrivate;
 typedef struct MooEditConfigClass    MooEditConfigClass;
 
-
-typedef enum
-{
-    MOO_EDIT_CONFIG_SOURCE_USER     = 0,
-    MOO_EDIT_CONFIG_SOURCE_FILE     = 10,
-    MOO_EDIT_CONFIG_SOURCE_FILENAME = 20,
-    MOO_EDIT_CONFIG_SOURCE_LANG     = 30,
-    MOO_EDIT_CONFIG_SOURCE_AUTO     = 40
-} MooEditConfigSource;
-
 struct MooEditConfig
 {
     GObject object;
