@@ -1,55 +1,45 @@
 /*
  *   mooeditdialogs.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
 #include "mooedit/mooeditdialogs.h"
-#include "mooedit/mooedit-private.h"
 #include "mooedit/mooeditprefs.h"
-#include "mooedit/mootextfind-glade.h"
-#include "mooedit/mooeditfileops.h"
-#include "mooedit/mooeditsavemult-glade.h"
+#include "mooedit/mooedit-fileops.h"
 #include "mooutils/moodialogs.h"
 #include "mooutils/moocompat.h"
 #include "mooutils/moostock.h"
-#include "mooutils/mooglade.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/mooencodings.h"
+#include "glade/mootextfind-prompt-gxml.h"
+#include "glade/mooeditsavemult-gxml.h"
 #include <gtk/gtk.h>
 #include <glib/gregex.h>
 #include <string.h>
 
 
-static void
-open_dialog_created (G_GNUC_UNUSED MooFileDialog *fd,
-                     GtkWidget *widget)
-{
-    _moo_window_set_remember_size (GTK_WINDOW (widget),
-                                   moo_edit_setting (MOO_EDIT_PREFS_DIALOG_OPEN),
-                                   TRUE);
-}
-
 GSList *
-_moo_edit_open_dialog (GtkWidget      *widget,
-                       MooFilterMgr   *mgr,
-                       MooEdit        *current_doc)
+_moo_edit_open_dialog (GtkWidget *widget,
+                       MooEdit   *current_doc)
 {
     MooFileDialog *dialog;
     const char *start_dir = NULL;
     const char *encoding;
     char *new_start;
     char *freeme = NULL;
-    GSList *filenames = NULL;
+    char **filenames = NULL, **p;
     GSList *infos = NULL;
-    GSList *l;
 
     moo_prefs_create_key (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR), MOO_PREFS_STATE, G_TYPE_STRING, NULL);
 
@@ -72,11 +62,10 @@ _moo_edit_open_dialog (GtkWidget      *widget,
                                   TRUE, GTK_STOCK_OPEN, start_dir,
                                   NULL);
     g_object_set (dialog, "enable-encodings", TRUE, NULL);
-    g_signal_connect (dialog, "dialog-created", G_CALLBACK (open_dialog_created), NULL);
     moo_file_dialog_set_help_id (dialog, "dialog-open");
+    moo_file_dialog_set_remember_size (dialog, moo_edit_setting (MOO_EDIT_PREFS_DIALOG_OPEN));
 
-    if (mgr)
-        moo_file_dialog_set_filter_mgr (dialog, mgr, "MooEdit");
+    moo_file_dialog_set_filter_mgr_id (dialog, "MooEdit");
 
     if (!moo_file_dialog_run (dialog))
         goto out;
@@ -89,30 +78,29 @@ _moo_edit_open_dialog (GtkWidget      *widget,
     filenames = moo_file_dialog_get_filenames (dialog);
     g_return_val_if_fail (filenames != NULL, NULL);
 
-    for (l = filenames; l != NULL; l = l->next)
-        infos = g_slist_prepend (infos, moo_edit_file_info_new (l->data, encoding));
+    for (p = filenames; *p != NULL; ++p)
+        infos = g_slist_prepend (infos, moo_edit_file_info_new_path (*p, encoding));
     infos = g_slist_reverse (infos);
 
-    new_start = g_path_get_dirname (filenames->data);
+    new_start = g_path_get_dirname (filenames[0]);
     moo_prefs_set_filename (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR), new_start);
     g_free (new_start);
 
 out:
     g_free (freeme);
     g_object_unref (dialog);
-    g_slist_foreach (filenames, (GFunc) g_free, NULL);
-    g_slist_free (filenames);
+    g_strfreev (filenames);
     return infos;
 }
 
 
 MooEditFileInfo *
-_moo_edit_save_as_dialog (MooEdit        *edit,
-                          MooFilterMgr   *mgr,
-                          const char     *display_basename)
+_moo_edit_save_as_dialog (MooEdit    *edit,
+                          const char *display_basename)
 {
     const char *start = NULL;
-    const char *filename = NULL;
+    char *filename = NULL;
+    char *freeme = NULL;
     const char *encoding;
     char *new_start;
     MooFileDialog *dialog;
@@ -120,7 +108,21 @@ _moo_edit_save_as_dialog (MooEdit        *edit,
 
     moo_prefs_create_key (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR),
                           MOO_PREFS_STATE, G_TYPE_STRING, NULL);
-    start = moo_prefs_get_filename (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR));
+
+    if (moo_prefs_get_bool (moo_edit_setting (MOO_EDIT_PREFS_DIALOGS_OPEN_FOLLOWS_DOC)))
+    {
+        char *this_filename = moo_edit_get_filename (edit);
+
+        if (this_filename)
+        {
+            freeme = g_path_get_dirname (this_filename);
+            start = freeme;
+            g_free (this_filename);
+        }
+    }
+
+    if (!start)
+        start = moo_prefs_get_filename (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR));
 
     dialog = moo_file_dialog_new (MOO_FILE_DIALOG_SAVE, GTK_WIDGET (edit),
                                   FALSE, GTK_STOCK_SAVE_AS, start, display_basename);
@@ -128,25 +130,27 @@ _moo_edit_save_as_dialog (MooEdit        *edit,
     moo_file_dialog_set_encoding (dialog, moo_edit_get_encoding (edit));
     moo_file_dialog_set_help_id (dialog, "dialog-save");
 
-    if (mgr)
-        moo_file_dialog_set_filter_mgr (dialog, mgr, "MooEdit");
+    moo_file_dialog_set_filter_mgr_id (dialog, "MooEdit");
 
     if (!moo_file_dialog_run (dialog))
     {
         g_object_unref (dialog);
+        g_free (freeme);
         return NULL;
     }
 
     encoding = moo_file_dialog_get_encoding (dialog);
     filename = moo_file_dialog_get_filename (dialog);
     g_return_val_if_fail (filename != NULL, NULL);
-    file_info = moo_edit_file_info_new (filename, encoding);
+    file_info = moo_edit_file_info_new_path (filename, encoding);
 
     new_start = g_path_get_dirname (filename);
     moo_prefs_set_filename (moo_edit_setting (MOO_EDIT_PREFS_LAST_DIR), new_start);
     g_free (new_start);
 
+    g_free (filename);
     g_object_unref (dialog);
+    g_free (freeme);
     return file_info;
 }
 
@@ -284,15 +288,13 @@ files_treeview_init (GtkTreeView *treeview, GtkWidget *dialog, GSList  *docs)
 
 
 static GSList *
-files_treeview_get_to_save (GtkWidget *treeview)
+files_treeview_get_to_save (GtkTreeView *treeview)
 {
     GtkTreeIter iter;
     GtkTreeModel *model;
     GSList *list = NULL;
 
-    g_return_val_if_fail (GTK_IS_TREE_VIEW (treeview), NULL);
-
-    model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
+    model = gtk_tree_view_get_model (treeview);
     g_return_val_if_fail (model != NULL, NULL);
 
     gtk_tree_model_get_iter_first (model, &iter);
@@ -323,11 +325,11 @@ _moo_edit_save_multiple_changes_dialog (GSList  *docs,
                                         GSList **to_save)
 {
     GSList *l;
-    GtkWidget *dialog, *label, *treeview;
+    GtkWidget *dialog;
     char *msg, *question;
     int response;
     MooSaveChangesDialogResponse retval;
-    MooGladeXML *xml;
+    SaveMultDialogXml *xml;
 
     g_return_val_if_fail (docs != NULL, MOO_SAVE_CHANGES_RESPONSE_CANCEL);
     g_return_val_if_fail (docs->next != NULL, MOO_SAVE_CHANGES_RESPONSE_CANCEL);
@@ -336,9 +338,8 @@ _moo_edit_save_multiple_changes_dialog (GSList  *docs,
     for (l = docs; l != NULL; l = l->next)
         g_return_val_if_fail (MOO_IS_EDIT (l->data), MOO_SAVE_CHANGES_RESPONSE_CANCEL);
 
-    xml = moo_glade_xml_new_from_buf (mooeditsavemult_glade_xml, -1,
-                                      "dialog", GETTEXT_PACKAGE, NULL);
-    dialog = moo_glade_xml_get_widget (xml, "dialog");
+    xml = save_mult_dialog_xml_new ();
+    dialog = GTK_WIDGET (xml->SaveMultDialog);
 
     moo_window_set_parent (dialog, docs->data);
 
@@ -355,7 +356,6 @@ _moo_edit_save_multiple_changes_dialog (GSList  *docs,
                                              GTK_RESPONSE_NO,
                                              GTK_RESPONSE_CANCEL, -1);
 
-    label = moo_glade_xml_get_widget (xml, "label");
     question = g_strdup_printf (dngettext (GETTEXT_PACKAGE,
                                            /* Translators: number of documents here is always greater than one, so
                                               ignore singular form (which is simply copy of the plural here) */
@@ -367,10 +367,9 @@ _moo_edit_save_multiple_changes_dialog (GSList  *docs,
                                 g_slist_length (docs));
     msg = g_markup_printf_escaped ("<span weight=\"bold\" size=\"larger\">%s</span>",
                                    question);
-    gtk_label_set_markup (GTK_LABEL (label), msg);
+    gtk_label_set_markup (xml->label, msg);
 
-    treeview = moo_glade_xml_get_widget (xml, "treeview");
-    files_treeview_init (GTK_TREE_VIEW (treeview), dialog, docs);
+    files_treeview_init (xml->treeview, dialog, docs);
 
     response = gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -380,7 +379,7 @@ _moo_edit_save_multiple_changes_dialog (GSList  *docs,
             retval = MOO_SAVE_CHANGES_RESPONSE_DONT_SAVE;
             break;
         case GTK_RESPONSE_YES:
-            *to_save = files_treeview_get_to_save (treeview);
+            *to_save = files_treeview_get_to_save (xml->treeview);
             retval = MOO_SAVE_CHANGES_RESPONSE_SAVE;
             break;
         default:
@@ -390,7 +389,6 @@ _moo_edit_save_multiple_changes_dialog (GSList  *docs,
     g_free (question);
     g_free (msg);
     gtk_widget_destroy (dialog);
-    g_object_unref (xml);
     return retval;
 }
 
@@ -400,84 +398,70 @@ _moo_edit_save_multiple_changes_dialog (GSList  *docs,
  */
 
 void
-_moo_edit_save_error_dialog (GtkWidget      *widget,
-                             const char     *filename,
-                             GError         *error)
+_moo_edit_save_error_dialog (GtkWidget *widget,
+                             GFile     *file,
+                             GError    *error)
 {
-    char *filename_utf8, *msg = NULL;
+    char *filename, *msg = NULL;
 
-    g_return_if_fail (filename != NULL);
+    filename = _moo_file_get_display_name (file);
 
-    filename_utf8 = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
-
-    if (!filename_utf8)
-        g_critical ("%s: could not convert filename '%s' to utf8", G_STRLOC, filename);
-
-    if (filename_utf8)
+    if (filename)
         /* Could not save file foo.txt */
-        msg = g_strdup_printf (_("Could not save file\n%s"), filename_utf8);
+        msg = g_strdup_printf (_("Could not save file\n%s"), filename);
     else
         msg = g_strdup (_("Could not save file"));
 
     moo_error_dialog (widget, msg, error ? error->message : NULL);
 
     g_free (msg);
-    g_free (filename_utf8);
+    g_free (filename);
 }
 
 void
 _moo_edit_save_error_enc_dialog (GtkWidget  *widget,
-                                 const char *filename,
+                                 GFile      *file,
                                  const char *encoding)
 {
-    char *filename_utf8, *msg = NULL;
+    char *filename, *msg = NULL;
     char *secondary;
 
-    g_return_if_fail (filename != NULL);
     g_return_if_fail (encoding != NULL);
 
-    filename_utf8 = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
+    filename = _moo_file_get_display_name (file);
 
-    if (!filename_utf8)
-        g_critical ("%s: could not convert filename '%s' to utf8", G_STRLOC, filename);
-
-    if (filename_utf8)
+    if (filename)
         /* Error saving file foo.txt */
-        msg = g_strdup_printf (_("Error saving file\n%s"), filename_utf8);
+        msg = g_strdup_printf (_("Error saving file\n%s"), filename);
     else
         msg = g_strdup (_("Error saving file"));
 
-    secondary = g_strdup_printf (_("Could not convert file to requested character "
-                                   "encoding %s. File was saved in UTF-8 encoding."),
+    secondary = g_strdup_printf (_("Could not convert file to encoding %s. "
+                                   "File was saved in UTF-8 encoding."),
                                  encoding);
 
     moo_error_dialog (widget, msg, secondary);
 
     g_free (msg);
     g_free (secondary);
-    g_free (filename_utf8);
+    g_free (filename);
 }
 
 
 void
-_moo_edit_open_error_dialog (GtkWidget      *widget,
-                             const char     *filename,
-                             const char     *encoding,
-                             GError         *error)
+_moo_edit_open_error_dialog (GtkWidget  *widget,
+                             GFile      *file,
+                             const char *encoding,
+                             GError     *error)
 {
-    char *filename_utf8, *msg = NULL;
+    char *filename, *msg = NULL;
     char *secondary;
 
-    g_return_if_fail (filename != NULL);
+    filename = _moo_file_get_display_name (file);
 
-    filename_utf8 = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
-
-    if (!filename_utf8)
-        g_critical ("%s: could not convert filename '%s' to utf8", G_STRLOC, filename);
-
-    if (filename_utf8)
+    if (filename)
         /* Could not open file foo.txt */
-        msg = g_strdup_printf (_("Could not open file\n%s"), filename_utf8);
+        msg = g_strdup_printf (_("Could not open file\n%s"), filename);
     else
         msg = g_strdup (_("Could not open file"));
 
@@ -502,7 +486,7 @@ _moo_edit_open_error_dialog (GtkWidget      *widget,
 
     g_free (msg);
     g_free (secondary);
-    g_free (filename_utf8);
+    g_free (filename);
 }
 
 
@@ -549,7 +533,7 @@ moo_edit_question_dialog (MooEdit    *edit,
     dialog = gtk_message_dialog_new (parent, GTK_DIALOG_MODAL,
                                      GTK_MESSAGE_WARNING,
                                      GTK_BUTTONS_NONE,
-                                     text);
+                                     "%s", text);
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
                                               "%s", secondary);
 
@@ -640,7 +624,7 @@ _moo_text_search_from_start_dialog (GtkWidget *widget,
 
     dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
                                      GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
-                                     msg);
+                                     "%s", msg);
     moo_window_set_parent (dialog, widget);
 
     gtk_dialog_add_buttons (GTK_DIALOG (dialog),
@@ -691,7 +675,7 @@ _moo_text_regex_error_dialog (GtkWidget  *parent,
     dialog = gtk_message_dialog_new (NULL,
                                      GTK_DIALOG_MODAL,
                                      GTK_MESSAGE_ERROR, GTK_BUTTONS_NONE,
-                                     msg_text);
+                                     "%s", msg_text);
     moo_window_set_parent (dialog, parent);
     gtk_dialog_add_buttons (GTK_DIALOG (dialog), GTK_STOCK_CLOSE,
                             GTK_RESPONSE_CANCEL, NULL);
@@ -704,19 +688,11 @@ _moo_text_regex_error_dialog (GtkWidget  *parent,
 }
 
 
-GtkWidget*
+GtkWidget *
 _moo_text_prompt_on_replace_dialog (GtkWidget *parent)
 {
-    GtkWidget *dialog;
-    MooGladeXML *xml;
-
-    xml = moo_glade_xml_new_from_buf (mootextfind_glade_xml, -1,
-                                      "prompt_on_replace_dialog",
-                                      GETTEXT_PACKAGE, NULL);
-    dialog = moo_glade_xml_get_widget (xml, "prompt_on_replace_dialog");
-    g_object_unref (xml);
-
-    moo_window_set_parent (dialog, parent);
-
-    return dialog;
+    FindPromptDialogXml *xml;
+    xml = find_prompt_dialog_xml_new ();
+    moo_window_set_parent (GTK_WIDGET (xml->FindPromptDialog), parent);
+    return GTK_WIDGET (xml->FindPromptDialog);
 }
