@@ -1,30 +1,31 @@
 /*
  *   mooeditdialogs.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_EDIT_DIALOGS_H
 #define MOO_EDIT_DIALOGS_H
 
-#include "mooedit/mooedit.h"
-#include "mooutils/moofiltermgr.h"
 #include "mooutils/moodialogs.h"
+#include "mooedit/mooedit.h"
+#include <gio/gio.h>
 
 G_BEGIN_DECLS
 
 
 MooEditFileInfo *_moo_edit_save_as_dialog           (MooEdit        *edit,
-                                                     MooFilterMgr   *mgr,
                                                      const char     *display_basename);
 GSList          *_moo_edit_open_dialog              (GtkWidget      *widget,
-                                                     MooFilterMgr   *mgr,
                                                      MooEdit        *current_doc);
 
 MooSaveChangesDialogResponse
@@ -37,13 +38,13 @@ gboolean         _moo_edit_reload_modified_dialog   (MooEdit        *edit);
 gboolean         _moo_edit_overwrite_modified_dialog (MooEdit        *edit);
 
 void             _moo_edit_save_error_dialog        (GtkWidget      *widget,
-                                                     const char     *filename,
+                                                     GFile          *file,
                                                      GError         *error);
 void             _moo_edit_save_error_enc_dialog    (GtkWidget      *widget,
-                                                     const char     *filename,
+                                                     GFile          *file,
                                                      const char     *encoding);
 void             _moo_edit_open_error_dialog        (GtkWidget      *widget,
-                                                     const char     *filename,
+                                                     GFile          *file,
                                                      const char     *encoding,
                                                      GError         *error);
 void             _moo_edit_reload_error_dialog      (MooEdit        *doc,
