@@ -1,13 +1,16 @@
 /*
  *   mooeditfiltersettings.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
@@ -19,9 +22,7 @@
 #include "mooutils/mooutils-debug.h"
 #include <glib/gregex.h>
 #include <string.h>
-#ifndef __WIN32__
 #include <fnmatch.h>
-#endif
 
 
 MOO_DEBUG_INIT(filters, FALSE)
@@ -100,7 +101,11 @@ _moo_edit_filter_new_regex (const char *string)
 
     g_return_val_if_fail (string != NULL, NULL);
 
-    regex = g_regex_new (string, G_REGEX_OPTIMIZE, 0, &error);
+    regex = g_regex_new (string, G_REGEX_OPTIMIZE
+#ifdef __WIN32__
+                                | G_REGEX_CASELESS
+#endif
+                         , 0, &error);
 
     if (!regex)
     {
@@ -293,7 +298,11 @@ filter_setting_new (const char *filter,
 
     setting = g_new0 (FilterSetting, 1);
 
-    setting->regex = g_regex_new (filter, G_REGEX_DOTALL | G_REGEX_OPTIMIZE, 0, &error);
+    setting->regex = g_regex_new (filter, G_REGEX_DOTALL | G_REGEX_OPTIMIZE
+#ifdef __WIN32__
+                                          | G_REGEX_CASELESS
+#endif
+                                  , 0, &error);
     setting->config = g_strdup (config);
 
     if (!setting->regex)
