@@ -1,23 +1,23 @@
 /*
  *   mooeditor.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_EDITOR_H
 #define MOO_EDITOR_H
 
 #include <mooedit/mooeditwindow.h>
-#include <mooedit/moolangmgr.h>
 #include <mooutils/moouixml.h>
-#include <mooutils/moohistorylist.h>
-#include <mooutils/moofiltermgr.h>
 
 G_BEGIN_DECLS
 
@@ -29,19 +29,19 @@ G_BEGIN_DECLS
 #define MOO_IS_EDITOR_CLASS(klass)      (G_TYPE_CHECK_CLASS_TYPE ((klass), MOO_TYPE_EDITOR))
 #define MOO_EDITOR_GET_CLASS(obj)       (G_TYPE_INSTANCE_GET_CLASS ((obj), MOO_TYPE_EDITOR, MooEditorClass))
 
-typedef struct _MooEditor               MooEditor;
-typedef struct _MooEditorPrivate        MooEditorPrivate;
-typedef struct _MooEditorClass          MooEditorClass;
+typedef struct MooEditor        MooEditor;
+typedef struct MooEditorPrivate MooEditorPrivate;
+typedef struct MooEditorClass   MooEditorClass;
 
-struct _MooEditor
+struct MooEditor
 {
-    GObject                 parent;
-    MooEditorPrivate       *priv;
+    GObject base;
+    MooEditorPrivate *priv;
 };
 
-struct _MooEditorClass
+struct MooEditorClass
 {
-    GObjectClass parent_class;
+    GObjectClass base_class;
 
     gboolean (*close_window) (MooEditor     *editor,
                               MooEditWindow *window,
@@ -52,7 +52,7 @@ struct _MooEditorClass
 GType            moo_editor_get_type        (void) G_GNUC_CONST;
 
 MooEditor       *moo_editor_instance        (void);
-MooEditor       *moo_editor_create_instance (void);
+MooEditor       *moo_editor_create_instance (gboolean        embedded);
 
 /* this creates 'windowless' MooEdit instance */
 MooEdit         *moo_editor_create_doc      (MooEditor      *editor,
@@ -90,6 +90,8 @@ MooEdit         *moo_editor_open_uri        (MooEditor      *editor,
 
 MooEdit         *moo_editor_get_doc         (MooEditor      *editor,
                                              const char     *filename);
+MooEdit         *moo_editor_get_doc_for_uri (MooEditor      *editor,
+                                             const char     *uri);
 
 MooEdit         *moo_editor_get_active_doc  (MooEditor      *editor);
 MooEditWindow   *moo_editor_get_active_window (MooEditor    *editor);
@@ -123,18 +125,14 @@ void             moo_editor_set_app_name    (MooEditor      *editor,
                                              const char     *name);
 const char      *moo_editor_get_app_name    (MooEditor      *editor);
 
-MooFilterMgr    *moo_editor_get_filter_mgr  (MooEditor      *editor);
-
-MooUIXML        *moo_editor_get_ui_xml      (MooEditor      *editor);
+MooUiXml        *moo_editor_get_ui_xml      (MooEditor      *editor);
 void             moo_editor_set_ui_xml      (MooEditor      *editor,
-                                             MooUIXML       *xml);
-MooUIXML        *moo_editor_get_doc_ui_xml  (MooEditor      *editor);
+                                             MooUiXml       *xml);
+MooUiXml        *moo_editor_get_doc_ui_xml  (MooEditor      *editor);
 
 MooEditor       *moo_edit_window_get_editor (MooEditWindow  *window);
 MooEditor       *moo_edit_get_editor        (MooEdit        *doc);
 
-MooLangMgr      *moo_editor_get_lang_mgr    (MooEditor      *editor);
-
 void             moo_editor_set_window_type (MooEditor      *editor,
                                              GType           type);
 void             moo_editor_set_edit_type   (MooEditor      *editor,
@@ -147,6 +145,7 @@ gboolean         moo_editor_save_copy       (MooEditor      *editor,
                                              GError        **error);
 
 void             moo_editor_apply_prefs     (MooEditor      *editor);
+void             moo_editor_queue_apply_prefs (MooEditor    *editor);
 
 void            _moo_editor_load_session    (MooEditor      *editor,
                                              MooMarkupNode  *xml);
@@ -155,18 +154,14 @@ void            _moo_editor_save_session    (MooEditor      *editor,
 
 
 enum {
-    MOO_EDIT_OPEN_NEW_WINDOW    = 1u << 0,
-    MOO_EDIT_OPEN_NEW_TAB       = 1u << 1
+    MOO_EDIT_OPEN_NEW_WINDOW = 1 << 0,
+    MOO_EDIT_OPEN_NEW_TAB    = 1 << 1,
+    MOO_EDIT_OPEN_RELOAD     = 1 << 2
 };
 
-char           *_moo_edit_filename_to_uri   (const char     *filename,
-                                             guint           line,
-                                             guint           options);
-char           *_moo_edit_uri_to_filename   (const char     *uri,
-                                             guint          *line,
-                                             guint          *options);
-void            _moo_editor_open_file       (MooEditor      *editor,
+void            _moo_editor_open_uri        (MooEditor      *editor,
                                              const char     *filename,
+                                             const char     *encoding,
                                              guint           line,
                                              guint           options);
 
