@@ -1,22 +1,26 @@
 /*
  *   mooeditprefs.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
 #include "mooedit/mooeditprefs.h"
-#include "mooedit/mooedit-private.h"
-#include "mooedit/mooeditfileops.h"
+#include "mooedit/mooedit-impl.h"
+#include "mooedit/mooedit-fileops.h"
 #include "mooedit/mootextview-private.h"
 #include "mooedit/mooedit-enums.h"
 #include "mooedit/mootextbuffer.h"
+#include "mooedit/moolangmgr.h"
 #include "mooutils/mooencodings.h"
 #include <string.h>
 
@@ -126,7 +130,6 @@ _moo_edit_init_prefs (void)
     NEW_KEY_BOOL (MOO_EDIT_PREFS_SHOW_LINE_NUMBERS, FALSE);
     NEW_KEY_BOOL (MOO_EDIT_PREFS_SHOW_TABS, FALSE);
     NEW_KEY_BOOL (MOO_EDIT_PREFS_SHOW_TRAILING_SPACES, FALSE);
-    NEW_KEY_BOOL (MOO_EDIT_PREFS_USE_DEFAULT_FONT, FALSE);
     NEW_KEY_STRING (MOO_EDIT_PREFS_FONT, "Monospace");
     NEW_KEY_FLAGS (MOO_EDIT_PREFS_QUICK_SEARCH_FLAGS,
                    MOO_TYPE_TEXT_SEARCH_FLAGS,
@@ -209,16 +212,12 @@ _moo_edit_apply_prefs (MooEdit *edit)
                   "backspace-indents", get_bool (MOO_EDIT_PREFS_BACKSPACE_INDENTS),
                   NULL);
 
-    if (get_bool (MOO_EDIT_PREFS_USE_DEFAULT_FONT))
-        moo_text_view_set_font_from_string (MOO_TEXT_VIEW (edit), NULL);
-    else
-        moo_text_view_set_font_from_string (MOO_TEXT_VIEW (edit),
-                                            get_string (MOO_EDIT_PREFS_FONT));
-
+    moo_text_view_set_font_from_string (MOO_TEXT_VIEW (edit),
+                                        get_string (MOO_EDIT_PREFS_FONT));
     _moo_text_view_set_line_numbers_font (MOO_TEXT_VIEW (edit),
                                           get_string (MOO_EDIT_PREFS_LINE_NUMBERS_FONT));
 
-    mgr = moo_editor_get_lang_mgr (edit->priv->editor);
+    mgr = moo_lang_mgr_default ();
     scheme = moo_lang_mgr_get_active_scheme (mgr);
 
     if (scheme)
