@@ -1,22 +1,25 @@
 /*
  *   mooeditwindow.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_EDIT_WINDOW_H
 #define MOO_EDIT_WINDOW_H
 
 #include <mooedit/mooedit.h>
+#include <mooedit/mooedit-enums.h>
 #include <mooutils/moowindow.h>
 #include <mooutils/moobigpaned.h>
-#include <gtk/gtkstatusbar.h>
 
 G_BEGIN_DECLS
 
@@ -28,18 +31,18 @@ G_BEGIN_DECLS
 #define MOO_IS_EDIT_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MOO_TYPE_EDIT_WINDOW))
 #define MOO_EDIT_WINDOW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), MOO_TYPE_EDIT_WINDOW, MooEditWindowClass))
 
-typedef struct _MooEditWindow           MooEditWindow;
-typedef struct _MooEditWindowPrivate    MooEditWindowPrivate;
-typedef struct _MooEditWindowClass      MooEditWindowClass;
+typedef struct MooEditWindow        MooEditWindow;
+typedef struct MooEditWindowPrivate MooEditWindowPrivate;
+typedef struct MooEditWindowClass   MooEditWindowClass;
 
-struct _MooEditWindow
+struct MooEditWindow
 {
     MooWindow               parent;
     MooEditWindowPrivate   *priv;
     MooBigPaned            *paned;
 };
 
-struct _MooEditWindowClass
+struct MooEditWindowClass
 {
     MooWindowClass          parent_class;
 
@@ -60,13 +63,6 @@ typedef gboolean (*MooActionCheckFunc)              (GtkAction      *action,
                                                      MooEditWindow  *window,
                                                      MooEdit        *doc,
                                                      gpointer        data);
-typedef enum
-{
-    MOO_ACTION_CHECK_SENSITIVE,
-    MOO_ACTION_CHECK_VISIBLE,
-    MOO_ACTION_CHECK_ACTIVE
-} MooActionCheckType;
-
 void         moo_edit_window_set_action_check       (const char     *action_id,
                                                      MooActionCheckType type,
                                                      MooActionCheckFunc func,
@@ -91,7 +87,7 @@ void         moo_edit_window_set_title_prefix       (MooEditWindow  *window,
                                                      const char     *prefix);
 
 /* sinks widget */
-gboolean     moo_edit_window_add_pane               (MooEditWindow  *window,
+MooPane     *moo_edit_window_add_pane               (MooEditWindow  *window,
                                                      const char     *user_id,
                                                      GtkWidget      *widget,
                                                      MooPaneLabel   *label,
@@ -114,9 +110,6 @@ void         moo_edit_window_present_output         (MooEditWindow  *window);
 GtkWidget   *moo_edit_window_get_output             (MooEditWindow  *window);
 GtkWidget   *moo_edit_window_get_output_pane        (MooEditWindow  *window);
 
-void         moo_edit_window_message                (MooEditWindow  *window,
-                                                     const char     *message);
-
 
 G_END_DECLS
 
