@@ -1,42 +1,176 @@
 /*
  *   moofold.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2008 by Daniel Poelzleithner <mooedit@poelzi.org>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
 #include "mooedit/mootext-private.h"
-#include "mooutils/moomarshals.h"
+#include "marshals.h"
 
 
-#ifdef MOO_DEBUG_ENABLED
+#ifdef MOO_DEBUG
 #define WANT_CHECKS 1
 #else
 #define WANT_CHECKS 0
 #endif
 
+static void     moo_fold_get_property       (GObject    *object,
+                                             guint       prop_id,
+                                             GValue     *value,
+                                             GParamSpec *pspec);
 static void     moo_fold_finalize           (GObject    *object);
 static void     moo_fold_free_recursively   (MooFold    *fold);
 
 static int      _moo_fold_get_end           (MooFold    *fold);
 
+enum {
+    PROP_0,
+    PROP_PARENT,
+    PROP_NEXT,
+    PROP_PREV,
+    PROP_CHILDREN,
+    PROP_N_CHILDREN,
+    PROP_MARK_START,
+    PROP_MARK_END,
+    PROP_COLLAPSED
+};
 
 /* MOO_TYPE_FOLD */
 G_DEFINE_TYPE (MooFold, moo_fold, G_TYPE_OBJECT)
 
-
 static void
 moo_fold_class_init (MooFoldClass *klass)
 {
     GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
     gobject_class->finalize = moo_fold_finalize;
+
+    gobject_class->get_property = moo_fold_get_property;
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_PARENT,
+                                     g_param_spec_object ("parent",
+                                             "parent",
+                                             "parent",
+                                             MOO_TYPE_FOLD,
+                                             G_PARAM_READABLE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_NEXT,
+                                     g_param_spec_object ("next",
+                                             "next",
+                                             "next",
+                                             MOO_TYPE_FOLD,
+                                             G_PARAM_READABLE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_PREV,
+                                     g_param_spec_object ("previous",
+                                             "previous",
+                                             "previous",
+                                             MOO_TYPE_FOLD,
+                                             G_PARAM_READABLE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_MARK_START,
+                                     g_param_spec_object ("mark_start",
+                                             "Start Marker",
+                                             "Marker of the start Marker",
+                                             MOO_TYPE_LINE_MARK,
+                                             G_PARAM_READABLE));
+
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_MARK_END,
+                                     g_param_spec_object ("mark_end",
+                                             "End Marker",
+                                             "Marker of the end Marker",
+                                             MOO_TYPE_LINE_MARK,
+                                             G_PARAM_READABLE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_CHILDREN,
+                                     g_param_spec_object ("children",
+                                             "Children",
+                                             "Children ordered by Line",
+                                             MOO_TYPE_FOLD,
+                                             G_PARAM_READABLE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_N_CHILDREN,
+                                     g_param_spec_int ("n_children",
+                                             "Children Number",
+                                             "Number of children",
+                                             0, G_MAXINT, 0,
+                                             G_PARAM_READABLE));
+
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_COLLAPSED,
+                                     g_param_spec_boolean ("collapsed",
+                                             "Fold Collapsed",
+                                             "True if the fold is collapsed",
+                                             FALSE,
+                                             G_PARAM_READABLE));
+}
+
+static void
+moo_fold_get_property (GObject        *object,
+                            guint           prop_id,
+                            GValue         *value,
+                            GParamSpec     *pspec)
+{
+    MooFold *fold = MOO_FOLD (object);
+
+    switch (prop_id)
+    {
+        case PROP_PARENT:
+            g_value_set_object (value, fold->parent);
+            break;
+
+        case PROP_NEXT:
+            g_value_set_object (value, fold->next);
+            break;
+
+        case PROP_PREV:
+            g_value_set_object (value, fold->prev);
+            break;
+
+        case PROP_MARK_START:
+            g_value_set_object (value, fold->start);
+            break;
+
+        case PROP_CHILDREN:
+            g_value_set_object (value, fold->children);
+            break;
+
+        case PROP_N_CHILDREN:
+            g_value_set_int (value, fold->n_children);
+            break;
+
+        case PROP_MARK_END:
+            g_value_set_object (value, fold->end);
+            break;
+
+        case PROP_COLLAPSED:
+            g_value_set_boolean (value, fold->collapsed);
+            break;
+
+        default:
+            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+            break;
+    }
 }
 
 
@@ -505,6 +639,64 @@ _moo_fold_tree_expand (MooFoldTree *tree,
     expand_check_visible (tree, fold);
 }
 
+#define MOO_FOLD_FOREACH_BEGIN(start, iter)             \
+do {                                                    \
+    MooFold *iter = (start);                            \
+    while (iter)                                        \
+    {
+
+#define MOO_FOLD_FOREACH_END(iter)                      \
+        if (iter->children)                             \
+            iter = iter->children;                      \
+        else if (iter->next)                            \
+            iter = iter->next;                          \
+        else                                            \
+        {                                               \
+            while (iter->parent && !iter->parent->next) \
+                iter = iter->parent;                    \
+            iter = iter && iter->parent ?               \
+                iter->parent->next : NULL;              \
+        }                                               \
+    }                                                   \
+} while (0)
+
+gboolean
+_moo_fold_tree_expand_all (MooFoldTree *tree)
+{
+    gboolean ret = FALSE;
+    MooFold *fold_first;
+
+    fold_first = NULL;
+
+    MOO_FOLD_FOREACH_BEGIN(tree->folds, fold)
+    {
+        if (!fold->deleted && fold->collapsed)
+        {
+            if (!fold_first)
+                fold_first = fold;
+            fold->collapsed = FALSE;
+        }
+    }
+    MOO_FOLD_FOREACH_END(fold);
+
+    if (fold_first)
+    {
+        GtkTextIter start, end;
+        GtkTextBuffer *buffer = GTK_TEXT_BUFFER (tree->buffer);
+        int line_first;
+
+        line_first = _moo_fold_get_start (fold_first);
+
+        gtk_text_buffer_get_iter_at_line (buffer, &start, line_first);
+        gtk_text_buffer_get_end_iter (buffer, &end);
+
+        gtk_text_buffer_remove_tag_by_name (buffer, MOO_FOLD_TAG, &start, &end);
+
+        ret = TRUE;
+    }
+
+    return ret;
+}
 
 void
 _moo_fold_tree_collapse (MooFoldTree *tree,
@@ -529,6 +721,57 @@ _moo_fold_tree_collapse (MooFoldTree *tree,
     gtk_text_buffer_apply_tag_by_name (buffer, MOO_FOLD_TAG, &start, &end);
 }
 
+gboolean
+_moo_fold_tree_collapse_all (MooFoldTree *tree)
+{
+    MooFold *fold;
+    gboolean ret = FALSE;
+
+    for (fold = tree->folds; fold != NULL; fold = fold->next)
+    {
+        if (!fold->collapsed)
+            ret = TRUE;
+        _moo_fold_tree_collapse (tree, fold);
+    }
+
+    return ret;
+}
+
+gboolean
+_moo_fold_tree_toggle (MooFoldTree *tree)
+{
+    gboolean seen_expanded = FALSE;
+    gboolean seen_collapsed = FALSE;
+
+    MOO_FOLD_FOREACH_BEGIN(tree->folds, fold)
+    {
+        if (!fold->deleted)
+        {
+            if (fold->collapsed)
+                seen_collapsed = TRUE;
+            else
+                seen_expanded = TRUE;
+        }
+
+        if (seen_collapsed && seen_expanded)
+            break;
+    }
+    MOO_FOLD_FOREACH_END(fold);
+
+    if (seen_collapsed)
+    {
+        _moo_fold_tree_expand_all (tree);
+        return TRUE;
+    }
+    else if (seen_expanded)
+    {
+        _moo_fold_tree_collapse_all (tree);
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
 
 static GSList *
 get_folds_in_range (MooFoldTree    *tree,
