@@ -1,13 +1,16 @@
 /*
  *   moofold.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOOEDIT_COMPILATION
@@ -73,10 +76,13 @@ GSList      *_moo_fold_tree_get     (MooFoldTree    *tree,
                                      int             first_line,
                                      int             last_line);
 
-void         _moo_fold_tree_expand  (MooFoldTree    *tree,
-                                     MooFold        *fold);
-void         _moo_fold_tree_collapse(MooFoldTree    *tree,
-                                     MooFold        *fold);
+void         _moo_fold_tree_expand          (MooFoldTree    *tree,
+                                             MooFold        *fold);
+void         _moo_fold_tree_collapse        (MooFoldTree    *tree,
+                                             MooFold        *fold);
+gboolean     _moo_fold_tree_toggle          (MooFoldTree    *tree);
+gboolean     _moo_fold_tree_expand_all      (MooFoldTree    *tree);
+gboolean     _moo_fold_tree_collapse_all    (MooFoldTree    *tree);
 
 
 G_END_DECLS
