@@ -1,13 +1,16 @@
 /*
  *   mooindenter.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_INDENTER_H
@@ -47,22 +50,21 @@ struct _MooIndenterClass
                                  guint           setting_id,
                                  GParamSpec     *pspec);
     void    (*character)        (MooIndenter    *indenter,
-                                 gunichar        inserted_char,
+                                 const char     *inserted_char,
                                  GtkTextIter    *where);
 };
 
 
 GType        moo_indenter_get_type              (void) G_GNUC_CONST;
 
-MooIndenter *moo_indenter_new                   (gpointer        doc,
-                                                 const char     *name);
+MooIndenter *moo_indenter_new                   (gpointer        doc);
 
 char        *moo_indenter_make_space            (MooIndenter    *indenter,
                                                  guint           len,
                                                  guint           start);
 
 void         moo_indenter_character             (MooIndenter    *indenter,
-                                                 gunichar        inserted_char,
+                                                 const char     *inserted_char,
                                                  GtkTextIter    *where);
 void         moo_indenter_tab                   (MooIndenter    *indenter,
                                                  GtkTextBuffer  *buffer);
