@@ -1,13 +1,16 @@
 /*
  *   moolangmgr-private.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOOEDIT_COMPILATION
@@ -28,11 +31,12 @@ G_BEGIN_DECLS
 #define MOO_IS_LANG_MGR_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), MOO_TYPE_LANG_MGR))
 #define MOO_LANG_MGR_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), MOO_TYPE_LANG_MGR, MooLangMgrClass))
 
-typedef struct _MooLangMgrClass MooLangMgrClass;
+typedef struct MooLangMgrClass MooLangMgrClass;
 
-struct _MooLangMgr {
-    GtkSourceLanguageManager base;
+struct MooLangMgr {
+    GObject base;
 
+    GtkSourceLanguageManager *lang_mgr;
     GtkSourceStyleSchemeManager *style_mgr;
 
     GHashTable *langs;
@@ -46,9 +50,9 @@ struct _MooLangMgr {
     gboolean modified;
 };
 
-struct _MooLangMgrClass
+struct MooLangMgrClass
 {
-    GtkSourceLanguageManagerClass base_class;
+    GObjectClass base_class;
 };
 
 
