@@ -1,13 +1,16 @@
 /*
  *   moolangmgr.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2009 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
@@ -20,8 +23,8 @@
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooprefs.h"
-#include "mooutils/moomarshals.h"
-#include "mooutils/xdgmime/xdgmime.h"
+#include "marshals.h"
+#include "mooutils/moo-mime.h"
 #include <string.h>
 
 #define LANGUAGE_DIR            "language-specs"
@@ -56,7 +59,7 @@ static MooLang *get_lang_for_mime_type  (MooLangMgr *mgr,
                                          const char *mime_type);
 
 
-G_DEFINE_TYPE (MooLangMgr, moo_lang_mgr, GTK_TYPE_SOURCE_LANGUAGE_MANAGER)
+G_DEFINE_TYPE (MooLangMgr, moo_lang_mgr, G_TYPE_OBJECT)
 
 
 static void
@@ -72,8 +75,9 @@ moo_lang_mgr_init (MooLangMgr *mgr)
     mgr->config = g_hash_table_new_full (g_str_hash, g_str_equal,
                                          g_free, g_free);
 
+    mgr->lang_mgr = gtk_source_language_manager_new ();
     dirs = moo_get_data_subdirs (LANGUAGE_DIR, MOO_DATA_SHARE, &n_dirs);
-    g_object_set (mgr, "search-path", dirs, NULL);
+    g_object_set (mgr->lang_mgr, "search-path", dirs, NULL);
     mgr->style_mgr = gtk_source_style_scheme_manager_new ();
     gtk_source_style_scheme_manager_set_search_path (mgr->style_mgr, dirs);
 
@@ -90,6 +94,7 @@ moo_lang_mgr_dispose (GObject *object)
 
     if (mgr->langs)
     {
+        g_object_unref (mgr->lang_mgr);
         g_object_unref (mgr->style_mgr);
         g_hash_table_destroy (mgr->langs);
         g_hash_table_destroy (mgr->config);
@@ -125,6 +130,15 @@ moo_lang_mgr_new (void)
     return g_object_new (MOO_TYPE_LANG_MGR, NULL);
 }
 
+MooLangMgr *
+moo_lang_mgr_default (void)
+{
+    static MooLangMgr *instance;
+    if (!instance)
+        instance = moo_lang_mgr_new ();
+    return instance;
+}
+
 
 static LangInfo *
 lang_info_new (void)
@@ -198,7 +212,7 @@ moo_lang_mgr_get_lang (MooLangMgr *mgr,
 {
     MooLang *lang;
 
-    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     if (!name)
         return NULL;
@@ -206,7 +220,7 @@ moo_lang_mgr_get_lang (MooLangMgr *mgr,
     lang = _moo_lang_mgr_find_lang (mgr, name);
 
     if (!lang)
-        g_warning ("could not find language '%s'", name);
+        moo_message ("could not find language '%s'", name);
 
     return lang;
 }
@@ -218,8 +232,8 @@ _moo_lang_mgr_find_lang (MooLangMgr *mgr,
     char *id;
     LangInfo *info;
 
-    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
-    g_return_val_if_fail (name != NULL, NULL);
+    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    moo_return_val_if_fail (name != NULL, NULL);
 
     if (!strcmp (name, MOO_LANG_NONE))
         return NULL;
@@ -249,15 +263,15 @@ get_lang_by_extension (MooLangMgr *mgr,
     GSList *langs, *l;
     gboolean found = FALSE;
 
-    g_return_val_if_fail (filename != NULL, NULL);
+    moo_return_val_if_fail (filename != NULL, NULL);
 
     read_langs (mgr);
 
     /* TODO: is this right? */
     basename = g_path_get_basename (filename);
-    g_return_val_if_fail (basename != NULL, NULL);
+    moo_return_val_if_fail (basename != NULL, NULL);
     utf8_basename = g_filename_display_name (basename);
-    g_return_val_if_fail (utf8_basename != NULL, NULL);
+    moo_return_val_if_fail (utf8_basename != NULL, NULL);
 
     langs = moo_lang_mgr_get_available_langs (mgr);
 
@@ -347,9 +361,9 @@ filename_blacklisted (MooLangMgr *mgr,
     read_langs (mgr);
 
     basename = g_path_get_basename (filename);
-    g_return_val_if_fail (basename != NULL, FALSE);
+    moo_return_val_if_fail (basename != NULL, FALSE);
     utf8_basename = g_filename_display_name (basename);
-    g_return_val_if_fail (utf8_basename != NULL, FALSE);
+    moo_return_val_if_fail (utf8_basename != NULL, FALSE);
 
     info = get_lang_info (mgr, MOO_LANG_NONE, FALSE);
 
@@ -379,8 +393,8 @@ moo_lang_mgr_get_lang_for_file (MooLangMgr *mgr,
     MooLang *lang = NULL;
     const char *mime_type;
 
-    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
-    g_return_val_if_fail (filename != NULL, NULL);
+    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    moo_return_val_if_fail (filename != NULL, NULL);
 
     read_langs (mgr);
 
@@ -397,21 +411,13 @@ moo_lang_mgr_get_lang_for_file (MooLangMgr *mgr,
     if (lang)
         return lang;
 
-#ifdef MOO_USE_XDGMIME
-    /* XXX: xdgmime wants utf8-encoded filename here. is it a problem? */
-    /* It's a big problem! */
-    mime_type = xdg_mime_get_mime_type_for_file (filename, NULL);
+    mime_type = moo_get_mime_type_for_file (filename, NULL);
 
-    if (mime_type != XDG_MIME_TYPE_UNKNOWN)
+    if (mime_type != MOO_MIME_TYPE_UNKNOWN)
         lang = get_lang_for_mime_type (mgr, mime_type);
 
     if (lang)
         return lang;
-#else
-#ifdef __GNUC__
-#warning "Implement moo_lang_mgr_get_lang_for_file()"
-#endif
-#endif /* MOO_USE_XDGMIME */
 
     return NULL;
 }
@@ -424,8 +430,8 @@ get_lang_for_filename (MooLangMgr *mgr,
     MooLang *lang = NULL;
     const char *mime_type;
 
-    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
-    g_return_val_if_fail (filename != NULL, NULL);
+    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    moo_return_val_if_fail (filename != NULL, NULL);
 
     read_langs (mgr);
 
@@ -437,22 +443,13 @@ get_lang_for_filename (MooLangMgr *mgr,
     if (lang)
         return lang;
 
-#ifdef MOO_USE_XDGMIME
-    /* XXX: xdgmime wants utf8-encoded filename here. is it a problem? */
-    /* It's a big problem! */
+    mime_type = moo_get_mime_type_for_filename (filename);
 
-    mime_type = xdg_mime_get_mime_type_from_file_name (filename);
-
-    if (mime_type != XDG_MIME_TYPE_UNKNOWN)
+    if (mime_type != MOO_MIME_TYPE_UNKNOWN)
         lang = get_lang_for_mime_type (mgr, mime_type);
 
     if (lang)
         return lang;
-#else
-#ifdef __GNUC__
-#warning "Implement moo_lang_mgr_get_lang_for_filename()"
-#endif
-#endif /* MOO_USE_XDGMIME */
 
     lang = lang_mgr_get_lang_for_bak_filename (mgr, filename);
 
@@ -463,12 +460,11 @@ get_lang_for_filename (MooLangMgr *mgr,
 }
 
 
-#ifdef MOO_USE_XDGMIME
 static int
 check_mime_subclass (const char *base,
                      const char *mime)
 {
-    return !xdg_mime_mime_type_subclass (mime, base);
+    return !moo_mime_type_is_subclass (mime, base);
 }
 
 
@@ -480,8 +476,8 @@ get_lang_for_mime_type (MooLangMgr *mgr,
     MooLang *lang = NULL;
     gboolean found = FALSE;
 
-    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
-    g_return_val_if_fail (mime != NULL, NULL);
+    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    moo_return_val_if_fail (mime != NULL, NULL);
 
     langs = moo_lang_mgr_get_available_langs (mgr);
 
@@ -515,17 +511,6 @@ get_lang_for_mime_type (MooLangMgr *mgr,
     g_slist_free (langs);
     return found ? lang : NULL;
 }
-#else /* MOO_USE_XDGMIME */
-static MooLang *
-get_lang_for_mime_type (MooLangMgr *mgr,
-                        const char *mime)
-{
-    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
-    g_return_val_if_fail (mime != NULL, NULL);
-    g_warning ("%s: implement me?", G_STRLOC);
-    return NULL;
-}
-#endif /* MOO_USE_XDGMIME */
 
 
 static void
@@ -541,7 +526,7 @@ read_langs (MooLangMgr *mgr)
     mgr->got_langs = TRUE;
 
 #ifdef MOO_USE_XML
-    ids = gtk_source_language_manager_get_language_ids (GTK_SOURCE_LANGUAGE_MANAGER (mgr));
+    ids = gtk_source_language_manager_get_language_ids (mgr->lang_mgr);
 #endif
 
     get_lang_info (mgr, MOO_LANG_NONE, TRUE);
@@ -553,7 +538,7 @@ read_langs (MooLangMgr *mgr)
         GtkSourceLanguage *lang;
 
         id = *ids;
-        lang = gtk_source_language_manager_get_language (GTK_SOURCE_LANGUAGE_MANAGER (mgr), id);
+        lang = gtk_source_language_manager_get_language (mgr->lang_mgr, id);
 
         info = get_lang_info (mgr, id, TRUE);
         info->lang = g_object_ref (lang);
@@ -602,7 +587,7 @@ load_lang_node (MooLangMgr    *mgr,
     MooMarkupNode *node;
 
     lang_id = moo_markup_get_prop (lang_node, PROP_LANG_ID);
-    g_return_if_fail (lang_id != NULL);
+    moo_return_if_fail (lang_id != NULL);
 
     for (node = lang_node->children; node != NULL; node = node->next)
     {
@@ -649,7 +634,7 @@ load_config (MooLangMgr *mgr)
     mgr->modified = FALSE;
 
     xml = moo_prefs_get_markup (MOO_PREFS_RC);
-    g_return_if_fail (xml != NULL);
+    moo_return_if_fail (xml != NULL);
 
     root = moo_markup_get_element (MOO_MARKUP_NODE (xml),
                                    ELEMENT_LANG_CONFIG);
@@ -678,7 +663,7 @@ load_config (MooLangMgr *mgr)
 MooTextStyleScheme *
 moo_lang_mgr_get_active_scheme (MooLangMgr *mgr)
 {
-    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_schemes (mgr);
 
@@ -692,7 +677,7 @@ _moo_lang_mgr_set_active_scheme (MooLangMgr *mgr,
 {
     MooTextStyleScheme *scheme = NULL;
 
-    g_return_if_fail (MOO_IS_LANG_MGR (mgr));
+    moo_return_if_fail (MOO_IS_LANG_MGR (mgr));
 
     read_schemes (mgr);
 
@@ -702,8 +687,7 @@ _moo_lang_mgr_set_active_scheme (MooLangMgr *mgr,
         scheme = g_hash_table_lookup (mgr->schemes, SCHEME_DEFAULT);
 
     if (!scheme)
-        g_warning ("%s: could not find style scheme '%s'",
-                   G_STRLOC, name ? name : "<NULL>");
+        moo_message ("could not find style scheme '%s'", name ? name : "<NULL>");
     else
         mgr->active_scheme = scheme;
 }
@@ -722,7 +706,7 @@ moo_lang_mgr_list_schemes (MooLangMgr *mgr)
 {
     GSList *list = NULL;
 
-    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_schemes (mgr);
     g_hash_table_foreach (mgr->schemes, (GHFunc) prepend_scheme, &list);
@@ -771,12 +755,12 @@ moo_lang_mgr_get_sections (MooLangMgr *mgr)
     GSList *sections = NULL;
     const char * const *ids = NULL;
 
-    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_langs (mgr);
 
 #ifdef MOO_USE_XML
-    ids = gtk_source_language_manager_get_language_ids (GTK_SOURCE_LANGUAGE_MANAGER (mgr));
+    ids = gtk_source_language_manager_get_language_ids (mgr->lang_mgr);
 #endif
 
     while (ids && *ids)
@@ -786,7 +770,7 @@ moo_lang_mgr_get_sections (MooLangMgr *mgr)
         const char *id;
 
         id = *ids;
-        lang = gtk_source_language_manager_get_language (GTK_SOURCE_LANGUAGE_MANAGER (mgr), id);
+        lang = gtk_source_language_manager_get_language (mgr->lang_mgr, id);
         section = _moo_lang_get_section (MOO_LANG (lang));
 
         if (section && !g_slist_find_custom (sections, section, (GCompareFunc) strcmp))
@@ -805,18 +789,18 @@ moo_lang_mgr_get_available_langs (MooLangMgr *mgr)
     GSList *list = NULL;
     const char *const *ids = NULL;
 
-    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_langs (mgr);
 
 #ifdef MOO_USE_XML
-    ids = gtk_source_language_manager_get_language_ids (GTK_SOURCE_LANGUAGE_MANAGER (mgr));
+    ids = gtk_source_language_manager_get_language_ids (mgr->lang_mgr);
 #endif
 
     while (ids && *ids)
     {
         GtkSourceLanguage *lang;
-        lang = gtk_source_language_manager_get_language (GTK_SOURCE_LANGUAGE_MANAGER (mgr), *ids);
+        lang = gtk_source_language_manager_get_language (mgr->lang_mgr, *ids);
         if (lang)
             list = g_slist_prepend (list, g_object_ref (lang));
         ids++;
@@ -833,13 +817,13 @@ _moo_lang_mgr_get_globs (MooLangMgr *mgr,
     char *id;
     LangInfo *info;
 
-    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_langs (mgr);
 
     id = _moo_lang_id_from_name (lang_id);
     info = get_lang_info (mgr, id, FALSE);
-    g_return_val_if_fail (info != NULL, NULL);
+    moo_return_val_if_fail (info != NULL, NULL);
 
     g_free (id);
     return string_list_copy (info->globs);
@@ -853,13 +837,13 @@ _moo_lang_mgr_get_mime_types (MooLangMgr *mgr,
     char *id;
     LangInfo *info;
 
-    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_langs (mgr);
 
     id = _moo_lang_id_from_name (lang_id);
     info = get_lang_info (mgr, id, FALSE);
-    g_return_val_if_fail (info != NULL, NULL);
+    moo_return_val_if_fail (info != NULL, NULL);
 
     g_free (id);
     return string_list_copy (info->mime_types);
@@ -901,7 +885,7 @@ set_globs_or_mime_types (MooLangMgr *mgr,
     id = _moo_lang_id_from_name (lang_id);
     info = get_lang_info (mgr, id, FALSE);
 
-    g_return_if_fail (info != NULL);
+    moo_return_if_fail (info != NULL);
 
     new = _moo_lang_parse_string_list (string);
     old = globs ? _moo_lang_mgr_get_globs (mgr, id) :
@@ -951,7 +935,7 @@ _moo_lang_mgr_set_globs (MooLangMgr *mgr,
                          const char *lang_id,
                          const char *globs)
 {
-    g_return_if_fail (MOO_IS_LANG_MGR (mgr));
+    moo_return_if_fail (MOO_IS_LANG_MGR (mgr));
     set_globs_or_mime_types (mgr, lang_id, globs, TRUE);
 }
 
@@ -961,7 +945,7 @@ _moo_lang_mgr_set_mime_types (MooLangMgr *mgr,
                               const char *lang_id,
                               const char *mime)
 {
-    g_return_if_fail (MOO_IS_LANG_MGR (mgr));
+    moo_return_if_fail (MOO_IS_LANG_MGR (mgr));
     set_globs_or_mime_types (mgr, lang_id, mime, FALSE);
 }
 
@@ -973,7 +957,7 @@ _moo_lang_mgr_get_config (MooLangMgr *mgr,
     char *id;
     const char *config;
 
-    g_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
+    moo_return_val_if_fail (MOO_IS_LANG_MGR (mgr), NULL);
 
     read_langs (mgr);
 
@@ -994,7 +978,7 @@ _moo_lang_mgr_set_config (MooLangMgr *mgr,
     char *norm = NULL;
     const char *old;
 
-    g_return_if_fail (MOO_IS_LANG_MGR (mgr));
+    moo_return_if_fail (MOO_IS_LANG_MGR (mgr));
 
     read_langs (mgr);
 
@@ -1026,8 +1010,8 @@ _moo_lang_mgr_update_config (MooLangMgr     *mgr,
 {
     const char *lang_config;
 
-    g_return_if_fail (MOO_IS_LANG_MGR (mgr));
-    g_return_if_fail (MOO_IS_EDIT_CONFIG (config));
+    moo_return_if_fail (MOO_IS_LANG_MGR (mgr));
+    moo_return_if_fail (MOO_IS_EDIT_CONFIG (config));
 
     read_langs (mgr);
 
@@ -1118,13 +1102,13 @@ _moo_lang_mgr_save_config (MooLangMgr *mgr)
         MooMarkupNode *root;
     } data;
 
-    g_return_if_fail (MOO_IS_LANG_MGR (mgr));
+    moo_return_if_fail (MOO_IS_LANG_MGR (mgr));
 
     if (!mgr->modified)
         return;
 
     xml = moo_prefs_get_markup (MOO_PREFS_RC);
-    g_return_if_fail (xml != NULL);
+    moo_return_if_fail (xml != NULL);
 
     mgr->modified = FALSE;
     root = moo_markup_get_element (MOO_MARKUP_NODE (xml), ELEMENT_LANG_CONFIG);
