@@ -1,13 +1,16 @@
 /*
  *   moolangmgr.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_LANG_MGR_H
@@ -24,12 +27,13 @@ G_BEGIN_DECLS
 #define MOO_LANG_MGR(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), MOO_TYPE_LANG_MGR, MooLangMgr))
 #define MOO_IS_LANG_MGR(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), MOO_TYPE_LANG_MGR))
 
-typedef struct _MooLangMgr MooLangMgr;
+typedef struct MooLangMgr MooLangMgr;
 
 
 GType           moo_lang_mgr_get_type               (void) G_GNUC_CONST;
 
 MooLangMgr     *moo_lang_mgr_new                    (void);
+MooLangMgr     *moo_lang_mgr_default                (void);
 
 MooLang        *moo_lang_mgr_get_lang               (MooLangMgr     *mgr,
                                                      const char     *lang_id);
