@@ -1,22 +1,21 @@
 /*
  *   moolinebuffer.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
 #include "mooedit/mootext-private.h"
-#include <string.h>
-
-
-#define PTRMOVE(dest_,src_,n_ptr_) g_memmove (dest_, src_, (n_ptr_) * sizeof(gpointer))
-#define PTRCPY(dest_,src_,n_ptr_)  memcpy (dest_, src_, (n_ptr_) * sizeof(gpointer))
+#include "mooutils/mooutils-mem.h"
 
 
 LineBuffer*
@@ -83,19 +82,12 @@ _moo_line_buffer_delete (LineBuffer *line_buf,
 
         if (n_old_marks)
         {
-            MooLineMark **tmp = g_new (MooLineMark*, n_old_marks + line->n_marks);
-
             if (moved_marks)
                 for (i = 0; i < n_old_marks; ++i)
                     *moved_marks = g_slist_prepend (*moved_marks, old_marks[i]);
 
-            if (line->n_marks)
-                PTRCPY (tmp, line->marks, line->n_marks);
-
-            PTRCPY (&tmp[line->n_marks], old_marks, n_old_marks);
-
-            g_free (line->marks);
-            line->marks = tmp;
+            MOO_ARRAY_GROW (MooLineMark*, line->marks, n_old_marks + line->n_marks);
+            MOO_ELMCPY (line->marks + line->n_marks, old_marks, n_old_marks);
 
             for (i = 0; i < n_old_marks; ++i)
                 _moo_line_mark_set_line (old_marks[i], line, move_to, line_buf->tree->stamp);
@@ -132,19 +124,7 @@ line_add_mark (LineBuffer  *line_buf,
                MooLineMark *mark,
                Line        *line)
 {
-    if (line->marks)
-    {
-        MooLineMark **tmp = g_new (MooLineMark*, line->n_marks + 1);
-        PTRCPY (tmp, line->marks, line->n_marks);
-        g_free (line->marks);
-        line->marks = tmp;
-    }
-    else
-    {
-        g_assert (!line->n_marks);
-        line->marks = g_new (MooLineMark*, 1);
-    }
-
+    MOO_ARRAY_GROW (MooLineMark*, line->marks, line->n_marks + 1);
     line->marks[line->n_marks] = mark;
     _moo_text_btree_update_n_marks (line_buf->tree, line, 1);
 }
@@ -189,7 +169,7 @@ line_remove_mark (LineBuffer  *line_buf,
     }
     else if (i < line->n_marks - 1)
     {
-        PTRMOVE (&line->marks[i], &line->marks[i+1], line->n_marks - i - 1);
+        MOO_ELMMOVE (line->marks + i, line->marks + i + 1, line->n_marks - i - 1);
         line->marks[line->n_marks - 1] = NULL;
     }
     else
@@ -206,7 +186,6 @@ _moo_line_buffer_remove_mark (LineBuffer     *line_buf,
                               MooLineMark    *mark)
 {
     Line *line;
-    G_GNUC_UNUSED int index;
 
     line = _moo_line_mark_get_line (mark);
 
@@ -238,12 +217,12 @@ node_get_marks (BTree  *tree,
                 int     node_offset)
 {
     GSList *total = NULL;
-    int i;
+    guint i;
 
     if (!node->n_marks)
         return NULL;
 
-    if (last_line < node_offset || first_line >= node_offset + node->count)
+    if (last_line < node_offset || first_line >= node_offset + (int) node->count)
         return NULL;
 
     if (node->is_bottom)
@@ -253,7 +232,7 @@ node_get_marks (BTree  *tree,
             Line *line;
             guint j;
 
-            if (i + node_offset > last_line)
+            if ((int) i + node_offset > last_line)
                 break;
 
             line = node->u.data[i];
@@ -328,7 +307,7 @@ line_get_index (BTData *line)
 
     while (node->parent)
     {
-        int i;
+        guint i;
 
         for (i = 0; i < node->parent->n_children; ++i)
         {
