@@ -1,19 +1,23 @@
 /*
  *   moolinemark.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2008 by Daniel Poelzleithner <mooedit@poelzi.org>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
 #include "mooedit/mootext-private.h"
 #include "mooedit/mootextbuffer.h"
-#include "mooutils/moomarshals.h"
+#include "marshals.h"
 #include "mooutils/mooutils-misc.h"
 
 
@@ -77,7 +81,8 @@ enum {
     PROP_PIXBUF,
     PROP_STOCK_ID,
     PROP_MARKUP,
-    PROP_VISIBLE
+    PROP_VISIBLE,
+    PROP_FOLD
 };
 
 
@@ -152,6 +157,14 @@ moo_line_mark_class_init (MooLineMarkClass *klass)
                                              NULL,
                                              G_PARAM_READWRITE));
 
+    g_object_class_install_property (gobject_class,
+                                     PROP_FOLD,
+                                     g_param_spec_object ("fold",
+                                             "fold",
+                                             "fold",
+                                             MOO_TYPE_FOLD,
+                                             G_PARAM_READABLE));
+
     signals[CHANGED] =
             g_signal_new ("changed",
                           G_OBJECT_CLASS_TYPE (klass),
@@ -272,6 +285,10 @@ moo_line_mark_get_property (GObject        *object,
             g_value_set_string (value, mark->priv->stock_id);
             break;
 
+        case PROP_FOLD:
+            g_value_set_object (value, mark->priv->fold);
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
             break;
@@ -388,8 +405,8 @@ moo_line_mark_get_markup (MooLineMark *mark)
 int
 moo_line_mark_get_line (MooLineMark *mark)
 {
-    g_return_val_if_fail (MOO_IS_LINE_MARK (mark), -1);
-    g_return_val_if_fail (mark->priv->line != NULL, -1);
+    moo_return_val_if_fail (MOO_IS_LINE_MARK (mark), -1);
+    moo_return_val_if_fail (mark->priv->line != NULL, -1);
 
     if (_moo_line_buffer_get_stamp (mark->priv->line_buf) != mark->priv->stamp)
     {
