@@ -1,13 +1,16 @@
 /*
  *   moooutputfilter.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_OUTPUT_FILTER_H
@@ -85,10 +88,10 @@ gboolean         moo_output_filter_cmd_exit         (MooOutputFilter    *filter,
 
 void             moo_output_filter_set_active_file  (MooOutputFilter    *filter,
                                                      const char         *path);
-void             moo_output_filter_set_active_dir   (MooOutputFilter    *filter,
-                                                     const char         *path);
+void             moo_output_filter_add_active_dirs  (MooOutputFilter    *filter,
+                                                     char              **dirs);
 const char      *moo_output_filter_get_active_file  (MooOutputFilter    *filter);
-const char      *moo_output_filter_get_active_dir   (MooOutputFilter    *filter);
+const char*const*moo_output_filter_get_active_dirs  (MooOutputFilter    *filter);
 void             moo_output_filter_set_window       (MooOutputFilter    *filter,
                                                      gpointer            window);
 
