@@ -1,13 +1,16 @@
 /*
  *   mooplugin-macro.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_PLUGIN_MACRO_H
@@ -35,6 +38,7 @@ static const MooPluginInfo plugin_name__##_plugin_info =                    \
 
 
 #define MOO_PLUGIN_DEFINE_FULL(Name__,name__,                               \
+                               class_init_code__,instance_init_code__,      \
                                attach_win__,detach_win__,                   \
                                attach_doc__,detach_doc__,                   \
                                prefs_page_func__,                           \
@@ -60,6 +64,10 @@ name__##_plugin_class_init (MooPluginClass *klass)                          \
     klass->attach_doc = attach_doc__;                                       \
     klass->detach_doc = detach_doc__;                                       \
     klass->create_prefs_page = prefs_page_func__;                           \
+                                                                            \
+    {                                                                       \
+        class_init_code__ ;                                                 \
+    }                                                                       \
 }                                                                           \
                                                                             \
 static void                                                                 \
@@ -67,6 +75,10 @@ name__##_plugin_instance_init (MooPlugin *plugin)                           \
 {                                                                           \
     plugin->win_plugin_type = WIN_PLUGIN_TYPE__;                            \
     plugin->doc_plugin_type = DOC_PLUGIN_TYPE__;                            \
+                                                                            \
+    {                                                                       \
+        instance_init_code__ ;                                              \
+    }                                                                       \
 }                                                                           \
                                                                             \
 static GType name__##_plugin_get_type (void) G_GNUC_CONST;                  \
@@ -92,12 +104,24 @@ name__##_plugin_get_type (void)                                             \
         };                                                                  \
                                                                             \
         type__ = g_type_register_static (MOO_TYPE_PLUGIN,                   \
-                                         #Name__ "Plugin", &info__, 0);     \
+                                         #Name__ "Plugin", &info__,         \
+                                         (GTypeFlags) 0);                   \
     }                                                                       \
                                                                             \
     return type__;                                                          \
 }
 
+#define MOO_PLUGIN_DEFINE(Name__,name__,                                    \
+                          attach_win__,detach_win__,                        \
+                          attach_doc__,detach_doc__,                        \
+                          prefs_page_func__,                                \
+                          WIN_PLUGIN_TYPE__,DOC_PLUGIN_TYPE__)              \
+MOO_PLUGIN_DEFINE_FULL(Name__,name__,                                       \
+                       {},{},                                               \
+                       attach_win__,detach_win__,                           \
+                       attach_doc__,detach_doc__,                           \
+                       prefs_page_func__,                                   \
+                       WIN_PLUGIN_TYPE__,DOC_PLUGIN_TYPE__)
 
 #define MOO_WIN_PLUGIN_DEFINE(Name__,name__)                                    \
                                                                                 \
@@ -140,7 +164,8 @@ name__##_window_plugin_get_type (void)
         };                                                                      \
                                                                                 \
         type__ = g_type_register_static (MOO_TYPE_WIN_PLUGIN,                   \
-                                         #Name__ "WindowPlugin",&info__, 0);    \
+                                         #Name__ "WindowPlugin",&info__,        \
+                                         (GTypeFlags) 0);                       \
     }                                                                           \
                                                                                 \
     return type__;                                                              \
@@ -188,7 +213,8 @@ name__##_doc_plugin_get_type (void)                                         \
         };                                                                  \
                                                                             \
         type__ = g_type_register_static (MOO_TYPE_DOC_PLUGIN,               \
-                                         #Name__ "DocPlugin", &info__, 0);  \
+                                         #Name__ "DocPlugin", &info__,      \
+                                         (GTypeFlags) 0);                   \
     }                                                                       \
                                                                             \
     return type__;                                                          \
