@@ -3,11 +3,14 @@
  *
  *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -16,14 +19,13 @@
 
 #include "mooedit/mooplugin.h"
 #include "mooedit/mooplugin-loader.h"
-#include "mooedit/moopluginprefs-glade.h"
-#include "mooedit/plugins/mooeditplugins.h"
-#include "moopython/moopython-builtin.h"
+#include "glade/moopluginprefs-gxml.h"
 #include "mooutils/mooprefsdialog.h"
 #include "mooutils/moostock.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-debug.h"
 #include "mooutils/moohelp.h"
+#include "mooutils/mootype-macros.h"
 #include "help-sections.h"
 #include <string.h>
 #include <gmodule.h>
@@ -50,6 +52,8 @@ static PluginStore *plugin_store = NULL;
 #define MOO_PLUGIN_QUARK plugin_store->plugin_quark
 #define MOO_PLUGIN_METHS_QUARK plugin_store->meths_quark
 
+MOO_DEFINE_BOXED_TYPE_C (MooPluginInfo, moo_plugin_info)
+MOO_DEFINE_BOXED_TYPE_C (MooPluginParams, moo_plugin_params)
 
 static void     plugin_store_init       (void);
 static void     plugin_store_add        (MooPlugin      *plugin);
@@ -905,24 +909,6 @@ moo_plugin_get_dirs (void)
 }
 
 
-static void
-moo_plugin_init_builtin (void)
-{
-    _moo_file_selector_plugin_init ();
-    _moo_file_list_plugin_init ();
-#ifndef __WIN32__
-#ifdef MOO_BUILD_CTAGS
-    _moo_ctags_plugin_init ();
-#endif
-    _moo_find_plugin_init ();
-#endif
-#if 0
-    _moo_completion_plugin_init ();
-    _moo_active_strings_plugin_init ();
-#endif
-}
-
-
 void
 moo_plugin_read_dirs (void)
 {
@@ -941,13 +927,6 @@ moo_plugin_read_dirs (void)
     g_strfreev (plugin_store->dirs);
     plugin_store->dirs = _moo_strv_reverse (dirs);
 
-    moo_plugin_init_builtin ();
-
-#ifdef MOO_PYTHON_BUILTIN
-    /* XXX move it elsewhere */
-    _moo_python_builtin_init ();
-#endif
-
     for (d = plugin_store->dirs; d && *d; ++d)
         moo_plugin_read_dir (*d);
 
@@ -1209,34 +1188,6 @@ moo_plugin_params_free (MooPluginParams *params)
 }
 
 
-GType
-moo_plugin_info_get_type (void)
-{
-    static GType type;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooPluginInfo",
-                                             (GBoxedCopyFunc) moo_plugin_info_copy,
-                                             (GBoxedFreeFunc) moo_plugin_info_free);
-
-    return type;
-}
-
-
-GType
-moo_plugin_params_get_type (void)
-{
-    static GType type;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooPluginParams",
-                                             (GBoxedCopyFunc) moo_plugin_params_copy,
-                                             (GBoxedFreeFunc) moo_plugin_params_free);
-
-    return type;
-}
-
-
 /***************************************************************************/
 /* Preferences dialog
  */
@@ -1250,14 +1201,12 @@ enum {
 
 
 static void
-selection_changed (GtkTreeSelection   *selection,
-                   MooPrefsDialogPage *page)
+selection_changed (GtkTreeSelection *selection,
+                   PrefsPageXml     *gxml)
 {
     MooPlugin *plugin = NULL;
     GtkTreeModel *model;
     GtkTreeIter iter;
-    GtkWidget *info;
-    GtkLabel *version, *author, *description;
 
     if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
@@ -1267,24 +1216,19 @@ selection_changed (GtkTreeSelection   *selection,
         g_free (id);
     }
 
-    info = moo_glade_xml_get_widget (page->xml, "info");
-    author = moo_glade_xml_get_widget (page->xml, "author");
-    version = moo_glade_xml_get_widget (page->xml, "version");
-    description = moo_glade_xml_get_widget (page->xml, "description");
-
-    gtk_widget_set_sensitive (info, plugin != NULL);
+    gtk_widget_set_sensitive (GTK_WIDGET (gxml->info), plugin != NULL);
 
     if (plugin)
     {
-        gtk_label_set_text (author, moo_plugin_author (plugin));
-        gtk_label_set_text (version, moo_plugin_version (plugin));
-        gtk_label_set_text (description, moo_plugin_description (plugin));
+        gtk_label_set_text (gxml->author, moo_plugin_author (plugin));
+        gtk_label_set_text (gxml->version, moo_plugin_version (plugin));
+        gtk_label_set_text (gxml->description, moo_plugin_description (plugin));
     }
     else
     {
-        gtk_label_set_text (author, "");
-        gtk_label_set_text (version, "");
-        gtk_label_set_text (description, "");
+        gtk_label_set_text (gxml->author, "");
+        gtk_label_set_text (gxml->version, "");
+        gtk_label_set_text (gxml->description, "");
     }
 }
 
@@ -1335,18 +1279,15 @@ sync_pages (MooPrefsDialog *dialog)
 
                 if (plugin_page)
                 {
-                    if (!MOO_IS_PREFS_DIALOG_PAGE (plugin_page))
+                    if (!MOO_IS_PREFS_PAGE (plugin_page))
                     {
                         g_critical ("%s: oops", G_STRLOC);
                     }
                     else
                     {
-                        MOO_PREFS_DIALOG_PAGE (plugin_page)->auto_apply = FALSE;
-
                         g_object_set_data_full (G_OBJECT (plugin_page), "moo-plugin-id",
                                                 g_strdup (moo_plugin_id (plugin)),
                                                 g_free);
-
                         plugin_pages = g_slist_append (plugin_pages, plugin_page);
                         moo_prefs_dialog_insert_page (dialog, plugin_page, -1);
                     }
@@ -1369,16 +1310,14 @@ sync_pages (MooPrefsDialog *dialog)
 
 
 static void
-prefs_init (MooPrefsDialog      *dialog,
-            MooPrefsDialogPage  *page)
+prefs_page_init (MooPrefsPage *page,
+                 PrefsPageXml *gxml)
 {
-    GtkTreeView *treeview;
     GtkListStore *store;
     GtkTreeModel *model;
     GSList *l, *plugins;
 
-    treeview = moo_glade_xml_get_widget (page->xml, "treeview");
-    model = gtk_tree_view_get_model (treeview);
+    model = gtk_tree_view_get_model (gxml->treeview);
     store = GTK_LIST_STORE (model);
 
     gtk_list_store_clear (store);
@@ -1400,24 +1339,24 @@ prefs_init (MooPrefsDialog      *dialog,
         }
     }
 
-    selection_changed (gtk_tree_view_get_selection (treeview), page);
+    selection_changed (gtk_tree_view_get_selection (gxml->treeview), gxml);
 
     g_slist_free (plugins);
-    sync_pages (dialog);
+    sync_pages (g_object_get_data (G_OBJECT (page), "moo-plugin-prefs-dialog"));
 }
 
 
 static void
-prefs_apply (MooPrefsDialog      *dialog,
-             MooPrefsDialogPage  *page)
+prefs_page_apply (MooPrefsPage *page,
+                  PrefsPageXml *gxml)
 {
-    GtkTreeView *treeview;
     GtkTreeModel *model;
     GtkTreeIter iter;
-    GSList *plugin_pages;
+    MooPrefsDialog *dialog;
 
-    treeview = moo_glade_xml_get_widget (page->xml, "treeview");
-    model = gtk_tree_view_get_model (treeview);
+    dialog = g_object_get_data (G_OBJECT (page), "moo-plugin-prefs-dialog");
+
+    model = gtk_tree_view_get_model (gxml->treeview);
 
     if (gtk_tree_model_get_iter_first (model, &iter)) do
     {
@@ -1446,15 +1385,6 @@ prefs_apply (MooPrefsDialog      *dialog,
     while (gtk_tree_model_iter_next (model, &iter));
 
     sync_pages (dialog);
-
-    plugin_pages = g_object_get_data (G_OBJECT (dialog),
-                                      "moo-plugin-prefs-pages");
-
-    while (plugin_pages)
-    {
-        g_signal_emit_by_name (plugin_pages->data, "apply");
-        plugin_pages = plugin_pages->next;
-    }
 }
 
 
@@ -1477,51 +1407,46 @@ enable_toggled (GtkCellRendererToggle *cell,
 void
 moo_plugin_attach_prefs (GtkWidget *dialog)
 {
-    MooPrefsDialogPage *page;
-    MooGladeXML *xml;
-    GtkTreeView *treeview;
+    GtkWidget *page;
     GtkCellRenderer *cell;
     GtkListStore *store;
     GtkTreeSelection *selection;
+    PrefsPageXml *gxml;
 
     g_return_if_fail (MOO_IS_PREFS_DIALOG (dialog));
 
-    page = moo_prefs_dialog_page_new_from_xml ("Plugins", MOO_STOCK_PLUGINS,
-                                               NULL, moopluginprefs_glade_xml,
-                                               "page", MOO_PLUGIN_PREFS_ROOT);
-    g_return_if_fail (page != NULL);
+    page = moo_prefs_page_new (_("Plugins"), MOO_STOCK_PLUGINS);
+    gxml = prefs_page_xml_new_with_root (page);
+    g_object_set_data (G_OBJECT (page), "moo-plugin-prefs-xml", gxml);
+    g_object_set_data (G_OBJECT (page), "moo-plugin-prefs-dialog", dialog);
+    g_signal_connect (page, "init", G_CALLBACK (prefs_page_init), gxml);
+    g_signal_connect (page, "apply", G_CALLBACK (prefs_page_apply), gxml);
 
     moo_help_set_id (GTK_WIDGET (page), HELP_SECTION_PREFS_PLUGINS);
 
-    xml = page->xml;
-
-    treeview = moo_glade_xml_get_widget (xml, "treeview");
-
-    selection = gtk_tree_view_get_selection (treeview);
+    selection = gtk_tree_view_get_selection (gxml->treeview);
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
-    g_signal_connect (selection, "changed", G_CALLBACK (selection_changed), page);
+    g_signal_connect (selection, "changed", G_CALLBACK (selection_changed), gxml);
 
     store = gtk_list_store_new (N_COLUMNS,
                                 G_TYPE_BOOLEAN,
                                 G_TYPE_STRING,
                                 G_TYPE_STRING);
-    gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (store));
+    gtk_tree_view_set_model (gxml->treeview, GTK_TREE_MODEL (store));
     g_object_unref (store);
 
     cell = gtk_cell_renderer_toggle_new ();
     g_object_set (cell, "activatable", TRUE, NULL);
     g_signal_connect (cell, "toggled", G_CALLBACK (enable_toggled), store);
-    gtk_tree_view_insert_column_with_attributes (treeview, 0, "Enabled", cell,
+    gtk_tree_view_insert_column_with_attributes (gxml->treeview, 0, "Enabled", cell,
                                                  "active", COLUMN_ENABLED, NULL);
 
     cell = gtk_cell_renderer_text_new ();
-    gtk_tree_view_insert_column_with_attributes (treeview, 1, "Plugin", cell,
+    gtk_tree_view_insert_column_with_attributes (gxml->treeview, 1, "Plugin", cell,
                                                  "text", COLUMN_PLUGIN_NAME, NULL);
 
-    g_signal_connect (dialog, "init", G_CALLBACK (prefs_init), page);
-    g_signal_connect (dialog, "apply", G_CALLBACK (prefs_apply), page);
-
     moo_prefs_dialog_append_page (MOO_PREFS_DIALOG (dialog), GTK_WIDGET (page));
+    prefs_page_init (MOO_PREFS_PAGE (page), gxml);
 }
 
 
