@@ -1,18 +1,21 @@
 /*
  *   mootextbtree.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
 #include "mooedit/mootext-private.h"
-#include <string.h>
+#include "mooutils/mooutils-mem.h"
 
 
 static BTNode  *bt_node_new         (BTNode     *parent,
@@ -27,7 +30,7 @@ static void     bt_data_free        (BTData     *data,
 
 #define NODE_IS_ROOT(node__) (!(node__)->parent)
 
-#if 0 && defined(MOO_DEBUG_ENABLED)
+#if 0 && defined(MOO_DEBUG)
 #define WANT_CHECK_INTEGRITY
 static void CHECK_INTEGRITY (BTree *tree, gboolean check_capacity);
 #else
@@ -201,18 +204,20 @@ node_get_index (BTNode *node, gpointer child)
 
 
 static void
-node_insert__ (BTNode *node, gpointer data, guint index_)
+node_insert__ (BTNode   *node,
+               gpointer  data,
+               guint     index)
 {
     g_assert (node != NULL);
     g_assert (node->n_children < BTREE_NODE_MAX_CAPACITY);
-    g_assert (index_ <= node->n_children);
+    g_assert (index <= node->n_children);
 
-    if (index_ < node->n_children)
-        memmove (node->u.children + index_ + 1,
-                 node->u.children + index_,
-                  (node->n_children - index_) * sizeof (BTData*));
+    if (index < node->n_children)
+        MOO_ELMMOVE (node->u.children + index + 1,
+                     node->u.children + index,
+                     node->n_children - index);
 
-    node->u.children[index_] = data;
+    node->u.children[index] = data;
     node->n_children++;
 }
 
@@ -228,9 +233,9 @@ node_remove__ (BTNode *node, gpointer data)
     index = node_get_index (node, data);
 
     if (index + 1 < node->n_children)
-        memmove (node->u.data + index,
-                 node->u.data + index + 1,
-                 (node->n_children - index - 1) * sizeof (BTData*));
+        MOO_ELMMOVE (node->u.data + index,
+                     node->u.data + index + 1,
+                     node->n_children - index - 1);
 
     node->n_children--;
 }
@@ -242,7 +247,8 @@ _moo_text_btree_insert (BTree   *tree,
 {
     BTNode *node, *tmp;
     BTData *data;
-    G_GNUC_UNUSED guint index_orig = index_;
+    guint index_orig = index_;
+    (void) index_orig;
 
     g_assert (tree != NULL);
     g_assert (index_ <= tree->root->count);
@@ -298,9 +304,9 @@ _moo_text_btree_insert (BTree   *tree,
         node_insert__ (node->parent, new_node, node_index + 1);
         g_assert (node_get_index (node->parent, new_node) == node_index + 1);
         g_assert (node_get_index (node->parent, node) == node_index);
-        memcpy (new_node->u.children,
-                node->u.children + BTREE_NODE_MIN_CAPACITY,
-                BTREE_NODE_MIN_CAPACITY * sizeof (BTNode*));
+        MOO_ELMCPY (new_node->u.children,
+                    node->u.children + BTREE_NODE_MIN_CAPACITY,
+                    BTREE_NODE_MIN_CAPACITY);
         node->n_children = BTREE_NODE_MIN_CAPACITY;
         for (i = 0; i < new_node->n_children; ++i)
             new_node->u.children[i]->parent = new_node;
@@ -338,9 +344,8 @@ merge_nodes (BTNode *parent, guint first)
     next = parent->u.children[first+1];
     g_assert (node->n_children + next->n_children < BTREE_NODE_MAX_CAPACITY);
 
-    memcpy (node->u.children + node->n_children,
-            next->u.children,
-            next->n_children * sizeof (BTNode*));
+    MOO_ELMCPY (node->u.children + node->n_children,
+                next->u.children, next->n_children);
 
     for (i = node->n_children; i < node->n_children + next->n_children; ++i)
         node->u.children[i]->parent = node;
