@@ -1,13 +1,16 @@
 /*
  *   mootextbuffer.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
@@ -15,7 +18,7 @@
 #include "mooedit/mootext-private.h"
 #include "mooedit/moolang-private.h"
 #include "mooedit/mootextstylescheme.h"
-#include "mooutils/moomarshals.h"
+#include "marshals.h"
 #include "mooutils/mooundo.h"
 #include "mooutils/mooutils-gobject.h"
 #include <string.h>
@@ -438,6 +441,21 @@ _moo_text_buffer_get_undo_stack (MooTextBuffer *buffer)
 }
 
 
+void
+moo_text_buffer_begin_not_undoable_action (MooTextBuffer *buffer)
+{
+    g_return_if_fail (MOO_IS_TEXT_BUFFER (buffer));
+    moo_undo_stack_freeze (buffer->priv->undo_stack);
+}
+
+void
+moo_text_buffer_end_not_undoable_action (MooTextBuffer *buffer)
+{
+    g_return_if_fail (MOO_IS_TEXT_BUFFER (buffer));
+    moo_undo_stack_thaw (buffer->priv->undo_stack);
+}
+
+
 static void
 moo_text_buffer_begin_user_action (GtkTextBuffer *text_buffer)
 {
@@ -2090,3 +2108,11 @@ moo_text_buffer_toggle_fold (MooTextBuffer *buffer,
 
     g_signal_emit (buffer, signals[FOLD_TOGGLED], 0, fold);
 }
+
+void
+moo_text_buffer_toggle_folds (MooTextBuffer *buffer)
+{
+    g_return_if_fail (MOO_IS_TEXT_BUFFER (buffer));
+    if (_moo_fold_tree_toggle (buffer->priv->fold_tree))
+        g_signal_emit (buffer, signals[FOLD_TOGGLED], 0, NULL);
+}
