@@ -1,17 +1,21 @@
 /*
  *   mootextcompletion.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooedit/mootextcompletion.h"
-#include "mooutils/moomarshals.h"
+#include "mooutils/mooaccel.h"
+#include "marshals.h"
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
@@ -260,7 +264,6 @@ finish:
     g_free (text);
     g_free (prefix);
     g_free (line);
-    return;
 }
 
 void
@@ -469,18 +472,14 @@ popup_key_press (MooTextCompletion *cmpl,
 {
     GtkTreeIter iter;
 
-    switch (event->keyval)
+    if (moo_accel_check_event (GTK_WIDGET (cmpl->priv->doc), event, GDK_Tab, 0) &&
+        moo_text_completion_unique (cmpl, &iter))
     {
-        case GDK_Tab:
-            if (moo_text_completion_unique (cmpl, &iter))
-            {
-                moo_text_completion_complete (cmpl, &iter);
-                return TRUE;
-            }
-        /* fall through */
-        default:
-            return FALSE;
+        moo_text_completion_complete (cmpl, &iter);
+        return TRUE;
     }
+
+    return FALSE;
 }
 
 
