@@ -1,18 +1,19 @@
 /*
  *   mootextfind.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooedit/mootextfind.h"
-#include "mooedit/mootextfind-glade.h"
-#include "mooedit/mootextgotoline-glade.h"
 #include "mooedit/mootextview.h"
 #include "mooedit/mooeditdialogs.h"
 #include "mooedit/mootextsearch-private.h"
@@ -23,6 +24,8 @@
 #include "mooutils/moodialogs.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/moohelp.h"
+#include "glade/mootextfind-gxml.h"
+#include "glade/mootextgotoline-gxml.h"
 #include "help-sections.h"
 #include <gtk/gtk.h>
 #include <glib/gprintf.h>
@@ -122,33 +125,22 @@ moo_find_class_init (MooFindClass *klass)
 static void
 moo_find_init (MooFind *find)
 {
-    GtkWidget *vbox;
     MooCombo *search, *replace;
 
-    find->xml = moo_glade_xml_new_empty (GETTEXT_PACKAGE);
-    moo_glade_xml_map_id (find->xml, "search_entry", MOO_TYPE_HISTORY_COMBO);
-    moo_glade_xml_map_id (find->xml, "replace_entry", MOO_TYPE_HISTORY_COMBO);
+    find->xml = moo_find_box_xml_new ();
+    g_return_if_fail (find->xml != NULL);
 
-    if (!moo_glade_xml_parse_memory (find->xml, mootextfind_glade_xml, -1, "vbox", NULL))
-    {
-        g_object_unref (find->xml);
-        find->xml = NULL;
-        g_return_if_reached ();
-    }
-
-    vbox = moo_glade_xml_get_widget (find->xml, "vbox");
-
-    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(find)->vbox), vbox);
+    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(find)->vbox),
+                       GTK_WIDGET (find->xml->MooFindBox));
     gtk_dialog_set_has_separator (GTK_DIALOG (find), FALSE);
 
-    search = moo_glade_xml_get_widget (find->xml, "search_entry");
-    replace = moo_glade_xml_get_widget (find->xml, "replace_entry");
-
+    search = MOO_COMBO (find->xml->search_entry);
+    replace = MOO_COMBO (find->xml->replace_entry);
     moo_entry_set_use_special_chars_menu (MOO_ENTRY (search->entry), TRUE);
     moo_entry_set_use_special_chars_menu (MOO_ENTRY (replace->entry), TRUE);
 
-    moo_history_combo_set_list (MOO_HISTORY_COMBO (search), search_history);
-    moo_history_combo_set_list (MOO_HISTORY_COMBO (replace), replace_history);
+    moo_history_combo_set_list (find->xml->search_entry, search_history);
+    moo_history_combo_set_list (find->xml->replace_entry, replace_history);
 }
 
 
@@ -178,11 +170,9 @@ moo_find_constructor (GType           type,
         title = "Find";
     }
 
-    gtk_widget_set_sensitive (moo_glade_xml_get_widget (find->xml, "backwards"), !use_replace);
-    g_object_set (moo_glade_xml_get_widget (find->xml, "replace_frame"),
-                  "visible", use_replace, NULL);
-    g_object_set (moo_glade_xml_get_widget (find->xml, "dont_prompt"),
-                  "visible", use_replace, NULL);
+    gtk_widget_set_sensitive (GTK_WIDGET (find->xml->backwards), !use_replace);
+    g_object_set (find->xml->replace_frame, "visible", use_replace, NULL);
+    g_object_set (find->xml->dont_prompt, "visible", use_replace, NULL);
 
     gtk_window_set_title (GTK_WINDOW (find), title);
     gtk_dialog_add_buttons (GTK_DIALOG (find),
@@ -250,7 +240,6 @@ moo_find_finalize (GObject *object)
 {
     MooFind *find = MOO_FIND (object);
 
-    g_object_unref (find->xml);
     REGEX_FREE (find->regex);
 
     G_OBJECT_CLASS(moo_find_parent_class)->finalize (object);
@@ -376,7 +365,6 @@ moo_find_setup (MooFind        *find,
 {
     GtkTextBuffer *buffer;
     GtkTextIter sel_start, sel_end;
-    MooCombo *search_entry, *replace_entry;
     char *search_term;
     gboolean has_selection;
 
@@ -386,15 +374,15 @@ moo_find_setup (MooFind        *find,
     moo_window_set_parent (GTK_WIDGET (find), GTK_WIDGET (view));
 
     buffer = gtk_text_view_get_buffer (view);
-    search_entry = moo_glade_xml_get_widget (find->xml, "search_entry");
-    replace_entry = moo_glade_xml_get_widget (find->xml, "replace_entry");
 
     search_term = get_search_term (view, TRUE, NULL, NULL);
 
     if (search_term && *search_term)
-        gtk_entry_set_text (GTK_ENTRY (search_entry->entry), search_term);
+        gtk_entry_set_text (GTK_ENTRY (MOO_COMBO (find->xml->search_entry)->entry),
+                            search_term);
     else if (last_search)
-        gtk_entry_set_text (GTK_ENTRY (search_entry->entry), last_search);
+        gtk_entry_set_text (GTK_ENTRY (MOO_COMBO (find->xml->search_entry)->entry),
+                            last_search);
 
     if (find->replace)
     {
@@ -405,7 +393,8 @@ moo_find_setup (MooFind        *find,
             replace_with = freeme = moo_history_list_get_last_item (replace_history);
 
         if (replace_with)
-            gtk_entry_set_text (GTK_ENTRY (replace_entry->entry), replace_with);
+            gtk_entry_set_text (GTK_ENTRY (MOO_COMBO (find->xml->replace_entry)->entry),
+                                replace_with);
 
         g_free (freeme);
     }
@@ -413,15 +402,15 @@ moo_find_setup (MooFind        *find,
     has_selection = gtk_text_buffer_get_selection_bounds (buffer, &sel_start, &sel_end);
 
     if (!has_selection)
-        gtk_widget_set_sensitive (moo_glade_xml_get_widget (find->xml, "selected"), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET (find->xml->selected), FALSE);
 
     if (find->replace && has_selection &&
         gtk_text_iter_get_line (&sel_start) != gtk_text_iter_get_line (&sel_end))
-            gtk_toggle_button_set_active (moo_glade_xml_get_widget (find->xml, "selected"), TRUE);
+            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (find->xml->selected), TRUE);
     else
-        gtk_toggle_button_set_active (moo_glade_xml_get_widget (find->xml, "selected"), FALSE);
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (find->xml->selected), FALSE);
 
-    moo_entry_clear_undo (MOO_ENTRY (search_entry->entry));
+    moo_entry_clear_undo (MOO_ENTRY (MOO_COMBO (find->xml->search_entry)->entry));
 
     moo_find_set_flags (find, last_search_flags);
 
@@ -442,7 +431,7 @@ print_messagev (MooFindMsgFunc func,
     g_free (msg);
 }
 
-static void
+static void G_GNUC_PRINTF(3,4)
 print_message (MooFindMsgFunc func,
                gpointer       data,
                const char    *format,
@@ -465,12 +454,8 @@ moo_find_run (MooFind        *find,
               MooFindMsgFunc  msg_func,
               gpointer        data)
 {
-    MooCombo *search_entry, *replace_entry;
     g_return_val_if_fail (MOO_IS_FIND (find), FALSE);
 
-    search_entry = moo_glade_xml_get_widget (find->xml, "search_entry");
-    replace_entry = moo_glade_xml_get_widget (find->xml, "replace_entry");
-
     REGEX_FREE (find->regex);
 
     while (TRUE)
@@ -481,8 +466,8 @@ moo_find_run (MooFind        *find,
         if (gtk_dialog_run (GTK_DIALOG (find)) != GTK_RESPONSE_OK)
             return FALSE;
 
-        search_for = moo_combo_entry_get_text (search_entry);
-        replace_with = moo_combo_entry_get_text (replace_entry);
+        search_for = moo_combo_entry_get_text (MOO_COMBO (find->xml->search_entry));
+        replace_with = moo_combo_entry_get_text (MOO_COMBO (find->xml->replace_entry));
 
         if (!search_for[0])
         {
@@ -554,25 +539,25 @@ moo_find_get_flags (MooFind *find)
 
     g_return_val_if_fail (MOO_IS_FIND (find), 0);
 
-    if (gtk_toggle_button_get_active (moo_glade_xml_get_widget (find->xml, "regex")))
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (find->xml->regex)))
         flags |= MOO_FIND_REGEX;
-    if (gtk_toggle_button_get_active (moo_glade_xml_get_widget (find->xml, "repl_literal")))
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (find->xml->repl_literal)))
         flags |= MOO_FIND_REPL_LITERAL;
-    if (gtk_toggle_button_get_active (moo_glade_xml_get_widget (find->xml, "whole_words")))
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (find->xml->whole_words)))
         flags |= MOO_FIND_WHOLE_WORDS;
-    if (gtk_toggle_button_get_active (moo_glade_xml_get_widget (find->xml, "from_cursor")))
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (find->xml->from_cursor)))
         flags |= MOO_FIND_FROM_CURSOR;
-    if (gtk_toggle_button_get_active (moo_glade_xml_get_widget (find->xml, "backwards")))
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (find->xml->backwards)))
         flags |= MOO_FIND_BACKWARDS;
-    if (gtk_toggle_button_get_active (moo_glade_xml_get_widget (find->xml, "selected")))
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (find->xml->selected)))
         flags |= MOO_FIND_IN_SELECTED;
-    if (gtk_toggle_button_get_active (moo_glade_xml_get_widget (find->xml, "dont_prompt")))
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (find->xml->dont_prompt)))
         flags |= MOO_FIND_DONT_PROMPT;
 
     if (!(flags & MOO_FIND_REGEX))
         flags |= MOO_FIND_REPL_LITERAL;
 
-    if (!gtk_toggle_button_get_active (moo_glade_xml_get_widget (find->xml, "case_sensitive")))
+    if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (find->xml->case_sensitive)))
         flags |= MOO_FIND_CASELESS;
 
     return flags;
@@ -585,22 +570,22 @@ moo_find_set_flags (MooFind        *find,
 {
     g_return_if_fail (MOO_IS_FIND (find));
 
-    gtk_toggle_button_set_active (moo_glade_xml_get_widget (find->xml, "regex"),
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (find->xml->regex),
                                   (flags & MOO_FIND_REGEX) ? TRUE : FALSE);
-    if ((flags & MOO_FIND_REPL_LITERAL) && (flags & MOO_FIND_REGEX))
-        gtk_toggle_button_set_active (moo_glade_xml_get_widget (find->xml, "repl_literal"), TRUE);
-    else
-        gtk_toggle_button_set_active (moo_glade_xml_get_widget (find->xml, "repl_literal"), FALSE);
-    gtk_toggle_button_set_active (moo_glade_xml_get_widget (find->xml, "whole_words"),
+
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (find->xml->repl_literal),
+                                  (flags & MOO_FIND_REPL_LITERAL) && (flags & MOO_FIND_REGEX));
+
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (find->xml->whole_words),
                                   (flags & MOO_FIND_WHOLE_WORDS) ? TRUE : FALSE);
-    gtk_toggle_button_set_active (moo_glade_xml_get_widget (find->xml, "from_cursor"),
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (find->xml->from_cursor),
                                   (flags & MOO_FIND_FROM_CURSOR) ? TRUE : FALSE);
-    gtk_toggle_button_set_active (moo_glade_xml_get_widget (find->xml, "backwards"),
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (find->xml->backwards),
                                   (flags & MOO_FIND_BACKWARDS) ? TRUE : FALSE);
-    gtk_toggle_button_set_active (moo_glade_xml_get_widget (find->xml, "dont_prompt"),
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (find->xml->dont_prompt),
                                   (flags & MOO_FIND_DONT_PROMPT) ? TRUE : FALSE);
 
-    gtk_toggle_button_set_active (moo_glade_xml_get_widget (find->xml, "case_sensitive"),
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (find->xml->case_sensitive),
                                   (flags & MOO_FIND_CASELESS) ? FALSE : TRUE);
 }
 
@@ -608,10 +593,8 @@ moo_find_set_flags (MooFind        *find,
 static char *
 moo_find_get_text (MooFind *find)
 {
-    MooCombo *entry;
     g_return_val_if_fail (MOO_IS_FIND (find), NULL);
-    entry = moo_glade_xml_get_widget (find->xml, "search_entry");
-    return g_strdup (moo_combo_entry_get_text (entry));
+    return g_strdup (moo_combo_entry_get_text (MOO_COMBO (find->xml->search_entry)));
 }
 
 
@@ -626,10 +609,8 @@ moo_find_get_regex (MooFind *find)
 static char *
 moo_find_get_replacement (MooFind *find)
 {
-    MooCombo *entry;
     g_return_val_if_fail (MOO_IS_FIND (find), NULL);
-    entry = moo_glade_xml_get_widget (find->xml, "replace_entry");
-    return g_strdup (moo_combo_entry_get_text (entry));
+    return g_strdup (moo_combo_entry_get_text (MOO_COMBO (find->xml->replace_entry)));
 }
 
 
@@ -1340,26 +1321,21 @@ update_scale_value (GtkSpinButton  *spin,
 void
 moo_text_view_run_goto_line (GtkTextView *view)
 {
-    GtkWidget *dialog;
+    GtkDialog *dialog;
     GtkTextBuffer *buffer;
     int line_count, line;
     GtkTextIter iter;
-    GtkRange *scale;
-    GtkSpinButton *spin;
-    MooGladeXML *xml;
+    GotoLineDialogXml *xml;
 
     g_return_if_fail (GTK_IS_TEXT_VIEW (view));
 
     buffer = gtk_text_view_get_buffer (view);
     line_count = gtk_text_buffer_get_line_count (buffer);
 
-    xml = moo_glade_xml_new_from_buf (mootextgotoline_glade_xml, -1,
-                                      NULL, GETTEXT_PACKAGE, NULL);
-    g_return_if_fail (xml != NULL);
+    xml = goto_line_dialog_xml_new ();
+    dialog = xml->GotoLineDialog;
 
-    dialog = moo_glade_xml_get_widget (xml, "dialog");
-
-    gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+    gtk_dialog_set_alternative_button_order (dialog,
                                              GTK_RESPONSE_OK,
                                              GTK_RESPONSE_CANCEL,
                                              -1);
@@ -1368,31 +1344,28 @@ moo_text_view_run_goto_line (GtkTextView *view)
                                       gtk_text_buffer_get_insert (buffer));
     line = gtk_text_iter_get_line (&iter);
 
-    scale = moo_glade_xml_get_widget (xml, "scale");
-    gtk_range_set_range (scale, 1, line_count + 1);
-    gtk_range_set_value (scale, line + 1);
+    gtk_range_set_range (GTK_RANGE (xml->scale), 1, line_count + 1);
+    gtk_range_set_value (GTK_RANGE (xml->scale), line + 1);
 
-    spin = moo_glade_xml_get_widget (xml, "spin");
-    gtk_entry_set_activates_default (GTK_ENTRY (spin), TRUE);
-    gtk_spin_button_set_range (spin, 1, line_count);
-    gtk_spin_button_set_value (spin, line + 1);
-    gtk_editable_select_region (GTK_EDITABLE (spin), 0, -1);
+    gtk_entry_set_activates_default (GTK_ENTRY (xml->spin), TRUE);
+    gtk_spin_button_set_range (xml->spin, 1, line_count);
+    gtk_spin_button_set_value (xml->spin, line + 1);
+    gtk_editable_select_region (GTK_EDITABLE (xml->spin), 0, -1);
 
-    g_signal_connect (scale, "value-changed", G_CALLBACK (update_spin_value), spin);
-    g_signal_connect (spin, "value-changed", G_CALLBACK (update_scale_value), scale);
+    g_signal_connect (xml->scale, "value-changed", G_CALLBACK (update_spin_value), xml->spin);
+    g_signal_connect (xml->spin, "value-changed", G_CALLBACK (update_scale_value), xml->scale);
 
-    moo_window_set_parent (dialog, GTK_WIDGET (view));
+    moo_window_set_parent (GTK_WIDGET (dialog), GTK_WIDGET (view));
 
-    g_object_unref (xml);
-
-    if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_OK)
+    if (gtk_dialog_run (dialog) != GTK_RESPONSE_OK)
     {
-        gtk_widget_destroy (dialog);
+        gtk_widget_destroy (GTK_WIDGET (dialog));
         return;
     }
 
-    line = gtk_spin_button_get_value (spin) - 1;
-    gtk_widget_destroy (dialog);
+    gtk_spin_button_update (xml->spin);
+    line = gtk_spin_button_get_value (xml->spin) - 1;
+    gtk_widget_destroy (GTK_WIDGET (dialog));
 
     if (MOO_IS_TEXT_VIEW (view))
     {
