@@ -1,13 +1,16 @@
 /*
  *   mootextfind.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_TEXT_FIND_H
@@ -15,8 +18,8 @@
 
 #include <gtk/gtktextview.h>
 #include <gtk/gtkdialog.h>
-#include "mooutils/moohistorylist.h"
-#include "mooutils/mooglade.h"
+#include <mooedit/mooedit-enums.h>
+#include <mooutils/moohistorylist.h>
 #include <glib/gregex.h>
 
 G_BEGIN_DECLS
@@ -33,22 +36,10 @@ G_BEGIN_DECLS
 typedef struct _MooFind       MooFind;
 typedef struct _MooFindClass  MooFindClass;
 
-typedef enum /*< flags >*/
-{
-    MOO_FIND_REGEX              = 1 << 0,
-    MOO_FIND_CASELESS           = 1 << 1,
-    MOO_FIND_IN_SELECTED        = 1 << 2,
-    MOO_FIND_BACKWARDS          = 1 << 3,
-    MOO_FIND_WHOLE_WORDS        = 1 << 4,
-    MOO_FIND_FROM_CURSOR        = 1 << 5,
-    MOO_FIND_DONT_PROMPT        = 1 << 6,
-    MOO_FIND_REPL_LITERAL       = 1 << 7
-} MooFindFlags;
-
 struct _MooFind
 {
     GtkDialog base;
-    MooGladeXML *xml;
+    struct MooFindBoxXml *xml;
     struct MooRegex *regex;
     guint replace : 1;
 };
