@@ -1,28 +1,31 @@
 /*
  *   mootextprint.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
 #include "mooedit/mootextprint-private.h"
 #include "mooedit/mooedit.h"
-#include "mooedit/mooedit-private.h"
-#include "mooedit/mooprint-glade.h"
+#include "mooedit/mooedit-impl.h"
 #include "mooedit/mooeditprefs.h"
 #include "mooedit/mootext-private.h"
 #include "mooedit/mooprintpreview.h"
-#include "mooutils/mooglade.h"
 #include "mooutils/moodialogs.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-debug.h"
+#include "mooutils/mootype-macros.h"
+#include "glade/mooprint-gxml.h"
 #include <sys/types.h>
 #include <time.h>
 #include <errno.h>
@@ -261,7 +264,7 @@ moo_print_settings_new_default (void)
 
 
 static MooPrintSettings *
-moo_print_settings_copy (MooPrintSettings *settings)
+_moo_print_settings_copy (MooPrintSettings *settings)
 {
     MooPrintSettings *copy;
 
@@ -277,9 +280,8 @@ moo_print_settings_copy (MooPrintSettings *settings)
     return copy;
 }
 
-
 static void
-moo_print_settings_free (MooPrintSettings *settings)
+_moo_print_settings_free (MooPrintSettings *settings)
 {
     if (settings)
     {
@@ -291,19 +293,7 @@ moo_print_settings_free (MooPrintSettings *settings)
     }
 }
 
-
-GType
-_moo_print_settings_get_type (void)
-{
-    static GType t = 0;
-
-    if (G_UNLIKELY (!t))
-        t = g_boxed_type_register_static ("MooPrintSettings",
-                                          (GBoxedCopyFunc) moo_print_settings_copy,
-                                          (GBoxedFreeFunc) moo_print_settings_free);
-
-    return t;
-}
+MOO_DEFINE_BOXED_TYPE_C (MooPrintSettings, _moo_print_settings)
 
 
 static void
@@ -314,10 +304,10 @@ moo_print_operation_set_settings (MooPrintOperation *op,
 
     if (op->priv->settings != settings)
     {
-        moo_print_settings_free (op->priv->settings);
+        _moo_print_settings_free (op->priv->settings);
 
         if (settings)
-            op->priv->settings = moo_print_settings_copy (settings);
+            op->priv->settings = _moo_print_settings_copy (settings);
         else
             op->priv->settings = moo_print_settings_new_default ();
 
@@ -339,7 +329,7 @@ moo_print_operation_finalize (GObject *object)
     if (op->priv->buffer)
         g_object_unref (op->priv->buffer);
 
-    moo_print_settings_free (op->priv->settings);
+    _moo_print_settings_free (op->priv->settings);
     g_free (op->priv->filename);
     g_free (op->priv->basename);
     g_free (op->priv->tm);
@@ -1470,7 +1460,7 @@ moo_print_operation_draw_page (GtkPrintOperation *operation,
     }
 #endif
 
-#if 0 && defined(MOO_DEBUG_ENABLED) && !defined(__WIN32__)
+#if 0 && defined(MOO_DEBUG) && !defined(__WIN32__)
     cairo_save (cr);
     cairo_set_line_width (cr, 1.);
     cairo_set_source_rgb (cr, 1., 0., 0.);
@@ -1557,12 +1547,12 @@ update_progress (GtkPrintOperation *operation,
     }
 
     if (window)
-        moo_edit_window_message (window, NULL);
+        moo_window_message (MOO_WINDOW (window), NULL);
 
     if (moo_edit_get_state (doc) == MOO_EDIT_STATE_PRINTING)
         _moo_edit_set_progress_text (doc, text);
     else if (window)
-        moo_edit_window_message (window, text);
+        moo_window_message (MOO_WINDOW (window), text);
 
     g_free (text);
 }
@@ -1628,7 +1618,7 @@ moo_print_operation_load_prefs (MooPrintOperation *op)
 {
     MooPrintSettings *settings = get_settings_from_prefs ();
     moo_print_operation_set_settings (op, settings);
-    moo_print_settings_free (settings);
+    _moo_print_settings_free (settings);
 }
 
 
@@ -1763,29 +1753,23 @@ _moo_print_operation_get_n_pages (MooPrintOperation *op)
 
 #define SET_TEXT(wid,setting)                                           \
 G_STMT_START {                                                          \
-    GtkEntry *entry__ = moo_glade_xml_get_widget (xml, wid);            \
     const char *s__ = moo_prefs_get_string (setting);                   \
-    gtk_entry_set_text (entry__, s__ ? s__ : "");                       \
+    gtk_entry_set_text (xml->wid, s__ ? s__ : "");                      \
 } G_STMT_END
 
 #define GET_TEXT(wid,setting)                                           \
 G_STMT_START {                                                          \
-    GtkEntry *entry__ = moo_glade_xml_get_widget (xml, wid);            \
-    const char *s__ = gtk_entry_get_text (entry__);                     \
+    const char *s__ = gtk_entry_get_text (xml->wid);                    \
     moo_prefs_set_string (setting, s__ && s__[0] ? s__ : NULL);         \
 } G_STMT_END
 
 #define SET_BOOL(wid,setting)                                           \
-G_STMT_START {                                                          \
-    gtk_toggle_button_set_active (moo_glade_xml_get_widget (xml, wid),  \
-                                  moo_prefs_get_bool (setting));        \
-} G_STMT_END
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (xml->wid),         \
+                                  moo_prefs_get_bool (setting))
 
 #define GET_BOOL(wid,setting)                                           \
-G_STMT_START {                                                          \
-    gpointer btn__ = moo_glade_xml_get_widget (xml, wid);               \
-    moo_prefs_set_bool (setting, gtk_toggle_button_get_active (btn__)); \
-} G_STMT_END
+    moo_prefs_set_bool (setting,                                        \
+        gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (xml->wid)))
 
 
 static void
@@ -1813,65 +1797,70 @@ moo_print_init_prefs (void)
 
 
 static void
-set_options (MooGladeXML *xml)
+set_options (PrintWidgetXml *xml)
 {
     const char *s;
 
+    g_return_if_fail (xml != NULL);
     moo_print_init_prefs ();
 
-    SET_BOOL ("print_header", PREFS_PRINT_HEADER);
-    SET_BOOL ("header_separator", PREFS_PRINT_HEADER_SEPARATOR);
+    SET_BOOL (print_header, PREFS_PRINT_HEADER);
+    SET_BOOL (header_separator, PREFS_PRINT_HEADER_SEPARATOR);
 
-    SET_TEXT ("header_left", PREFS_HEADER_LEFT);
-    SET_TEXT ("header_center", PREFS_HEADER_CENTER);
-    SET_TEXT ("header_right", PREFS_HEADER_RIGHT);
+    SET_TEXT (header_left, PREFS_HEADER_LEFT);
+    SET_TEXT (header_center, PREFS_HEADER_CENTER);
+    SET_TEXT (header_right, PREFS_HEADER_RIGHT);
 
-    SET_BOOL ("print_footer", PREFS_PRINT_FOOTER);
-    SET_BOOL ("footer_separator", PREFS_PRINT_FOOTER_SEPARATOR);
+    SET_BOOL (print_footer, PREFS_PRINT_FOOTER);
+    SET_BOOL (footer_separator, PREFS_PRINT_FOOTER_SEPARATOR);
 
-    SET_TEXT ("footer_left", PREFS_FOOTER_LEFT);
-    SET_TEXT ("footer_center", PREFS_FOOTER_CENTER);
-    SET_TEXT ("footer_right", PREFS_FOOTER_RIGHT);
+    SET_TEXT (footer_left, PREFS_FOOTER_LEFT);
+    SET_TEXT (footer_center, PREFS_FOOTER_CENTER);
+    SET_TEXT (footer_right, PREFS_FOOTER_RIGHT);
 
-    SET_BOOL ("use_styles", PREFS_USE_STYLES);
-    SET_BOOL ("use_custom_font", PREFS_USE_CUSTOM_FONT);
-    SET_BOOL ("wrap", PREFS_WRAP);
-    SET_BOOL ("ellipsize", PREFS_ELLIPSIZE);
-    SET_BOOL ("line_numbers", PREFS_LINE_NUMBERS);
+    SET_BOOL (use_styles, PREFS_USE_STYLES);
+    SET_BOOL (use_custom_font, PREFS_USE_CUSTOM_FONT);
+    SET_BOOL (wrap, PREFS_WRAP);
+    SET_BOOL (ellipsize, PREFS_ELLIPSIZE);
+    SET_BOOL (line_numbers, PREFS_LINE_NUMBERS);
 
-    gtk_spin_button_set_value (moo_glade_xml_get_widget (xml, "line_numbers_step"),
+    gtk_spin_button_set_value (xml->line_numbers_step,
                                moo_prefs_get_int (PREFS_LINE_NUMBERS_STEP));
 
     if ((s = moo_prefs_get_string (PREFS_FONT)))
-        gtk_font_button_set_font_name (moo_glade_xml_get_widget (xml, "font"), s);
+        gtk_font_button_set_font_name (xml->font, s);
 }
 
 
 static void
-get_options (MooGladeXML *xml)
+get_options (PrintWidgetXml *xml)
 {
-    GET_BOOL ("print_header", PREFS_PRINT_HEADER);
-    GET_BOOL ("header_separator", PREFS_PRINT_HEADER_SEPARATOR);
-    GET_TEXT ("header_left", PREFS_HEADER_LEFT);
-    GET_TEXT ("header_center", PREFS_HEADER_CENTER);
-    GET_TEXT ("header_right", PREFS_HEADER_RIGHT);
+    g_return_if_fail (xml != NULL);
 
-    GET_BOOL ("print_footer", PREFS_PRINT_FOOTER);
-    GET_BOOL ("footer_separator", PREFS_PRINT_FOOTER_SEPARATOR);
-    GET_TEXT ("footer_left", PREFS_FOOTER_LEFT);
-    GET_TEXT ("footer_center", PREFS_FOOTER_CENTER);
-    GET_TEXT ("footer_right", PREFS_FOOTER_RIGHT);
+    GET_BOOL (print_header, PREFS_PRINT_HEADER);
+    GET_BOOL (header_separator, PREFS_PRINT_HEADER_SEPARATOR);
+    GET_TEXT (header_left, PREFS_HEADER_LEFT);
+    GET_TEXT (header_center, PREFS_HEADER_CENTER);
+    GET_TEXT (header_right, PREFS_HEADER_RIGHT);
 
-    GET_BOOL ("use_styles", PREFS_USE_STYLES);
-    GET_BOOL ("use_custom_font", PREFS_USE_CUSTOM_FONT);
-    GET_BOOL ("wrap", PREFS_WRAP);
-    GET_BOOL ("ellipsize", PREFS_ELLIPSIZE);
-    GET_BOOL ("line_numbers", PREFS_LINE_NUMBERS);
+    GET_BOOL (print_footer, PREFS_PRINT_FOOTER);
+    GET_BOOL (footer_separator, PREFS_PRINT_FOOTER_SEPARATOR);
+    GET_TEXT (footer_left, PREFS_FOOTER_LEFT);
+    GET_TEXT (footer_center, PREFS_FOOTER_CENTER);
+    GET_TEXT (footer_right, PREFS_FOOTER_RIGHT);
 
-    if (gtk_toggle_button_get_active (moo_glade_xml_get_widget (xml, "line_numbers")))
+    GET_BOOL (use_styles, PREFS_USE_STYLES);
+    GET_BOOL (use_custom_font, PREFS_USE_CUSTOM_FONT);
+    GET_BOOL (wrap, PREFS_WRAP);
+    GET_BOOL (ellipsize, PREFS_ELLIPSIZE);
+    GET_BOOL (line_numbers, PREFS_LINE_NUMBERS);
+
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (xml->line_numbers)))
     {
-        GtkSpinButton *btn = moo_glade_xml_get_widget (xml, "line_numbers_step");
-        int step = gtk_spin_button_get_value_as_int (btn);
+        GtkSpinButton *btn = xml->line_numbers_step;
+        int step;
+        gtk_spin_button_update (btn);
+        step = gtk_spin_button_get_value_as_int (btn);
         if (step < 1)
         {
             step = 1;
@@ -1880,9 +1869,8 @@ get_options (MooGladeXML *xml)
         moo_prefs_set_int (PREFS_LINE_NUMBERS_STEP, step);
     }
 
-    if (gtk_toggle_button_get_active (moo_glade_xml_get_widget (xml, "use_custom_font")))
-        moo_prefs_set_string (PREFS_FONT,
-                              gtk_font_button_get_font_name (moo_glade_xml_get_widget (xml, "font")));
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (xml->use_custom_font)))
+        moo_prefs_set_string (PREFS_FONT, gtk_font_button_get_font_name (xml->font));
 }
 
 
@@ -1891,7 +1879,7 @@ void
 _moo_edit_print_options_dialog (GtkWidget *parent)
 {
     GtkWidget *dialog;
-    MooGladeXML *xml;
+    PrintWidgetXml *xml;
 
     xml = moo_glade_xml_new_from_buf (MOO_PRINT_GLADE_XML, -1, NULL, GETTEXT_PACKAGE, NULL);
     g_return_if_fail (xml != NULL);
@@ -1915,27 +1903,18 @@ _moo_edit_print_options_dialog (GtkWidget *parent)
 static GtkWidget *
 moo_print_operation_create_custom_widget (G_GNUC_UNUSED GtkPrintOperation *operation)
 {
-    GtkWidget *page, *font_button, *line_numbers_hbox;
-    MooGladeXML *xml;
+    PrintWidgetXml *xml;
 
-    xml = moo_glade_xml_new_from_buf (mooprint_glade_xml, -1, "page", GETTEXT_PACKAGE, NULL);
-    g_return_val_if_fail (xml != NULL, NULL);
+    xml = print_widget_xml_new ();
 
-    font_button = moo_glade_xml_get_widget (xml, "font");
-    moo_bind_sensitive (moo_glade_xml_get_widget (xml, "use_custom_font"),
-                        &font_button, 1, FALSE);
+    moo_bind_sensitive (GTK_WIDGET (xml->wrap), GTK_WIDGET (xml->ellipsize), TRUE);
+    moo_bind_sensitive (GTK_WIDGET (xml->print_header), GTK_WIDGET (xml->header_alignment), FALSE);
+    moo_bind_sensitive (GTK_WIDGET (xml->print_footer), GTK_WIDGET (xml->footer_alignment), FALSE);
+    moo_bind_sensitive (GTK_WIDGET (xml->use_custom_font), GTK_WIDGET (xml->font), FALSE);
+    moo_bind_sensitive (GTK_WIDGET (xml->line_numbers), GTK_WIDGET (xml->line_numbers_hbox), FALSE);
 
-    line_numbers_hbox = moo_glade_xml_get_widget (xml, "line_numbers_hbox");
-    moo_bind_sensitive (moo_glade_xml_get_widget (xml, "line_numbers"),
-                        &line_numbers_hbox, 1, FALSE);
-
-    page = moo_glade_xml_get_widget (xml, "page");
-    g_return_val_if_fail (page != NULL, NULL);
-
-    g_object_set_data_full (G_OBJECT (page), "moo-glade-xml",
-                            xml, g_object_unref);
     set_options (xml);
-    return page;
+    return GTK_WIDGET (xml->PrintWidget);
 }
 
 
@@ -1943,12 +1922,7 @@ static void
 moo_print_operation_custom_widget_apply (G_GNUC_UNUSED GtkPrintOperation *print,
                                          GtkWidget *widget)
 {
-    MooGladeXML *xml;
-
-    xml = g_object_get_data (G_OBJECT (widget), "moo-glade-xml");
-    g_return_if_fail (xml != NULL);
-
-    get_options (xml);
+    get_options (print_widget_xml_get (widget));
 }
 
 
