@@ -1,13 +1,16 @@
 /*
  *   mootextsearch.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooedit/mootextsearch-private.h"
@@ -58,7 +61,7 @@ _moo_regex_new (GRegex *re)
 
     g_return_val_if_fail (re != NULL, NULL);
 
-    regex = _moo_new (MooRegex);
+    regex = moo_new (MooRegex);
     regex->re = g_regex_ref (re);
     regex->ref_count_ = 1;
 
@@ -103,7 +106,7 @@ _moo_regex_unref (MooRegex *regex)
     if (!--regex->ref_count_)
     {
         g_regex_unref (regex->re);
-        _moo_free (MooRegex, regex);
+        moo_free (MooRegex, regex);
     }
 }
 
