@@ -1,31 +1,27 @@
 /*
  *   mootextsearch.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_TEXT_SEARCH_H
 #define MOO_TEXT_SEARCH_H
 
 #include <mooedit/mootextiter.h>
+#include <mooedit/mooedit-enums.h>
 
 G_BEGIN_DECLS
 
 
-typedef enum /*< flags >*/
-{
-    MOO_TEXT_SEARCH_CASELESS        = 1 << 0,
-    MOO_TEXT_SEARCH_REGEX           = 1 << 1,
-    MOO_TEXT_SEARCH_WHOLE_WORDS     = 1 << 2,
-    MOO_TEXT_SEARCH_REPL_LITERAL    = 1 << 3
-} MooTextSearchFlags;
-
 gboolean moo_text_search_forward            (const GtkTextIter      *start,
                                              const char             *str,
                                              MooTextSearchFlags      flags,
