@@ -1,13 +1,16 @@
 /*
  *   mootextview-private.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOOEDIT_COMPILATION
@@ -32,6 +35,7 @@ void        _moo_text_view_move_cursor          (GtkTextView        *text_view,
                                                  GtkMovementStep     step,
                                                  gint                count,
                                                  gboolean            extend_selection);
+void        _moo_text_view_ensure_primary       (GtkTextView        *text_view);
 #if !GTK_CHECK_VERSION(2,12,0)
 void        _moo_text_view_page_horizontally    (GtkTextView        *text_view,
                                                  int                 count,
@@ -61,6 +65,11 @@ int         _moo_text_view_get_line_height      (MooTextView        *view);
 void        _moo_text_view_set_line_numbers_font (MooTextView       *view,
                                                  const char         *name);
 
+void        _moo_text_view_update_text_cursor   (MooTextView        *view,
+                                                 int                 x,
+                                                 int                 y);
+
+extern gpointer _moo_text_view_parent_class;
 
 typedef enum {
     MOO_TEXT_VIEW_DRAG_NONE = 0,
@@ -83,8 +92,15 @@ typedef enum {
     MOO_TEXT_VIEW_N_COLORS
 } MooTextViewColor;
 
+typedef struct {
+    MooTextView *view;
+    char *text;
+} MooTextViewClipboard;
+
 struct _MooTextViewPrivate {
     gboolean constructed;
+    GType buffer_type;
+    GtkTextBuffer *buffer;
 
     GdkRectangle *update_rectangle;
     gboolean in_expose;
@@ -93,6 +109,7 @@ struct _MooTextViewPrivate {
 
     /* Clipboard */
     gboolean manage_clipboard;
+    MooTextViewClipboard *clipboard;
 
 #if !GTK_CHECK_VERSION(2,12,0)
     /* Overwrite mode cursor */
@@ -102,6 +119,8 @@ struct _MooTextViewPrivate {
     guint blink_timeout;
 #endif
 
+    MooTextCursor text_cursor;
+
     /***********************************************************************/
     /* Drawing
      */
@@ -157,7 +176,7 @@ struct _MooTextViewPrivate {
     /* key press handler sets this flag in order to distinguish typed in
        characters in buffer's insert-text signal */
     gboolean in_key_press;
-    gunichar char_inserted;
+    char *char_inserted;
     int char_inserted_offset;
 
     gunichar *word_chars;
