@@ -1,13 +1,16 @@
 /*
  *   mootextview.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_TEXT_VIEW_H
@@ -18,6 +21,7 @@
 #include <mooedit/moolang.h>
 #include <mooedit/mootextsearch.h>
 #include <mooedit/mootextstylescheme.h>
+#include <mooedit/mooedit-enums.h>
 
 G_BEGIN_DECLS
 
@@ -34,13 +38,6 @@ typedef struct _MooTextView         MooTextView;
 typedef struct _MooTextViewPrivate  MooTextViewPrivate;
 typedef struct _MooTextViewClass    MooTextViewClass;
 
-typedef enum
-{
-    MOO_TEXT_SELECT_CHARS,
-    MOO_TEXT_SELECT_WORDS,
-    MOO_TEXT_SELECT_LINES
-} MooTextSelectionType;
-
 struct _MooTextView
 {
     GtkTextView  parent;
@@ -57,7 +54,7 @@ struct _MooTextViewClass
 
     gboolean (* char_inserted)      (MooTextView    *view,
                                      GtkTextIter    *where, /* points to position after the char */
-                                     guint           character); /* gunichar */
+                                     const char     *character); /* single character as string */
 
     gboolean (* line_mark_clicked)  (MooTextView    *view,
                                      int             line);
@@ -82,12 +79,18 @@ struct _MooTextViewClass
 
     void (*apply_style_scheme)      (MooTextView    *view,
                                      MooTextStyleScheme *scheme);
+
+    MooTextCursor (*get_text_cursor)(MooTextView    *view,
+                                     int             x,  /* buffer coordinates */
+                                     int             y); /* buffer coordinates */
 };
 
 
 GType        moo_text_view_get_type                 (void) G_GNUC_CONST;
 
 GtkWidget   *moo_text_view_new                      (void);
+void         moo_text_view_set_buffer_type          (MooTextView        *view,
+                                                     GType               type);
 
 void         moo_text_view_select_all               (MooTextView        *view);
 
