@@ -1,13 +1,16 @@
 /*
  *   moobookmarkmgr.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_BOOKMARK_MGR_H
@@ -84,7 +87,7 @@ GtkWidget      *_moo_bookmark_mgr_get_editor(MooBookmarkMgr *mgr);
 void            _moo_bookmark_mgr_add_user  (MooBookmarkMgr *mgr,
                                              gpointer        user, /* GObject* */
                                              MooActionCollection *actions,
-                                             MooUIXML       *xml,
+                                             MooUiXml       *xml,
                                              const char     *path);
 void            _moo_bookmark_mgr_remove_user(MooBookmarkMgr *mgr,
                                              gpointer        user); /* GObject* */
