@@ -1,13 +1,16 @@
 /*
  *   moofile-private.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_FILE_VIEW_COMPILATION
@@ -25,8 +28,6 @@ G_BEGIN_DECLS
 
 
 #define MOO_TYPE_FILE               (_moo_file_get_type ())
-#define MOO_TYPE_FILE_INFO          (_moo_file_info_get_type ())
-#define MOO_TYPE_FILE_FLAGS         (_moo_file_flags_get_type ())
 
 #ifdef __WIN32__
 /* FILETIME */
@@ -38,6 +39,7 @@ typedef GTime MooFileTime;
 #endif
 
 typedef gint64 MooFileSize;
+typedef struct MooCollationKey MooCollationKey;
 
 struct _MooFile
 {
@@ -45,7 +47,7 @@ struct _MooFile
     char           *link_target;
     char           *display_name; /* normalized */
     char           *case_display_name;
-    char           *collation_key;
+    MooCollationKey *collation_key;
     MooFileInfo     info;
     MooFileFlags    flags;
     guint8          icon;
@@ -56,8 +58,6 @@ struct _MooFile
 
 
 GType        _moo_file_get_type         (void) G_GNUC_CONST;
-GType        _moo_file_flags_get_type   (void) G_GNUC_CONST;
-GType        _moo_file_info_get_type    (void) G_GNUC_CONST;
 
 MooFile     *_moo_file_new              (const char     *dirname,
                                          const char     *basename);
@@ -74,8 +74,12 @@ GdkPixbuf   *_moo_file_get_icon         (const MooFile  *file,
                                          GtkWidget      *widget,
                                          GtkIconSize     size);
 
-const char  *_moo_file_collation_key    (const MooFile  *file);
-const char  *_moo_file_case_display_name(const MooFile *file);
+gsize        _moo_collation_key_size    (MooCollationKey *key);
+int          _moo_collation_key_cmp     (const MooCollationKey *key1,
+                                         const MooCollationKey *key2);
+const MooCollationKey *_moo_file_collation_key (const MooFile *file);
+
+const char  *_moo_file_case_display_name(const MooFile  *file);
 
 #ifndef __WIN32__
 const char  *_moo_file_link_get_target  (const MooFile  *file);
