@@ -1,13 +1,16 @@
 /*
  *   moofile.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -21,13 +24,15 @@
 #define MOO_FILE_VIEW_COMPILATION
 #include "moofileview/moofilesystem.h"
 #include "moofileview/moofile-private.h"
-#include "moofileview/moofileicons.h"
+#include "mooutils/moofileicon.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mootype-macros.h"
 #include "mooutils/mooutils-debug.h"
+#include "mooutils/moostat.h"
 #include "mooutils/moocompat.h"
-#include "mooutils/moomarshals.h"
-#include "mooutils/xdgmime/xdgmime.h"
+#include "marshals.h"
+#include "mooutils/moo-mime.h"
 #include <glib/gstdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -37,16 +42,14 @@
 #endif
 #include <errno.h>
 #include <time.h>
+#if defined(HAVE_CARBON)
+#include <CoreServices/CoreServices.h>
+#endif
 #include <gtk/gtkicontheme.h>
 #include <gtk/gtkiconfactory.h>
 #include <gtk/gtkstock.h>
 
-#ifdef MOO_US_XDGMIME
-#define MIME_TYPE_UNKNOWN xdg_mime_type_unknown
-#else
-static const char *mime_type_unknown = "application/octet-stream";
-#define MIME_TYPE_UNKNOWN mime_type_unknown
-#endif
+MOO_DEBUG_INIT(file, FALSE)
 
 #define NORMAL_PRIORITY         G_PRIORITY_DEFAULT_IDLE
 #define NORMAL_TIMEOUT          0.04
@@ -59,36 +62,190 @@ G_STMT_START {              \
     g_timer_stop (timer);   \
 } G_STMT_END
 
-static MooIconType   _get_folder_icon           (const char     *path);
-static MooIconEmblem _get_icon_flags            (const MooFile  *file);
+static MooIconType      get_folder_icon (const char     *path);
+static MooIconEmblem    get_icon_flags  (const MooFile  *file);
 
-static GdkPixbuf    *render_icon                (const MooFile  *file,
-                                                 GtkWidget      *widget,
-                                                 GtkIconSize     size);
-static GdkPixbuf    *render_icon_for_path       (const char     *path,
-                                                 GtkWidget      *widget,
-                                                 GtkIconSize     size);
+MOO_DEFINE_BOXED_TYPE_R (MooFile, _moo_file)
 
 #define MAKE_PATH(dirname,file) g_build_filename (dirname, file->name, NULL)
 
-
 void
 _moo_file_find_mime_type (MooFile    *file,
                           const char *path)
 {
-    file->mime_type = xdg_mime_get_mime_type_for_file (path, file->statbuf);
+    file->mime_type = moo_get_mime_type_for_file (path, file->statbuf);
 
     if (!file->mime_type || !file->mime_type[0])
     {
         /* this should not happen */
         _moo_message ("%s: oops, %s", G_STRLOC, file->display_name);
-        file->mime_type = MIME_TYPE_UNKNOWN;
+        file->mime_type = MOO_MIME_TYPE_UNKNOWN;
     }
 
     file->flags |= MOO_FILE_HAS_MIME_TYPE;
 }
 
 
+#if defined(HAVE_CARBON)
+
+struct MooCollationKey {
+    guint len;
+    UCCollationValue buf[1];
+};
+
+static MooCollationKey *
+create_collation_key (CollatorRef  collator,
+                      const gchar *str,
+                      gssize       len)
+{
+  MooCollationKey *key = NULL;
+  UniChar *str_utf16 = NULL;
+  ItemCount actual_len;
+  ItemCount try_len;
+  glong len_utf16;
+  OSStatus ret;
+  UCCollationValue buf[512];
+
+  if (!collator)
+    goto error;
+
+  str_utf16 = g_utf8_to_utf16 (str, len, NULL, &len_utf16, NULL);
+  try_len = len_utf16 * 5 + 2;
+
+  if (try_len <= sizeof buf)
+    {
+      ret = UCGetCollationKey (collator, str_utf16, len_utf16,
+                               sizeof buf, &actual_len, buf);
+
+      if (ret == 0)
+        {
+          key = g_malloc (sizeof (MooCollationKey) + (actual_len - 1) * sizeof (UCCollationValue));
+          memcpy (key->buf, buf, actual_len * sizeof (UCCollationValue));
+        }
+      else if (ret == kCollateBufferTooSmall)
+        try_len *= 2;
+      else
+        goto error;
+    }
+
+  if (!key)
+    {
+      key = g_malloc (sizeof (MooCollationKey) + (try_len - 1) * sizeof (UCCollationValue));
+      ret = UCGetCollationKey (collator, str_utf16, len_utf16,
+                               try_len, &actual_len, key->buf);
+
+      if (ret == kCollateBufferTooSmall)
+        {
+          try_len *= 2;
+          key = g_realloc (key, sizeof (MooCollationKey) + (actual_len - 1) * sizeof (UCCollationValue));
+          ret = UCGetCollationKey (collator, str_utf16, len_utf16,
+                                   try_len, &actual_len, key->buf);
+        }
+
+      if (ret != 0)
+        goto error;
+    }
+
+  key->len = actual_len;
+
+  g_free (str_utf16);
+  return key;
+
+error:
+  g_free (str_utf16);
+  g_free (key);
+  key = g_new (MooCollationKey, 1);
+  key->len = 0;
+  return key;
+}
+
+G_GNUC_UNUSED static MooCollationKey *
+moo_get_collation_key (const char *str,
+                       gssize      len)
+{
+  static CollatorRef collator;
+
+  if (G_UNLIKELY (!collator))
+    {
+      UCCreateCollator (NULL, 0, kUCCollateStandardOptions, &collator);
+
+      if (!collator)
+        g_warning ("%s: UCCreateCollator failed", G_STRLOC);
+    }
+
+  return create_collation_key (collator, str, len);
+}
+
+static MooCollationKey *
+moo_get_collation_key_for_filename (const char *str,
+                                    gssize      len)
+{
+  static CollatorRef collator;
+
+  if (G_UNLIKELY (!collator))
+    {
+      /* http://developer.apple.com/qa/qa2004/qa1159.html */
+      UCCreateCollator (NULL, 0,
+                        kUCCollateComposeInsensitiveMask
+                         | kUCCollateWidthInsensitiveMask
+                         | kUCCollateCaseInsensitiveMask
+                         | kUCCollateDigitsOverrideMask
+                         | kUCCollateDigitsAsNumberMask
+                         | kUCCollatePunctuationSignificantMask,
+                        &collator);
+
+      if (!collator)
+        g_warning ("%s: UCCreateCollator failed", G_STRLOC);
+    }
+
+  return create_collation_key (collator, str, len);
+}
+
+int
+_moo_collation_key_cmp (const MooCollationKey *key1,
+                        const MooCollationKey *key2)
+{
+  SInt32 order = 0;
+
+  g_return_val_if_fail (key1 != NULL, 0);
+  g_return_val_if_fail (key2 != NULL, 0);
+
+  UCCompareCollationKeys (key1->buf, key1->len,
+                          key2->buf, key2->len,
+                          NULL, &order);
+  return order;
+}
+
+gsize
+_moo_collation_key_size (MooCollationKey *key)
+{
+    return key ?
+            sizeof (MooCollationKey) +
+                key->len * sizeof (UCCollationValue) -
+                sizeof (UCCollationValue) :
+            0;
+}
+
+#else
+
+#define moo_get_collation_key_for_filename(fn,len) ((MooCollationKey*)g_utf8_collate_key_for_filename(fn,len))
+
+int
+_moo_collation_key_cmp (const MooCollationKey *key1,
+                        const MooCollationKey *key2)
+{
+  return strcmp ((const char*) key1, (const char*) key2);
+}
+
+gsize
+_moo_collation_key_size (MooCollationKey *key)
+{
+    return key ? strlen ((char*) key) + 1 : 0;
+}
+
+#endif
+
+
 /********************************************************************/
 /* MooFile
  */
@@ -116,14 +273,14 @@ _moo_file_new (const char *dirname,
         g_return_val_if_fail (display_name != NULL, NULL);
     }
 
-    file = _moo_new0 (MooFile);
+    file = moo_new0 (MooFile);
     file->ref_count = 1;
 
     file->name = g_strdup (basename);
 
     file->display_name = g_utf8_normalize (display_name, -1, G_NORMALIZE_ALL);
     file->case_display_name = g_utf8_casefold (file->display_name, -1);
-    file->collation_key = g_utf8_collate_key_for_filename (file->display_name, -1);
+    file->collation_key = moo_get_collation_key_for_filename (file->display_name, -1);
 
     g_free (path);
     g_free (display_name);
@@ -156,8 +313,8 @@ _moo_file_unref (MooFile *file)
         g_free (file->case_display_name);
         g_free (file->collation_key);
         g_free (file->link_target);
-        _moo_free (struct stat, file->statbuf);
-        _moo_free (MooFile, file);
+        moo_free (struct stat, file->statbuf);
+        moo_free (MooFile, file);
     }
 }
 
@@ -181,13 +338,13 @@ _moo_file_stat (MooFile    *file,
     errno = 0;
 
     if (!file->statbuf)
-        file->statbuf = _moo_new (struct stat);
+        file->statbuf = moo_new (struct stat);
 
-    if (g_lstat (fullname, file->statbuf) != 0)
+    if (moo_lstat (fullname, file->statbuf) != 0)
     {
         if (errno == ENOENT)
         {
-            MOO_DEBUG({
+            MOO_DEBUG_CODE({
                 gchar *display_name = g_filename_display_name (fullname);
                 _moo_message ("%s: file '%s' does not exist",
                               G_STRLOC, display_name);
@@ -197,7 +354,7 @@ _moo_file_stat (MooFile    *file,
         }
         else
         {
-            MOO_DEBUG({
+            MOO_DEBUG_CODE({
                 int save_errno = errno;
                 gchar *display_name = g_filename_display_name (fullname);
                 _moo_message ("%s: error getting information for '%s': %s",
@@ -220,11 +377,11 @@ _moo_file_stat (MooFile    *file,
             file->info |= MOO_FILE_INFO_IS_LINK;
             errno = 0;
 
-            if (g_stat (fullname, file->statbuf) != 0)
+            if (moo_stat (fullname, file->statbuf) != 0)
             {
                 if (errno == ENOENT)
                 {
-                    MOO_DEBUG({
+                    MOO_DEBUG_CODE({
                         gchar *display_name = g_filename_display_name (fullname);
                         _moo_message ("%s: file '%s' is a broken link",
                                       G_STRLOC, display_name);
@@ -234,7 +391,7 @@ _moo_file_stat (MooFile    *file,
                 }
                 else
                 {
-                    MOO_DEBUG({
+                    MOO_DEBUG_CODE({
                         int save_errno = errno;
                         gchar *display_name = g_filename_display_name (fullname);
                         _moo_message ("%s: error getting information for '%s': %s",
@@ -251,7 +408,7 @@ _moo_file_stat (MooFile    *file,
 
             if (len == -1)
             {
-                MOO_DEBUG({
+                MOO_DEBUG_CODE({
                     int save_errno = errno;
                     gchar *display_name = g_filename_display_name (fullname);
                     _moo_message ("%s: error getting link target for '%s': %s",
@@ -309,7 +466,7 @@ void
 _moo_file_free_statbuf (MooFile *file)
 {
     g_return_if_fail (file != NULL);
-    _moo_free (struct stat, file->statbuf);
+    moo_free (struct stat, file->statbuf);
     file->statbuf = NULL;
     file->flags &= ~MOO_FILE_HAS_STAT;
 }
@@ -331,7 +488,7 @@ _moo_file_display_name (const MooFile *file)
     return file->display_name;
 }
 
-const char *
+const MooCollationKey *
 _moo_file_collation_key (const MooFile *file)
 {
     g_return_val_if_fail (file != NULL, NULL);
@@ -366,19 +523,15 @@ _moo_file_get_icon (const MooFile  *file,
                     GtkWidget      *widget,
                     GtkIconSize     size)
 {
+    MooFileIcon icon = {0};
+
     g_return_val_if_fail (file != NULL, NULL);
     g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
-    return render_icon (file, widget, size);
-}
 
-
-GdkPixbuf *
-_moo_get_icon_for_path (const char     *path,
-                        GtkWidget      *widget,
-                        GtkIconSize     size)
-{
-    g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
-    return render_icon_for_path (path, widget, size);
+    icon.mime_type = file->mime_type;
+    icon.type = file->icon;
+    icon.emblem = get_icon_flags (file);
+    return moo_file_icon_get_pixbuf (&icon, widget, size);
 }
 
 
@@ -420,97 +573,6 @@ _moo_file_get_stat (const MooFile *file)
 #endif
 
 
-GType
-_moo_file_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooFile",
-                                             (GBoxedCopyFunc) _moo_file_ref,
-                                             (GBoxedFreeFunc) _moo_file_unref);
-
-    return type;
-}
-
-
-GType
-_moo_file_flags_get_type (void)
-{
-    static GType type = 0;
-
-    static const GFlagsValue values[] = {
-        { MOO_FILE_HAS_MIME_TYPE, (char*)"MOO_FILE_HAS_MIME_TYPE", (char*)"has-mime-type" },
-        { MOO_FILE_HAS_ICON, (char*)"MOO_FILE_HAS_ICON", (char*)"has-icon" },
-        { MOO_FILE_HAS_STAT, (char*)"MOO_FILE_HAS_STAT", (char*)"has-stat" },
-        { MOO_FILE_ALL_FLAGS, (char*)"MOO_FILE_ALL_FLAGS", (char*)"all-flags" },
-        { 0, NULL, NULL }
-    };
-
-    if (G_UNLIKELY (!type))
-        type = g_flags_register_static ("MooFileFlags", values);
-
-    return type;
-}
-
-
-GType
-_moo_file_info_get_type (void)
-{
-    static GType type = 0;
-
-    static const GFlagsValue values[] = {
-        { MOO_FILE_INFO_EXISTS, (char*)"MOO_FILE_INFO_EXISTS", (char*)"exists" },
-        { MOO_FILE_INFO_IS_DIR, (char*)"MOO_FILE_INFO_IS_DIR", (char*)"is-folder" },
-        { MOO_FILE_INFO_IS_HIDDEN, (char*)"MOO_FILE_INFO_IS_HIDDEN", (char*)"is-hidden" },
-        { MOO_FILE_INFO_IS_LINK, (char*)"MOO_FILE_INFO_IS_LINK", (char*)"is-link" },
-        { 0, NULL, NULL }
-    };
-
-    if (G_UNLIKELY (!type))
-        type = g_flags_register_static ("MooFileInfo", values);
-
-    return type;
-}
-
-
-static GdkPixbuf *
-render_icon (const MooFile  *file,
-             GtkWidget      *widget,
-             GtkIconSize     size)
-{
-    GdkPixbuf *pixbuf = _moo_get_icon (widget, file->icon, file->mime_type,
-                                       _get_icon_flags (file), size);
-    g_assert (pixbuf != NULL);
-    return pixbuf;
-}
-
-
-static GdkPixbuf *
-render_icon_for_path (const char     *path,
-                      GtkWidget      *widget,
-                      GtkIconSize     size)
-{
-    const char *mime_type = MIME_TYPE_UNKNOWN;
-
-#ifndef __WIN32__
-    if (path)
-    {
-        mime_type = xdg_mime_get_mime_type_for_file (path, NULL);
-
-        if (!mime_type || !mime_type[0])
-            mime_type = MIME_TYPE_UNKNOWN;
-    }
-#else
-#ifdef __GNUC__
-#warning "Implement render_icon_for_path()"
-#endif
-#endif
-
-    return _moo_get_icon (widget, MOO_ICON_MIME, mime_type, 0, size);
-}
-
-
 guint8
 _moo_file_get_icon_type (MooFile    *file,
                          const char *dirname)
@@ -524,7 +586,7 @@ _moo_file_get_icon_type (MooFile    *file,
     if (MOO_FILE_IS_DIR (file))
     {
         char *path = MAKE_PATH (dirname, file);
-        MooIconType icon = _get_folder_icon (path);
+        MooIconType icon = get_folder_icon (path);
         g_free (path);
         return icon;
     }
@@ -556,7 +618,7 @@ _moo_file_icon_blank (void)
 
 
 static MooIconType
-_get_folder_icon (const char *path)
+get_folder_icon (const char *path)
 {
     static const char *home_path = NULL;
     static char *desktop_path = NULL;
@@ -590,7 +652,7 @@ _get_folder_icon (const char *path)
 
 
 static MooIconEmblem
-_get_icon_flags (const MooFile *file)
+get_icon_flags (const MooFile *file)
 {
     return
 #if 0
