@@ -1,13 +1,16 @@
 /*
  *   moofile.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_FILE_H
@@ -56,13 +59,7 @@ typedef enum {
 
 gboolean     _moo_file_test             (const MooFile  *file,
                                          MooFileInfo     test);
-
 const char  *_moo_file_get_mime_type    (const MooFile  *file);
-
-GdkPixbuf   *_moo_get_icon_for_path     (const char     *path,
-                                         GtkWidget      *widget,
-                                         GtkIconSize     size);
-
 const char  *_moo_file_display_name     (const MooFile  *file);
 
 
