@@ -1,13 +1,16 @@
 /*
  *   moofileentry.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* XXX use MooCombo */
@@ -17,17 +20,17 @@
 #include "moofilesystem.h"
 #include "moofoldermodel.h"
 #include "moofileview-aux.h"
-#include "mooutils/moomarshals.h"
+#include "marshals.h"
 #include "mooutils/mooutils-gobject.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-debug.h"
+#include "mooutils/mooaccel.h"
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
 #define MOD_MASK() (gtk_accelerator_get_default_mod_mask ())
 
 #define COMPLETION_POPUP_LEN 15
-#define COLUMN_FILE MOO_FOLDER_MODEL_COLUMN_FILE
 
 #ifndef __WIN32__
 #define CASE_SENSITIVE_DEFAULT   TRUE
@@ -578,6 +581,8 @@ completion_parse_text (MooFileEntryCompletion *cmpl,
         TAKE_STRING (cmpl->priv->dirname, dirname);
 
         completion_connect_folder (cmpl, folder);
+
+        g_object_unref (folder);
     }
     else if (refilter)
     {
@@ -1508,7 +1513,8 @@ completion_entry_key_press (GtkEntry               *entry,
 {
     g_return_val_if_fail (entry == cmpl->priv->entry, FALSE);
 
-    if (cmpl->priv->enabled && !(event->state & MOD_MASK()) && event->keyval == GDK_Tab)
+    if (cmpl->priv->enabled &&
+        moo_accel_check_event (GTK_WIDGET (entry), event, GDK_Tab, 0))
     {
         completion_tab_key (cmpl);
         return TRUE;
@@ -1559,6 +1565,19 @@ _moo_file_entry_completion_set_path (MooFileEntryCompletion *cmpl,
     g_free (path_utf8);
 }
 
+char *
+_moo_file_entry_completion_get_path (MooFileEntryCompletion *cmpl)
+{
+    const char *text;
+
+    g_return_val_if_fail (MOO_IS_FILE_ENTRY_COMPLETION (cmpl), NULL);
+    g_return_val_if_fail (cmpl->priv->entry != NULL, NULL);
+
+    text = gtk_entry_get_text (GTK_ENTRY (cmpl->priv->entry));
+    return _moo_file_system_get_absolute_path (cmpl->priv->fs, text,
+                                               cmpl->priv->current_dir);
+}
+
 
 /*****************************************************************************/
 /* MooFileEntry
