@@ -1,13 +1,16 @@
 /*
  *   moofileentry.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_FILE_ENTRY_H
@@ -78,6 +81,7 @@ void        _moo_file_entry_completion_set_entry        (MooFileEntryCompletion
 /* converts path to utf8 and sets entry content */
 void        _moo_file_entry_completion_set_path         (MooFileEntryCompletion *completion,
                                                          const char             *path);
+char       *_moo_file_entry_completion_get_path         (MooFileEntryCompletion *completion);
 
 
 G_END_DECLS
