@@ -1,13 +1,16 @@
 /*
  *   moofilesystem.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -16,20 +19,24 @@
 #include "moofilesystem.h"
 #include "moofolder-private.h"
 #include "mooutils/mooutils-fs.h"
-#include "mooutils/moomarshals.h"
-#include <string.h>
+#include "mooutils/mooutils-mem.h"
+#include "mooutils/mooutils-macros.h"
+#include "marshals.h"
+#include <gio/gio.h>
 #include <errno.h>
 #include <stdio.h>
 #ifndef __WIN32__
 #include <sys/wait.h>
 #else
+#include <windows.h>
 #include <io.h>
+#include <shellapi.h>
 #endif
 
-#if 0 && MOO_DEBUG_ENABLED
+#if 0 && MOO_DEBUG
 #define DEBUG_MESSAGE g_message
 #else
-static void DEBUG_MESSAGE (G_GNUC_UNUSED const char *format, ...)
+static void G_GNUC_PRINTF(1,2) DEBUG_MESSAGE (G_GNUC_UNUSED const char *format, ...)
 {
 }
 #endif
@@ -70,7 +77,7 @@ static MooFolder   *get_parent_folder       (MooFileSystem  *fs,
                                              MooFileFlags    flags);
 static gboolean     delete_file             (MooFileSystem  *fs,
                                              const char     *path,
-                                             gboolean        recursive,
+                                             MooDeleteFileFlags flags,
                                              GError        **error);
 
 static gboolean     move_file_unix          (MooFileSystem  *fs,
@@ -331,14 +338,14 @@ _moo_file_system_create_folder (MooFileSystem  *fs,
 
 
 gboolean
-_moo_file_system_delete_file (MooFileSystem  *fs,
-                              const char     *path,
-                              gboolean        recursive,
-                              GError        **error)
+_moo_file_system_delete_file (MooFileSystem       *fs,
+                              const char          *path,
+                              MooDeleteFileFlags   flags,
+                              GError             **error)
 {
     g_return_val_if_fail (MOO_IS_FILE_SYSTEM (fs), FALSE);
     g_return_val_if_fail (path != NULL, FALSE);
-    return MOO_FILE_SYSTEM_GET_CLASS(fs)->delete_file (fs, path, recursive, error);
+    return MOO_FILE_SYSTEM_GET_CLASS(fs)->delete_file (fs, path, flags, error);
 }
 
 
@@ -464,7 +471,7 @@ get_folder (MooFileSystem  *fs,
 #endif /* __WIN32__ */
 
     /* XXX check the caller */
-    if (!g_path_is_absolute (path))
+    if (!_moo_path_is_absolute (path))
     {
         g_set_error (error, MOO_FILE_ERROR,
                      MOO_FILE_ERROR_BAD_FILENAME,
@@ -541,7 +548,7 @@ create_folder (G_GNUC_UNUSED MooFileSystem *fs,
     g_return_val_if_fail (path != NULL, FALSE);
 
     /* XXX check the caller */
-    if (!g_path_is_absolute (path))
+    if (!_moo_path_is_absolute (path))
     {
         g_set_error (error, MOO_FILE_ERROR,
                      MOO_FILE_ERROR_BAD_FILENAME,
@@ -657,19 +664,14 @@ normalize_path (const char *path)
     }
     else
     {
-        char *tmp = g_strjoinv (G_DIR_SEPARATOR_S, (char**) comps->pdata);
+        normpath = g_strjoinv (G_DIR_SEPARATOR_S, (char**) comps->pdata);
 
         if (first_slash)
         {
-            guint len = strlen (tmp);
-            normpath = g_new (char, len + 2);
-            memcpy (normpath + 1, tmp, len + 1);
+            gsize len = strlen (normpath);
+            normpath = g_renew (char, normpath, len + 2);
+            memmove (normpath + 1, normpath, len + 1);
             normpath[0] = G_DIR_SEPARATOR;
-            g_free (tmp);
-        }
-        else
-        {
-            normpath = tmp;
         }
     }
 
@@ -681,16 +683,71 @@ normalize_path (const char *path)
 }
 
 
+#if defined(__WIN32__) && 0
+
+#ifndef FOF_NORECURSION
+#define FOF_NORECURSION 0x1000
+#endif
+
+static gboolean
+delete_file_win32 (const char     *path,
+                   gboolean        recursive,
+                   GError        **error)
+{
+    gboolean success;
+    SHFILEOPSTRUCT op = {0};
+    gunichar2 *path_utf16;
+    long len;
+
+    if (!(path_utf16 = g_utf8_to_utf16 (path, -1, NULL, &len, error)))
+        return FALSE;
+
+    path_utf16 = g_renew (gunichar2, path_utf16, len + 2);
+    path_utf16[len + 1] = 0;
+
+    op.wFunc = FO_DELETE;
+    op.pFrom = path_utf16;
+    op.fFlags = FOF_ALLOWUNDO;
+    if (!recursive)
+        op.fFlags |= FOF_NORECURSION;
+
+    success = SHFileOperation (&op) == 0;
+
+    if (!success)
+        g_set_error (error, MOO_FILE_ERROR,
+                     MOO_FILE_ERROR_FAILED,
+                     "Could not delete '%s'",
+                     path);
+
+    g_free (path_utf16);
+    return success;
+}
+
+#endif
+
 static gboolean
 delete_file (G_GNUC_UNUSED MooFileSystem *fs,
-             const char     *path,
-             gboolean        recursive,
-             GError        **error)
+             const char          *path,
+             MooDeleteFileFlags   flags,
+             GError             **error)
 {
     gboolean isdir;
 
     g_return_val_if_fail (path != NULL, FALSE);
-    g_return_val_if_fail (g_path_is_absolute (path), FALSE);
+    g_return_val_if_fail (_moo_path_is_absolute (path), FALSE);
+    g_return_val_if_fail (!error || !*error, FALSE);
+
+    if (flags & MOO_DELETE_TO_TRASH)
+    {
+        GFile *file = g_file_new_for_path (path);
+        gboolean retval = g_file_trash (file, NULL, error);
+        g_object_unref (file);
+        return retval;
+    }
+
+#if defined(__WIN32__) && 0
+    return delete_file_win32 (path, (flags & MOO_DELETE_RECURSIVE) != 0, error);
+#endif
 
     if (g_file_test (path, G_FILE_TEST_IS_SYMLINK))
         isdir = FALSE;
@@ -698,7 +755,7 @@ delete_file (G_GNUC_UNUSED MooFileSystem *fs,
         isdir = g_file_test (path, G_FILE_TEST_IS_DIR);
 
     if (isdir)
-        return _moo_remove_dir (path, recursive, error);
+        return _moo_remove_dir (path, (flags & MOO_DELETE_RECURSIVE) != 0, error);
 
     if (_moo_remove (path))
     {
@@ -727,8 +784,8 @@ move_file_unix (G_GNUC_UNUSED MooFileSystem *fs,
                 GError        **error)
 {
     g_return_val_if_fail (old_path && new_path, FALSE);
-    g_return_val_if_fail (g_path_is_absolute (old_path), FALSE);
-    g_return_val_if_fail (g_path_is_absolute (new_path), FALSE);
+    g_return_val_if_fail (_moo_path_is_absolute (old_path), FALSE);
+    g_return_val_if_fail (_moo_path_is_absolute (new_path), FALSE);
 
     /* XXX */
     if (_moo_rename (old_path, new_path))
@@ -767,7 +824,7 @@ make_path_unix (G_GNUC_UNUSED MooFileSystem *fs,
     g_return_val_if_fail (display_name != NULL, NULL);
 
     /* XXX check the caller */
-    if (!g_path_is_absolute (base_path))
+    if (!_moo_path_is_absolute (base_path))
     {
         g_set_error (error, MOO_FILE_ERROR,
                      MOO_FILE_ERROR_BAD_FILENAME,
@@ -795,7 +852,7 @@ make_path_unix (G_GNUC_UNUSED MooFileSystem *fs,
 }
 
 
-/* TODO: error checking, etc. */
+/* XXX make sure error is set TODO: error checking, etc. */
 static char *
 normalize_path_unix (G_GNUC_UNUSED MooFileSystem *fs,
                      const char     *path,
@@ -803,30 +860,23 @@ normalize_path_unix (G_GNUC_UNUSED MooFileSystem *fs,
                      G_GNUC_UNUSED GError **error)
 {
     guint len;
-    char *normpath, *tmp;
+    char *normpath;
 
     g_return_val_if_fail (path != NULL, NULL);
 
-    tmp = normalize_path (path);
+    normpath = normalize_path (path);
 
     if (!is_folder)
-        return tmp;
+        return normpath;
 
-    len = strlen (tmp);
-    g_return_val_if_fail (len > 0, tmp);
+    len = strlen (normpath);
+    g_return_val_if_fail (len > 0, normpath);
 
-    if (tmp[len-1] != G_DIR_SEPARATOR)
+    if (normpath[len-1] != G_DIR_SEPARATOR)
     {
-        normpath = g_new (char, len + 2);
-        memcpy (normpath, tmp, len);
+        normpath = g_renew (char, normpath, len + 2);
         normpath[len] = G_DIR_SEPARATOR;
         normpath[len+1] = 0;
-        g_free (tmp);
-    }
-    else
-    {
-        g_assert (1 || len == 1);
-        normpath = tmp;
     }
 
 #if 0
@@ -854,7 +904,7 @@ parse_path_unix (MooFileSystem  *fs,
     g_return_val_if_fail (path_utf8 && path_utf8[0], FALSE);
 
     /* XXX check the caller */
-    if (!g_path_is_absolute (path_utf8))
+    if (!_moo_path_is_absolute (path_utf8))
     {
         g_set_error (error, MOO_FILE_ERROR,
                      MOO_FILE_ERROR_BAD_FILENAME,
@@ -927,7 +977,7 @@ get_absolute_path_unix (G_GNUC_UNUSED MooFileSystem *fs,
             return g_strdup (home);
     }
 
-    if (g_path_is_absolute (short_name))
+    if (_moo_path_is_absolute (short_name))
         return g_strdup (short_name);
 
     if (current_dir)
@@ -948,9 +998,7 @@ static MooFolder *
 get_root_folder_win32 (MooFileSystem  *fs,
                        MooFileFlags    wanted)
 {
-#ifdef __GNUC__
-#warning "Implement get_root_folder_win32()"
-#endif
+    MOO_IMPLEMENT_ME
     return _moo_file_system_get_folder (fs, "c:\\", wanted, NULL);
 }
 
@@ -962,9 +1010,7 @@ move_file_win32 (G_GNUC_UNUSED MooFileSystem  *fs,
                  G_GNUC_UNUSED const char *new_path,
                  GError        **error)
 {
-#ifdef __GNUC__
-#warning "Implement move_file_win32()"
-#endif
+    MOO_IMPLEMENT_ME
     g_set_error (error, MOO_FILE_ERROR,
                  MOO_FILE_ERROR_NOT_IMPLEMENTED,
                  "Renaming files is not implemented on win32");
@@ -999,6 +1045,7 @@ normalize_path_win32 (G_GNUC_UNUSED MooFileSystem *fs,
 {
     char *drive, *path, *normpath;
     guint slashes;
+    gboolean drive_slashes = FALSE;
 
     g_return_val_if_fail (fullpath != NULL, NULL);
 
@@ -1020,6 +1067,7 @@ normalize_path_win32 (G_GNUC_UNUSED MooFileSystem *fs,
         drive = g_strndup (path, slashes);
         path = g_strdup (tmp + slashes);
         g_free (tmp);
+        drive_slashes = TRUE;
     }
 //     else if (path[0] == '\\')
 //     {
@@ -1046,7 +1094,7 @@ normalize_path_win32 (G_GNUC_UNUSED MooFileSystem *fs,
     {
         char *tmp = normpath;
 
-        if (normpath[0] == '\\')
+        if (normpath[0] == '\\' || drive_slashes)
             normpath = g_strdup_printf ("%s%s", drive, normpath);
         else
             normpath = g_strdup_printf ("%s\\%s", drive, normpath);
@@ -1078,7 +1126,7 @@ make_path_win32 (G_GNUC_UNUSED MooFileSystem *fs,
                  const char     *display_name,
                  G_GNUC_UNUSED GError **error)
 {
-    g_return_val_if_fail (g_path_is_absolute (base_path), NULL);
+    g_return_val_if_fail (_moo_path_is_absolute (base_path), NULL);
     g_return_val_if_fail (display_name != NULL, NULL);
     return g_strdup_printf ("%s\\%s", base_path, display_name);
 }
@@ -1092,15 +1140,13 @@ parse_path_win32 (MooFileSystem  *fs,
                   char          **display_basename_p,
                   GError        **error)
 {
-#ifdef __GNUC__
-#warning "Implement parse_path_win32()"
-#endif
+    MOO_IMPLEMENT_ME
     const char *separator;
     char *norm_dirname = NULL, *dirname = NULL, *basename = NULL;
     gsize len;
 
     g_return_val_if_fail (path_utf8 && path_utf8[0], FALSE);
-    g_return_val_if_fail (g_path_is_absolute (path_utf8), FALSE);
+    g_return_val_if_fail (_moo_path_is_absolute (path_utf8), FALSE);
 
     separator = strrchr (path_utf8, '\\');
     g_return_val_if_fail (separator != NULL, FALSE);
@@ -1149,7 +1195,7 @@ get_absolute_path_win32 (G_GNUC_UNUSED MooFileSystem *fs,
 {
     g_return_val_if_fail (short_name && short_name[0], NULL);
 
-    if (g_path_is_absolute (short_name))
+    if (_moo_path_is_absolute (short_name))
         return g_strdup (short_name);
 
     if (current_dir)
