@@ -1,13 +1,16 @@
 /*
  *   moofilesystem.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -42,6 +45,11 @@ typedef struct _MooFileSystem         MooFileSystem;
 typedef struct _MooFileSystemPrivate  MooFileSystemPrivate;
 typedef struct _MooFileSystemClass    MooFileSystemClass;
 
+typedef enum {
+    MOO_DELETE_RECURSIVE = 1 << 0,
+    MOO_DELETE_TO_TRASH  = 1 << 1
+} MooDeleteFileFlags;
+
 struct _MooFileSystem
 {
     GObject         parent;
@@ -75,7 +83,7 @@ struct _MooFileSystemClass
                                      GError        **error);
     gboolean    (*delete_file)      (MooFileSystem  *fs,
                                      const char     *path,
-                                     gboolean        recursive,
+                                     MooDeleteFileFlags flags,
                                      GError        **error);
     gboolean    (*move_file)        (MooFileSystem  *fs,
                                      const char     *old_path,
@@ -112,7 +120,7 @@ gboolean     _moo_file_system_create_folder     (MooFileSystem  *fs,
                                                  GError        **error);
 gboolean     _moo_file_system_delete_file       (MooFileSystem  *fs,
                                                  const char     *path,
-                                                 gboolean        recursive,
+                                                 MooDeleteFileFlags flags,
                                                  GError        **error);
 gboolean     _moo_file_system_move_file         (MooFileSystem  *fs,
                                                  const char     *old_path,
