@@ -1,26 +1,29 @@
 /*
  *   moofileview-dialogs.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOO_FILE_VIEW_COMPILATION
 #include "moofileview/moofileview-dialogs.h"
 #include "moofileview/moofilesystem.h"
 #include "moofileview/moofile-private.h"
-#include "moofileview/moofileprops-glade.h"
-#include "moofileview/moocreatefolder-glade.h"
-#include "moofileview/moofileview-drop-glade.h"
 #include "mooutils/mooentry.h"
 #include "mooutils/mooutils-gobject.h"
 #include "mooutils/moodialogs.h"
 #include "mooutils/mooi18n.h"
+#include "glade/moofileprops-gxml.h"
+#include "glade/moocreatefolder-gxml.h"
+#include "glade/moofileview-drop-gxml.h"
 #include <time.h>
 #include <string.h>
 #include <gtk/gtk.h>
@@ -51,20 +54,12 @@ _moo_file_props_dialog_class_init (MooFilePropsDialogClass *klass)
 static void
 _moo_file_props_dialog_init (MooFilePropsDialog *dialog)
 {
-    dialog->xml = moo_glade_xml_new_empty (GETTEXT_PACKAGE);
-    moo_glade_xml_map_class (dialog->xml, "GtkEntry", MOO_TYPE_ENTRY);
+    dialog->xml = moo_file_props_xml_new ();
 
-    if (!moo_glade_xml_parse_memory (dialog->xml, moofileprops_glade_xml, -1, "notebook", NULL))
-    {
-        g_object_unref (dialog->xml);
-        dialog->xml = NULL;
-        g_return_if_reached ();
-    }
-
-    dialog->notebook = moo_glade_xml_get_widget (dialog->xml, "notebook");
-    dialog->icon = moo_glade_xml_get_widget (dialog->xml, "icon");
-    dialog->entry = moo_glade_xml_get_widget (dialog->xml, "entry");
-    dialog->table = moo_glade_xml_get_widget (dialog->xml, "table");
+    dialog->notebook = GTK_WIDGET (dialog->xml->MooFileProps);
+    dialog->icon = GTK_WIDGET (dialog->xml->icon);
+    dialog->entry = GTK_WIDGET (dialog->xml->entry);
+    dialog->table = GTK_WIDGET (dialog->xml->table);
 
     gtk_container_add (GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), dialog->notebook);
     gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
@@ -276,7 +271,6 @@ moo_file_props_dialog_destroy (GtkObject *object)
 
     if (dialog->xml)
     {
-        g_object_unref (dialog->xml);
         if (dialog->file)
             _moo_file_unref (dialog->file);
         if (dialog->folder)
@@ -320,41 +314,34 @@ char*
 _moo_file_view_create_folder_dialog (GtkWidget  *parent,
                                      MooFolder  *folder)
 {
-    MooGladeXML *xml;
-    GtkWidget *dialog, *entry, *label;
+    CreateFolderXml *xml;
+    GtkWidget *dialog;
     char *text, *path, *new_folder_name = NULL;
 
     g_return_val_if_fail (MOO_IS_FOLDER (folder), NULL);
 
-    xml = moo_glade_xml_new_empty (GETTEXT_PACKAGE);
-    moo_glade_xml_map_class (xml, "GtkEntry", MOO_TYPE_ENTRY);
-    moo_glade_xml_parse_memory (xml, moocreatefolder_glade_xml, -1, NULL, NULL);
-
-    dialog = moo_glade_xml_get_widget (xml, "dialog");
-    g_return_val_if_fail (dialog != NULL, NULL);
+    xml = create_folder_xml_new ();
+    dialog = GTK_WIDGET (xml->CreateFolder);
 
     moo_window_set_parent (dialog, parent);
 
-    label = moo_glade_xml_get_widget (xml, "label");
     path = g_filename_display_name (_moo_folder_get_path (folder));
     text = g_strdup_printf ("Create new folder in %s", path);
-    gtk_label_set_text (GTK_LABEL (label), text);
+    gtk_label_set_text (xml->label, text);
     g_free (path);
     g_free (text);
 
-    entry = moo_glade_xml_get_widget (xml, "entry");
-    gtk_entry_set_text (GTK_ENTRY (entry), "New Folder");
-    moo_entry_clear_undo (MOO_ENTRY (entry));
+    gtk_entry_set_text (GTK_ENTRY (xml->entry), "New Folder");
+    moo_entry_clear_undo (xml->entry);
     gtk_widget_show_all (dialog);
-    gtk_widget_grab_focus (entry);
-    gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
+    gtk_widget_grab_focus (GTK_WIDGET (xml->entry));
+    gtk_editable_select_region (GTK_EDITABLE (xml->entry), 0, -1);
 
     if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
-        new_folder_name = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
+        new_folder_name = g_strdup (gtk_entry_get_text (GTK_ENTRY (xml->entry)));
     else
         new_folder_name = NULL;
 
-    g_object_unref (xml);
     gtk_widget_destroy (dialog);
     return new_folder_name;
 }
@@ -420,26 +407,22 @@ find_available_clip_name (const char *dirname)
 
 
 char *
-_moo_file_view_save_drop_dialog (GtkWidget          *parent,
-                                 const char         *dirname)
+_moo_file_view_save_drop_dialog (GtkWidget  *parent,
+                                 const char *dirname)
 {
-    MooGladeXML *xml;
-    GtkWidget *dialog, *button;
+    DropXml *xml;
+    GtkWidget *dialog;
     GtkEntry *entry;
     char *start_name, *fullname = NULL;
 
     g_return_val_if_fail (dirname != NULL, NULL);
 
-    xml = moo_glade_xml_new_empty (GETTEXT_PACKAGE);
-    moo_glade_xml_map_class (xml, "GtkEntry", MOO_TYPE_ENTRY);
-    moo_glade_xml_parse_memory (xml, moofileview_drop_glade_xml, -1, NULL, NULL);
-
-    dialog = moo_glade_xml_get_widget (xml, "save_drop_dialog");
-    g_return_val_if_fail (dialog != NULL, NULL);
+    xml = drop_xml_new ();
+    dialog = GTK_WIDGET (xml->Drop);
 
     moo_position_window_at_pointer (dialog, parent);
 
-    entry = moo_glade_xml_get_widget (xml, "entry");
+    entry = GTK_ENTRY (xml->entry);
 
     start_name = find_available_clip_name (dirname);
     gtk_entry_set_text (entry, start_name);
@@ -448,8 +431,7 @@ _moo_file_view_save_drop_dialog (GtkWidget          *parent,
     gtk_widget_show_all (dialog);
     gtk_widget_grab_focus (GTK_WIDGET (entry));
 
-    button = moo_glade_xml_get_widget (xml, "ok_button");
-    moo_bind_bool_property (button, "sensitive", entry, "empty", TRUE);
+    moo_bind_bool_property (xml->ok_button, "sensitive", entry, "empty", TRUE);
 
     while (TRUE)
     {
@@ -497,7 +479,6 @@ _moo_file_view_save_drop_dialog (GtkWidget          *parent,
     }
 
 out:
-    g_object_unref (xml);
     gtk_widget_destroy (dialog);
     g_free (start_name);
     return fullname;
