@@ -1,13 +1,16 @@
 /*
  *   moofileview-dialogs.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_FILE_VIEW_COMPILATION
@@ -18,7 +21,6 @@
 #define MOO_FILE_VIEW_DIALOGS_H
 
 #include "moofileview/moofile.h"
-#include "mooutils/mooglade.h"
 #include <gtk/gtkdialog.h>
 
 G_BEGIN_DECLS
@@ -38,7 +40,7 @@ typedef struct _MooFilePropsDialogClass   MooFilePropsDialogClass;
 struct _MooFilePropsDialog
 {
     GtkDialog base;
-    MooGladeXML *xml;
+    struct MooFilePropsXml *xml;
     GtkWidget *notebook;
     GtkWidget *icon;
     GtkWidget *entry;
