@@ -1,13 +1,16 @@
 /*
  *   moofileview-tools.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -22,9 +25,8 @@
 #include "mooutils/mooaction.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mootype-macros.h"
-#ifdef MOO_USE_XDGMIME
-#include "mooutils/xdgmime/xdgmime.h"
-#endif
+#include "mooutils/mooi18n.h"
+#include "mooutils/moo-mime.h"
 #include <string.h>
 
 
@@ -128,7 +130,7 @@ tools_info_free (ToolsInfo *info)
 
 static void
 remove_old_tools (MooFileView    *fileview,
-                  MooUIXML       *xml,
+                  MooUiXml       *xml,
                   GtkActionGroup *group)
 {
     ToolsInfo *info;
@@ -218,7 +220,7 @@ _moo_file_view_tools_load (MooFileView *fileview)
     ToolsInfo *info;
     MooMarkupDoc *doc;
     MooMarkupNode *root, *child;
-    MooUIXML *xml;
+    MooUiXml *xml;
     MooActionCollection *actions;
     GtkActionGroup *group;
     MooUINode *ph;
@@ -234,17 +236,17 @@ _moo_file_view_tools_load (MooFileView *fileview)
     doc = moo_prefs_get_markup (MOO_PREFS_RC);
     g_return_if_fail (doc != NULL);
 
-    root = moo_markup_get_element (MOO_MARKUP_NODE (doc), "MooFileView/Tools");
-
-    if (!root)
-        return;
+    info = g_new0 (ToolsInfo, 1);
+    info->merge_id = moo_ui_xml_new_merge_id (xml);
+    g_object_set_data_full (G_OBJECT (fileview), "moo-file-view-tools-info", info,
+                            (GDestroyNotify) tools_info_free);
 
     ph = moo_ui_xml_find_placeholder (xml, "OpenWith");
     g_return_if_fail (ph != NULL);
 
-    info = g_new0 (ToolsInfo, 1);
+    root = moo_markup_get_element (MOO_MARKUP_NODE (doc), "MooFileView/Tools");
 
-    for (child = root->children; child != NULL; child = child->next)
+    for (child = root ? root->children : NULL; child != NULL; child = child->next)
     {
         GtkAction *action;
         const char *label, *extensions, *mimetypes;
@@ -284,16 +286,18 @@ _moo_file_view_tools_load (MooFileView *fileview)
         info->actions = g_slist_prepend (info->actions, action);
     }
 
-    if (!info->actions)
     {
-        g_free (info);
-        return;
+        GtkAction *action = tool_action_new (fileview, Q_("Open with|Default Application"), "*", NULL,
+#ifndef __WIN32__
+                                             "xdg-open %f"
+#else
+                                             "explorer %f"
+#endif
+                                            );
+        info->actions = g_slist_prepend (info->actions, action);
     }
 
     info->actions = g_slist_reverse (info->actions);
-    info->merge_id = moo_ui_xml_new_merge_id (xml);
-    g_object_set_data_full (G_OBJECT (fileview), "moo-file-view-tools-info", info,
-                            (GDestroyNotify) tools_info_free);
 
     for (l = info->actions; l != NULL; l = l->next)
     {
@@ -326,17 +330,15 @@ action_check_one (ToolAction *action,
         if (_moo_glob_match_simple (l->data, _moo_file_display_name (file)))
             return TRUE;
 
-#ifdef MOO_USE_XDGMIME
     mime = _moo_file_get_mime_type (file);
     g_return_val_if_fail (mime != NULL, FALSE);
 
-    if (mime == XDG_MIME_TYPE_UNKNOWN)
+    if (mime == MOO_MIME_TYPE_UNKNOWN)
         return FALSE;
 
     for (l = action->mimetypes; l != NULL; l = l->next)
-        if (xdg_mime_mime_type_subclass (mime, l->data))
+        if (moo_mime_type_is_subclass (mime, l->data))
             return TRUE;
-#endif
 
     return FALSE;
 }
