@@ -1,13 +1,16 @@
 /*
  *   moofileview.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_FILE_VIEW_H
@@ -25,8 +28,8 @@ G_BEGIN_DECLS
 #define MOO_IS_FILE_VIEW_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), MOO_TYPE_FILE_VIEW))
 #define MOO_FILE_VIEW_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), MOO_TYPE_FILE_VIEW, MooFileViewClass))
 
-typedef struct _MooFileView      MooFileView;
-typedef struct _MooFileViewClass MooFileViewClass;
+typedef struct MooFileView      MooFileView;
+typedef struct MooFileViewClass MooFileViewClass;
 
 GType                   moo_file_view_get_type      (void) G_GNUC_CONST;
 
@@ -34,7 +37,7 @@ gboolean                moo_file_view_chdir         (MooFileView    *fileview,
                                                      const char     *dir,
                                                      GError        **error);
 
-MooUIXML               *moo_file_view_get_ui_xml    (MooFileView    *fileview);
+MooUiXml               *moo_file_view_get_ui_xml    (MooFileView    *fileview);
 MooActionCollection    *moo_file_view_get_actions   (MooFileView    *fileview);
 
 
