@@ -1,13 +1,16 @@
 /*
  *   moofolder.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -19,7 +22,7 @@
 #include "moofileview/moofolder-private.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-misc.h"
-#include "mooutils/moomarshals.h"
+#include "marshals.h"
 #include <glib/gstdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -47,7 +50,7 @@ G_STMT_START {              \
 #if 0
 #define PRINT_TIMES g_print
 #else
-static void PRINT_TIMES (G_GNUC_UNUSED const char *format, ...)
+static void G_GNUC_PRINTF(1,2) PRINT_TIMES (G_GNUC_UNUSED const char *format, ...)
 {
 }
 #endif
@@ -240,7 +243,7 @@ add_file_size (G_GNUC_UNUSED const char *filename,
     *mem += STRING_SIZE (file->link_target);
     *mem += STRING_SIZE (file->display_name);
     *mem += STRING_SIZE (file->case_display_name);
-    *mem += STRING_SIZE (file->collation_key);
+    *mem += _moo_collation_key_size (file->collation_key);
 #undef STRING_SIZE
 }
 
@@ -1061,7 +1064,7 @@ moo_file_get_mtime_string (MooFile *file)
 
     g_return_val_if_fail (file->statbuf != NULL, NULL);
 
-    if (strftime (buf, 1024, "%x %X", localtime ((time_t*)&file->statbuf->st_mtime)))
+    if (strftime (buf, 1024, "%x %X", localtime (&file->statbuf->st_mtime)))
         return g_strdup (buf);
     else
         return NULL;
