@@ -1,13 +1,16 @@
 /*
  *   moofoldermodel-private.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_FILE_VIEW_COMPILATION
@@ -90,7 +93,7 @@ static void      _hash_table_remove     (FileList       *flist,
                                          MooFile        *file);
 
 
-#ifdef MOO_DEBUG_ENABLED
+#ifdef MOO_DEBUG
 #if 0
 #define DEFINE_CHECK_FILE_LIST_INTEGRITY
 static void CHECK_FILE_LIST_INTEGRITY (FileList *flist)
@@ -114,7 +117,7 @@ static void CHECK_FILE_LIST_INTEGRITY (FileList *flist)
     }
 }
 #endif
-#endif /* MOO_DEBUG_ENABLED */
+#endif /* MOO_DEBUG */
 
 #ifndef DEFINE_CHECK_FILE_LIST_INTEGRITY
 #define CHECK_FILE_LIST_INTEGRITY(flist)
@@ -336,20 +339,56 @@ static GList    *_list_find             (FileList       *flist,
 }
 
 
-static int      moo_file_case_cmp       (MooFile        *f1,
-                                         MooFile        *f2)
+static int
+moo_file_case_cmp (MooFile *f1,
+                   MooFile *f2)
+{
+    if (!strcmp (_moo_file_name (f1), ".."))
+        return strcmp (_moo_file_name (f2), "..") ? -1 : 0;
+    else if (!strcmp (_moo_file_name (f2), ".."))
+        return 1;
+    else if (MOO_FILE_IS_DIR (f1) && !MOO_FILE_IS_DIR (f2))
+        return -1;
+    else if (!MOO_FILE_IS_DIR (f1) && MOO_FILE_IS_DIR (f2))
+        return 1;
+    else
+        return _moo_collation_key_cmp (_moo_file_collation_key (f1),
+                                       _moo_file_collation_key (f2));
+}
+
+static int
+moo_file_cmp (MooFile *f1,
+              MooFile *f2)
+{
+    if (!strcmp (_moo_file_name (f1), ".."))
+        return strcmp (_moo_file_name (f2), "..") ? -1 : 0;
+    else if (!strcmp (_moo_file_name (f2), ".."))
+        return 1;
+    else if (MOO_FILE_IS_DIR (f1) && !MOO_FILE_IS_DIR (f2))
+        return -1;
+    else if (!MOO_FILE_IS_DIR (f1) && MOO_FILE_IS_DIR (f2))
+        return 1;
+    else
+        return strcmp (_moo_file_display_name (f1),
+                       _moo_file_display_name (f2));
+}
+
+static int
+moo_file_case_cmp_fi (MooFile *f1,
+                      MooFile *f2)
 {
     if (!strcmp (_moo_file_name (f1), ".."))
         return strcmp (_moo_file_name (f2), "..") ? -1 : 0;
     else if (!strcmp (_moo_file_name (f2), ".."))
         return 1;
     else
-        return strcmp (_moo_file_collation_key (f1),
-                       _moo_file_collation_key (f2));
+        return _moo_collation_key_cmp (_moo_file_collation_key (f1),
+                                       _moo_file_collation_key (f2));
 }
 
-static int      moo_file_cmp            (MooFile        *f1,
-                                         MooFile        *f2)
+static int
+moo_file_cmp_fi (MooFile *f1,
+                 MooFile *f2)
 {
     if (!strcmp (_moo_file_name (f1), ".."))
         return strcmp (_moo_file_name (f2), "..") ? -1 : 0;
