@@ -1,13 +1,16 @@
 /*
  *   moofoldermodel.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_FILE_VIEW_COMPILATION
@@ -18,16 +21,10 @@
 #define MOO_FOLDER_MODEL_H
 
 #include <moofileview/moofile.h>
-#include <gtk/gtktreemodelfilter.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
-#ifndef __WIN32__
-#define MOO_FOLDER_MODEL_SORT_CASE_SENSITIVE_DEFAULT      FALSE
-#else /* __WIN32__ */
-#define MOO_FOLDER_MODEL_SORT_CASE_SENSITIVE_DEFAULT      FALSE
-#endif /* __WIN32__ */
-
 #define MOO_TYPE_FOLDER_MODEL            (_moo_folder_model_get_type ())
 #define MOO_FOLDER_MODEL(object)         (G_TYPE_CHECK_INSTANCE_CAST ((object), MOO_TYPE_FOLDER_MODEL, MooFolderModel))
 #define MOO_FOLDER_MODEL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), MOO_TYPE_FOLDER_MODEL, MooFolderModelClass))
@@ -58,10 +55,23 @@ typedef enum {
 #define MOO_FOLDER_MODEL_N_COLUMNS 1
 
 
+typedef enum {
+    MOO_FOLDER_MODEL_SORT_CASE_SENSITIVE = 1 << 0,
+    MOO_FOLDER_MODEL_SORT_FOLDERS_FIRST  = 1 << 1,
+#if defined(GDK_WINDOWING_QUARTZ) && 0
+    MOO_FOLDER_MODEL_SORT_FLAGS_DEFAULT  = 0
+#else
+    MOO_FOLDER_MODEL_SORT_FLAGS_DEFAULT  = MOO_FOLDER_MODEL_SORT_FOLDERS_FIRST
+#endif
+} MooFolderModelSortFlags;
+
+
 GType            _moo_folder_model_get_type      (void) G_GNUC_CONST;
 GtkTreeModel    *_moo_folder_model_new           (MooFolder      *folder);
 void             _moo_folder_model_set_folder    (MooFolderModel *model,
-                                                 MooFolder      *folder);
+                                                  MooFolder      *folder);
+void             _moo_folder_model_set_sort_flags(MooFolderModel *model,
+                                                  MooFolderModelSortFlags flags);
 
 gboolean         _moo_folder_model_get_iter_by_name
                                                 (MooFolderModel *model,
