@@ -1,13 +1,16 @@
 /*
  *   mooiconview.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_FILE_VIEW_COMPILATION
@@ -33,17 +36,17 @@ G_BEGIN_DECLS
 #define MOO_ICON_VIEW_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), MOO_TYPE_ICON_VIEW, MooIconViewClass))
 
 
-typedef struct _MooIconView         MooIconView;
-typedef struct _MooIconViewPrivate  MooIconViewPrivate;
-typedef struct _MooIconViewClass    MooIconViewClass;
+typedef struct MooIconView        MooIconView;
+typedef struct MooIconViewPrivate MooIconViewPrivate;
+typedef struct MooIconViewClass   MooIconViewClass;
 
-struct _MooIconView
+struct MooIconView
 {
     GtkVBox             vbox;
     MooIconViewPrivate *priv;
 };
 
-struct _MooIconViewClass
+struct MooIconViewClass
 {
     GtkVBoxClass        vbox_class;
 
