@@ -1,24 +1,24 @@
 /*
  *   mooaccelbutton.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/mooaccelbutton.h"
-#include "mooutils/mooaccelbutton-glade.h"
-#include "mooutils/mooglade.h"
-#include "mooutils/moomarshals.h"
+#include "marshals.h"
 #include "mooutils/moodialogs.h"
 #include "mooutils/mooi18n.h"
-#include <gtk/gtkaccelgroup.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtklabel.h>
+#include "glade/mooaccelbutton-gxml.h"
+#include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
 
@@ -208,7 +208,7 @@ _moo_accel_button_set_accel (MooAccelButton *button,
     {
         gtk_accelerator_parse (accel, &accel_key, &accel_mods);
 
-        if (!accel_key && !accel_mods)
+        if (!accel_key || accel_key == GDK_VoidSymbol)
             return FALSE;
     }
 
@@ -309,17 +309,14 @@ static void
 moo_accel_button_clicked (GtkButton *gtkbutton)
 {
     MooAccelButton *button = MOO_ACCEL_BUTTON (gtkbutton);
-    MooGladeXML *xml;
-    GtkWidget *dialog, *ok_button, *cancel_button, *eventbox, *label;
+    AccelDialogXml *xml;
+    GtkWidget *dialog;
     GtkWidget *parent;
     Stuff s = {0, 0, NULL, NULL, 0};
     int response;
 
-    xml = moo_glade_xml_new_from_buf (mooaccelbutton_glade_xml, -1,
-                                      "dialog", GETTEXT_PACKAGE, NULL);
-    g_return_if_fail (xml != NULL);
-
-    dialog = moo_glade_xml_get_widget (xml, "dialog");
+    xml = accel_dialog_xml_new ();
+    dialog = GTK_WIDGET (xml->AccelDialog);
 
     parent = gtk_widget_get_toplevel (GTK_WIDGET (gtkbutton));
     gtk_window_set_transient_for (GTK_WINDOW (dialog),
@@ -334,24 +331,18 @@ moo_accel_button_clicked (GtkButton *gtkbutton)
     if (button->title)
         gtk_window_set_title (GTK_WINDOW (dialog), button->title);
 
-    ok_button = moo_glade_xml_get_widget (xml, "ok");
-    cancel_button = moo_glade_xml_get_widget (xml, "cancel");
-    eventbox = moo_glade_xml_get_widget (xml, "eventbox");
-    label = moo_glade_xml_get_widget (xml, "label");
+    gtk_button_set_use_underline (GTK_BUTTON (xml->ok), FALSE);
+    gtk_button_set_use_underline (GTK_BUTTON (xml->cancel), FALSE);
 
-    gtk_button_set_use_underline (GTK_BUTTON (ok_button), FALSE);
-    gtk_button_set_use_underline (GTK_BUTTON (cancel_button), FALSE);
-
-    s.label = GTK_LABEL (label);
+    s.label = xml->label;
     s.dialog = GTK_DIALOG (dialog);
 
-    g_signal_connect (eventbox, "key-press-event", G_CALLBACK (key_event), &s);
+    g_signal_connect (xml->eventbox, "key-press-event", G_CALLBACK (key_event), &s);
 
     response = gtk_dialog_run (GTK_DIALOG (dialog));
     remove_commit_timeout (&s);
 
     gtk_widget_destroy (dialog);
-    g_object_unref (xml);
 
     if (response == GTK_RESPONSE_OK)
     {
