@@ -1,19 +1,23 @@
 /*
  *   mooactionbase.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/mooactionbase-private.h"
 #include "mooutils/mooaction-private.h"
 #include "mooutils/mooactiongroup.h"
-#include "mooutils/moomarshals.h"
+#include "mooutils/mooaccel.h"
+#include "marshals.h"
 #include <gtk/gtk.h>
 #include <string.h>
 
@@ -27,76 +31,55 @@ enum {
 };
 
 
-static void
-base_init (void)
-{
-    static gboolean done = FALSE;
-
-    if (!done)
-    {
-        done = TRUE;
-
-        g_signal_new ("connect-proxy",
-                      MOO_TYPE_ACTION_BASE,
-                      G_SIGNAL_RUN_LAST,
-                      G_STRUCT_OFFSET (MooActionBaseClass, connect_proxy),
-                      NULL, NULL,
-                      _moo_marshal_VOID__OBJECT,
-                      G_TYPE_NONE, 1, GTK_TYPE_WIDGET);
-
-        g_signal_new ("disconnect-proxy",
-                      MOO_TYPE_ACTION_BASE,
-                      G_SIGNAL_RUN_LAST,
-                      G_STRUCT_OFFSET (MooActionBaseClass, disconnect_proxy),
-                      NULL, NULL,
-                      _moo_marshal_VOID__OBJECT,
-                      G_TYPE_NONE, 1, GTK_TYPE_WIDGET);
-    }
-}
-
 static void
 class_init (gpointer g_iface)
 {
     g_object_interface_install_property (g_iface,
-                                         g_param_spec_string ("display-name", "display-name", "display-name",
-                                                              NULL,
-                                                              G_PARAM_READWRITE));
+        g_param_spec_string ("display-name", "display-name", "display-name",
+                             NULL, G_PARAM_READWRITE));
     g_object_interface_install_property (g_iface,
-                                         g_param_spec_string ("accel", "accel", "accel",
-                                                              NULL,
-                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+        g_param_spec_string ("default-accel", "default-accel", "default-accel",
+                             NULL, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
     g_object_interface_install_property (g_iface,
-                                         g_param_spec_boolean ("connect-accel", "connect-accel", "connect-accel",
-                                                               FALSE,
-                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+        g_param_spec_boolean ("connect-accel", "connect-accel", "connect-accel",
+                              FALSE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
     g_object_interface_install_property (g_iface,
-                                         g_param_spec_boolean ("no-accel", "no-accel", "no-accel",
-                                                               FALSE,
-                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+        g_param_spec_boolean ("no-accel", "no-accel", "no-accel",
+                              FALSE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
     g_object_interface_install_property (g_iface,
-                                         g_param_spec_boolean ("force-accel-label", "force-accel-label", "force-accel-label",
-                                                               FALSE,
-                                                               G_PARAM_READWRITE));
+        g_param_spec_boolean ("force-accel-label", "force-accel-label", "force-accel-label",
+                              FALSE, G_PARAM_READWRITE));
     g_object_interface_install_property (g_iface,
-                                         g_param_spec_boolean ("accel-editable", "accel-editable", "accel-editable",
-                                                               TRUE,
-                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+        g_param_spec_boolean ("accel-editable", "accel-editable", "accel-editable",
+                              TRUE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
     g_object_interface_install_property (g_iface,
-                                         g_param_spec_boolean ("dead", "dead", "dead",
-                                                               FALSE,
-                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+        g_param_spec_boolean ("dead", "dead", "dead",
+                              FALSE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
     g_object_interface_install_property (g_iface,
-                                         g_param_spec_boolean ("active", "active", "active",
-                                                               TRUE,
-                                                               G_PARAM_WRITABLE));
+        g_param_spec_boolean ("active", "active", "active",
+                              TRUE, G_PARAM_WRITABLE));
     g_object_interface_install_property (g_iface,
-                                         g_param_spec_boolean ("has-submenu", "has-submenu", "has-submenu",
-                                                               FALSE,
-                                                               G_PARAM_READWRITE));
+        g_param_spec_boolean ("has-submenu", "has-submenu", "has-submenu",
+                              FALSE, G_PARAM_READWRITE));
     g_object_interface_install_property (g_iface,
-                                         g_param_spec_boolean ("use-underline", "use-underline", "use-underline",
-                                                               TRUE,
-                                                               G_PARAM_READWRITE));
+        g_param_spec_boolean ("use-underline", "use-underline", "use-underline",
+                              TRUE, G_PARAM_READWRITE));
+
+    g_signal_new ("connect-proxy",
+                  MOO_TYPE_ACTION_BASE,
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (MooActionBaseClass, connect_proxy),
+                  NULL, NULL,
+                  _moo_marshal_VOID__OBJECT,
+                  G_TYPE_NONE, 1, GTK_TYPE_WIDGET);
+
+    g_signal_new ("disconnect-proxy",
+                  MOO_TYPE_ACTION_BASE,
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (MooActionBaseClass, disconnect_proxy),
+                  NULL, NULL,
+                  _moo_marshal_VOID__OBJECT,
+                  G_TYPE_NONE, 1, GTK_TYPE_WIDGET);
 }
 
 
@@ -104,45 +87,35 @@ void
 _moo_action_base_init_class (GObjectClass *klass)
 {
     g_object_class_install_property (klass, MOO_ACTION_BASE_PROP_DISPLAY_NAME,
-                                     g_param_spec_string ("display-name", "display-name", "display-name",
-                                                          NULL,
-                                                          G_PARAM_READWRITE));
-    g_object_class_install_property (klass, MOO_ACTION_BASE_PROP_ACCEL,
-                                     g_param_spec_string ("accel", "accel", "accel",
-                                                          NULL,
-                                                          G_PARAM_READWRITE));
+        g_param_spec_string ("display-name", "display-name", "display-name",
+                             NULL, G_PARAM_READWRITE));
+    g_object_class_install_property (klass, MOO_ACTION_BASE_PROP_DEFAULT_ACCEL,
+        g_param_spec_string ("default-accel", "default-accel", "default-accel",
+                             NULL, G_PARAM_READWRITE));
     g_object_class_install_property (klass, MOO_ACTION_BASE_PROP_CONNECT_ACCEL,
-                                     g_param_spec_boolean ("connect-accel", "connect-accel", "connect-accel",
-                                                           FALSE,
-                                                           G_PARAM_READWRITE));
+        g_param_spec_boolean ("connect-accel", "connect-accel", "connect-accel",
+                              FALSE, G_PARAM_READWRITE));
     g_object_class_install_property (klass, MOO_ACTION_BASE_PROP_NO_ACCEL,
-                                     g_param_spec_boolean ("no-accel", "no-accel", "no-accel",
-                                                           FALSE,
-                                                           G_PARAM_READWRITE));
+        g_param_spec_boolean ("no-accel", "no-accel", "no-accel",
+                              FALSE, G_PARAM_READWRITE));
     g_object_class_install_property (klass, MOO_ACTION_BASE_PROP_ACCEL_EDITABLE,
-                                     g_param_spec_boolean ("accel-editable", "accel-editable", "accel-editable",
-                                                           TRUE,
-                                                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+        g_param_spec_boolean ("accel-editable", "accel-editable", "accel-editable",
+                              TRUE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
     g_object_class_install_property (klass, MOO_ACTION_BASE_PROP_FORCE_ACCEL_LABEL,
-                                     g_param_spec_boolean ("force-accel-label", "force-accel-label", "force-accel-label",
-                                                           FALSE,
-                                                           G_PARAM_READWRITE));
+        g_param_spec_boolean ("force-accel-label", "force-accel-label", "force-accel-label",
+                              FALSE, G_PARAM_READWRITE));
     g_object_class_install_property (klass, MOO_ACTION_BASE_PROP_DEAD,
-                                     g_param_spec_boolean ("dead", "dead", "dead",
-                                                           FALSE,
-                                                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+        g_param_spec_boolean ("dead", "dead", "dead",
+                              FALSE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
     g_object_class_install_property (klass, MOO_ACTION_BASE_PROP_ACTIVE,
-                                     g_param_spec_boolean ("active", "active", "active",
-                                                           TRUE,
-                                                           G_PARAM_WRITABLE));
+        g_param_spec_boolean ("active", "active", "active",
+                              TRUE, G_PARAM_WRITABLE));
     g_object_class_install_property (klass, MOO_ACTION_BASE_PROP_HAS_SUBMENU,
-                                     g_param_spec_boolean ("has-submenu", "has-submenu", "has-submenu",
-                                                           FALSE,
-                                                           G_PARAM_READWRITE));
+        g_param_spec_boolean ("has-submenu", "has-submenu", "has-submenu",
+                              FALSE, G_PARAM_READWRITE));
     g_object_class_install_property (klass, MOO_ACTION_BASE_PROP_USE_UNDERLINE,
-                                     g_param_spec_boolean ("use-underline", "use-underline", "use-underline",
-                                                           TRUE,
-                                                           G_PARAM_READWRITE));
+        g_param_spec_boolean ("use-underline", "use-underline", "use-underline",
+                              TRUE, G_PARAM_READWRITE));
 
     g_object_class_override_property (klass,
                                       MOO_ACTION_BASE_PROP_LABEL,
@@ -162,7 +135,7 @@ moo_action_base_get_type (void)
     {
         static const GTypeInfo info = {
             sizeof (MooActionBaseClass), /* class_size */
-            (GBaseInitFunc) base_init,
+            NULL, /* base_init */
             NULL, /* base_finalize */
             (GClassInitFunc) class_init,
             NULL, /* class_finalize */
@@ -271,26 +244,26 @@ _moo_action_get_display_name (gpointer action)
 
 
 static void
-moo_action_base_set_accel (MooActionBase *ab,
-                           const char    *accel)
+moo_action_base_set_default_accel (MooActionBase *ab,
+                                   const char    *accel)
 {
     g_return_if_fail (MOO_IS_ACTION_BASE (ab));
 
     if (accel && !accel[0])
         accel = NULL;
 
-    set_string (ab, "moo-action-accel", accel);
-    g_object_notify (G_OBJECT (ab), "accel");
+    set_string (ab, "moo-action-default-accel", accel);
+    g_object_notify (G_OBJECT (ab), "default-accel");
 }
 
 static const char *
-moo_action_base_get_accel (MooActionBase *ab)
+moo_action_base_get_default_accel (MooActionBase *ab)
 {
     const char *accel;
 
     g_return_val_if_fail (MOO_IS_ACTION_BASE (ab), "");
 
-    accel = get_string (ab, "moo-action-accel");
+    accel = get_string (ab, "moo-action-default-accel");
 
     if (!accel)
         accel = "";
@@ -492,8 +465,8 @@ _moo_action_base_set_property (GObject      *object,
         case MOO_ACTION_BASE_PROP_DISPLAY_NAME:
             moo_action_base_set_display_name (ab, g_value_get_string (value));
             break;
-        case MOO_ACTION_BASE_PROP_ACCEL:
-            moo_action_base_set_accel (ab, g_value_get_string (value));
+        case MOO_ACTION_BASE_PROP_DEFAULT_ACCEL:
+            moo_action_base_set_default_accel (ab, g_value_get_string (value));
             break;
         case MOO_ACTION_BASE_PROP_CONNECT_ACCEL:
             moo_action_base_set_connect_accel (ab, g_value_get_boolean (value));
@@ -545,8 +518,8 @@ _moo_action_base_get_property (GObject    *object,
         case MOO_ACTION_BASE_PROP_DISPLAY_NAME:
             g_value_set_string (value, _moo_action_get_display_name (ab));
             break;
-        case MOO_ACTION_BASE_PROP_ACCEL:
-            g_value_set_string (value, moo_action_base_get_accel (ab));
+        case MOO_ACTION_BASE_PROP_DEFAULT_ACCEL:
+            g_value_set_string (value, moo_action_base_get_default_accel (ab));
             break;
         case MOO_ACTION_BASE_PROP_CONNECT_ACCEL:
             g_value_set_boolean (value, _moo_action_get_connect_accel (ab));
@@ -648,7 +621,7 @@ const char *
 _moo_action_get_default_accel (gpointer action)
 {
     g_return_val_if_fail (MOO_IS_ACTION_BASE (action), "");
-    return moo_action_base_get_accel (action);
+    return moo_action_base_get_default_accel (action);
 }
 
 
