@@ -1,13 +1,16 @@
 /*
  *   mooactionfactory.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/mooactionfactory.h"
@@ -242,7 +245,7 @@ param_array_concatenate (GParameter *props1,
 }
 
 
-gpointer
+GtkAction *
 moo_action_factory_create_action (MooActionFactory   *factory,
                                   gpointer            data,
                                   const char         *prop_name,
@@ -290,7 +293,7 @@ moo_action_factory_create_action (MooActionFactory   *factory,
     _moo_param_array_free (props, n_props);
     _moo_param_array_free (add_props, n_add_props);
 
-    return object;
+    return GTK_ACTION (object);
 }
 
 
