@@ -1,13 +1,16 @@
 /*
  *   mooactionfactory.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_ACTION_FACTORY_H
@@ -66,10 +69,10 @@ MooActionFactory   *moo_action_factory_new_a        (GType               object_
                                                      GParameter         *params,
                                                      guint               n_params);
 
-gpointer            moo_action_factory_create_action(MooActionFactory   *factory,
+GtkAction          *moo_action_factory_create_action(MooActionFactory   *factory,
                                                      gpointer            data,
                                                      const char         *additional_prop_name,
-                                                     ...);
+                                                     ...) G_GNUC_NULL_TERMINATED;
 
 
 GtkAction          *moo_action_group_add_action     (GtkActionGroup     *group,
