@@ -1,17 +1,24 @@
 /*
  *   mooclosure.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/mooclosure.h"
-#include "mooutils/moomarshals.h"
+#include "mooutils/mootype-macros.h"
+#include "marshals.h"
+
+
+MOO_DEFINE_BOXED_TYPE_R (MooClosure, moo_closure)
 
 
 MooClosure*
@@ -114,20 +121,6 @@ moo_closure_invalidate (MooClosure *closure)
 }
 
 
-GType
-moo_closure_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooClosure",
-                                             (GBoxedCopyFunc) moo_closure_ref,
-                                             (GBoxedFreeFunc) moo_closure_unref);
-
-    return type;
-}
-
-
 /******************************************************************/
 /* MooClosureSignal
  */
@@ -187,7 +180,7 @@ moo_closure_signal_new (gpointer    object,
                         GCallback   proxy_func)
 {
     guint signal_id = 0;
-    GSignalQuery query;
+    GSignalQuery query = {0}; /* make MSVC happy */
     MooClosureSignal *cl;
 
     g_return_val_if_fail (G_IS_OBJECT (object), NULL);
