@@ -1,13 +1,16 @@
 /*
  *   moocompat.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -682,7 +685,7 @@ g_listenv (void)
 #endif /* !GLIB_CHECK_VERSION(2,8,0) */
 
 
-#if !GLIB_CHECK_VERSION(2,10,0)
+#if !GTK_CHECK_VERSION(2,10,0)
 
 GdkAtom
 gdk_atom_intern_static_string (const char *atom_name)
@@ -690,11 +693,6 @@ gdk_atom_intern_static_string (const char *atom_name)
     return gdk_atom_intern (atom_name, FALSE);
 }
 
-#endif /* !GLIB_CHECK_VERSION(2,10,0) */
-
-
-#if !GTK_CHECK_VERSION(2,10,0)
-
 GType
 gtk_unit_get_type (void)
 {
@@ -717,3 +715,102 @@ gtk_unit_get_type (void)
 }
 
 #endif /* !GTK_CHECK_VERSION(2,10,0) */
+
+
+#if !GTK_CHECK_VERSION(2,12,0)
+
+static void
+set_cursor_colors (GtkWidget      *widget,
+		   const GdkColor *primary,
+		   const GdkColor *secondary)
+{
+    char *rc_string;
+    char *widget_name;
+
+    widget_name = g_strdup_printf ("gtk-source-view-%p", (gpointer) widget);
+
+    rc_string = g_strdup_printf (
+    	"style \"%p\"\n"
+    	"{\n"
+    	"   GtkWidget::cursor-color = \"#%02x%02x%02x\"\n"
+    	"   GtkWidget::secondary-cursor-color = \"#%02x%02x%02x\"\n"
+    	"}\n"
+    	"widget \"*.%s\" style \"%p\"\n",
+    	(gpointer) widget,
+    	primary->red >> 8, primary->green >> 8, primary->blue >> 8,
+    	secondary->red >> 8, secondary->green >> 8, secondary->blue >> 8,
+    	widget_name,
+    	(gpointer) widget
+    );
+
+    gtk_rc_parse_string (rc_string);
+
+    if (strcmp (widget_name, gtk_widget_get_name (widget)) != 0)
+    	gtk_widget_set_name (widget, widget_name);
+
+    g_free (rc_string);
+    g_free (widget_name);
+}
+
+void
+gtk_widget_modify_cursor (GtkWidget *widget,
+                          GdkColor  *primary,
+                          GdkColor  *secondary)
+{
+    g_return_if_fail (GTK_IS_WIDGET (widget));
+    g_return_if_fail (!primary || secondary);
+
+    if (!primary)
+    {
+        primary = &widget->style->text[GTK_STATE_NORMAL],
+        secondary = &widget->style->text_aa[GTK_STATE_NORMAL];
+    }
+
+    set_cursor_colors (widget, primary, secondary);
+}
+
+#endif /* !GTK_CHECK_VERSION(2,12,0) */
+
+
+#if !GLIB_CHECK_VERSION(2,16,0)
+
+#include <libintl.h>
+
+const char *
+g_dpgettext (const char *domain,
+             const char *msgctxtid,
+             gsize       msgidoffset)
+{
+    const char *translation;
+
+    translation = dgettext (domain, msgctxtid);
+
+    if (translation == msgctxtid)
+    {
+        const char *sep;
+
+        if (msgidoffset > 0)
+            return msgctxtid + msgidoffset;
+
+        sep = strchr (msgctxtid, '|');
+
+        if (sep)
+        {
+            /* try with '\004' instead of '|', in case
+            * xgettext -kQ_:1g was used
+            */
+            gchar *tmp = g_alloca (strlen (msgctxtid) + 1);
+            strcpy (tmp, msgctxtid);
+            tmp[sep - msgctxtid] = '\004';
+
+            translation = dgettext (domain, tmp);
+
+            if (translation == tmp)
+                return sep + 1;
+        }
+    }
+
+    return translation;
+}
+
+#endif /* !GLIB_CHECK_VERSION(2,16,0) */
