@@ -1,13 +1,16 @@
 /*
  *   moocompat.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOOUTILS_COMPAT_H
@@ -24,7 +27,7 @@ G_BEGIN_DECLS
 
 #if !GLIB_CHECK_VERSION(2,8,0)
 #define g_mkdir_with_parents            _moo_g_mkdir_with_parents
-#define g_g_listenv                     _moo_g_listenv
+#define g_listenv                       _moo_g_listenv
 #define g_utf8_collate_key_for_filename _moo_g_utf8_collate_key_for_filename
 #define g_file_set_contents             _moo_g_file_set_contents
 
@@ -52,9 +55,6 @@ G_END_DECLS
 G_BEGIN_DECLS
 
 #define g_intern_string _moo_intern_string
-#define gdk_atom_intern_static_string _moo_gdk_atom_intern_static_string
-
-GdkAtom     gdk_atom_intern_static_string   (const char *atom_name);
 
 #endif /* !GLIB_CHECK_VERSION(2,10,0) */
 
@@ -75,6 +75,9 @@ typedef enum {
 #define GTK_TYPE_UNIT (gtk_unit_get_type ())
 GType gtk_unit_get_type (void) G_GNUC_CONST;
 
+#define gdk_atom_intern_static_string _moo_gdk_atom_intern_static_string
+GdkAtom gdk_atom_intern_static_string (const char *atom_name);
+
 #endif /* !GTK_CHECK_VERSION(2,10,0) */
 
 
@@ -84,9 +87,24 @@ GType gtk_unit_get_type (void) G_GNUC_CONST;
 #define GTK_STOCK_DISCARD  "gtk-discard"
 #endif
 
+#define gtk_widget_modify_cursor _moo_gtk_widget_modify_cursor
+void gtk_widget_modify_cursor (GtkWidget *widget,
+                               GdkColor  *primary,
+                               GdkColor  *secondary);
+
 #endif /* !GTK_CHECK_VERSION(2,10,0) */
 
 
+#if !GLIB_CHECK_VERSION(2,16,0)
+
+#define g_dpgettext _moo_compat_g_dpgettext
+const char *g_dpgettext (const char *domain,
+                         const char *msgctxtid,
+                         gsize       msgidoffset);
+
+#endif /* !GLIB_CHECK_VERSION(2,16,0) */
+
+
 G_END_DECLS
 
 #endif /* MOOUTILS_COMPAT_H */
