@@ -1,13 +1,16 @@
 /*
  *   moodialogs.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <gtk/gtk.h>
@@ -285,6 +288,7 @@ moo_overwrite_file_dialog (GtkWidget  *parent,
     GtkWidget *dialog, *button, *toplevel = NULL;
 
     g_return_val_if_fail (display_name != NULL, FALSE);
+    g_return_val_if_fail (display_dirname != NULL, FALSE);
 
     if (parent)
         parent = gtk_widget_get_toplevel (parent);
@@ -517,6 +521,8 @@ set_initial_size (GtkWindow    *window,
 void
 _moo_window_set_remember_size (GtkWindow  *window,
                                const char *prefs_key,
+                               int         default_width,
+                               int         default_height,
                                gboolean    remember_position)
 {
     PositionInfo *pinfo;
@@ -537,8 +543,8 @@ _moo_window_set_remember_size (GtkWindow  *window,
     pinfo->remember_position = remember_position;
 
     moo_prefs_create_key (pinfo->key_maximized, MOO_PREFS_STATE, G_TYPE_BOOLEAN, FALSE);
-    moo_prefs_create_key (pinfo->key_width, MOO_PREFS_STATE, G_TYPE_INT, -1);
-    moo_prefs_create_key (pinfo->key_height, MOO_PREFS_STATE, G_TYPE_INT, -1);
+    moo_prefs_create_key (pinfo->key_width, MOO_PREFS_STATE, G_TYPE_INT, default_width);
+    moo_prefs_create_key (pinfo->key_height, MOO_PREFS_STATE, G_TYPE_INT, default_height);
 
     if (remember_position)
     {
