@@ -1,13 +1,16 @@
 /*
  *   moodialogs.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOOUTILS_DIALOGS_H
@@ -55,6 +58,8 @@ void        moo_warning_dialog              (GtkWidget  *parent,
 
 void        _moo_window_set_remember_size   (GtkWindow  *window,
                                              const char *prefs_key,
+                                             int         default_width,
+                                             int         default_height,
                                              gboolean    remember_position);
 
 
