@@ -1,19 +1,24 @@
 /*
  *   mooentry.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "mooutils/moomarshals.h"
+#include "marshals.h"
+#include "mooutils/mooaccel.h"
 #include "mooutils/mooentry.h"
 #include "mooutils/mooundo.h"
 #include "mooutils/mooutils-gobject.h"
+#include "mooutils/mooeditops.h"
 #include <gtk/gtkbindings.h>
 #include <gtk/gtkimagemenuitem.h>
 #include <gtk/gtkseparatormenuitem.h>
@@ -39,6 +44,7 @@ static guint DELETE_ACTION_TYPE;
 
 static void     moo_entry_class_init        (MooEntryClass      *klass);
 static void     moo_entry_editable_init     (GtkEditableClass   *klass);
+static void     moo_entry_undo_ops_init     (MooUndoOpsIface    *iface);
 
 static void     moo_entry_init              (MooEntry           *entry);
 static void     moo_entry_finalize          (GObject            *object);
@@ -111,15 +117,17 @@ moo_entry_get_type (void)
             NULL
         };
 
-        static const GInterfaceInfo editable_info =
-        {
-            (GInterfaceInitFunc) moo_entry_editable_init,
-            NULL,
-            NULL
+        static const GInterfaceInfo editable_info = {
+            (GInterfaceInitFunc) moo_entry_editable_init, NULL, NULL
+        };
+
+        static const GInterfaceInfo undo_ops_info = {
+            (GInterfaceInitFunc) moo_entry_undo_ops_init, NULL, NULL
         };
 
         type = g_type_register_static (GTK_TYPE_ENTRY, "MooEntry", &info, 0);
         g_type_add_interface_static (type, GTK_TYPE_EDITABLE, &editable_info);
+        g_type_add_interface_static (type, MOO_TYPE_UNDO_OPS, &undo_ops_info);
     }
 
     return type;
@@ -250,10 +258,10 @@ moo_entry_class_init (MooEntryClass *klass)
 
     binding_set = gtk_binding_set_by_class (klass);
     gtk_binding_entry_add_signal (binding_set, GDK_z,
-                                  GDK_CONTROL_MASK,
+                                  MOO_ACCEL_CTRL_MASK,
                                   "undo", 0);
     gtk_binding_entry_add_signal (binding_set, GDK_z,
-                                  GDK_CONTROL_MASK | GDK_SHIFT_MASK,
+                                  MOO_ACCEL_CTRL_MASK | GDK_SHIFT_MASK,
                                   "redo", 0);
     gtk_binding_entry_add_signal (binding_set, GDK_u,
                                   GDK_CONTROL_MASK,
@@ -387,22 +395,28 @@ moo_entry_changed (GtkEditable *editable)
 
 
 void
-moo_entry_undo (MooEntry       *entry)
+moo_entry_undo (MooEntry *entry)
 {
     g_return_if_fail (MOO_IS_ENTRY (entry));
+
     if (entry->priv->enable_undo && moo_undo_stack_can_undo (entry->priv->undo_stack))
         moo_undo_stack_undo (entry->priv->undo_stack);
-}
 
+    moo_undo_ops_can_undo_changed (G_OBJECT (entry));
+    moo_undo_ops_can_redo_changed (G_OBJECT (entry));
+}
 
 void
-moo_entry_redo (MooEntry       *entry)
+moo_entry_redo (MooEntry *entry)
 {
     g_return_if_fail (MOO_IS_ENTRY (entry));
+
     if (entry->priv->enable_undo && moo_undo_stack_can_redo (entry->priv->undo_stack))
         moo_undo_stack_redo (entry->priv->undo_stack);
-}
 
+    moo_undo_ops_can_undo_changed (G_OBJECT (entry));
+    moo_undo_ops_can_redo_changed (G_OBJECT (entry));
+}
 
 void
 moo_entry_begin_undo_group (MooEntry *entry)
@@ -411,7 +425,6 @@ moo_entry_begin_undo_group (MooEntry *entry)
     moo_undo_stack_start_group (entry->priv->undo_stack);
 }
 
-
 void
 moo_entry_end_undo_group (MooEntry *entry)
 {
@@ -419,7 +432,6 @@ moo_entry_end_undo_group (MooEntry *entry)
     moo_undo_stack_end_group (entry->priv->undo_stack);
 }
 
-
 void
 moo_entry_clear_undo (MooEntry *entry)
 {
@@ -427,6 +439,31 @@ moo_entry_clear_undo (MooEntry *entry)
     moo_undo_stack_clear (entry->priv->undo_stack);
 }
 
+static gboolean
+undo_ops_can_undo (MooUndoOps *obj)
+{
+    MooEntry *entry = MOO_ENTRY (obj);
+    return entry->priv->enable_undo &&
+            moo_undo_stack_can_undo (entry->priv->undo_stack);
+}
+
+static gboolean
+undo_ops_can_redo (MooUndoOps *obj)
+{
+    MooEntry *entry = MOO_ENTRY (obj);
+    return entry->priv->enable_undo &&
+            moo_undo_stack_can_redo (entry->priv->undo_stack);
+}
+
+static void
+moo_entry_undo_ops_init (MooUndoOpsIface *iface)
+{
+    iface->undo = (void(*)(MooUndoOps*)) moo_entry_undo;
+    iface->redo = (void(*)(MooUndoOps*)) moo_entry_redo;
+    iface->can_undo = undo_ops_can_undo;
+    iface->can_redo = undo_ops_can_redo;
+}
+
 
 GtkWidget*
 moo_entry_new (void)
@@ -598,6 +635,7 @@ moo_entry_do_insert_text (GtkEditable        *editable,
                                  INSERT_ACTION_TYPE,
                                  insert_action_new (editable, text, length, position));
         parent_editable_iface->do_insert_text (editable, text, length, position);
+        moo_undo_ops_can_undo_changed (G_OBJECT (editable));
     }
 }
 
@@ -625,9 +663,10 @@ moo_entry_do_delete_text (GtkEditable        *editable,
     if (start_pos < end_pos)
     {
         moo_undo_stack_add_action (MOO_ENTRY(editable)->priv->undo_stack,
-                                 DELETE_ACTION_TYPE,
-                                 delete_action_new (editable, start_pos, end_pos));
+                                   DELETE_ACTION_TYPE,
+                                   delete_action_new (editable, start_pos, end_pos));
         parent_editable_iface->do_delete_text (editable, start_pos, end_pos);
+        moo_undo_ops_can_undo_changed (G_OBJECT (editable));
     }
 }
 
