@@ -1,13 +1,16 @@
 /*
  *   moofilewatch.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -23,7 +26,6 @@
 #endif
 
 #ifdef __WIN32__
-#include "mooutils/mooutils-thread.h"
 #include <windows.h>
 #include <io.h>
 #include <fcntl.h>
@@ -37,16 +39,19 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <errno.h>
-#include <string.h>
 /* sys/stat.h macros */
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mooutils-mem.h"
 #include "mooutils/moofilewatch.h"
-#include "mooutils/moomarshals.h"
+#include "mooutils/moostat.h"
+#include "mooutils/mootype-macros.h"
+#include "marshals.h"
+#include "mooutils/mooutils-thread.h"
 
 
 #if 1
-static void DEBUG_PRINT (G_GNUC_UNUSED const char *format, ...)
+static void  G_GNUC_PRINTF(1,2) DEBUG_PRINT (G_GNUC_UNUSED const char *format, ...)
 {
 }
 #elif defined(__WIN32__)
@@ -99,9 +104,6 @@ struct _MooFileWatch {
     guint alive : 1;
 };
 
-
-#define MOO_FILE_WATCH_ERROR (moo_file_watch_error_quark ())
-
 typedef enum
 {
     MOO_FILE_WATCH_ERROR_CLOSED,
@@ -115,8 +117,10 @@ typedef enum
     MOO_FILE_WATCH_ERROR_ACCESS_DENIED
 } MooFileWatchError;
 
-static GQuark moo_file_watch_error_quark (void);
+MOO_DEFINE_BOXED_TYPE_R (MooFileWatch, moo_file_watch)
 
+#define MOO_FILE_WATCH_ERROR (moo_file_watch_error_quark ())
+MOO_DEFINE_QUARK_STATIC (moo-file-watch-error, moo_file_watch_error_quark)
 
 #ifdef MOO_USE_FAM
 static gboolean watch_fam_start             (MooFileWatch   *watch,
@@ -262,30 +266,6 @@ moo_file_watch_new (GError **error)
 }
 
 
-GType
-moo_file_watch_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooFileWatch",
-                                             (GBoxedCopyFunc) moo_file_watch_ref,
-                                             (GBoxedFreeFunc) moo_file_watch_unref);
-
-    return type;
-}
-
-
-static GQuark
-moo_file_watch_error_quark (void)
-{
-    static GQuark quark = 0;
-    if (G_UNLIKELY (!quark))
-        quark = g_quark_from_static_string ("moo-file-watch-error");
-    return quark;
-}
-
-
 static MooFileEvent *
 moo_file_event_new (const char      *filename,
                     guint            monitor_id,
@@ -329,39 +309,7 @@ moo_file_event_free (MooFileEvent *event)
     }
 }
 
-GType
-moo_file_event_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooFileEvent",
-                                             (GBoxedCopyFunc) moo_file_event_copy,
-                                             (GBoxedFreeFunc) moo_file_event_free);
-
-    return type;
-}
-
-
-GType
-moo_file_event_code_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-    {
-        static const GEnumValue values[] = {
-            { MOO_FILE_EVENT_CHANGED, (char*) "MOO_FILE_EVENT_CHANGED", (char*) "changed"},
-            { MOO_FILE_EVENT_DELETED, (char*) "MOO_FILE_EVENT_DELETED", (char*) "deleted"},
-            { MOO_FILE_EVENT_ERROR, (char*) "MOO_FILE_EVENT_ERROR", (char*) "error"},
-            { 0, NULL, NULL }
-        };
-
-        type = g_enum_register_static ("MooFileEventCode", values);
-    }
-
-    return type;
-}
+MOO_DEFINE_BOXED_TYPE_C (MooFileEvent, moo_file_event)
 
 
 gboolean
@@ -861,7 +809,7 @@ watch_stat_start_monitor (MooFileWatch   *watch,
 
     errno = 0;
 
-    if (g_stat (monitor->filename, &buf) != 0)
+    if (moo_stat (monitor->filename, &buf) != 0)
     {
         int saved_errno = errno;
         g_set_error (error, MOO_FILE_WATCH_ERROR,
@@ -885,7 +833,7 @@ watch_stat_start_monitor (MooFileWatch   *watch,
 #endif
 
     monitor->id = get_new_monitor_id ();
-    memcpy (&monitor->statbuf, &buf, sizeof (buf));
+    monitor->statbuf = buf;
 
     return TRUE;
 }
@@ -933,7 +881,7 @@ do_stat (MooFileWatch *watch)
         event.filename = monitor->filename;
         event.error = NULL;
 
-        if (g_stat (monitor->filename, &monitor->statbuf) != 0)
+        if (moo_stat (monitor->filename, &monitor->statbuf) != 0)
         {
             if (errno == ENOENT)
             {
@@ -1094,7 +1042,7 @@ fam_thread_add_path (FAMThread  *thr,
 {
     gunichar2 *win_path;
 
-    if (thr->n_events == MAXIMUM_WAIT_OBJECTS)
+    if (thr->n_events >= MAXIMUM_WAIT_OBJECTS)
     {
         g_critical ("%s: too many folders watched", G_STRLOC);
         return;
@@ -1150,10 +1098,12 @@ fam_thread_remove_path (FAMThread  *thr,
 
             if (i < thr->n_events - 1)
             {
-                memmove (&thr->watches[i], &thr->watches[i+1],
-                         (thr->n_events - i - 1) * sizeof(thr->watches[i]));
-                memmove (&thr->events[i], &thr->events[i+1],
-                         (thr->n_events - i - 1) * sizeof(thr->events[i]));
+                MOO_ELMMOVE (thr->watches + i,
+                             thr->watches + i + 1,
+                             thr->n_events - i - 1);
+                MOO_ELMMOVE (thr->events + i,
+                             thr->events + i + 1,
+                             thr->n_events - i - 1);
             }
 
             thr->n_events -= 1;
@@ -1174,7 +1124,7 @@ fam_thread_check_dir (FAMThread *thr,
 
     errno = 0;
 
-    if (g_stat (thr->watches[idx].path, &buf) != 0 &&
+    if (moo_stat (thr->watches[idx].path, &buf) != 0 &&
         errno == ENOENT)
     {
         fam_thread_event (MOO_FILE_EVENT_DELETED,
@@ -1426,7 +1376,7 @@ fam_win32_init (void)
         fam->running = FALSE;
 
         fam->thread_data.n_events = 1;
-        fam->thread_data.events[0] = CreateEvent (NULL, TRUE, FALSE, NULL);;
+        fam->thread_data.events[0] = CreateEvent (NULL, TRUE, FALSE, NULL);
 
         if (!fam->thread_data.events[0])
         {
@@ -1501,7 +1451,7 @@ watch_win32_start_monitor (MooFileWatch   *watch,
 
     errno = 0;
 
-    if (g_stat (monitor->filename, &buf) != 0)
+    if (moo_stat (monitor->filename, &buf) != 0)
     {
         int saved_errno = errno;
         g_set_error (error, MOO_FILE_WATCH_ERROR,
