@@ -1,16 +1,19 @@
 /*
  *   moofilewatch.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2009 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* Files and directory monitor. Uses FAM if present, or stat() otherwise.
+/* Files and directory monitor. Uses stat().
    On win32 does FindFirstChangeNotification and ReadDirectoryChangesW. */
 
 #ifndef MOO_FILE_WATCH_H
@@ -23,7 +26,6 @@ G_BEGIN_DECLS
 
 #define MOO_TYPE_FILE_WATCH         (moo_file_watch_get_type ())
 #define MOO_TYPE_FILE_EVENT         (moo_file_event_get_type ())
-#define MOO_TYPE_FILE_EVENT_CODE    (moo_file_event_code_get_type ())
 
 typedef enum {
     MOO_FILE_EVENT_CHANGED,
@@ -49,7 +51,6 @@ typedef void (*MooFileWatchCallback) (MooFileWatch *watch,
 
 GType           moo_file_watch_get_type             (void) G_GNUC_CONST;
 GType           moo_file_event_get_type             (void) G_GNUC_CONST;
-GType           moo_file_event_code_get_type        (void) G_GNUC_CONST;
 
 /* FAMOpen */
 MooFileWatch   *moo_file_watch_new                  (GError        **error);
