@@ -1,21 +1,22 @@
 /*
  *   moofiltermgr.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_FILTER_MGR_H
 #define MOO_FILTER_MGR_H
 
-#include <gtk/gtkfilefilter.h>
-#include <gtk/gtkfilechooser.h>
-#include <mooutils/moocombo.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
@@ -48,9 +49,10 @@ struct _MooFilterMgrClass
 GType            moo_filter_mgr_get_type            (void) G_GNUC_CONST;
 
 MooFilterMgr    *moo_filter_mgr_new                 (void);
+MooFilterMgr    *moo_filter_mgr_default             (void);
 
 void             moo_filter_mgr_init_filter_combo   (MooFilterMgr   *mgr,
-                                                     MooCombo       *combo,
+                                                     GtkComboBox    *combo,
                                                      const char     *user_id);
 void             moo_filter_mgr_attach              (MooFilterMgr   *mgr,
                                                      GtkFileChooser *filechooser,
