@@ -1,13 +1,16 @@
 /*
  *   mooglade.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef GTK_DISABLE_DEPRECATED
@@ -25,21 +28,25 @@
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooprefsdialog.h"
 #include "mooutils/mooprefs.h"
+#include "mooutils/moofontsel.h"
+#include "mooutils/moohistorycombo.h"
+#include "mooutils/mooentry.h"
+#include "mooutils/mooaccelbutton.h"
 #include "mooutils/mooi18n.h"
+#include "mooutils/mootype-macros.h"
 #include <gtk/gtk.h>
 #include <string.h>
 #include <errno.h>
 #include <stdlib.h>
 
-G_DEFINE_TYPE (MooGladeXML, moo_glade_xml, G_TYPE_OBJECT)
-
 typedef enum {
     MOO_GLADE_XML_ERROR_FAILED
 } MooGladeXMLError;
 
 #define MOO_GLADE_XML_ERROR (moo_glade_xml_error_quark ())
-static GQuark moo_glade_xml_error_quark (void) G_GNUC_CONST;
+MOO_DEFINE_QUARK_STATIC (moo-glade-xml-error, moo_glade_xml_error_quark)
 
+G_DEFINE_TYPE (MooGladeXML, moo_glade_xml, G_TYPE_OBJECT)
 
 #define FOREACH_ELM_START(parent,elm)                           \
 G_STMT_START {                                                  \
@@ -408,7 +415,7 @@ set_moo_sensitive (MooGladeXML *xml,
     btn = moo_glade_xml_get_widget (xml, value);
     g_return_if_fail (GTK_IS_TOGGLE_BUTTON (btn));
 
-    moo_bind_sensitive (btn, &widget, 1, invert);
+    moo_bind_sensitive (btn, widget, invert);
 }
 
 
@@ -464,7 +471,7 @@ connect_special_signal (MooGladeXML    *xml,
                            G_STRLOC, signal->object);
         }
 
-        moo_bind_sensitive (btn, &node->widget, 1, invert);
+        moo_bind_sensitive (btn, node->widget, invert);
 
         return TRUE;
     }
@@ -1088,6 +1095,21 @@ packing_props_free (PackingProps *props)
 }
 
 
+static GType
+moo_glade_xml_get_type_by_name (MooGladeXML *xml, const char *class_name)
+{
+    GType type;
+
+    g_return_val_if_fail (class_name != NULL, 0);
+
+    type = GPOINTER_TO_SIZE (g_hash_table_lookup (xml->priv->class_to_type, class_name));
+
+    if (!type)
+        type = get_type_by_name (class_name);
+
+    return type;
+}
+
 static Widget*
 widget_new (MooGladeXML    *xml,
             Child          *parent,
@@ -1096,9 +1118,10 @@ widget_new (MooGladeXML    *xml,
 {
     Widget *widget;
     WidgetProps *props;
-    const char *id, *class_name;
+    const char *id;
+    const char *class_name = NULL;
+    char *freeme = NULL;
     GType type;
-    gboolean ignore_errors = FALSE;
 
     g_return_val_if_fail (NODE_IS_WIDGET (node), NULL);
 
@@ -1114,15 +1137,31 @@ widget_new (MooGladeXML    *xml,
 
         if (!type)
         {
-            class_name = moo_markup_get_prop (node, "class");
-            g_return_val_if_fail (id != NULL && class_name != NULL, NULL);
+            const char *colon = strchr (id, ':');
 
-            type = GPOINTER_TO_SIZE (g_hash_table_lookup (xml->priv->class_to_type, class_name));
-
-            if (!type)
+            if (colon != NULL)
             {
-                ignore_errors = FALSE;
-                type = get_type_by_name (class_name);
+                const char *old_id;
+
+                old_id = id;
+                freeme = g_strndup (id, colon - id);
+                id = freeme;
+                class_name = colon + 1;
+
+                type = moo_glade_xml_get_type_by_name (xml, class_name);
+
+                if (!type)
+                {
+                    g_free (freeme);
+                    freeme = NULL;
+                    id = old_id;
+                }
+            }
+            else
+            {
+                class_name = moo_markup_get_prop (node, "class");
+                g_return_val_if_fail (id != NULL && class_name != NULL, NULL);
+                type = moo_glade_xml_get_type_by_name (xml, class_name);
             }
         }
 
@@ -1138,17 +1177,20 @@ widget_new (MooGladeXML    *xml,
 
     props = widget_props_new (node, type,
                               g_hash_table_lookup (xml->priv->props, id),
-                              ignore_errors, xml->priv->translation_domain,
+                              FALSE, xml->priv->translation_domain,
                               error);
     g_return_val_if_fail (props != NULL, NULL);
 
     widget = g_new0 (Widget, 1);
     widget->parent_node = parent;
-    widget->id = g_strdup (id);
     widget->type = type;
     widget->props = props;
     widget->children = NULL;
 
+    widget->id = g_strdup (id);
+    id = NULL;
+    g_free (freeme);
+
     collect_signals (node, widget);
 
     FOREACH_ELM_START (node, elm)
@@ -1169,6 +1211,7 @@ widget_new (MooGladeXML    *xml,
     FOREACH_ELM_END;
 
     widget->children = g_slist_reverse (widget->children);
+
     return widget;
 }
 
@@ -1795,72 +1838,22 @@ parse_property (GParamSpec     *param_spec,
     }
     else if (G_TYPE_IS_ENUM (param_spec->value_type))
     {
-        GEnumClass *enum_class;
-        GEnumValue *val;
-
-        enum_class = g_type_class_ref (param_spec->value_type);
-
-        val = g_enum_get_value_by_name (enum_class, value);
-
-        if (!val)
-            val = g_enum_get_value_by_nick (enum_class, value);
-
-        g_type_class_unref (enum_class);
-
-        if (!val)
+        if (!_moo_value_convert_from_string (value, &param->value))
         {
             const char *typename = g_type_name (param_spec->value_type);
             g_warning ("%s: can not convert string '%s' to a value of type %s",
                        G_STRLOC, value, typename ? typename : "<unknown>");
             return FALSE;
         }
-
-        g_value_set_enum (&param->value, val->value);
     }
     else if (G_TYPE_IS_FLAGS (param_spec->value_type))
     {
-        if (!value[0])
+        if (!_moo_value_convert_from_string (value, &param->value))
         {
-            g_value_set_flags (&param->value, 0);
-        }
-        else
-        {
-            GFlagsClass *flags_class;
-            char **pieces;
-            guint i, len;
-            int result = 0;
-
-            pieces = g_strsplit (value, "|", 0);
-            len = g_strv_length (pieces);
-
-            flags_class = g_type_class_ref (param_spec->value_type);
-
-            for (i = 0; i < len; ++i)
-            {
-                GFlagsValue *val;
-
-                val = g_flags_get_value_by_name (flags_class, pieces[i]);
-
-                if (!val)
-                    val = g_flags_get_value_by_nick (flags_class, pieces[i]);
-
-                if (!val)
-                {
-                    const char *typename = g_type_name (param_spec->value_type);
-                    g_warning ("%s: can not convert string '%s' to a value of type %s",
-                               G_STRLOC, pieces[i], typename ? typename : "<unknown>");
-                    g_type_class_unref (flags_class);
-                    return FALSE;
-                }
-                else
-                {
-                    result |= val->value;
-                }
-            }
-
-            g_value_set_flags (&param->value, result);
-            g_type_class_unref (flags_class);
-            g_strfreev (pieces);
+            const char *typename = g_type_name (param_spec->value_type);
+            g_warning ("%s: can not convert string '%s' to a value of type %s",
+                       G_STRLOC, value, typename ? typename : "<unknown>");
+            return FALSE;
         }
     }
     else if (param_spec->value_type == GDK_TYPE_PIXBUF)
@@ -2284,9 +2277,10 @@ moo_glade_xml_parse_markup (MooGladeXML  *xml,
                 break;
             }
         }
-        else if (!strcmp (elm->name, "requires"))
+        else if (strcmp (elm->name, "requires") == 0 ||
+                 strcmp (elm->name, "requires-version") == 0)
         {
-            g_message ("%s: ignoring '%s'", G_STRLOC, elm->name);
+//             g_message ("%s: ignoring '%s'", G_STRLOC, elm->name);
         }
         else
         {
@@ -2476,9 +2470,28 @@ gpointer
 moo_glade_xml_get_widget (MooGladeXML    *xml,
                           const char     *id)
 {
+    gpointer widget;
+    const char *colon;
+
     g_return_val_if_fail (xml != NULL, NULL);
     g_return_val_if_fail (id != NULL, NULL);
-    return g_hash_table_lookup (xml->priv->widgets, id);
+
+    widget = g_hash_table_lookup (xml->priv->widgets, id);
+
+    if (!widget && (colon = strchr (id, ':')))
+    {
+        char *real_id = g_strndup (id, colon - id);
+        widget = g_hash_table_lookup (xml->priv->widgets, real_id);
+        g_free (real_id);
+    }
+
+    return widget;
+}
+
+
+void
+moo_glade_xml_register_type (G_GNUC_UNUSED GType type)
+{
 }
 
 
@@ -2591,269 +2604,190 @@ func_data_pair_free (FuncDataPair *pair)
 }
 
 
-static GQuark
-moo_glade_xml_error_quark (void)
-{
-    return g_quark_from_static_string ("moo-glade-xml-error");
-}
-
-
-#define REGISTER_TYPE(class_name,type)  \
-    if (!strcmp (name, class_name))     \
-        return type
-
-
 static GType
 get_type_by_name (const char *name)
 {
+    static GHashTable *types;
     GType type = g_type_from_name (name);
+    GType (*type_func) (void);
 
     if (type)
         return type;
 
-    REGISTER_TYPE ("GtkAboutDialog", GTK_TYPE_ABOUT_DIALOG);
-    REGISTER_TYPE ("GtkCellRendererCombo", GTK_TYPE_CELL_RENDERER_COMBO);
-    REGISTER_TYPE ("GtkCellRendererProgress", GTK_TYPE_CELL_RENDERER_PROGRESS);
-    REGISTER_TYPE ("GtkCellView", GTK_TYPE_CELL_VIEW);
-    REGISTER_TYPE ("GtkIconView", GTK_TYPE_ICON_VIEW);
-    REGISTER_TYPE ("GtkMenuToolButton", GTK_TYPE_MENU_TOOL_BUTTON);
-    REGISTER_TYPE ("GtkAccelMap", GTK_TYPE_ACCEL_MAP);
-    REGISTER_TYPE ("GtkAction", GTK_TYPE_ACTION);
-    REGISTER_TYPE ("GtkActionGroup", GTK_TYPE_ACTION_GROUP);
-    REGISTER_TYPE ("GtkCellLayout", GTK_TYPE_CELL_LAYOUT);
-    REGISTER_TYPE ("GtkComboBox", GTK_TYPE_COMBO_BOX);
-    REGISTER_TYPE ("GtkComboBoxEntry", GTK_TYPE_COMBO_BOX_ENTRY);
-    REGISTER_TYPE ("GtkEntryCompletion", GTK_TYPE_ENTRY_COMPLETION);
-    REGISTER_TYPE ("GtkExpander", GTK_TYPE_EXPANDER);
-    REGISTER_TYPE ("GtkFileChooser", GTK_TYPE_FILE_CHOOSER);
-    REGISTER_TYPE ("GtkFileChooserAction", GTK_TYPE_FILE_CHOOSER_ACTION);
-    REGISTER_TYPE ("GtkFileChooserDialog", GTK_TYPE_FILE_CHOOSER_DIALOG);
-    REGISTER_TYPE ("GtkFileChooserError", GTK_TYPE_FILE_CHOOSER_ERROR);
-    REGISTER_TYPE ("GtkFileChooserWidget", GTK_TYPE_FILE_CHOOSER_WIDGET);
-    REGISTER_TYPE ("GtkFileFilter", GTK_TYPE_FILE_FILTER);
-    REGISTER_TYPE ("GtkFileFilterFlags", GTK_TYPE_FILE_FILTER_FLAGS);
-    REGISTER_TYPE ("GtkIconInfo", GTK_TYPE_ICON_INFO);
-    REGISTER_TYPE ("GtkIconLookupFlags", GTK_TYPE_ICON_LOOKUP_FLAGS);
-    REGISTER_TYPE ("GtkIconTheme", GTK_TYPE_ICON_THEME);
-    REGISTER_TYPE ("GtkIconThemeError", GTK_TYPE_ICON_THEME_ERROR);
-    REGISTER_TYPE ("GtkRadioAction", GTK_TYPE_RADIO_ACTION);
-    REGISTER_TYPE ("GtkRadioToolButton", GTK_TYPE_RADIO_TOOL_BUTTON);
-    REGISTER_TYPE ("GtkScrollStep", GTK_TYPE_SCROLL_STEP);
-    REGISTER_TYPE ("GtkSeparatorToolItem", GTK_TYPE_SEPARATOR_TOOL_ITEM);
-    REGISTER_TYPE ("GtkToggleAction", GTK_TYPE_TOGGLE_ACTION);
-    REGISTER_TYPE ("GtkToggleToolButton", GTK_TYPE_TOGGLE_TOOL_BUTTON);
-    REGISTER_TYPE ("GtkToolButton", GTK_TYPE_TOOL_BUTTON);
-    REGISTER_TYPE ("GtkToolItem", GTK_TYPE_TOOL_ITEM);
-    REGISTER_TYPE ("GtkTreeModelFilter", GTK_TYPE_TREE_MODEL_FILTER);
-    REGISTER_TYPE ("GtkUiManager", GTK_TYPE_UI_MANAGER);
-    REGISTER_TYPE ("GtkUiManagerItemType", GTK_TYPE_UI_MANAGER_ITEM_TYPE);
-    REGISTER_TYPE ("GtkAccelFlags", GTK_TYPE_ACCEL_FLAGS);
-    REGISTER_TYPE ("GtkAccelGroup", GTK_TYPE_ACCEL_GROUP);
-    REGISTER_TYPE ("GtkAccelLabel", GTK_TYPE_ACCEL_LABEL);
-    REGISTER_TYPE ("GtkAccessible", GTK_TYPE_ACCESSIBLE);
-    REGISTER_TYPE ("GtkAdjustment", GTK_TYPE_ADJUSTMENT);
-    REGISTER_TYPE ("GtkAlignment", GTK_TYPE_ALIGNMENT);
-    REGISTER_TYPE ("GtkAnchorType", GTK_TYPE_ANCHOR_TYPE);
-    REGISTER_TYPE ("GtkArgFlags", GTK_TYPE_ARG_FLAGS);
-    REGISTER_TYPE ("GtkArrow", GTK_TYPE_ARROW);
-    REGISTER_TYPE ("GtkArrowType", GTK_TYPE_ARROW_TYPE);
-    REGISTER_TYPE ("GtkAspectFrame", GTK_TYPE_ASPECT_FRAME);
-    REGISTER_TYPE ("GtkAttachOptions", GTK_TYPE_ATTACH_OPTIONS);
-    REGISTER_TYPE ("GtkBin", GTK_TYPE_BIN);
-    REGISTER_TYPE ("GtkBorder", GTK_TYPE_BORDER);
-    REGISTER_TYPE ("GtkBox", GTK_TYPE_BOX);
-    REGISTER_TYPE ("GtkButton", GTK_TYPE_BUTTON);
-    REGISTER_TYPE ("GtkButtonAction", GTK_TYPE_BUTTON_ACTION);
-    REGISTER_TYPE ("GtkButtonBox", GTK_TYPE_BUTTON_BOX);
-    REGISTER_TYPE ("GtkButtonBoxStyle", GTK_TYPE_BUTTON_BOX_STYLE);
-    REGISTER_TYPE ("GtkButtonsType", GTK_TYPE_BUTTONS_TYPE);
-    REGISTER_TYPE ("GtkCalendar", GTK_TYPE_CALENDAR);
-    REGISTER_TYPE ("GtkCalendarDisplayOptions", GTK_TYPE_CALENDAR_DISPLAY_OPTIONS);
-    REGISTER_TYPE ("GtkCellEditable", GTK_TYPE_CELL_EDITABLE);
-    REGISTER_TYPE ("GtkCellRenderer", GTK_TYPE_CELL_RENDERER);
-    REGISTER_TYPE ("GtkCellRendererMode", GTK_TYPE_CELL_RENDERER_MODE);
-    REGISTER_TYPE ("GtkCellRendererPixbuf", GTK_TYPE_CELL_RENDERER_PIXBUF);
-    REGISTER_TYPE ("GtkCellRendererState", GTK_TYPE_CELL_RENDERER_STATE);
-    REGISTER_TYPE ("GtkCellRendererText", GTK_TYPE_CELL_RENDERER_TEXT);
-    REGISTER_TYPE ("GtkCellRendererToggle", GTK_TYPE_CELL_RENDERER_TOGGLE);
-    REGISTER_TYPE ("GtkCellType", GTK_TYPE_CELL_TYPE);
-    REGISTER_TYPE ("GtkCheckButton", GTK_TYPE_CHECK_BUTTON);
-    REGISTER_TYPE ("GtkCheckMenuItem", GTK_TYPE_CHECK_MENU_ITEM);
-    REGISTER_TYPE ("GtkClipboard", GTK_TYPE_CLIPBOARD);
-    REGISTER_TYPE ("GtkColorButton", GTK_TYPE_COLOR_BUTTON);
-    REGISTER_TYPE ("GtkColorSelection", GTK_TYPE_COLOR_SELECTION);
-    REGISTER_TYPE ("GtkColorSelectionDialog", GTK_TYPE_COLOR_SELECTION_DIALOG);
-    REGISTER_TYPE ("GtkCombo", GTK_TYPE_COMBO);
-    REGISTER_TYPE ("GtkContainer", GTK_TYPE_CONTAINER);
-    REGISTER_TYPE ("GtkCornerType", GTK_TYPE_CORNER_TYPE);
-    REGISTER_TYPE ("GtkCurve", GTK_TYPE_CURVE);
-    REGISTER_TYPE ("GtkCurveType", GTK_TYPE_CURVE_TYPE);
-    REGISTER_TYPE ("GtkDeleteType", GTK_TYPE_DELETE_TYPE);
-    REGISTER_TYPE ("GtkDestDefaults", GTK_TYPE_DEST_DEFAULTS);
-    REGISTER_TYPE ("GtkDialog", GTK_TYPE_DIALOG);
-    REGISTER_TYPE ("GtkDialogFlags", GTK_TYPE_DIALOG_FLAGS);
-    REGISTER_TYPE ("GtkDirectionType", GTK_TYPE_DIRECTION_TYPE);
-    REGISTER_TYPE ("GtkDrawingArea", GTK_TYPE_DRAWING_AREA);
-    REGISTER_TYPE ("GtkEditable", GTK_TYPE_EDITABLE);
-    REGISTER_TYPE ("GtkEntry", GTK_TYPE_ENTRY);
-    REGISTER_TYPE ("GtkEventBox", GTK_TYPE_EVENT_BOX);
-    REGISTER_TYPE ("GtkExpanderStyle", GTK_TYPE_EXPANDER_STYLE);
-    REGISTER_TYPE ("GtkFixed", GTK_TYPE_FIXED);
-    REGISTER_TYPE ("GtkFontButton", GTK_TYPE_FONT_BUTTON);
-    REGISTER_TYPE ("GtkFontSelection", GTK_TYPE_FONT_SELECTION);
-    REGISTER_TYPE ("GtkFontSelectionDialog", GTK_TYPE_FONT_SELECTION_DIALOG);
-    REGISTER_TYPE ("GtkFrame", GTK_TYPE_FRAME);
-    REGISTER_TYPE ("GtkGammaCurve", GTK_TYPE_GAMMA_CURVE);
-    REGISTER_TYPE ("GtkHBox", GTK_TYPE_HBOX);
-    REGISTER_TYPE ("GtkHButtonBox", GTK_TYPE_HBUTTON_BOX);
-    REGISTER_TYPE ("GtkHPaned", GTK_TYPE_HPANED);
-    REGISTER_TYPE ("GtkHRuler", GTK_TYPE_HRULER);
-    REGISTER_TYPE ("GtkHScale", GTK_TYPE_HSCALE);
-    REGISTER_TYPE ("GtkHScrollbar", GTK_TYPE_HSCROLLBAR);
-    REGISTER_TYPE ("GtkHSeparator", GTK_TYPE_HSEPARATOR);
-    REGISTER_TYPE ("GtkHandleBox", GTK_TYPE_HANDLE_BOX);
-    REGISTER_TYPE ("GtkIconFactory", GTK_TYPE_ICON_FACTORY);
-    REGISTER_TYPE ("GtkIconSet", GTK_TYPE_ICON_SET);
-    REGISTER_TYPE ("GtkIconSize", GTK_TYPE_ICON_SIZE);
-    REGISTER_TYPE ("GtkIconSource", GTK_TYPE_ICON_SOURCE);
-    REGISTER_TYPE ("GtkIdentifier", GTK_TYPE_IDENTIFIER);
-    REGISTER_TYPE ("GtkImContext", GTK_TYPE_IM_CONTEXT);
-    REGISTER_TYPE ("GtkImContextSimple", GTK_TYPE_IM_CONTEXT_SIMPLE);
-    REGISTER_TYPE ("GtkImMulticontext", GTK_TYPE_IM_MULTICONTEXT);
-    REGISTER_TYPE ("GtkImPreeditStyle", GTK_TYPE_IM_PREEDIT_STYLE);
-    REGISTER_TYPE ("GtkImStatusStyle", GTK_TYPE_IM_STATUS_STYLE);
-    REGISTER_TYPE ("GtkImage", GTK_TYPE_IMAGE);
-    REGISTER_TYPE ("GtkImageMenuItem", GTK_TYPE_IMAGE_MENU_ITEM);
-    REGISTER_TYPE ("GtkImageType", GTK_TYPE_IMAGE_TYPE);
-    REGISTER_TYPE ("GtkInputDialog", GTK_TYPE_INPUT_DIALOG);
-    REGISTER_TYPE ("GtkInvisible", GTK_TYPE_INVISIBLE);
-    REGISTER_TYPE ("GtkItem", GTK_TYPE_ITEM);
-    REGISTER_TYPE ("GtkJustification", GTK_TYPE_JUSTIFICATION);
-    REGISTER_TYPE ("GtkLabel", GTK_TYPE_LABEL);
-    REGISTER_TYPE ("GtkLayout", GTK_TYPE_LAYOUT);
-    REGISTER_TYPE ("GtkList", GTK_TYPE_LIST);
-    REGISTER_TYPE ("GtkListItem", GTK_TYPE_LIST_ITEM);
-    REGISTER_TYPE ("GtkListStore", GTK_TYPE_LIST_STORE);
-    REGISTER_TYPE ("GtkMatchType", GTK_TYPE_MATCH_TYPE);
-    REGISTER_TYPE ("GtkMenu", GTK_TYPE_MENU);
-    REGISTER_TYPE ("GtkMenuBar", GTK_TYPE_MENU_BAR);
-    REGISTER_TYPE ("GtkMenuDirectionType", GTK_TYPE_MENU_DIRECTION_TYPE);
-    REGISTER_TYPE ("GtkMenuItem", GTK_TYPE_MENU_ITEM);
-    REGISTER_TYPE ("GtkMenuShell", GTK_TYPE_MENU_SHELL);
-    REGISTER_TYPE ("GtkMessageDialog", GTK_TYPE_MESSAGE_DIALOG);
-    REGISTER_TYPE ("GtkMessageType", GTK_TYPE_MESSAGE_TYPE);
-    REGISTER_TYPE ("GtkMetricType", GTK_TYPE_METRIC_TYPE);
-    REGISTER_TYPE ("GtkMisc", GTK_TYPE_MISC);
-    REGISTER_TYPE ("GtkMovementStep", GTK_TYPE_MOVEMENT_STEP);
-    REGISTER_TYPE ("GtkNotebook", GTK_TYPE_NOTEBOOK);
-    REGISTER_TYPE ("GtkNotebookTab", GTK_TYPE_NOTEBOOK_TAB);
-    REGISTER_TYPE ("GtkObject", GTK_TYPE_OBJECT);
-    REGISTER_TYPE ("GtkObjectFlags", GTK_TYPE_OBJECT_FLAGS);
-    REGISTER_TYPE ("GtkOptionMenu", GTK_TYPE_OPTION_MENU);
-    REGISTER_TYPE ("GtkOrientation", GTK_TYPE_ORIENTATION);
-    REGISTER_TYPE ("GtkPackType", GTK_TYPE_PACK_TYPE);
-    REGISTER_TYPE ("GtkPaned", GTK_TYPE_PANED);
-    REGISTER_TYPE ("GtkPathPriorityType", GTK_TYPE_PATH_PRIORITY_TYPE);
-    REGISTER_TYPE ("GtkPathType", GTK_TYPE_PATH_TYPE);
-    REGISTER_TYPE ("GtkPolicyType", GTK_TYPE_POLICY_TYPE);
-    REGISTER_TYPE ("GtkPositionType", GTK_TYPE_POSITION_TYPE);
-    REGISTER_TYPE ("GtkPrivateFlags", GTK_TYPE_PRIVATE_FLAGS);
-    REGISTER_TYPE ("GtkProgressBar", GTK_TYPE_PROGRESS_BAR);
-    REGISTER_TYPE ("GtkProgressBarOrientation", GTK_TYPE_PROGRESS_BAR_ORIENTATION);
-    REGISTER_TYPE ("GtkProgressBarStyle", GTK_TYPE_PROGRESS_BAR_STYLE);
-    REGISTER_TYPE ("GtkRadioButton", GTK_TYPE_RADIO_BUTTON);
-    REGISTER_TYPE ("GtkRadioMenuItem", GTK_TYPE_RADIO_MENU_ITEM);
-    REGISTER_TYPE ("GtkRange", GTK_TYPE_RANGE);
-    REGISTER_TYPE ("GtkRcFlags", GTK_TYPE_RC_FLAGS);
-    REGISTER_TYPE ("GtkRcStyle", GTK_TYPE_RC_STYLE);
-    REGISTER_TYPE ("GtkRcTokenType", GTK_TYPE_RC_TOKEN_TYPE);
-    REGISTER_TYPE ("GtkReliefStyle", GTK_TYPE_RELIEF_STYLE);
-    REGISTER_TYPE ("GtkRequisition", GTK_TYPE_REQUISITION);
-    REGISTER_TYPE ("GtkResizeMode", GTK_TYPE_RESIZE_MODE);
-    REGISTER_TYPE ("GtkResponseType", GTK_TYPE_RESPONSE_TYPE);
-    REGISTER_TYPE ("GtkRuler", GTK_TYPE_RULER);
-    REGISTER_TYPE ("GtkScale", GTK_TYPE_SCALE);
-    REGISTER_TYPE ("GtkScrollType", GTK_TYPE_SCROLL_TYPE);
-    REGISTER_TYPE ("GtkScrollbar", GTK_TYPE_SCROLLBAR);
-    REGISTER_TYPE ("GtkScrolledWindow", GTK_TYPE_SCROLLED_WINDOW);
-    REGISTER_TYPE ("GtkSelectionData", GTK_TYPE_SELECTION_DATA);
-    REGISTER_TYPE ("GtkSelectionMode", GTK_TYPE_SELECTION_MODE);
-    REGISTER_TYPE ("GtkSeparator", GTK_TYPE_SEPARATOR);
-    REGISTER_TYPE ("GtkSeparatorMenuItem", GTK_TYPE_SEPARATOR_MENU_ITEM);
-    REGISTER_TYPE ("GtkSettings", GTK_TYPE_SETTINGS);
-    REGISTER_TYPE ("GtkShadowType", GTK_TYPE_SHADOW_TYPE);
-    REGISTER_TYPE ("GtkSideType", GTK_TYPE_SIDE_TYPE);
-    REGISTER_TYPE ("GtkSignalRunType", GTK_TYPE_SIGNAL_RUN_TYPE);
-    REGISTER_TYPE ("GtkSizeGroup", GTK_TYPE_SIZE_GROUP);
-    REGISTER_TYPE ("GtkSizeGroupMode", GTK_TYPE_SIZE_GROUP_MODE);
-    REGISTER_TYPE ("GtkSortType", GTK_TYPE_SORT_TYPE);
-    REGISTER_TYPE ("GtkSpinButton", GTK_TYPE_SPIN_BUTTON);
-    REGISTER_TYPE ("GtkSpinButtonUpdatePolicy", GTK_TYPE_SPIN_BUTTON_UPDATE_POLICY);
-    REGISTER_TYPE ("GtkSpinType", GTK_TYPE_SPIN_TYPE);
-    REGISTER_TYPE ("GtkStateType", GTK_TYPE_STATE_TYPE);
-    REGISTER_TYPE ("GtkStatusbar", GTK_TYPE_STATUSBAR);
-    REGISTER_TYPE ("GtkStyle", GTK_TYPE_STYLE);
-    REGISTER_TYPE ("GtkSubmenuDirection", GTK_TYPE_SUBMENU_DIRECTION);
-    REGISTER_TYPE ("GtkSubmenuPlacement", GTK_TYPE_SUBMENU_PLACEMENT);
-    REGISTER_TYPE ("GtkTable", GTK_TYPE_TABLE);
-    REGISTER_TYPE ("GtkTargetFlags", GTK_TYPE_TARGET_FLAGS);
-    REGISTER_TYPE ("GtkTearoffMenuItem", GTK_TYPE_TEAROFF_MENU_ITEM);
-    REGISTER_TYPE ("GtkTextAttributes", GTK_TYPE_TEXT_ATTRIBUTES);
-    REGISTER_TYPE ("GtkTextBuffer", GTK_TYPE_TEXT_BUFFER);
-    REGISTER_TYPE ("GtkTextChildAnchor", GTK_TYPE_TEXT_CHILD_ANCHOR);
-    REGISTER_TYPE ("GtkTextDirection", GTK_TYPE_TEXT_DIRECTION);
-    REGISTER_TYPE ("GtkTextIter", GTK_TYPE_TEXT_ITER);
-    REGISTER_TYPE ("GtkTextMark", GTK_TYPE_TEXT_MARK);
-    REGISTER_TYPE ("GtkTextSearchFlags", GTK_TYPE_TEXT_SEARCH_FLAGS);
-    REGISTER_TYPE ("GtkTextTag", GTK_TYPE_TEXT_TAG);
-    REGISTER_TYPE ("GtkTextTagTable", GTK_TYPE_TEXT_TAG_TABLE);
-    REGISTER_TYPE ("GtkTextView", GTK_TYPE_TEXT_VIEW);
-    REGISTER_TYPE ("GtkTextWindowType", GTK_TYPE_TEXT_WINDOW_TYPE);
-    REGISTER_TYPE ("GtkToggleButton", GTK_TYPE_TOGGLE_BUTTON);
-    REGISTER_TYPE ("GtkToolbar", GTK_TYPE_TOOLBAR);
-    REGISTER_TYPE ("GtkToolbarChildType", GTK_TYPE_TOOLBAR_CHILD_TYPE);
-    REGISTER_TYPE ("GtkToolbarSpaceStyle", GTK_TYPE_TOOLBAR_SPACE_STYLE);
-    REGISTER_TYPE ("GtkToolbarStyle", GTK_TYPE_TOOLBAR_STYLE);
-    REGISTER_TYPE ("GtkTreeDragDest", GTK_TYPE_TREE_DRAG_DEST);
-    REGISTER_TYPE ("GtkTreeDragSource", GTK_TYPE_TREE_DRAG_SOURCE);
-    REGISTER_TYPE ("GtkTreeIter", GTK_TYPE_TREE_ITER);
-    REGISTER_TYPE ("GtkTreeModel", GTK_TYPE_TREE_MODEL);
-    REGISTER_TYPE ("GtkTreeModelFlags", GTK_TYPE_TREE_MODEL_FLAGS);
-    REGISTER_TYPE ("GtkTreeModelSort", GTK_TYPE_TREE_MODEL_SORT);
-    REGISTER_TYPE ("GtkTreePath", GTK_TYPE_TREE_PATH);
-    REGISTER_TYPE ("GtkTreeRowReference", GTK_TYPE_TREE_ROW_REFERENCE);
-    REGISTER_TYPE ("GtkTreeSelection", GTK_TYPE_TREE_SELECTION);
-    REGISTER_TYPE ("GtkTreeSortable", GTK_TYPE_TREE_SORTABLE);
-    REGISTER_TYPE ("GtkTreeStore", GTK_TYPE_TREE_STORE);
-    REGISTER_TYPE ("GtkTreeView", GTK_TYPE_TREE_VIEW);
-    REGISTER_TYPE ("GtkTreeViewColumn", GTK_TYPE_TREE_VIEW_COLUMN);
-    REGISTER_TYPE ("GtkTreeViewColumnSizing", GTK_TYPE_TREE_VIEW_COLUMN_SIZING);
-    REGISTER_TYPE ("GtkTreeViewDropPosition", GTK_TYPE_TREE_VIEW_DROP_POSITION);
-    REGISTER_TYPE ("GtkTreeViewMode", GTK_TYPE_TREE_VIEW_MODE);
-    REGISTER_TYPE ("GtkUpdateType", GTK_TYPE_UPDATE_TYPE);
-    REGISTER_TYPE ("GtkVBox", GTK_TYPE_VBOX);
-    REGISTER_TYPE ("GtkVButtonBox", GTK_TYPE_VBUTTON_BOX);
-    REGISTER_TYPE ("GtkViewport", GTK_TYPE_VIEWPORT);
-    REGISTER_TYPE ("GtkVisibility", GTK_TYPE_VISIBILITY);
-    REGISTER_TYPE ("GtkVPaned", GTK_TYPE_VPANED);
-    REGISTER_TYPE ("GtkVRuler", GTK_TYPE_VRULER);
-    REGISTER_TYPE ("GtkVScale", GTK_TYPE_VSCALE);
-    REGISTER_TYPE ("GtkVScrollbar", GTK_TYPE_VSCROLLBAR);
-    REGISTER_TYPE ("GtkVSeparator", GTK_TYPE_VSEPARATOR);
-    REGISTER_TYPE ("GtkWidget", GTK_TYPE_WIDGET);
-    REGISTER_TYPE ("GtkWidgetFlags", GTK_TYPE_WIDGET_FLAGS);
-    REGISTER_TYPE ("GtkWidgetHelpType", GTK_TYPE_WIDGET_HELP_TYPE);
-    REGISTER_TYPE ("GtkWindow", GTK_TYPE_WINDOW);
-    REGISTER_TYPE ("GtkWindowGroup", GTK_TYPE_WINDOW_GROUP);
-    REGISTER_TYPE ("GtkWindowPosition", GTK_TYPE_WINDOW_POSITION);
-    REGISTER_TYPE ("GtkWindowType", GTK_TYPE_WINDOW_TYPE);
-    REGISTER_TYPE ("GtkWrapMode", GTK_TYPE_WRAP_MODE);
+    if (!types)
+    {
+#define add_type(class_name, func) g_hash_table_insert (types, g_strdup (class_name), func)
+
+        types = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+
+        add_type ("MooFontButton", moo_font_button_get_type);
+        add_type ("MooFontSelection", moo_font_selection_get_type);
+        add_type ("MooFontSelectionDialog", moo_font_selection_dialog_get_type);
+        add_type ("MooCombo", moo_combo_get_type);
+        add_type ("MooHistoryCombo", moo_history_combo_get_type);
+        add_type ("MooEntry", moo_entry_get_type);
+        add_type ("MooPrefsPage", moo_prefs_page_get_type);
+        add_type ("MooAccelButton", _moo_accel_button_get_type);
+
+        add_type ("GtkAboutDialog", gtk_about_dialog_get_type);
+        add_type ("GtkCellRendererCombo", gtk_cell_renderer_combo_get_type);
+        add_type ("GtkCellRendererProgress", gtk_cell_renderer_progress_get_type);
+        add_type ("GtkCellView", gtk_cell_view_get_type);
+        add_type ("GtkIconView", gtk_icon_view_get_type);
+        add_type ("GtkMenuToolButton", gtk_menu_tool_button_get_type);
+        add_type ("GtkAccelMap", gtk_accel_map_get_type);
+        add_type ("GtkAction", gtk_action_get_type);
+        add_type ("GtkActionGroup", gtk_action_group_get_type);
+        add_type ("GtkComboBox", gtk_combo_box_get_type);
+        add_type ("GtkComboBoxEntry", gtk_combo_box_entry_get_type);
+        add_type ("GtkExpander", gtk_expander_get_type);
+        add_type ("GtkFileChooser", gtk_file_chooser_get_type);
+        add_type ("GtkFileChooserDialog", gtk_file_chooser_dialog_get_type);
+        add_type ("GtkFileChooserWidget", gtk_file_chooser_widget_get_type);
+        add_type ("GtkFileFilter", gtk_file_filter_get_type);
+        add_type ("GtkRadioAction", gtk_radio_action_get_type);
+        add_type ("GtkRadioToolButton", gtk_radio_tool_button_get_type);
+        add_type ("GtkSeparatorToolItem", gtk_separator_tool_item_get_type);
+        add_type ("GtkToggleAction", gtk_toggle_action_get_type);
+        add_type ("GtkToggleToolButton", gtk_toggle_tool_button_get_type);
+        add_type ("GtkToolButton", gtk_tool_button_get_type);
+        add_type ("GtkToolItem", gtk_tool_item_get_type);
+        add_type ("GtkTreeModelFilter", gtk_tree_model_filter_get_type);
+        add_type ("GtkUiManager", gtk_ui_manager_get_type);
+        add_type ("GtkAccelGroup", gtk_accel_group_get_type);
+        add_type ("GtkAccelLabel", gtk_accel_label_get_type);
+        add_type ("GtkAdjustment", gtk_adjustment_get_type);
+        add_type ("GtkAlignment", gtk_alignment_get_type);
+        add_type ("GtkArrow", gtk_arrow_get_type);
+        add_type ("GtkAspectFrame", gtk_aspect_frame_get_type);
+        add_type ("GtkBin", gtk_bin_get_type);
+        add_type ("GtkBox", gtk_box_get_type);
+        add_type ("GtkButton", gtk_button_get_type);
+        add_type ("GtkButtonBox", gtk_button_box_get_type);
+        add_type ("GtkCalendar", gtk_calendar_get_type);
+        add_type ("GtkCellRenderer", gtk_cell_renderer_get_type);
+        add_type ("GtkCellRendererMode", gtk_cell_renderer_mode_get_type);
+        add_type ("GtkCellRendererPixbuf", gtk_cell_renderer_pixbuf_get_type);
+        add_type ("GtkCellRendererState", gtk_cell_renderer_state_get_type);
+        add_type ("GtkCellRendererText", gtk_cell_renderer_text_get_type);
+        add_type ("GtkCellRendererToggle", gtk_cell_renderer_toggle_get_type);
+        add_type ("GtkCheckButton", gtk_check_button_get_type);
+        add_type ("GtkCheckMenuItem", gtk_check_menu_item_get_type);
+        add_type ("GtkClipboard", gtk_clipboard_get_type);
+        add_type ("GtkColorButton", gtk_color_button_get_type);
+        add_type ("GtkColorSelection", gtk_color_selection_get_type);
+        add_type ("GtkColorSelectionDialog", gtk_color_selection_dialog_get_type);
+        add_type ("GtkCombo", gtk_combo_get_type);
+        add_type ("GtkContainer", gtk_container_get_type);
+        add_type ("GtkCurve", gtk_curve_get_type);
+        add_type ("GtkDialog", gtk_dialog_get_type);
+        add_type ("GtkDrawingArea", gtk_drawing_area_get_type);
+        add_type ("GtkEditable", gtk_editable_get_type);
+        add_type ("GtkEntry", gtk_entry_get_type);
+        add_type ("GtkEventBox", gtk_event_box_get_type);
+        add_type ("GtkFixed", gtk_fixed_get_type);
+        add_type ("GtkFontButton", gtk_font_button_get_type);
+        add_type ("GtkFontSelection", gtk_font_selection_get_type);
+        add_type ("GtkFontSelectionDialog", gtk_font_selection_dialog_get_type);
+        add_type ("GtkFrame", gtk_frame_get_type);
+        add_type ("GtkGammaCurve", gtk_gamma_curve_get_type);
+        add_type ("GtkHBox", gtk_hbox_get_type);
+        add_type ("GtkHButtonBox", gtk_hbutton_box_get_type);
+        add_type ("GtkHPaned", gtk_hpaned_get_type);
+        add_type ("GtkHRuler", gtk_hruler_get_type);
+        add_type ("GtkHScale", gtk_hscale_get_type);
+        add_type ("GtkHScrollbar", gtk_hscrollbar_get_type);
+        add_type ("GtkHSeparator", gtk_hseparator_get_type);
+        add_type ("GtkHandleBox", gtk_handle_box_get_type);
+        add_type ("GtkIconFactory", gtk_icon_factory_get_type);
+        add_type ("GtkIconSet", gtk_icon_set_get_type);
+        add_type ("GtkIconSource", gtk_icon_source_get_type);
+        add_type ("GtkImContext", gtk_im_context_get_type);
+        add_type ("GtkImContextSimple", gtk_im_context_simple_get_type);
+        add_type ("GtkImMulticontext", gtk_im_multicontext_get_type);
+        add_type ("GtkImage", gtk_image_get_type);
+        add_type ("GtkImageMenuItem", gtk_image_menu_item_get_type);
+        add_type ("GtkInputDialog", gtk_input_dialog_get_type);
+        add_type ("GtkInvisible", gtk_invisible_get_type);
+        add_type ("GtkItem", gtk_item_get_type);
+        add_type ("GtkLabel", gtk_label_get_type);
+        add_type ("GtkLayout", gtk_layout_get_type);
+        add_type ("GtkList", gtk_list_get_type);
+        add_type ("GtkListStore", gtk_list_store_get_type);
+        add_type ("GtkMenu", gtk_menu_get_type);
+        add_type ("GtkMenuBar", gtk_menu_bar_get_type);
+        add_type ("GtkMenuItem", gtk_menu_item_get_type);
+        add_type ("GtkMenuShell", gtk_menu_shell_get_type);
+        add_type ("GtkMessageDialog", gtk_message_dialog_get_type);
+        add_type ("GtkMisc", gtk_misc_get_type);
+        add_type ("GtkNotebook", gtk_notebook_get_type);
+        add_type ("GtkObject", gtk_object_get_type);
+        add_type ("GtkOptionMenu", gtk_option_menu_get_type);
+        add_type ("GtkPaned", gtk_paned_get_type);
+        add_type ("GtkProgressBar", gtk_progress_bar_get_type);
+        add_type ("GtkRadioButton", gtk_radio_button_get_type);
+        add_type ("GtkRadioMenuItem", gtk_radio_menu_item_get_type);
+        add_type ("GtkRange", gtk_range_get_type);
+        add_type ("GtkRuler", gtk_ruler_get_type);
+        add_type ("GtkScale", gtk_scale_get_type);
+        add_type ("GtkScrollbar", gtk_scrollbar_get_type);
+        add_type ("GtkScrolledWindow", gtk_scrolled_window_get_type);
+        add_type ("GtkSeparator", gtk_separator_get_type);
+        add_type ("GtkSeparatorMenuItem", gtk_separator_menu_item_get_type);
+        add_type ("GtkSettings", gtk_settings_get_type);
+        add_type ("GtkSizeGroup", gtk_size_group_get_type);
+        add_type ("GtkSpinButton", gtk_spin_button_get_type);
+        add_type ("GtkStatusbar", gtk_statusbar_get_type);
+        add_type ("GtkStyle", gtk_style_get_type);
+        add_type ("GtkTable", gtk_table_get_type);
+        add_type ("GtkTearoffMenuItem", gtk_tearoff_menu_item_get_type);
+        add_type ("GtkTextBuffer", gtk_text_buffer_get_type);
+        add_type ("GtkTextChildAnchor", gtk_text_child_anchor_get_type);
+        add_type ("GtkTextMark", gtk_text_mark_get_type);
+        add_type ("GtkTextTag", gtk_text_tag_get_type);
+        add_type ("GtkTextTagTable", gtk_text_tag_table_get_type);
+        add_type ("GtkTextView", gtk_text_view_get_type);
+        add_type ("GtkToggleButton", gtk_toggle_button_get_type);
+        add_type ("GtkToolbar", gtk_toolbar_get_type);
+        add_type ("GtkTreeDragDest", gtk_tree_drag_dest_get_type);
+        add_type ("GtkTreeDragSource", gtk_tree_drag_source_get_type);
+        add_type ("GtkTreeIter", gtk_tree_iter_get_type);
+        add_type ("GtkTreeModel", gtk_tree_model_get_type);
+        add_type ("GtkTreeModelSort", gtk_tree_model_sort_get_type);
+        add_type ("GtkTreePath", gtk_tree_path_get_type);
+        add_type ("GtkTreeRowReference", gtk_tree_row_reference_get_type);
+        add_type ("GtkTreeSelection", gtk_tree_selection_get_type);
+        add_type ("GtkTreeSortable", gtk_tree_sortable_get_type);
+        add_type ("GtkTreeStore", gtk_tree_store_get_type);
+        add_type ("GtkTreeView", gtk_tree_view_get_type);
+        add_type ("GtkTreeViewColumn", gtk_tree_view_column_get_type);
+        add_type ("GtkTreeViewMode", gtk_tree_view_mode_get_type);
+        add_type ("GtkVBox", gtk_vbox_get_type);
+        add_type ("GtkVButtonBox", gtk_vbutton_box_get_type);
+        add_type ("GtkViewport", gtk_viewport_get_type);
+        add_type ("GtkVPaned", gtk_vpaned_get_type);
+        add_type ("GtkVRuler", gtk_vruler_get_type);
+        add_type ("GtkVScale", gtk_vscale_get_type);
+        add_type ("GtkVScrollbar", gtk_vscrollbar_get_type);
+        add_type ("GtkVSeparator", gtk_vseparator_get_type);
+        add_type ("GtkWidget", gtk_widget_get_type);
+        add_type ("GtkWidgetFlags", gtk_widget_flags_get_type);
+        add_type ("GtkWidgetHelpType", gtk_widget_help_type_get_type);
+        add_type ("GtkWindow", gtk_window_get_type);
+        add_type ("GtkWindowGroup", gtk_window_group_get_type);
 
 #ifndef __WIN32__
-    REGISTER_TYPE ("GtkPlug", GTK_TYPE_PLUG);
-    REGISTER_TYPE ("GtkSocket", GTK_TYPE_SOCKET);
+        add_type ("GtkPlug", gtk_plug_get_type);
+        add_type ("GtkSocket", gtk_socket_get_type);
 #endif
 
+#undef add_type
+    }
+
+    type_func = g_hash_table_lookup (types, name);
+    if (type_func)
+        return type_func ();
+
     return 0;
 }
 
