@@ -1,13 +1,16 @@
 /*
  *   mooglade.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_GLADE_H
@@ -62,6 +65,7 @@ GType        moo_glade_xml_get_type         (void) G_GNUC_CONST;
 
 MooGladeXML *moo_glade_xml_new_empty        (const char     *domain);
 
+void         moo_glade_xml_register_type    (GType           type);
 void         moo_glade_xml_map_class        (MooGladeXML    *xml,
                                              const char     *class_name,
                                              GType           use_type);
