@@ -1,8 +1,25 @@
+/*
+ *   moohelp.h
+ *
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
+ *
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #ifndef MOO_HELP_H
 #define MOO_HELP_H
 
 #include <gtk/gtkdialog.h>
 
+G_BEGIN_DECLS
+
 #define MOO_HELP_ID_CONTENTS "contents"
 #define MOO_HELP_ID_INDEX    "index"
 
@@ -25,4 +42,6 @@ void        moo_help_set_func_full      (GtkWidget     *widget,
 
 void        moo_help_connect_keys       (GtkWidget     *widget);
 
+G_END_DECLS
+
 #endif /* MOO_HELP_H */
