@@ -1,18 +1,22 @@
 /*
  *   moohistorylist.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "mooutils/moomarshals.h"
+#include "marshals.h"
 #include "mooutils/moohistorylist.h"
 #include "mooutils/mooprefs.h"
+#include "mooutils/mootype-macros.h"
 #include <gtk/gtk.h>
 #include <string.h>
 
@@ -43,6 +47,8 @@ struct _MooHistoryListPrivate {
     gboolean allow_empty;
 };
 
+MOO_DEFINE_BOXED_TYPE_C (MooHistoryItem, moo_history_item)
+
 static GHashTable *named_lists;
 
 static void     moo_history_list_class_init     (MooHistoryListClass *klass);
@@ -164,6 +170,7 @@ moo_history_list_init (MooHistoryList *list)
     list->priv->use_separator = TRUE;
 
     list->priv->mgr = moo_menu_mgr_new ();
+    moo_menu_mgr_set_use_mnemonic (list->priv->mgr, FALSE);
     g_signal_connect_swapped (list->priv->mgr, "item-activated",
                               G_CALLBACK (menu_item_activated), list);
 
@@ -440,6 +447,7 @@ moo_history_list_add_full (MooHistoryList *list,
     Item *new_entry;
     GtkTreeIter iter;
     int index;
+    gboolean was_empty;
 
     g_return_if_fail (MOO_IS_HISTORY_LIST (list));
     g_return_if_fail (entry != NULL);
@@ -448,6 +456,8 @@ moo_history_list_add_full (MooHistoryList *list,
     if (!list->priv->allow_empty && !entry[0])
         return;
 
+    was_empty = moo_history_list_is_empty (list);
+
     _moo_history_list_load (list);
 
     if (moo_history_list_find (list, entry, &iter))
@@ -489,7 +499,9 @@ moo_history_list_add_full (MooHistoryList *list,
     list_save_recent (list);
 
     g_signal_emit (list, signals[CHANGED], 0);
-    g_object_notify (G_OBJECT (list), "empty");
+
+    if (was_empty)
+        g_object_notify (G_OBJECT (list), "empty");
 
     moo_history_item_free (new_entry);
 }
@@ -555,6 +567,9 @@ moo_history_list_remove (MooHistoryList *list,
 
     list_save_recent (list);
 
+    if (moo_history_list_is_empty (list))
+        g_object_notify (G_OBJECT (list), "empty");
+
     moo_history_item_free (item);
     return;
 }
@@ -718,20 +733,6 @@ moo_history_list_get (const char *user_id)
 }
 
 
-GType
-moo_history_item_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooHistoryItem",
-                                             (GBoxedCopyFunc) moo_history_item_copy,
-                                             (GBoxedFreeFunc) moo_history_item_free);
-
-    return type;
-}
-
-
 GtkTreeModel*
 moo_history_list_get_model (MooHistoryList *list)
 {
