@@ -1,17 +1,21 @@
 /*
  *   mooi18n.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/mooi18n.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/moocompat.h"
 #include <glib.h>
 
 
@@ -46,6 +50,53 @@ moo_gettext (const char *string)
 #endif /* !ENABLE_NLS */
 }
 
+const char *
+moo_pgettext (const char *msgctxtid, G_GNUC_UNUSED gsize msgidoffset)
+{
+#ifdef ENABLE_NLS
+    g_return_val_if_fail (msgctxtid != NULL, NULL);
+    init_gettext ();
+    return moo_dpgettext (GETTEXT_PACKAGE, msgctxtid, msgidoffset);
+#else /* !ENABLE_NLS */
+    return msgctxtid;
+#endif /* !ENABLE_NLS */
+}
+
+const char *
+moo_pgettext2 (G_GNUC_UNUSED const char *context, const char *msgctxtid)
+{
+#ifdef ENABLE_NLS
+    char *tmp;
+    const char *translation;
+
+    g_return_val_if_fail (msgctxtid != NULL, NULL);
+    init_gettext ();
+
+    tmp = g_strjoin (context, "\004", msgctxtid, NULL);
+    translation = dgettext (GETTEXT_PACKAGE, tmp);
+
+    if (translation == tmp)
+        translation = msgctxtid;
+
+    g_free (tmp);
+    return translation;
+#else /* !ENABLE_NLS */
+    return msgctxtid;
+#endif /* !ENABLE_NLS */
+}
+
+const char *
+moo_dpgettext (const char *domain, const char *msgctxtid, gsize msgidoffset)
+{
+    g_return_val_if_fail (domain != NULL, NULL);
+    g_return_val_if_fail (msgctxtid != NULL, NULL);
+#ifdef ENABLE_NLS
+    return g_dpgettext (GETTEXT_PACKAGE, msgctxtid, msgidoffset);
+#else
+    return msgctxtid + msgidoffset;
+#endif
+}
+
 const char *
 _moo_gsv_gettext (const char *string)
 {
@@ -59,7 +110,7 @@ _moo_gsv_gettext (const char *string)
 }
 
 char *
-_moo_gsv_dgettext (const char *domain, const char *string)
+_moo_gsv_dgettext (G_GNUC_UNUSED const char *domain, const char *string)
 {
 #ifdef ENABLE_NLS
     gchar *tmp;
