@@ -1,63 +1,34 @@
 /*
  *   moomarkup.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/moomarkup.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-gobject.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mootype-macros.h"
 #include <string.h>
 #include <glib.h>
 
 
-GType
-moo_markup_doc_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooMarkupDoc",
-                                             (GBoxedCopyFunc) moo_markup_doc_ref,
-                                             (GBoxedFreeFunc) moo_markup_doc_unref);
-
-    return type;
-}
-
-
-typedef void (*markup_start_element_func)   (GMarkupParseContext    *context,
-                                             const gchar            *element_name,
-                                             const gchar           **attribute_names,
-                                             const gchar           **attribute_values,
-                                             gpointer                user_data,
-                                             GError                **error);
-typedef void (*markup_end_element_func)     (GMarkupParseContext    *context,
-                                             const gchar            *element_name,
-                                             gpointer                user_data,
-                                             GError                **error);
-typedef void (*markup_text_func)            (GMarkupParseContext    *context,
-                                             const gchar            *text,
-                                             gsize                   text_len,
-                                             gpointer                user_data,
-                                             GError                **error);
-typedef void (*markup_passthrough_func)     (GMarkupParseContext    *context,
-                                             const gchar            *passthrough_text,
-                                             gsize                   text_len,
-                                             gpointer                user_data,
-                                             GError                **error);
-
 typedef struct {
     MooMarkupDoc   *doc;
     MooMarkupNode  *current;
 } ParserState;
 
+MOO_DEFINE_BOXED_TYPE_R (MooMarkupDoc, moo_markup_doc)
+MOO_DEFINE_QUARK (moo-parse-error, moo_parse_error_quark)
 
 static MooMarkupDoc     *moo_markup_doc_new_priv    (const char         *name);
 static void              moo_markup_doc_set_name    (MooMarkupDoc       *doc,
@@ -128,7 +99,7 @@ set_modified (MooMarkupNode *node,
 {
     if (node->doc)
     {
-#ifdef MOO_DEBUG_ENABLED
+#ifdef MOO_DEBUG
         if (modified && !node->doc->modified && node->doc->track_modified)
         {
             g_message ("markup doc %s modified",
@@ -173,10 +144,10 @@ markup_parse_memory (const char     *buffer,
                      G_GNUC_UNUSED const char *filename,
                      GError        **error)
 {
-    GMarkupParser parser = { (markup_start_element_func) start_element,
-                             (markup_end_element_func) end_element,
-                             (markup_text_func) text,
-                             (markup_passthrough_func) passthrough,
+    GMarkupParser parser = { (MooMarkupStartElementFunc) start_element,
+                             (MooMarkupEndElementFunc) end_element,
+                             (MooMarkupTextFunc) text,
+                             (MooMarkupPassthroughFunc) passthrough,
                              NULL};
 
     ParserState state;
@@ -223,6 +194,8 @@ moo_markup_parse_file (const char     *filename,
 
     g_return_val_if_fail (filename != NULL, NULL);
 
+    /* XXX use moo_parse_markup_file here */
+
     if (!g_file_get_contents (filename, &content, &len, error))
         return NULL;
 
@@ -304,7 +277,7 @@ passthrough (G_GNUC_UNUSED GMarkupParseContext    *ctx,
 static MooMarkupDoc *
 moo_markup_doc_new_priv (const char *name)
 {
-    MooMarkupDoc *doc = _moo_new0 (MooMarkupDoc);
+    MooMarkupDoc *doc = moo_new0 (MooMarkupDoc);
 
     doc->type = MOO_MARKUP_DOC_NODE;
     doc->name = g_strdup (name ? name : "");
@@ -340,7 +313,7 @@ moo_markup_element_new (MooMarkupDoc   *doc,
 {
     MooMarkupElement *elm;
 
-    elm = _moo_new0 (MooMarkupElement);
+    elm = moo_new0 (MooMarkupElement);
     add_node (doc, parent, MOO_MARKUP_NODE (elm));
 
     elm->type = MOO_MARKUP_ELEMENT_NODE;
@@ -368,7 +341,7 @@ moo_markup_text_node_new_take_string (MooMarkupNodeType   type,
 
     g_assert (type == MOO_MARKUP_TEXT_NODE || type == MOO_MARKUP_COMMENT_NODE);
 
-    node = _moo_new0 (MooMarkupText);
+    node = moo_new0 (MooMarkupText);
     add_node (doc, parent, MOO_MARKUP_NODE (node));
 
     node->type = type;
@@ -438,21 +411,17 @@ moo_markup_text_node_add_text (MooMarkupText  *node,
                                const char     *text,
                                gssize          text_len)
 {
-    char *tmp;
-
     if (text_len < 0)
         text_len = strlen (text);
 
     if (text_len == 0)
         return;
 
-    tmp = g_new (char, node->size + text_len + 1);
-    memcpy (tmp, node->text, node->size);
-    memcpy (tmp + node->size, text, text_len);
-    g_free (node->text);
-    node->text = tmp;
+    node->text = g_renew (char, node->text, node->size + text_len + 1);
+    memcpy (node->text + node->size, text, text_len);
     node->size += text_len;
     node->text[node->size] = 0;
+
     set_modified (MOO_MARKUP_NODE (node), TRUE);
 }
 
@@ -512,19 +481,19 @@ moo_markup_node_free (MooMarkupNode *node)
     {
         case MOO_MARKUP_DOC_NODE:
             moo_markup_doc_unref_private (MOO_MARKUP_DOC (node));
-            _moo_free (MooMarkupDoc, (MooMarkupDoc*) node);
+            moo_free (MooMarkupDoc, (MooMarkupDoc*) node);
             break;
         case MOO_MARKUP_ELEMENT_NODE:
             moo_markup_element_free (MOO_MARKUP_ELEMENT (node));
-            _moo_free (MooMarkupElement, (MooMarkupElement*) node);
+            moo_free (MooMarkupElement, (MooMarkupElement*) node);
             break;
         case MOO_MARKUP_TEXT_NODE:
             moo_markup_text_node_free (node);
-            _moo_free (MooMarkupText, (MooMarkupText*) node);
+            moo_free (MooMarkupText, (MooMarkupText*) node);
             break;
         case MOO_MARKUP_COMMENT_NODE:
             moo_markup_text_node_free (node);
-            _moo_free (MooMarkupComment, (MooMarkupComment*) node);
+            moo_free (MooMarkupComment, (MooMarkupComment*) node);
             break;
         default:
             g_assert_not_reached ();
@@ -675,7 +644,7 @@ moo_markup_comment_node_print (MooMarkupNode *node,
         *p = 0;
     }
 
-    g_string_append_printf (str, "<!--%s-->", text->text);
+    g_string_append (str, text->text);
 }
 
 
@@ -853,9 +822,9 @@ moo_markup_set_prop (MooMarkupNode      *elm,
 
 
 int
-moo_markup_get_int_prop (MooMarkupNode      *node,
-                         const char         *prop_name,
-                         int                 default_val)
+moo_markup_int_prop (MooMarkupNode *node,
+                     const char    *prop_name,
+                     int            default_val)
 {
     g_return_val_if_fail (MOO_MARKUP_IS_ELEMENT (node), default_val);
     g_return_val_if_fail (prop_name != NULL, default_val);
@@ -863,21 +832,21 @@ moo_markup_get_int_prop (MooMarkupNode      *node,
                                        default_val);
 }
 
-
-void
-moo_markup_set_int_prop (MooMarkupNode      *node,
-                         const char         *prop_name,
-                         int                 val)
+guint
+moo_markup_uint_prop (MooMarkupNode *node,
+                      const char    *prop_name,
+                      guint          default_val)
 {
-    moo_markup_set_prop (node, prop_name,
-                         _moo_convert_int_to_string (val));
+    g_return_val_if_fail (MOO_MARKUP_IS_ELEMENT (node), default_val);
+    g_return_val_if_fail (prop_name != NULL, default_val);
+    return _moo_convert_string_to_uint (moo_markup_get_prop (node, prop_name),
+                                        default_val);
 }
 
-
 gboolean
-moo_markup_get_bool_prop (MooMarkupNode      *node,
-                          const char         *prop_name,
-                          gboolean            default_val)
+moo_markup_bool_prop (MooMarkupNode *node,
+                      const char    *prop_name,
+                      gboolean       default_val)
 {
     g_return_val_if_fail (MOO_MARKUP_IS_ELEMENT (node), default_val);
     g_return_val_if_fail (prop_name != NULL, default_val);
@@ -887,9 +856,9 @@ moo_markup_get_bool_prop (MooMarkupNode      *node,
 
 
 void
-moo_markup_set_bool_prop (MooMarkupNode      *node,
-                          const char         *prop_name,
-                          gboolean            val)
+moo_markup_set_bool_prop (MooMarkupNode *node,
+                          const char    *prop_name,
+                          gboolean       val)
 {
     moo_markup_set_prop (node, prop_name,
                          _moo_convert_bool_to_string (val));
@@ -1158,28 +1127,22 @@ moo_markup_save (MooMarkupDoc       *doc,
 
 
 #define INDENT_CHAR ' '
-#ifdef __WIN32__
-#define LINE_SEPARATOR "\r\n"
-#elif defined(MOO_OS_DARWIN)
-#define LINE_SEPARATOR "\r"
-#else
-#define LINE_SEPARATOR "\n"
-#endif
 
-static void
-format_pretty_element (MooMarkupElement *elm,
-                       GString          *str,
-                       guint             indent,
-                       guint             indent_size)
+static gboolean
+write_pretty_element (MooMarkupElement *elm,
+                      MooFileWriter    *writer,
+                      guint             indent,
+                      guint             indent_size)
 {
     gboolean isdir = FALSE;
     gboolean empty = TRUE;
     MooMarkupNode *child;
+    gboolean result = TRUE;
     char *fill;
     guint i;
 
-    g_return_if_fail (MOO_MARKUP_IS_ELEMENT (elm));
-    g_return_if_fail (str != NULL);
+    g_return_val_if_fail (MOO_MARKUP_IS_ELEMENT (elm), FALSE);
+    g_return_val_if_fail (writer != NULL, FALSE);
 
     for (child = elm->children; child != NULL; child = child->next)
     {
@@ -1196,89 +1159,68 @@ format_pretty_element (MooMarkupElement *elm,
 
     fill = g_strnfill (indent, INDENT_CHAR);
 
-    g_string_append_len (str, fill, indent);
-    g_string_append_printf (str, "<%s", elm->name);
-    for (i = 0; i < elm->n_attrs; ++i)
+    moo_file_writer_write (writer, fill, indent);
+    moo_file_writer_printf (writer, "<%s", elm->name);
+
+    for (i = 0; result && i < elm->n_attrs; ++i)
     {
         char *attr = g_markup_escape_text (elm->attr_vals[i], -1);
-        g_string_append_printf (str, " %s=\"%s\"",
-                                elm->attr_names[i],
-                                attr);
+        result = moo_file_writer_printf (writer, " %s=\"%s\"",
+                                         elm->attr_names[i],
+                                         attr);
         g_free (attr);
     }
 
     if (empty)
     {
-        g_string_append (str, "/>" LINE_SEPARATOR);
+        moo_file_writer_write (writer, "/>\n", -1);
     }
     else if (isdir)
     {
-        g_string_append (str, ">" LINE_SEPARATOR);
+        moo_file_writer_write (writer, ">\n", -1);
 
-        for (child = elm->children; child != NULL; child = child->next)
+        for (child = elm->children; result && child != NULL; child = child->next)
             if (MOO_MARKUP_IS_ELEMENT (child))
-                format_pretty_element (MOO_MARKUP_ELEMENT (child), str,
-                                       indent + indent_size, indent_size);
+                result = write_pretty_element (MOO_MARKUP_ELEMENT (child), writer,
+                                               indent + indent_size, indent_size);
 
-        g_string_append_printf (str, "%s</%s>" LINE_SEPARATOR,
+        moo_file_writer_printf (writer, "%s</%s>\n",
                                 fill, elm->name);
     }
     else
     {
         char *escaped = g_markup_escape_text (elm->content, -1);
-        g_string_append_printf (str, ">%s</%s>" LINE_SEPARATOR,
+        moo_file_writer_printf (writer, ">%s</%s>\n",
                                 escaped, elm->name);
         g_free (escaped);
     }
 
     g_free (fill);
+    return result;
 }
 #undef INDENT_CHAR
-#undef LINE_SEPARATOR
 
 
 gboolean
-moo_markup_save_pretty (MooMarkupDoc       *doc,
-                        const char         *filename,
-                        guint               indent,
-                        GError            **error)
+moo_markup_write_pretty (MooMarkupDoc  *doc,
+                         MooFileWriter *writer,
+                         guint          indent)
 {
-    GString *str;
     MooMarkupNode *child;
-    gboolean result;
+    gboolean result = TRUE;
 
     g_return_val_if_fail (MOO_MARKUP_IS_DOC (doc), FALSE);
-    g_return_val_if_fail (filename != NULL, FALSE);
+    g_return_val_if_fail (writer != NULL, FALSE);
 
-    str = g_string_new ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
+    result = moo_file_writer_write (writer, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n", -1);
 
-    for (child = doc->children; child != NULL; child = child->next)
+    for (child = doc->children; result && child != NULL; child = child->next)
         if (MOO_MARKUP_IS_ELEMENT (child))
-            format_pretty_element (MOO_MARKUP_ELEMENT (child), str, 0, indent);
+            result = write_pretty_element (MOO_MARKUP_ELEMENT (child), writer, 0, indent);
 
-    result = _moo_save_file_utf8 (filename, str->str, -1, error);
-    g_string_free (str, TRUE);
     return result;
 }
 
-char *
-moo_markup_format_pretty (MooMarkupDoc *doc,
-                          guint         indent)
-{
-    GString *str;
-    MooMarkupNode *child;
-
-    g_return_val_if_fail (MOO_MARKUP_IS_DOC (doc), FALSE);
-
-    str = g_string_new ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
-
-    for (child = doc->children; child != NULL; child = child->next)
-        if (MOO_MARKUP_IS_ELEMENT (child))
-            format_pretty_element (MOO_MARKUP_ELEMENT (child), str, 0, indent);
-
-    return g_string_free (str, FALSE);
-}
-
 
 #if 0
 char *
@@ -1349,3 +1291,52 @@ MOO_MARKUP_COMMENT_CHECK_CAST (gpointer n)
     g_return_val_if_fail (node->type == MOO_MARKUP_COMMENT_NODE, NULL);
     return (MooMarkupComment*) n;
 }
+
+
+gboolean
+moo_parse_markup_file (const char         *filename,
+                       const GMarkupParser *parser,
+                       gpointer            data,
+                       GError            **error)
+{
+    GMarkupParseContext *ctx;
+    MooFileReader *reader;
+    char buf[1024];
+    gsize size;
+    gboolean seen_error = FALSE;
+
+    g_return_val_if_fail (filename != NULL, FALSE);
+    g_return_val_if_fail (parser != NULL, FALSE);
+    g_return_val_if_fail (!error || !*error, FALSE);
+
+    if (!(reader = moo_text_reader_new (filename, error)))
+        return FALSE;
+
+    ctx = g_markup_parse_context_new (parser, 0, data, NULL);
+
+    while (TRUE)
+    {
+        if (!moo_file_reader_read (reader, buf, sizeof buf, &size, error))
+        {
+            seen_error = TRUE;
+            break;
+        }
+
+        if (!size)
+            break;
+
+        if (!g_markup_parse_context_parse (ctx, buf, size, error))
+        {
+            seen_error = TRUE;
+            break;
+        }
+    }
+
+    if (!seen_error && !g_markup_parse_context_end_parse (ctx, error))
+        seen_error = TRUE;
+
+    g_markup_parse_context_free (ctx);
+    moo_file_reader_close (reader);
+
+    return !seen_error;
+}
