@@ -1,25 +1,33 @@
 /*
  *   moomarkup.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_MARKUP_H
 #define MOO_MARKUP_H
 
-#include <glib/gerror.h>
-#include <glib-object.h>
+#include <mooutils/mooutils-file.h>
 
 G_BEGIN_DECLS
 
 #define MOO_TYPE_MARKUP_DOC (moo_markup_doc_get_type())
 
+#define MOO_PARSE_ERROR (moo_parse_error_quark ())
+
+enum {
+    MOO_PARSE_ERROR_INVALID_CONTENT
+};
+
 typedef enum {
     MOO_MARKUP_DOC_NODE,
     MOO_MARKUP_ELEMENT_NODE,
@@ -34,6 +42,26 @@ typedef struct _MooMarkupElement MooMarkupElement;
 typedef struct _MooMarkupText MooMarkupText;
 typedef struct _MooMarkupText MooMarkupComment;
 
+typedef void (*MooMarkupStartElementFunc)   (GMarkupParseContext    *context,
+                                             const gchar            *element_name,
+                                             const gchar           **attribute_names,
+                                             const gchar           **attribute_values,
+                                             gpointer                user_data,
+                                             GError                **error);
+typedef void (*MooMarkupEndElementFunc)     (GMarkupParseContext    *context,
+                                             const gchar            *element_name,
+                                             gpointer                user_data,
+                                             GError                **error);
+typedef void (*MooMarkupTextFunc)           (GMarkupParseContext    *context,
+                                             const gchar            *text,
+                                             gsize                   text_len,
+                                             gpointer                user_data,
+                                             GError                **error);
+typedef void (*MooMarkupPassthroughFunc)    (GMarkupParseContext    *context,
+                                             const gchar            *passthrough_text,
+                                             gsize                   text_len,
+                                             gpointer                user_data,
+                                             GError                **error);
 
 #if 0 && defined(ENABLE_DEBUG)
 #define MOO_MARKUP_NODE(n)      (MOO_MARKUP_NODE_CHECK_CAST(n))
@@ -115,13 +143,14 @@ struct _MooMarkupText {
 };
 
 
-GType               moo_markup_doc_get_type         (void);
+GType               moo_markup_doc_get_type         (void) G_GNUC_CONST;
+GQuark              moo_parse_error_quark           (void) G_GNUC_CONST;
 
-MooMarkupNode      *MOO_MARKUP_NODE_CHECK_CAST     (gpointer node);
-MooMarkupDoc       *MOO_MARKUP_DOC_CHECK_CAST      (gpointer node);
-MooMarkupElement   *MOO_MARKUP_ELEMENT_CHECK_CAST  (gpointer node);
-MooMarkupText      *MOO_MARKUP_TEXT_CHECK_CAST     (gpointer node);
-MooMarkupComment   *MOO_MARKUP_COMMENT_CHECK_CAST  (gpointer node);
+MooMarkupNode      *MOO_MARKUP_NODE_CHECK_CAST      (gpointer node);
+MooMarkupDoc       *MOO_MARKUP_DOC_CHECK_CAST       (gpointer node);
+MooMarkupElement   *MOO_MARKUP_ELEMENT_CHECK_CAST   (gpointer node);
+MooMarkupText      *MOO_MARKUP_TEXT_CHECK_CAST      (gpointer node);
+MooMarkupComment   *MOO_MARKUP_COMMENT_CHECK_CAST   (gpointer node);
 
 MooMarkupDoc       *moo_markup_doc_new              (const char         *name);
 
@@ -130,11 +159,8 @@ MooMarkupDoc       *moo_markup_parse_file           (const char         *filenam
 MooMarkupDoc       *moo_markup_parse_memory         (const char         *buffer,
                                                      int                 size,
                                                      GError            **error);
-gboolean            moo_markup_save_pretty          (MooMarkupDoc       *doc,
-                                                     const char         *filename,
-                                                     guint               indent,
-                                                     GError            **error);
-char               *moo_markup_format_pretty        (MooMarkupDoc       *doc,
+gboolean            moo_markup_write_pretty         (MooMarkupDoc       *doc,
+                                                     MooFileWriter      *writer,
                                                      guint               indent);
 char               *moo_markup_node_get_string      (MooMarkupNode      *node);
 
@@ -153,13 +179,13 @@ void                moo_markup_set_prop             (MooMarkupNode      *node,
                                                      const char         *prop_name,
                                                      const char         *val);
 
-int                 moo_markup_get_int_prop         (MooMarkupNode      *node,
+int                 moo_markup_int_prop             (MooMarkupNode      *node,
                                                      const char         *prop_name,
                                                      int                 default_val);
-void                moo_markup_set_int_prop         (MooMarkupNode      *node,
+guint               moo_markup_uint_prop            (MooMarkupNode      *node,
                                                      const char         *prop_name,
-                                                     int                 val);
-gboolean            moo_markup_get_bool_prop        (MooMarkupNode      *node,
+                                                     guint               default_val);
+gboolean            moo_markup_bool_prop            (MooMarkupNode      *node,
                                                      const char         *prop_name,
                                                      gboolean            default_val);
 void                moo_markup_set_bool_prop        (MooMarkupNode      *node,
@@ -184,6 +210,11 @@ MooMarkupNode      *moo_markup_create_text_element  (MooMarkupNode      *parent,
                                                      const char         *path,
                                                      const char         *content);
 
+gboolean            moo_parse_markup_file           (const char         *filename,
+                                                     const GMarkupParser *parser,
+                                                     gpointer            data,
+                                                     GError            **error);
+
 void                _moo_markup_set_modified        (MooMarkupDoc       *doc,
                                                      gboolean            modified);
 void                _moo_markup_set_track_modified  (MooMarkupDoc       *doc,
