@@ -3,18 +3,21 @@
  *
  *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include "moomarshals.h"
+#include "marshals.h"
 #include "moopaned.h"
 #include "stock-moo.h"
 
@@ -89,7 +92,7 @@ _moo_window_set_icon_from_stock (GtkWindow  *window,
 #endif
 
 
-struct _MooPane {
+struct MooPane {
     GtkObject base;
 
     char         *id;
@@ -100,6 +103,7 @@ struct _MooPane {
     MooPaneLabel *label;
     GtkWidget    *frame;
     GtkWidget    *handle;
+    GtkWidget    *small_handle;
     GtkWidget    *button;
     GtkWidget    *label_widget;
     GtkWidget    *icon_widget;
@@ -107,6 +111,11 @@ struct _MooPane {
     GtkWidget    *detach_button;
     GtkWidget    *close_button;
 
+    GtkWidget    *frame_label_embedded;
+    GtkWidget    *frame_label_window;
+    char         *frame_label_text;
+    gboolean      frame_label_markup;
+
     /* XXX weak pointer */
     gpointer      focus_child;
 
@@ -126,7 +135,7 @@ struct _MooPane {
     guint         params_changed_blocked : 1;
 };
 
-struct _MooPaneClass {
+struct MooPaneClass {
     GtkObjectClass base_class;
     gboolean (*remove) (MooPane *pane);
 };
@@ -382,6 +391,7 @@ moo_pane_init (MooPane *pane)
     pane->child_holder = NULL;
     pane->frame = NULL;
     pane->handle = NULL;
+    pane->small_handle = NULL;
     pane->button = NULL;
     pane->label_widget = NULL;
     pane->icon_widget = NULL;
@@ -416,7 +426,6 @@ moo_pane_dispose (GObject *object)
     {
         GtkWidget *tmp = pane->child;
         pane->child = NULL;
-        gtk_widget_destroy (tmp);
         g_object_unref (tmp);
     }
 
@@ -535,6 +544,51 @@ update_sticky_button (MooPane *pane)
 }
 
 
+void
+moo_pane_set_frame_markup (MooPane    *pane,
+                           const char *text)
+{
+    char *tmp;
+
+    g_return_if_fail (MOO_IS_PANE (pane));
+
+    if (!text)
+    {
+        moo_pane_set_frame_text (pane, NULL);
+        return;
+    }
+
+    tmp = pane->frame_label_text;
+    pane->frame_label_text = g_strdup (text);
+    pane->frame_label_markup = TRUE;
+    g_free (tmp);
+
+    if (pane->frame_label_embedded)
+        gtk_label_set_markup (GTK_LABEL (pane->frame_label_embedded), text);
+    if (pane->frame_label_window)
+        gtk_label_set_markup (GTK_LABEL (pane->frame_label_window), text);
+}
+
+void
+moo_pane_set_frame_text (MooPane    *pane,
+                         const char *text)
+{
+    char *tmp;
+
+    g_return_if_fail (MOO_IS_PANE (pane));
+
+    tmp = pane->frame_label_text;
+    pane->frame_label_text = g_strdup (text);
+    pane->frame_label_markup = FALSE;
+    g_free (tmp);
+
+    if (pane->frame_label_embedded)
+        gtk_label_set_text (GTK_LABEL (pane->frame_label_embedded), text);
+    if (pane->frame_label_window)
+        gtk_label_set_text (GTK_LABEL (pane->frame_label_window), text);
+}
+
+
 static GtkWidget *
 create_button (MooPane      *pane,
                GtkWidget    *toolbar,
@@ -570,6 +624,7 @@ create_frame_widget (MooPane        *pane,
                      gboolean        embedded)
 {
     GtkWidget *vbox, *toolbar, *separator, *handle, *table, *child_holder;
+    GtkWidget *handle_hbox, *frame_label;
 
     vbox = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (vbox);
@@ -581,6 +636,29 @@ create_frame_widget (MooPane        *pane,
     gtk_box_pack_start (GTK_BOX (toolbar), handle, TRUE, TRUE, 3);
     pane->handle = handle;
 
+    handle_hbox = gtk_hbox_new (FALSE, 0);
+    gtk_widget_show (handle_hbox);
+    gtk_container_add (GTK_CONTAINER (pane->handle), handle_hbox);
+
+    frame_label = gtk_label_new (NULL);
+    gtk_widget_show (frame_label);
+    gtk_box_pack_start (GTK_BOX (handle_hbox), frame_label, TRUE, TRUE, 0);
+    gtk_misc_set_alignment (GTK_MISC (frame_label), .0, .5);
+    gtk_misc_set_padding (GTK_MISC (frame_label), 6, 0);
+    gtk_label_set_ellipsize (GTK_LABEL (frame_label), PANGO_ELLIPSIZE_END);
+    if (pane->frame_label_markup)
+        gtk_label_set_markup (GTK_LABEL (frame_label), pane->frame_label_text);
+    else
+        gtk_label_set_text (GTK_LABEL (frame_label), pane->frame_label_text);
+    if (embedded)
+        pane->frame_label_embedded = frame_label;
+    else
+        pane->frame_label_window = frame_label;
+
+    pane->small_handle = gtk_event_box_new ();
+    gtk_widget_show (pane->small_handle);
+    gtk_box_pack_start (GTK_BOX (handle_hbox), pane->small_handle, TRUE, TRUE, 0);
+
     if (embedded)
     {
         GtkWidget *hide_button;
@@ -1043,11 +1121,14 @@ _moo_pane_get_button (MooPane *pane)
     return pane->button;
 }
 
-GtkWidget *
-_moo_pane_get_handle (MooPane *pane)
+void
+_moo_pane_get_handle (MooPane    *pane,
+                      GtkWidget **big,
+                      GtkWidget **small)
 {
-    g_return_val_if_fail (MOO_IS_PANE (pane), NULL);
-    return pane->handle;
+    g_return_if_fail (MOO_IS_PANE (pane));
+    *big = pane->handle;
+    *small = pane->small_handle;
 }
 
 GtkWidget *
@@ -1121,6 +1202,7 @@ _moo_pane_unparent (MooPane *pane)
         pane->child_holder = NULL;
         pane->frame = NULL;
         pane->handle = NULL;
+        pane->small_handle = NULL;
         pane->button = NULL;
         pane->label_widget = NULL;
         pane->icon_widget = NULL;
@@ -1247,6 +1329,8 @@ create_pane_window (MooPane *pane)
 #endif
 
     set_pane_window_icon_and_title (pane);
+    gtk_window_set_type_hint (GTK_WINDOW (pane->window),
+                              GDK_WINDOW_TYPE_HINT_UTILITY);
 
     switch (_moo_paned_get_position (pane->parent))
     {
@@ -1274,14 +1358,15 @@ create_pane_window (MooPane *pane)
     g_object_set_data (G_OBJECT (pane->window), "moo-pane", pane);
     g_object_set_data (G_OBJECT (pane->keep_on_top_button), "moo-pane", pane);
 
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pane->keep_on_top_button),
+                                  pane->params->keep_on_top);
     g_signal_connect (pane->keep_on_top_button, "toggled",
                       G_CALLBACK (keep_on_top_button_toggled), pane);
     g_signal_connect (pane->window, "configure-event",
                       G_CALLBACK (pane_window_configure), pane);
 }
 
-/* XXX gtk_widget_reparent() doesn't work here for some reasons */
-/* shouldn't it work now, as I fixed GTK_NO_WINDOW flag? */
+/* FIXME use gtk_widget_reparent(), it does work now */
 static void
 reparent (GtkWidget *widget,
           GtkWidget *old_container,
