@@ -3,15 +3,18 @@
  *
  *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/mooprefs.h"
-#include "mooutils/moomarshals.h"
+#include "marshals.h"
 #include "mooutils/moomarkup.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-gobject.h"
@@ -367,14 +370,6 @@ static void
 moo_prefs_set_modified (gboolean modified)
 {
     MooPrefs *prefs = instance ();
-
-#ifdef MOO_DEBUG_ENABLED
-    if (modified && !prefs->rc_modified)
-    {
-        g_message ("%s: prefs modified", G_STRLOC);
-    }
-#endif
-
     prefs->rc_modified = modified;
 }
 
@@ -1213,6 +1208,9 @@ sync_xml (MooPrefsType prefs_type)
         case MOO_PREFS_STATE:
             xml_ptr = &prefs->xml_state;
             break;
+        default:
+            g_return_if_reached ();
+            break;
     }
 
     if (!*xml_ptr)
@@ -1258,6 +1256,7 @@ save_file (const char    *file,
     MooMarkupNode *node;
     gboolean empty;
     MooPrefs *prefs = instance ();
+    MooFileWriter *writer;
 
     if (!check_modified (prefs_type))
         return TRUE;
@@ -1290,7 +1289,13 @@ save_file (const char    *file,
         return TRUE;
     }
 
-    return moo_markup_save_pretty (xml, file, 2, error);
+    if ((writer = moo_config_writer_new (file, FALSE, error)))
+    {
+        moo_markup_write_pretty (xml, writer, 2);
+        return moo_file_writer_close (writer, error);
+    }
+
+    return FALSE;
 }
 
 
@@ -1701,45 +1706,6 @@ moo_prefs_set_flags (const char     *key,
 }
 
 
-GType
-moo_prefs_match_type_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-    {
-        static const GFlagsValue values[] = {
-            { MOO_PREFS_MATCH_KEY, (char*)"MOO_PREFS_MATCH_KEY", (char*)"match-key" },
-            { MOO_PREFS_MATCH_PREFIX, (char*)"MOO_PREFS_MATCH_PREFIX", (char*)"match-prefix" },
-            { MOO_PREFS_MATCH_REGEX, (char*)"MOO_PREFS_MATCH_REGEX", (char*)"match-regex" },
-            { 0, NULL, NULL }
-        };
-        type = g_flags_register_static ("MooPrefsMatchType", values);
-    }
-
-    return type;
-}
-
-
-GType
-moo_prefs_type_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-    {
-        static const GEnumValue values[] = {
-            { MOO_PREFS_RC, (char*)"MOO_PREFS_RC", (char*)"rc" },
-            { MOO_PREFS_STATE, (char*)"MOO_PREFS_STATE", (char*)"state" },
-            { 0, NULL, NULL }
-        };
-        type = g_enum_register_static ("MooPrefsType", values);
-    }
-
-    return type;
-}
-
-
 #if 0
 static void
 add_key (const char  *key,
