@@ -1,13 +1,16 @@
 /*
  *   mooprefs.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_PREFS_H
@@ -28,10 +31,6 @@
 G_BEGIN_DECLS
 
 
-#define MOO_TYPE_PREFS_TYPE         (moo_prefs_type_get_type ())
-#define MOO_TYPE_PREFS_MATCH_TYPE   (moo_prefs_match_type_get_type ())
-
-
 typedef enum {
     MOO_PREFS_RC,
     MOO_PREFS_STATE
@@ -44,9 +43,6 @@ typedef enum {
 } MooPrefsMatchType;
 
 
-GType           moo_prefs_type_get_type (void) G_GNUC_CONST;
-GType           moo_prefs_match_type_get_type (void) G_GNUC_CONST;
-
 gboolean        moo_prefs_load          (char          **sys_files,
                                          const char     *file_rc,
                                          const char     *file_state,
