@@ -1,20 +1,24 @@
 /*
  *   mooprefsdialog.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
+#include "marshals.h"
 #include "mooutils/mooprefsdialog.h"
-#include "mooutils/moomarshals.h"
 #include "mooutils/moodialogs.h"
 #include "mooutils/moohelp.h"
+#include "mooutils/mooutils-treeview.h"
 #include "help-sections.h"
 
 
@@ -46,7 +50,7 @@ static void setup_pages_list                (MooPrefsDialog *dialog);
 static void pages_list_selection_changed    (MooPrefsDialog *dialog,
                                              GtkTreeSelection *selection);
 
-static void init_page                       (MooPrefsDialogPage *page);
+static void init_page                       (MooPrefsPage   *page);
 
 static gboolean moo_prefs_dialog_help       (GtkWidget      *widget);
 
@@ -121,23 +125,19 @@ moo_prefs_dialog_init (MooPrefsDialog *dialog)
     moo_help_set_func (GTK_WIDGET (dialog), moo_prefs_dialog_help);
     moo_help_connect_keys (GTK_WIDGET (dialog));
 
-    _moo_window_set_remember_size (GTK_WINDOW (dialog), "Dialogs/Preferences", FALSE);
+    _moo_window_set_remember_size (GTK_WINDOW (dialog), "Dialogs/Preferences", -1, -1, FALSE);
 
     gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
 
     gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-#ifdef MOO_ENABLE_HELP
                             GTK_STOCK_HELP, GTK_RESPONSE_HELP,
-#endif
                             GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                             GTK_STOCK_OK, GTK_RESPONSE_OK,
                             NULL);
 #if GTK_MINOR_VERSION >= 6
     gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-#ifdef MOO_ENABLE_HELP
                                              GTK_RESPONSE_HELP,
-#endif
                                              GTK_RESPONSE_OK,
                                              GTK_RESPONSE_CANCEL,
                                              GTK_RESPONSE_APPLY,
@@ -221,7 +221,7 @@ setup_pages_list (MooPrefsDialog *dialog)
                                         GDK_TYPE_PIXBUF,
                                         G_TYPE_STRING,
                                         G_TYPE_STRING,
-                                        MOO_TYPE_PREFS_DIALOG_PAGE);
+                                        MOO_TYPE_PREFS_PAGE);
     tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (dialog->store));
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree), FALSE);
     gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tree), FALSE);
@@ -262,7 +262,7 @@ pages_list_selection_changed (MooPrefsDialog *dialog,
 
     if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
-        MooPrefsDialogPage *page = NULL;
+        MooPrefsPage *page = NULL;
         gtk_tree_model_get (model, &iter, PAGE_COLUMN, &page, -1);
         g_return_if_fail (page != NULL);
 
@@ -285,13 +285,13 @@ pages_list_selection_changed (MooPrefsDialog *dialog,
 
 
 static gboolean
-page_initialized (MooPrefsDialogPage *page)
+page_initialized (MooPrefsPage *page)
 {
     return g_object_get_data (G_OBJECT (page), "moo-prefs-dialog-page-initialized") != NULL;
 }
 
 static void
-init_page (MooPrefsDialogPage *page)
+init_page (MooPrefsPage *page)
 {
     if (!g_object_get_data (G_OBJECT (page), "moo-prefs-dialog-page-initialized"))
     {
@@ -350,7 +350,7 @@ moo_prefs_dialog_get_property (GObject    *object,
  */
 
 GtkWidget*
-moo_prefs_dialog_new (const char         *title)
+moo_prefs_dialog_new (const char *title)
 {
     return GTK_WIDGET (g_object_new (MOO_TYPE_PREFS_DIALOG,
                                      "title", title, NULL));
@@ -364,6 +364,7 @@ moo_prefs_dialog_run (MooPrefsDialog *dialog,
     g_return_if_fail (MOO_IS_PREFS_DIALOG (dialog));
 
     moo_window_set_parent (GTK_WIDGET (dialog), parent);
+    _moo_tree_view_select_first (dialog->pages_list);
 
     dialog->running = TRUE;
     g_signal_emit_by_name (dialog, "init");
@@ -434,7 +435,7 @@ moo_prefs_dialog_apply (MooPrefsDialog *dialog)
 
     while (list)
     {
-        MooPrefsDialogPage *page = list->data;
+        MooPrefsPage *page = list->data;
 
         if (!(GTK_OBJECT_FLAGS (page) & GTK_IN_DESTRUCTION) &&
             page->auto_apply &&
@@ -467,7 +468,7 @@ moo_prefs_dialog_insert_page (MooPrefsDialog     *dialog,
     GtkTreePath *path = NULL;
 
     g_return_if_fail (MOO_IS_PREFS_DIALOG (dialog));
-    g_return_if_fail (MOO_IS_PREFS_DIALOG_PAGE (page));
+    g_return_if_fail (MOO_IS_PREFS_PAGE (page));
     g_return_if_fail (page->parent == NULL);
 
     if (position < 0)
@@ -521,7 +522,7 @@ moo_prefs_dialog_remove_page (MooPrefsDialog     *dialog,
     GtkTreeIter iter;
 
     g_return_if_fail (MOO_IS_PREFS_DIALOG (dialog));
-    g_return_if_fail (MOO_IS_PREFS_DIALOG_PAGE (page));
+    g_return_if_fail (MOO_IS_PREFS_PAGE (page));
 
     ref = g_object_get_data (G_OBJECT (page), "moo-prefs-dialog-row");
     g_return_if_fail (ref && gtk_tree_row_reference_valid (ref));
