@@ -1,19 +1,22 @@
 /*
  *   mooprefsdialog.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_PREFS_DIALOG_H
 #define MOO_PREFS_DIALOG_H
 
-#include <mooutils/mooprefsdialogpage.h>
+#include <mooutils/mooprefspage.h>
 
 G_BEGIN_DECLS
 
@@ -26,10 +29,10 @@ G_BEGIN_DECLS
 #define MOO_PREFS_DIALOG_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), MOO_TYPE_PREFS_DIALOG, MooPrefsDialogClass))
 
 
-typedef struct _MooPrefsDialog        MooPrefsDialog;
-typedef struct _MooPrefsDialogClass   MooPrefsDialogClass;
+typedef struct MooPrefsDialog      MooPrefsDialog;
+typedef struct MooPrefsDialogClass MooPrefsDialogClass;
 
-struct _MooPrefsDialog
+struct MooPrefsDialog
 {
     GtkDialog     dialog;
     GtkNotebook  *notebook;
@@ -39,7 +42,7 @@ struct _MooPrefsDialog
     gboolean      running;
 };
 
-struct _MooPrefsDialogClass
+struct MooPrefsDialogClass
 {
     GtkDialogClass parent_class;
 
