@@ -1,17 +1,20 @@
 /*
  *   moospawn.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/moospawn.h"
-#include "mooutils/moomarshals.h"
+#include "marshals.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/moocompat.h"
 #include <string.h>
@@ -22,7 +25,6 @@
 #include <unistd.h>
 #include <signal.h>
 #else
-#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #endif
 
@@ -238,6 +240,7 @@ command_exit (GPid            pid,
         g_return_if_fail (cmd->priv->child_watch_data == data);
         cmd->priv->exit_status = status;
         cmd->priv->child_watch_data = NULL;
+        cmd->priv->pid = 0;
         moo_cmd_check_stop (cmd);
     }
 }
@@ -257,55 +260,52 @@ child_watch_removed (ChildWatchData *data)
 
 static void
 process_line (MooCmd      *cmd,
+              gboolean     err,
               const char  *line,
-              gboolean     err)
+              gssize       len)
 {
     gboolean dummy = FALSE;
-    const char *real_line = line;
-    char *freeme = NULL;
+    char *real_line = NULL;
+    const char *end;
 
-    if (cmd->priv->cmd_flags & MOO_CMD_UTF8_OUTPUT)
+    g_return_if_fail (line != NULL);
+
+    if (len < 0)
+        len = strlen (line);
+
+    if ((cmd->priv->cmd_flags & MOO_CMD_UTF8_OUTPUT) &&
+        !g_utf8_validate (line, len, &end))
     {
         const char *charset;
 
-        real_line = NULL;
-
         if (g_get_charset (&charset))
         {
-            const char *end;
+            g_warning ("%s: invalid unicode:\n%s", G_STRLOC, line);
 
-            if (!g_utf8_validate (line, -1, &end))
-            {
-                g_warning ("%s: invalid unicode:\n%s", G_STRLOC, line);
-
-                if (end > line)
-                {
-                    freeme = g_strndup (line, end - line);
-                    real_line = freeme;
-                }
-            }
-            else
-            {
-                real_line = line;
-            }
+            if (end > line)
+                real_line = g_strndup (line, end - line);
         }
         else
         {
             GError *error = NULL;
+            gsize bytes_written;
 
-            freeme = g_convert_with_fallback (line, -1, "UTF-8", charset,
-                                              NULL, NULL, NULL, &error);
+            real_line = g_convert_with_fallback (line, len, "UTF-8", charset,
+                                                 NULL, NULL, &bytes_written,
+                                                 &error);
 
-            if (!freeme)
+            if (!real_line)
             {
-                g_warning ("%s: could not convert text to UTF-8:\n%s",
-                           G_STRLOC, line);
+                g_warning ("%s: could not convert text to UTF-8:\n%.*s",
+                           G_STRLOC, (int) len, line);
                 g_warning ("%s: %s", G_STRLOC, error->message);
             }
-
-            real_line = freeme;
         }
     }
+    else
+    {
+        real_line = g_strndup (line, len);
+    }
 
     if (real_line)
     {
@@ -315,7 +315,7 @@ process_line (MooCmd      *cmd,
             g_signal_emit (cmd, signals[STDOUT_LINE], 0, real_line, &dummy);
     }
 
-    g_free (freeme);
+    g_free (real_line);
 }
 
 
@@ -327,7 +327,7 @@ command_out_or_err (MooCmd         *cmd,
 {
     GSList *lines;
     GError *error = NULL;
-    GIOStatus status;
+    GIOStatus status = G_IO_STATUS_NORMAL;
     gsize count;
 
     count = 0;
@@ -352,7 +352,7 @@ command_out_or_err (MooCmd         *cmd,
 
     while (lines)
     {
-        process_line (cmd, lines->data, !out);
+        process_line (cmd, !out, lines->data, -1);
         g_free (lines->data);
         lines = g_slist_delete_link (lines, lines);
     }
@@ -403,18 +403,16 @@ try_channel_leftover (MooCmd      *cmd,
 
     if (text)
     {
-        char **lines, **p;
+        MooLineReader lr;
+        const char *line;
+        gsize line_len;
 
-        lines = moo_splitlines (text);
-
-        if (lines)
+        for (moo_line_reader_init (&lr, text, -1);
+             (line = moo_line_reader_get_line (&lr, &line_len, NULL)); )
         {
-            for (p = lines; *p != NULL; p++)
-                if (**p)
-                    process_line (cmd, *p, !out);
+            process_line (cmd, !out, line, line_len);
         }
 
-        g_strfreev (lines);
         g_free (text);
     }
 }
@@ -668,8 +666,6 @@ moo_cmd_cleanup (MooCmd *cmd)
 #else
         TerminateProcess (cmd->priv->pid, 1);
 #endif
-        g_spawn_close_pid (cmd->priv->pid);
-        cmd->priv->pid = 0;
     }
 
     cmd->priv->pid = 0;
@@ -691,6 +687,8 @@ moo_cmd_abort_real (MooCmd *cmd)
 
     g_return_val_if_fail (cmd->priv->pid != 0, TRUE);
 
+    /* TODO: on windows it gets exit status of 0 for some reason */
+
 #ifndef __WIN32__
     kill (-cmd->priv->pid, SIGHUP);
 #else
@@ -699,12 +697,18 @@ moo_cmd_abort_real (MooCmd *cmd)
 
     if (cmd->priv->stdout_watch)
     {
+        /* close it to avoid final read after watch is removed */
+        g_io_channel_shutdown (cmd->priv->stdout_io, FALSE, NULL);
+        cmd->priv->stdout_io = NULL;
         g_source_remove (cmd->priv->stdout_watch);
         cmd->priv->stdout_watch = 0;
     }
 
     if (cmd->priv->stderr_watch > 0)
     {
+        /* close it to avoid final read after watch is removed */
+        g_io_channel_shutdown (cmd->priv->stderr_io, FALSE, NULL);
+        cmd->priv->stderr_io = NULL;
         g_source_remove (cmd->priv->stderr_watch);
         cmd->priv->stderr_watch = 0;
     }
