@@ -1,13 +1,16 @@
 /*
  *   moostock.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/moostock.h"
@@ -15,7 +18,7 @@
 #include "mooutils/stock-terminal-24.h"
 #include "mooutils/stock-file-selector-24.h"
 #include "mooutils/stock-file-24.h"
-#include "mooutils/stock-medit.h"
+#include "stock-medit.h"
 #include "mooutils/mooi18n.h"
 #include <gtk/gtk.h>
 #include <string.h>
@@ -220,7 +223,8 @@ _moo_stock_init (void)
 
     add_icon (factory, MOO_STOCK_TERMINAL, "terminal", 24, MOO_GNOME_TERMINAL_ICON);
 
-    add_icon (factory, MOO_STOCK_MEDIT, "medit", 24, MEDIT_ICON);
+    add_default_image (24, MEDIT_ICON, "medit", NULL);
+    add_default_image (48, MEDIT_ICON, "medit", NULL);
 
 #if !GTK_CHECK_VERSION(2,10,0)
     add_icon2 (factory, GTK_STOCK_SELECT_ALL, "edit-select-all",
