@@ -1,13 +1,16 @@
 /*
  *   moostock.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOOUTILS_STOCK_H
@@ -19,7 +22,6 @@
 G_BEGIN_DECLS
 
 
-#define MOO_STOCK_MEDIT                 "moo-medit"
 #define MOO_STOCK_TERMINAL              "moo-terminal"
 #define MOO_STOCK_KEYBOARD              GTK_STOCK_SELECT_FONT
 #define MOO_STOCK_MENU                  GTK_STOCK_INDEX
