@@ -1,17 +1,20 @@
 /*
  *   mooundo.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/mooundo.h"
-#include "mooutils/moomarshals.h"
+#include "marshals.h"
 #include <string.h>
 
 
@@ -232,19 +235,12 @@ moo_undo_stack_get_property (GObject        *object,
 guint
 moo_undo_action_register (MooUndoActionClass *klass)
 {
-    MooUndoActionClass *tmp;
-
     g_return_val_if_fail (klass != NULL, 0);
     g_return_val_if_fail (klass->undo != NULL && klass->redo != NULL, 0);
     g_return_val_if_fail (klass->merge != NULL && klass->destroy != NULL, 0);
 
-    tmp = g_new (MooUndoActionClass, last_type + 2);
-    if (last_type)
-        memcpy (tmp, types, sizeof(MooUndoActionClass) * (last_type + 1));
-    g_free (types);
-    types = tmp;
-
-    memcpy (&types[last_type+1], klass, sizeof(MooUndoActionClass));
+    types = g_renew (MooUndoActionClass, types, last_type + 2);
+    types[last_type+1] = *klass;
 
     return ++last_type;
 }
