@@ -3,11 +3,14 @@
  *
  *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -15,22 +18,32 @@
 #endif
 
 #include "mooutils/mooutils-fs.h"
+#include "mooutils/moostat.h"
 #include "mooutils/mooutils-debug.h"
+#include "mooutils/mooutils-mem.h"
+#include "mooutils/mootype-macros.h"
 #include "mooutils/moocompat.h"
+#include "mooutils/mooi18n.h"
 
-#include <string.h>
 #include <stdio.h>
+#include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <stdlib.h>
 #include <errno.h>
 #include <glib/gstdio.h>
+/* for glib-2.6 */
+#include <glib/gmappedfile.h>
 
 #ifdef __WIN32__
-#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <io.h>
 #include <direct.h>
+#include <shellapi.h>
+#include <gdk/gdkwin32.h>
+#endif
+
+#ifndef S_IRWXU
+#define S_IRWXU 0
 #endif
 
 #ifdef HAVE_SYS_WAIT_H
@@ -42,9 +55,7 @@
 
 #define BROKEN_NAME "<" "????" ">"
 
-
-static int  _moo_rmdir  (const char *path);
-
+MOO_DEFINE_QUARK (moo-file-error-quark, _moo_file_error_quark)
 
 /* XXX fix this */
 gboolean
@@ -86,6 +97,111 @@ _moo_save_file_utf8 (const char *name,
 }
 
 
+#ifdef __WIN32__
+
+static wchar_t *
+filename_to_double_null_terminated_string (const char *filename)
+{
+    long len;
+    gunichar2 *ret;
+
+    ret = g_utf8_to_utf16 (filename, -1, NULL, &len, NULL);
+
+    if (!ret)
+        return NULL;
+
+    ret = g_renew (gunichar2, ret, len + 2);
+    ret[len + 1] = 0;
+
+    return (wchar_t*) ret;
+}
+
+static wchar_t *
+filename_list_to_double_null_terminated_string (GList *filenames)
+{
+    GArray *ret;
+    gunichar2 wzero = 0;
+
+    ret = g_array_new (FALSE, FALSE, sizeof (gunichar2));
+
+    while (filenames)
+    {
+        guint old_len;
+        long len;
+        gunichar2 *wstr;
+
+        wstr = g_utf8_to_utf16 (filenames->data, -1, NULL, &len, NULL);
+        filenames = filenames->next;
+
+        if (!wstr)
+        {
+            g_array_free (ret, TRUE);
+            return NULL;
+        }
+
+        old_len = ret->len;
+        g_array_set_size (ret, old_len + len + 1);
+        memcpy (ret->data + old_len * sizeof (gunichar2), wstr, (len + 1) * sizeof (gunichar2));
+
+        g_free (wstr);
+    }
+
+    g_array_append_val (ret, wzero);
+    return (wchar_t*) g_array_free (ret, FALSE);
+}
+
+static gboolean
+move_or_copy_files_ui (GList      *filenames,
+                       const char *destdir,
+                       gboolean    copy,
+                       GtkWidget  *parent)
+{
+    SHFILEOPSTRUCT shop = {0};
+    gboolean ret;
+    wchar_t *from, *to;
+
+    from = filename_list_to_double_null_terminated_string (filenames);
+    to = filename_to_double_null_terminated_string (destdir);
+
+    if (!from || !to)
+    {
+        g_free (from);
+        g_free (to);
+        return FALSE;
+    }
+
+    shop.hwnd = parent && parent->window ? GDK_WINDOW_HWND (parent->window) : NULL;
+    shop.wFunc = copy ? FO_COPY : FO_MOVE;
+    shop.pFrom = from;
+    shop.pTo = to;
+    shop.fFlags = FOF_ALLOWUNDO;
+
+    ret = SHFileOperation(&shop) == 0 && !shop.fAnyOperationsAborted;
+
+    g_free (from);
+    g_free (to);
+    return ret;
+}
+
+gboolean
+_moo_copy_files_ui (GList      *filenames,
+                    const char *destdir,
+                    GtkWidget  *parent)
+{
+    return move_or_copy_files_ui (filenames, destdir, TRUE, parent);
+}
+
+gboolean
+_moo_move_files_ui (GList      *filenames,
+                    const char *destdir,
+                    GtkWidget  *parent)
+{
+    return move_or_copy_files_ui (filenames, destdir, FALSE, parent);
+}
+
+#endif /* __WIN32__ */
+
+
 #ifndef __WIN32__
 static gboolean
 rm_fr (const char *path,
@@ -106,7 +222,9 @@ rm_fr (const char *path,
                        NULL, NULL, NULL, &child_err, &status, &error_here))
     {
         g_set_error (error, MOO_FILE_ERROR, MOO_FILE_ERROR_FAILED,
-                     "Could not run 'rm' command: %s",
+                     /* This is error message in file selector when rm
+                        fails for some strange reason */
+                     _("Could not run 'rm' command: %s"),
                      error_here->message);
         g_error_free (error_here);
         g_strfreev (argv);
@@ -117,10 +235,16 @@ rm_fr (const char *path,
 
     if (!WIFEXITED (status) || WEXITSTATUS (status))
     {
+        if (child_err && strlen (child_err) > 5000)
+            strcpy (child_err + 4997, "...");
+
         g_set_error (error, MOO_FILE_ERROR,
                      MOO_FILE_ERROR_FAILED,
-                     "'rm' command failed: %s",
+                     /* This is error message in file selector when rm
+                        fails to delete a file or folder */
+                     _("'rm' command failed: %s"),
                      child_err ? child_err : "");
+
         g_free (child_err);
         return FALSE;
     }
@@ -153,26 +277,23 @@ rm_r (const char *path,
     {
         char *file_path = g_build_filename (path, file, NULL);
 
-        errno = 0;
-
-        if (_moo_remove (file_path))
+        if (g_file_test (file_path, G_FILE_TEST_IS_DIR))
         {
-            int err = errno;
+            if (!rm_r (file_path, error))
+                success = FALSE;
+        }
+        else
+        {
+            errno = 0;
 
-            switch (err)
+            if (_moo_remove (file_path) != 0)
             {
-                case ENOTEMPTY:
-                case EEXIST:
-                    if (!rm_r (file_path, error))
-                        success = FALSE;
-                    break;
-
-                default:
-                    success = FALSE;
-                    g_set_error (error, MOO_FILE_ERROR,
-                                 _moo_file_error_from_errno (err),
-                                 "Could not remove '%s': %s", file_path,
-                                 g_strerror (err));
+                int err = errno;
+                success = FALSE;
+                g_set_error (error, MOO_FILE_ERROR,
+                             _moo_file_error_from_errno (err),
+                             _("Could not remove %s: %s"), file_path,
+                             g_strerror (err));
             }
         }
 
@@ -191,7 +312,7 @@ rm_r (const char *path,
             success = FALSE;
             g_set_error (error, MOO_FILE_ERROR,
                          _moo_file_error_from_errno (err),
-                         "Could not remove '%s': %s", path,
+                         _("Could not remove %s: %s"), path,
                          g_strerror (err));
         }
     }
@@ -212,13 +333,13 @@ _moo_remove_dir (const char *path,
     {
         errno = 0;
 
-        if (_moo_rmdir (path) != 0)
+        if (_moo_remove (path) != 0)
         {
             int err = errno;
             char *path_utf8 = g_filename_display_name (path);
             g_set_error (error, MOO_FILE_ERROR,
                          _moo_file_error_from_errno (err),
-                         "Could not remove '%s': %s",
+                         _("Could not remove %s: %s"),
                          path_utf8, g_strerror (err));
             g_free (path_utf8);
             return FALSE;
@@ -255,7 +376,7 @@ _moo_create_dir (const char *path,
 
     errno = 0;
 
-    if (g_stat (path, &buf) != 0 && errno != ENOENT)
+    if (moo_stat (path, &buf) != 0 && errno != ENOENT)
     {
         int err_code = errno;
         utf8_path = g_filename_display_name (path);
@@ -263,7 +384,7 @@ _moo_create_dir (const char *path,
         g_set_error (error,
                      MOO_FILE_ERROR,
                      _moo_file_error_from_errno (err_code),
-                     "Could not create directory '%s': %s",
+                     _("Could not create folder %s: %s"),
                      utf8_path, g_strerror (err_code));
 
         g_free (utf8_path);
@@ -282,7 +403,7 @@ _moo_create_dir (const char *path,
             g_set_error (error,
                          MOO_FILE_ERROR,
                          _moo_file_error_from_errno (err_code),
-                         "Could not create directory '%s': %s",
+                         _("Could not create folder %s: %s"),
                          utf8_path, g_strerror (err_code));
 
             g_free (utf8_path);
@@ -298,7 +419,7 @@ _moo_create_dir (const char *path,
     utf8_path = g_filename_display_name (path);
     g_set_error (error, MOO_FILE_ERROR,
                  MOO_FILE_ERROR_ALREADY_EXISTS,
-                 "Could not create directory '%s': %s",
+                 _("Could not create folder %s: %s"),
                  utf8_path, g_strerror (EEXIST));
     g_free (utf8_path);
 
@@ -311,6 +432,8 @@ _moo_rename_file (const char *path,
                   const char *new_path,
                   GError    **error)
 {
+    // Do not break this for directories!
+
     g_return_val_if_fail (path != NULL, FALSE);
     g_return_val_if_fail (new_path != NULL, FALSE);
 
@@ -325,7 +448,7 @@ _moo_rename_file (const char *path,
         g_set_error (error,
                      MOO_FILE_ERROR,
                      _moo_file_error_from_errno (err_code),
-                     "Could not rename file '%s' to '%s': %s",
+                     _("Could not rename file %s to %s: %s"),
                      utf8_path, utf8_new_path, g_strerror (err_code));
 
         g_free (utf8_path);
@@ -337,18 +460,6 @@ _moo_rename_file (const char *path,
 }
 
 
-GQuark
-_moo_file_error_quark (void)
-{
-    static GQuark quark = 0;
-
-    if (quark == 0)
-        quark = g_quark_from_static_string ("moo-file-error-quark");
-
-    return quark;
-}
-
-
 MooFileError
 _moo_file_error_from_errno (int code)
 {
@@ -413,6 +524,28 @@ moo_filename_from_locale (const char *file)
 #endif
 }
 
+char *
+_moo_filename_to_uri (const char *file,
+                      GError    **error)
+{
+    char *uri;
+    char *freeme = NULL;
+
+    g_return_val_if_fail (file != NULL, NULL);
+
+    if (!_moo_path_is_absolute (file))
+    {
+        char *cd = g_get_current_dir ();
+        file = freeme = g_build_filename (cd, file, NULL);
+        g_free (cd);
+    }
+
+    uri = g_filename_to_uri (file, NULL, error);
+
+    g_free (freeme);
+    return uri;
+}
+
 
 #if 0
 static int
@@ -437,7 +570,7 @@ _moo_chdir (const char *path)
 //     working_dir = g_get_current_dir ();
 //     g_return_val_if_fail (working_dir != NULL, g_strdup (filename));
 //
-//     if (!g_path_is_absolute (filename))
+//     if (!_moo_path_is_absolute (filename))
 //     {
 //         freeme = g_build_filename (working_dir, filename, NULL);
 //         filename = freeme;
@@ -494,6 +627,8 @@ _moo_chdir (const char *path)
 #endif
 
 
+#ifndef __WIN32__
+
 static char *
 normalize_path_string (const char *path)
 {
@@ -560,10 +695,9 @@ normalize_path_string (const char *path)
         if (first_slash)
         {
             guint len = strlen (tmp);
-            normpath = g_new (char, len + 2);
-            memcpy (normpath + 1, tmp, len + 1);
+            normpath = g_renew (char, tmp, len + 2);
+            memmove (normpath + 1, normpath, len + 1);
             normpath[0] = G_DIR_SEPARATOR;
-            g_free (tmp);
         }
         else
         {
@@ -578,158 +712,162 @@ normalize_path_string (const char *path)
     return normpath;
 }
 
-
-#ifndef __WIN32__
 static char *
-normalize_full_path (const char *path,
-                     gboolean    is_folder)
+normalize_full_path_unix (const char *path)
 {
     guint len;
-    char *normpath, *tmp;
+    char *normpath;
 
     g_return_val_if_fail (path != NULL, NULL);
 
-    tmp = normalize_path_string (path);
+    normpath = normalize_path_string (path);
+    g_return_val_if_fail (normpath != NULL, NULL);
 
-    if (!is_folder)
-        return tmp;
+    len = strlen (normpath);
+    g_return_val_if_fail (len > 0, normpath);
 
-    len = strlen (tmp);
-    g_return_val_if_fail (len > 0, tmp);
-
-    if (tmp[len-1] != G_DIR_SEPARATOR)
-    {
-        normpath = g_new (char, len + 2);
-        memcpy (normpath, tmp, len);
-        normpath[len] = G_DIR_SEPARATOR;
-        normpath[len+1] = 0;
-        g_free (tmp);
-    }
-    else
-    {
-        g_assert (1 || len == 1);
-        normpath = tmp;
-    }
+    if (len > 1 && normpath[len-1] == G_DIR_SEPARATOR)
+        normpath[len-1] = 0;
 
     return normpath;
 }
-#else
-static void
-splitdrive (const char *fullpath,
-            char      **drive,
-            char      **path)
+
+#endif /* !__WIN32__ */
+
+#if defined(__WIN32__) || defined(MOO_ENABLE_UNIT_TESTS)
+
+static char *
+normalize_full_path_win32 (const char *fullpath)
 {
+    char *prefix;
+    char *path;
+    guint slashes;
+
+    g_return_val_if_fail (fullpath && fullpath[0], g_strdup (fullpath));
+
     if (fullpath[0] && fullpath[1] == ':')
     {
-        *drive = g_strndup (fullpath, 2);
-        *path = g_strdup (fullpath + 2);
+        prefix = g_strndup (fullpath, 3);
+        prefix[2] = '\\';
+
+        if (fullpath[2] == '\\')
+            path = g_strdup (fullpath + 3);
+        else
+            path = g_strdup (fullpath + 2);
     }
     else
     {
-        *drive = NULL;
-        *path = g_strdup (fullpath);
+        prefix = NULL;
+        path = g_strdup (fullpath);
     }
-}
 
-static char *
-normalize_full_path (const char *fullpath,
-                     gboolean    is_folder)
-{
-    char *drive, *path, *normpath;
-    guint slashes;
-
-    g_return_val_if_fail (fullpath != NULL, NULL);
-
-    splitdrive (fullpath, &drive, &path);
     g_strdelimit (path, "/", '\\');
 
     for (slashes = 0; path[slashes] == '\\'; ++slashes) ;
 
-    if (drive && path[0] != '\\')
+    if (!prefix && slashes)
+        prefix = g_strndup (path, slashes);
+
+    if (slashes)
+        memmove (path, path + slashes, strlen (path) + 1 - slashes);
+
+    g_assert (prefix || *path);
+    g_assert (*path != '\\');
+
+    if (*path)
     {
-        char *tmp = path;
-        path = g_strdup_printf ("\\%s", path);
-        g_free (tmp);
-    }
+        char **comps;
+        guint n_comps, i;
 
-    if (!drive)
-    {
-        char *tmp = path;
-        drive = g_strndup (path, slashes);
-        path = g_strdup (tmp + slashes);
-        g_free (tmp);
-    }
-#if 0
-//     else if (path[0] == '\\')
-//     {
-//         char *tmp;
-//
-//         tmp = drive;
-//         drive = g_strdup_printf ("%s\\", drive);
-//         g_free (tmp);
-//
-//         tmp = path;
-//         path = g_strdup (path + slashes);
-//         g_free (tmp);
-//     }
-#endif
+        comps = g_strsplit (path, "\\", 0);
+        n_comps = g_strv_length (comps);
 
-    normpath = normalize_path_string (path);
-
-    if (!normpath[0] && !drive)
-    {
-        char *tmp = normpath;
-        normpath = g_strdup (".");
-        g_free (tmp);
-    }
-    else if (drive)
-    {
-        char *tmp = normpath;
-
-        if (normpath[0] == '\\')
-            normpath = g_strdup_printf ("%s%s", drive, normpath);
-        else
-            normpath = g_strdup_printf ("%s\\%s", drive, normpath);
-
-        g_free (tmp);
-    }
-
-    if (is_folder)
-    {
-        guint len = strlen (normpath);
-
-        if (!len || normpath[len -1] != '\\')
+        for (i = 0; i < n_comps; )
         {
-            char *tmp = normpath;
-            normpath = g_strdup_printf ("%s\\", normpath);
-            g_free (tmp);
+            if (strcmp (comps[i], "") == 0 || strcmp (comps[i], ".") == 0 ||
+                (i == 0 && strcmp (comps[i], "..") == 0))
+            {
+                g_free (comps[i]);
+                MOO_ELMMOVE (comps + i, comps + i + 1, n_comps - i);
+                n_comps -= 1;
+            }
+            else if (strcmp (comps[i], "..") == 0)
+            {
+                g_free (comps[i-1]);
+                g_free (comps[i]);
+                MOO_ELMMOVE (comps + i - 1, comps + i + 1, n_comps - i);
+                n_comps -= 2;
+                i -= 1;
+            }
+            else
+            {
+                i += 1;
+            }
+        }
+
+        if (n_comps > 0)
+        {
+            char *tmp = g_strjoinv ("\\", comps);
+            g_free (path);
+            path = tmp;
+        }
+        else
+        {
+            path[0] = 0;
+        }
+
+        g_strfreev (comps);
+    }
+
+    if (*path)
+    {
+        if (prefix)
+        {
+            char *tmp = g_strconcat (prefix, path, NULL);
+            g_free (path);
+            path = tmp;
         }
     }
+    else if (prefix)
+    {
+        g_free (path);
+        path = prefix;
+        prefix = NULL;
+    }
+    else
+    {
+        g_free (path);
+        path = g_strdup (".");
+    }
 
-    g_free (drive);
-    g_free (path);
-    return normpath;
+    g_free (prefix);
+    return path;
 }
+
 #endif
 
 static char *
-normalize_path (const char *filename,
-                gboolean    is_folder)
+normalize_path (const char *filename)
 {
     char *freeme = NULL;
     char *norm_filename;
 
-    g_return_val_if_fail (filename != NULL, NULL);
+    g_assert (filename && filename[0]);
 
-    if (!g_path_is_absolute (filename))
+    if (!_moo_path_is_absolute (filename))
     {
         char *working_dir = g_get_current_dir ();
         g_return_val_if_fail (working_dir != NULL, g_strdup (filename));
         freeme = g_build_filename (working_dir, filename, NULL);
         filename = freeme;
+        g_free (working_dir);
     }
 
-    norm_filename = normalize_full_path (filename, is_folder);
+#ifdef __WIN32__
+    norm_filename = normalize_full_path_win32 (filename);
+#else
+    norm_filename = normalize_full_path_unix (filename);
+#endif
 
     g_free (freeme);
     return norm_filename;
@@ -738,16 +876,277 @@ normalize_path (const char *filename,
 char *
 _moo_normalize_file_path (const char *filename)
 {
-    return normalize_path (filename, FALSE);
+    g_return_val_if_fail (filename != NULL, NULL);
+    /* empty filename is an error, but we don't want to crash here */
+    g_return_val_if_fail (filename[0] != 0, g_strdup (""));
+    return normalize_path (filename);
 }
 
-char *
-_moo_normalize_dir_path (const char *filename)
+gboolean
+_moo_path_is_absolute (const char *path)
 {
-    return normalize_path (filename, TRUE);
+    g_return_val_if_fail (path != NULL, FALSE);
+    return g_path_is_absolute (path)
+#ifdef __WIN32__
+        /* 'C:' is an absolute path even if glib doesn't like it */
+        /* This will match nonsense like 1:23:23 too, but that's not
+         * a valid path, and it's better to have "1:23:23" in the error
+         * message than "c:\some\silly\current\dir\1:23:23" */
+        || path[1] == ':'
+#endif
+        ;
 }
 
 
+#ifdef MOO_ENABLE_UNIT_TESTS
+
+#include <mooutils/mooutils-tests.h>
+
+static void
+test_normalize_path_one (const char *path,
+                         const char *expected,
+                         char *(*func) (const char*),
+                         const char *func_name)
+{
+    char *result;
+
+    TEST_EXPECT_WARNING (!(expected && expected[0]),
+                         "%s(%s)", func_name, TEST_FMT_STR (path));
+    result = func (path);
+    TEST_CHECK_WARNING ();
+
+    TEST_ASSERT_STR_EQ_MSG (result, expected, "%s(%s)",
+                            func_name, TEST_FMT_STR (path));
+
+    g_free (result);
+}
+
+static GPtrArray *
+make_cases (gboolean unix_paths)
+{
+    guint i;
+    char *current_dir;
+    GPtrArray *paths = g_ptr_array_new ();
+
+    const char *abs_files_common[] = {
+        NULL, NULL,
+        "", "",
+    };
+
+    const char *abs_files_unix[] = {
+        "/usr", "/usr",
+        "/usr/", "/usr",
+        "/usr///", "/usr",
+        "///usr////", "/usr",
+        "/", "/",
+        "//", "/",
+        "/usr/../bin/../usr", "/usr",
+        "/usr/bin/../bin/.", "/usr/bin",
+        "/..", "/",
+        "/../../../", "/",
+        "/../whatever/..", "/",
+        "/../whatever/.././././", "/",
+    };
+
+    const char *abs_files_win32[] = {
+        "C:", "C:\\",
+        "C:\\", "C:\\",
+        "C:\\\\", "C:\\",
+        "C:\\foobar", "C:\\foobar",
+        "C:\\foobar\\", "C:\\foobar",
+        "C:\\foobar\\\\\\", "C:\\foobar",
+        "C:/", "C:\\",
+        "C:/foobar", "C:\\foobar",
+        "C:/foobar//", "C:\\foobar",
+        "C:/foobar///", "C:\\foobar",
+        "\\\\.host\\dir\\root", "\\\\.host\\dir\\root",
+        "\\\\.host\\dir\\root\\", "\\\\.host\\dir\\root",
+        "\\\\.host\\dir\\root\\..\\..\\", "\\\\.host",
+    };
+
+    const char *rel_files_common[] = {
+        ".", NULL,
+        "././././/", NULL,
+        "foobar", "foobar",
+        "foobar/", "foobar",
+        "foobar//", "foobar",
+        "foobar/..", NULL,
+        "foobar/./..", NULL,
+        "foobar/../", NULL,
+        "foobar/./../", NULL,
+    };
+
+#ifndef __WIN32__
+    const char *rel_files_unix[] = {
+        "foobar/com", "foobar/com",
+    };
+#endif
+
+#ifdef __WIN32__
+    const char *rel_files_win32[] = {
+        "foobar/com", "foobar\\com",
+        ".\\.\\.\\.\\\\", NULL,
+        "foobar\\", "foobar",
+        "foobar\\\\", "foobar",
+        "foobar\\..", NULL,
+        "foobar\\.\\..", NULL,
+        "foobar\\..\\", NULL,
+        "foobar\\.\\..\\", NULL,
+    };
+#endif
+
+    for (i = 0; i < G_N_ELEMENTS (abs_files_common); i += 2)
+    {
+        g_ptr_array_add (paths, g_strdup (abs_files_common[i]));
+        g_ptr_array_add (paths, g_strdup (abs_files_common[i+1]));
+    }
+
+    if (unix_paths)
+        for (i = 0; i < G_N_ELEMENTS (abs_files_unix); i += 2)
+        {
+            g_ptr_array_add (paths, g_strdup (abs_files_unix[i]));
+            g_ptr_array_add (paths, g_strdup (abs_files_unix[i+1]));
+        }
+    else
+        for (i = 0; i < G_N_ELEMENTS (abs_files_win32); i += 2)
+        {
+            g_ptr_array_add (paths, g_strdup (abs_files_win32[i]));
+            g_ptr_array_add (paths, g_strdup (abs_files_win32[i+1]));
+        }
+
+    current_dir = g_get_current_dir ();
+
+#ifndef __WIN32__
+    if (unix_paths)
+#endif
+        for (i = 0; i < G_N_ELEMENTS (rel_files_common); i += 2)
+        {
+            g_ptr_array_add (paths, g_strdup (rel_files_common[i]));
+            if (rel_files_common[i+1])
+                g_ptr_array_add (paths, g_build_filename (current_dir, rel_files_common[i+1], NULL));
+            else
+                g_ptr_array_add (paths, g_strdup (current_dir));
+        }
+
+#ifndef __WIN32__
+    if (unix_paths)
+        for (i = 0; i < G_N_ELEMENTS (rel_files_unix); i += 2)
+        {
+            g_ptr_array_add (paths, g_strdup (rel_files_unix[i]));
+            if (rel_files_unix[i+1])
+                g_ptr_array_add (paths, g_build_filename (current_dir, rel_files_unix[i+1], NULL));
+            else
+                g_ptr_array_add (paths, g_strdup (current_dir));
+        }
+#endif
+
+#ifdef __WIN32__
+    for (i = 0; i < G_N_ELEMENTS (rel_files_win32); i += 2)
+    {
+        g_ptr_array_add (paths, g_strdup (rel_files_win32[i]));
+        if (rel_files_win32[i+1])
+            g_ptr_array_add (paths, g_build_filename (current_dir, rel_files_win32[i+1], NULL));
+        else
+            g_ptr_array_add (paths, g_strdup (current_dir));
+    }
+#endif
+
+#ifndef __WIN32__
+    if (unix_paths)
+#endif
+    {
+        char *parent_dir = g_path_get_dirname (current_dir);
+        g_ptr_array_add (paths, g_strdup (".."));
+        g_ptr_array_add (paths, g_strdup (parent_dir));
+        g_ptr_array_add (paths, g_strdup ("../"));
+        g_ptr_array_add (paths, g_strdup (parent_dir));
+        g_ptr_array_add (paths, g_strdup ("../."));
+        g_ptr_array_add (paths, g_strdup (parent_dir));
+        g_ptr_array_add (paths, g_strdup (".././"));
+        g_ptr_array_add (paths, g_strdup (parent_dir));
+        g_ptr_array_add (paths, g_strdup ("..//"));
+        g_ptr_array_add (paths, g_strdup (parent_dir));
+#ifdef __WIN32__
+        g_ptr_array_add (paths, g_strdup ("..\\"));
+        g_ptr_array_add (paths, g_strdup (parent_dir));
+        g_ptr_array_add (paths, g_strdup ("..\\."));
+        g_ptr_array_add (paths, g_strdup (parent_dir));
+        g_ptr_array_add (paths, g_strdup ("..\\.\\"));
+        g_ptr_array_add (paths, g_strdup (parent_dir));
+        g_ptr_array_add (paths, g_strdup ("..\\\\"));
+        g_ptr_array_add (paths, g_strdup (parent_dir));
+#endif
+        g_free (parent_dir);
+    }
+
+    g_free (current_dir);
+    return paths;
+}
+
+static void
+test_normalize_file_path (void)
+{
+    GPtrArray *paths;
+    guint i;
+
+#ifndef __WIN32__
+    paths = make_cases (TRUE);
+#else
+    paths = make_cases (FALSE);
+#endif
+
+    for (i = 0; i < paths->len; i += 2)
+    {
+        test_normalize_path_one (paths->pdata[i], paths->pdata[i+1],
+                                 _moo_normalize_file_path,
+                                 "_moo_normalize_file_path");
+        g_free (paths->pdata[i]);
+        g_free (paths->pdata[i+1]);
+    }
+
+    g_ptr_array_free (paths, TRUE);
+}
+
+#ifndef __WIN32__
+static void
+test_normalize_file_path_win32 (void)
+{
+    GPtrArray *paths;
+    guint i;
+
+    paths = make_cases (FALSE);
+
+    for (i = 0; i < paths->len; i += 2)
+    {
+        test_normalize_path_one (paths->pdata[i], paths->pdata[i+1],
+                                 normalize_full_path_win32,
+                                 "normalize_full_path_win32");
+        g_free (paths->pdata[i]);
+        g_free (paths->pdata[i+1]);
+    }
+
+    g_ptr_array_free (paths, TRUE);
+}
+#endif
+
+void
+moo_test_mooutils_fs (void)
+{
+    MooTestSuite *suite;
+
+    suite = moo_test_suite_new ("mooutils/mooutils-fs.c", NULL, NULL, NULL);
+
+    moo_test_suite_add_test (suite, "test of _moo_normalize_file_path()",
+                             (MooTestFunc) test_normalize_file_path, NULL);
+#ifndef __WIN32__
+    moo_test_suite_add_test (suite, "test of normalize_full_path_win32()",
+                             (MooTestFunc) test_normalize_file_path_win32, NULL);
+#endif
+}
+
+#endif /* MOO_ENABLE_UNIT_TESTS */
+
+
 /**********************************************************************/
 /* MSLU for poor
  */
@@ -805,22 +1204,7 @@ G_STMT_START {                                                  \
 int
 _moo_unlink (const char *path)
 {
-#ifdef __WIN32__
-    CCALL_1 (unlink, _wunlink, path);
-#else
-    return unlink (path);
-#endif
-}
-
-
-static int
-_moo_rmdir (const char *path)
-{
-#ifdef __WIN32__
-    CCALL_1 (rmdir, _wrmdir, path);
-#else
-    return rmdir (path);
-#endif
+    return g_unlink (path);
 }
 
 
@@ -838,43 +1222,7 @@ _moo_mkdir (const char *path)
 int
 _moo_remove (const char *path)
 {
-#ifdef __WIN32__
-    gboolean use_wide_char_api;
-    gpointer converted;
-    int retval;
-    int save_errno;
-
-    converted = convert_filename (path, &use_wide_char_api);
-
-    if (!converted)
-    {
-        errno = EINVAL;
-        return -1;
-    }
-
-    errno = 0;
-
-    if (use_wide_char_api)
-        retval = _wremove (converted);
-    else
-        retval = remove (converted);
-
-    if (retval && errno == ENOENT)
-    {
-        if (use_wide_char_api)
-            retval = _wrmdir (converted);
-        else
-            retval = rmdir (converted);
-    }
-
-    save_errno = errno;
-    g_free (converted);
-    errno = save_errno;
-
-    return retval;
-#else
-    return remove (path);
-#endif
+    return g_remove (path);
 }
 
 
@@ -981,12 +1329,8 @@ _moo_rename (const char *old_name,
 /* Glob matching
  */
 
-#if 0 && defined(HAVE_FNMATCH_H)
-#include <fnmatch.h>
-#else
 #define MOO_GLOB_REGEX
 #include <glib/gregex.h>
-#endif
 
 typedef struct _MooGlob {
 #ifdef MOO_GLOB_REGEX
