@@ -1,19 +1,23 @@
 /*
  *   mooutils-fs.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_UTILS_FS_H
 #define MOO_UTILS_FS_H
 
 #include <glib.h>
+#include <gtk/gtkwidget.h>
 
 /* MSVC */
 #if defined(WIN32) && !defined(__GNUC__)
@@ -24,6 +28,10 @@
 #endif
 #endif
 
+#ifndef O_BINARY
+#define O_BINARY 0x0
+#endif
+
 G_BEGIN_DECLS
 
 
@@ -63,11 +71,20 @@ gboolean        _moo_rename_file            (const char *path,
                                              const char *new_path,
                                              GError    **error);
 
-char          **moo_filenames_from_locale   (char      **files);
-char           *moo_filename_from_locale    (const char *file);
+char           **moo_filenames_from_locale  (char      **files);
+char            *moo_filename_from_locale   (const char *file);
+char           *_moo_filename_to_uri        (const char *file,
+                                             GError    **error);
 
 char           *_moo_normalize_file_path    (const char *filename);
-char           *_moo_normalize_dir_path     (const char *filename);
+gboolean        _moo_path_is_absolute       (const char *path);
+
+gboolean        _moo_copy_files_ui          (GList      *filenames,
+                                             const char *destdir,
+                                             GtkWidget  *parent);
+gboolean        _moo_move_files_ui          (GList      *filenames,
+                                             const char *destdir,
+                                             GtkWidget  *parent);
 
 /*
  * C library and WinAPI functions wrappers analogous to glib/gstdio.h
