@@ -1,13 +1,16 @@
 /*
  *   mooutils-gobject.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_UTILS_GOBJECT_H
@@ -58,6 +61,8 @@ gboolean        _moo_convert_string_to_bool (const char     *string,
                                              gboolean        default_val);
 int             _moo_convert_string_to_int  (const char     *string,
                                              int             default_val);
+guint           _moo_convert_string_to_uint (const char     *string,
+                                             guint           default_val);
 const char     *_moo_convert_bool_to_string (gboolean        value);
 const char     *_moo_convert_int_to_string  (int             value);
 
@@ -119,8 +124,7 @@ MooObjectWatch *_moo_object_watch_alloc (gsize                size,
     ((Type_*) _moo_object_watch_alloc (sizeof (Type_), klass_, src_, tgt_, notify_, data_))
 
 void        moo_bind_sensitive      (GtkWidget          *toggle_btn,
-                                     GtkWidget         **dependent,
-                                     int                 num_dependent,
+                                     GtkWidget          *dependent,
                                      gboolean            invert);
 
 guint       moo_bind_bool_property  (gpointer            target,
