@@ -1,27 +1,31 @@
 /*
  *   mooutils-misc.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
-
+#include "config-dirs.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-fs.h"
+#include "mooutils/mooutils-file.h"
 #include "mooutils/mooutils-debug.h"
-#include "mooutils/moologwindow-glade.h"
-#include "mooutils/mooglade.h"
 #include "mooutils/mooi18n.h"
+#include "mooutils/mooatom.h"
+#include "mooutils/mooonce.h"
+#include "mooutils/mooutils-misc.h"
+#include "glade/moologwindow-gxml.h"
 #include <gtk/gtk.h>
 #include <glib/gmappedfile.h>
+#include <glib/gstdio.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -31,26 +35,30 @@
 #include <sys/types.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <stdarg.h>
 
 #ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
 #endif
 
+#ifdef MOO_USE_QUARTZ
+#include <gdk/gdkquartz.h>
+#endif
+
 #ifdef __WIN32__
-#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <shellapi.h>
 #endif
 
 
 G_LOCK_DEFINE_STATIC (moo_user_data_dir);
-G_LOCK_DEFINE_STATIC (moo_temp_dir);
 static char *moo_app_instance_name;
 static char *moo_user_data_dir;
 static char *moo_temp_dir;
 
-static void     moo_install_atexit      (void);
-static void     moo_remove_tempdir      (void);
+static void         moo_install_atexit      (void);
+static void         moo_remove_tempdir      (void);
+static const char  *moo_get_prgname         (void);
 
 
 #ifdef __WIN32__
@@ -194,7 +202,7 @@ moo_open_file (const char *path)
 
     g_return_val_if_fail (path != NULL, FALSE);
 
-    uri = g_filename_to_uri (path, NULL, NULL);
+    uri = _moo_filename_to_uri (path, NULL);
     g_return_val_if_fail (uri != NULL, FALSE);
 
     ret = open_uri (uri, FALSE);
@@ -444,7 +452,6 @@ _moo_window_is_hidden (GtkWindow  *window)
 
 #elif defined(__WIN32__)
 
-#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <gdk/gdkwin32.h>
 
@@ -530,13 +537,19 @@ _moo_get_top_window (GSList *windows)
     return GTK_WINDOW (l->data);
 }
 
-#else /* neither X nor WIN32 ?? */
+#else /* neither X nor WIN32 */
 
-GtkWindow*
+/* XXX implement me */
+
+GtkWindow *
 _moo_get_top_window (GSList *windows)
 {
     g_return_val_if_fail (windows != NULL, NULL);
-    g_critical ("%s: don't know how to do it", G_STRLOC);
+
+    if (!windows->next)
+        return GTK_WINDOW (windows->data);
+
+    g_message ("%s: don't know how to do it", G_STRFUNC);
     return GTK_WINDOW (windows->data);
 }
 
@@ -570,10 +583,13 @@ moo_window_present (GtkWindow *window,
 {
     g_return_if_fail (GTK_IS_WINDOW (window));
 
-#if !defined(GDK_WINDOWING_X11)
-    gtk_window_present (window);
-#else
+#if defined(GDK_WINDOWING_X11)
     present_window_x11 (window, stamp);
+#elif defined(MOO_USE_QUARTZ)
+    gtk_window_present (window);
+    [[NSApplication sharedApplication] activateIgnoringOtherApps:YES];
+#else
+    gtk_window_present (window);
 #endif
 }
 
@@ -643,16 +659,15 @@ static GtkWidget    *moo_log_window_get_widget  (void);
 static MooLogWindow*
 moo_log_window_new (void)
 {
-    MooGladeXML *xml;
     MooLogWindow *log;
     PangoFontDescription *font;
+    LogWindowXml *xml;
 
-    xml = moo_glade_xml_new_from_buf (moologwindow_glade_xml, -1,
-                                      NULL, GETTEXT_PACKAGE, NULL);
+    xml = log_window_xml_new ();
     log = g_new (MooLogWindow, 1);
 
-    log->window = moo_glade_xml_get_widget (xml, "window");
-    log->textview = moo_glade_xml_get_widget (xml, "textview");
+    log->window = GTK_WIDGET (xml->LogWindow);
+    log->textview = xml->textview;
 
     g_signal_connect (log->window, "delete-event",
                       G_CALLBACK (gtk_widget_hide_on_delete), NULL);
@@ -735,11 +750,8 @@ moo_log_window_insert (MooLogWindow *log,
 static GLogFunc moo_log_func;
 static GPrintFunc moo_print_func;
 static GPrintFunc moo_printerr_func;
-static char *moo_log_file;
-static gboolean moo_log_file_written;
 static gboolean moo_log_handlers_set;
 
-
 static void
 set_print_funcs (GLogFunc   log_func,
                  GPrintFunc print_func,
@@ -768,9 +780,7 @@ set_print_funcs (GLogFunc   log_func,
     set (NULL);
 #undef set
 
-#if GLIB_CHECK_VERSION(2,6,0)
     g_log_set_default_handler (log_func, NULL);
-#endif /* GLIB_CHECK_VERSION(2,6,0) */
 }
 
 
@@ -803,6 +813,23 @@ moo_reset_log_func (void)
  * Display log messages in a window
  */
 
+#if !defined(__WIN32__) || !defined(DEBUG)
+
+static void
+on_warning_or_critical_message (G_GNUC_UNUSED const char *message)
+{
+}
+
+#else /* __WIN32__ && DEBUG */
+
+static void
+on_warning_or_critical_message (const char *message)
+{
+    _moo_abort_debug_ignore (MOO_CODE_LOC_UNKNOWN, message);
+}
+
+#endif
+
 static void
 log_func_window (const gchar    *log_domain,
                  GLogLevelFlags  flags,
@@ -810,8 +837,6 @@ log_func_window (const gchar    *log_domain,
                  G_GNUC_UNUSED gpointer dummy)
 {
     char *text;
-    GtkTextTag *tag;
-    MooLogWindow *log;
 
 #ifdef __WIN32__
     if (flags & (G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION))
@@ -821,21 +846,40 @@ log_func_window (const gchar    *log_domain,
     }
 #endif /* __WIN32__ */
 
+    if (!g_utf8_validate (message, -1, NULL))
+        message = "<corrupted string, invalid UTF8>";
+
     if (log_domain)
         text = g_strdup_printf ("%s: %s\n", log_domain, message);
     else
         text = g_strdup_printf ("%s\n", message);
 
-    log = moo_log_window ();
+    if (flags < G_LOG_LEVEL_MESSAGE)
+        on_warning_or_critical_message (text);
 
-    if (flags >= G_LOG_LEVEL_MESSAGE)
-        tag = log->message_tag;
-    else if (flags >= G_LOG_LEVEL_WARNING)
-        tag = log->warning_tag;
-    else
-        tag = log->critical_tag;
+    {
+        GtkTextTag *tag;
+        MooLogWindow *log;
+
+        gdk_threads_enter ();
+
+        if ((log = moo_log_window ()))
+        {
+            if (flags >= G_LOG_LEVEL_MESSAGE)
+                tag = log->message_tag;
+            else if (flags >= G_LOG_LEVEL_WARNING)
+                tag = log->warning_tag;
+            else
+                tag = log->critical_tag;
+
+            moo_log_window_insert (log, text, tag);
+            if (flags <= G_LOG_LEVEL_WARNING)
+                gtk_window_present (GTK_WINDOW (log->window));
+        }
+
+        gdk_threads_leave ();
+    }
 
-    moo_log_window_insert (log, text, tag);
     g_free (text);
 }
 
@@ -843,21 +887,29 @@ log_func_window (const gchar    *log_domain,
 static void
 print_func_window (const char *string)
 {
-    MooLogWindow *log = moo_log_window ();
-    moo_log_window_insert (log, string, NULL);
+    MooLogWindow *log;
+    gdk_threads_enter ();
+    if ((log = moo_log_window ()))
+        moo_log_window_insert (log, string, NULL);
+    gdk_threads_leave ();
 }
 
 static void
 printerr_func_window (const char *string)
 {
-    MooLogWindow *log = moo_log_window ();
-    moo_log_window_insert (log, string, log->warning_tag);
+    MooLogWindow *log;
+    gdk_threads_enter ();
+    if ((log = moo_log_window ()))
+        moo_log_window_insert (log, string, log->warning_tag);
+    gdk_threads_leave ();
 }
 
 
 void
 moo_set_log_func_window (gboolean show_now)
 {
+    gtk_init (NULL, NULL);
+
     if (show_now)
         gtk_widget_show (GTK_WIDGET (moo_log_window()->window));
 
@@ -869,21 +921,31 @@ moo_set_log_func_window (gboolean show_now)
  * Write to a file
  */
 
+static char *moo_log_file;
+static gboolean moo_log_file_written;
+static GStaticRecMutex moo_log_file_mutex = G_STATIC_REC_MUTEX_INIT;
+
 static void
 print_func_file (const char *string)
 {
     FILE *file;
 
-    g_return_if_fail (moo_log_file != NULL);
+    g_static_rec_mutex_lock (&moo_log_file_mutex);
+
+    if (!moo_log_file)
+    {
+        g_static_rec_mutex_unlock (&moo_log_file_mutex);
+        g_return_if_reached ();
+    }
 
     if (!moo_log_file_written)
     {
-        file = fopen (moo_log_file, "wb+");
+        file = fopen (moo_log_file, "w+");
         moo_log_file_written = TRUE;
     }
     else
     {
-        file = fopen (moo_log_file, "ab+");
+        file = fopen (moo_log_file, "a+");
     }
 
     if (file)
@@ -895,6 +957,8 @@ print_func_file (const char *string)
     {
         /* TODO ??? */
     }
+
+    g_static_rec_mutex_unlock (&moo_log_file_mutex);
 }
 
 
@@ -927,9 +991,11 @@ log_func_file (const char       *log_domain,
 void
 moo_set_log_func_file (const char *log_file)
 {
+    g_static_rec_mutex_lock (&moo_log_file_mutex);
     g_free (moo_log_file);
     moo_log_file = g_strdup (log_file);
     set_print_funcs (log_func_file, print_func_file, print_func_file);
+    g_static_rec_mutex_unlock (&moo_log_file_mutex);
 }
 
 
@@ -970,11 +1036,18 @@ moo_set_log_func_silent (void)
 /* Very useful function
  */
 
-void
+void MOO_NORETURN
 moo_segfault (void)
 {
     char *var = (char*) -1;
     var[18] = 8;
+    moo_abort ();
+}
+
+void MOO_NORETURN
+moo_abort (void)
+{
+    abort ();
 }
 
 
@@ -983,9 +1056,9 @@ moo_segfault (void)
  */
 
 void
-_moo_selection_data_set_pointer (GtkSelectionData *data,
-                                 GdkAtom           type,
-                                 gpointer          ptr)
+moo_selection_data_set_pointer (GtkSelectionData *data,
+                                GdkAtom           type,
+                                gpointer          ptr)
 {
     g_return_if_fail (data != NULL);
     gtk_selection_data_set (data, type, 8, /* 8 bits per byte */
@@ -994,8 +1067,8 @@ _moo_selection_data_set_pointer (GtkSelectionData *data,
 
 
 gpointer
-_moo_selection_data_get_pointer (GtkSelectionData *data,
-                                 GdkAtom           type)
+moo_selection_data_get_pointer (GtkSelectionData *data,
+                                GdkAtom           type)
 {
     GdkWindow *owner;
     gpointer result = NULL;
@@ -1108,21 +1181,17 @@ _moo_menu_item_set_label (GtkWidget  *item,
 const char *
 _moo_get_pid_string (void)
 {
-    G_LOCK_DEFINE_STATIC (moo_pid_string);
     static char *moo_pid_string;
 
-    G_LOCK (moo_pid_string);
+    MOO_DO_ONCE_BEGIN
 
-    if (!moo_pid_string)
-    {
 #ifdef __WIN32__
-        moo_pid_string = g_strdup_printf ("%ld", GetCurrentProcessId ());
+    moo_pid_string = g_strdup_printf ("%ld", GetCurrentProcessId ());
 #else
-        moo_pid_string = g_strdup_printf ("%ld", (long) getpid ());
+    moo_pid_string = g_strdup_printf ("%ld", (long) getpid ());
 #endif
-    }
 
-    G_UNLOCK (moo_pid_string);
+    MOO_DO_ONCE_END
 
     return moo_pid_string;
 }
@@ -1138,7 +1207,7 @@ moo_get_prgname (void)
     if (!name)
     {
         g_critical ("%s: program name not set", G_STRLOC);
-        name = "ggap";
+        name = "medit";
     }
 
     return name;
@@ -1153,9 +1222,7 @@ moo_tempnam (void)
     static int counter;
     G_LOCK_DEFINE_STATIC (counter);
 
-    G_LOCK (moo_temp_dir);
-
-    if (!moo_temp_dir)
+    MOO_DO_ONCE_BEGIN
     {
         char *dirname = NULL;
         const char *short_name;
@@ -1170,41 +1237,35 @@ moo_tempnam (void)
             dirname = g_build_filename (g_get_tmp_dir (), basename, NULL);
             g_free (basename);
 
-            if (_moo_mkdir (dirname))
-            {
-                g_free (dirname);
-                dirname = NULL;
-            }
-            else
-            {
+            if (_moo_mkdir (dirname) == 0)
                 break;
-            }
+
+            g_free (dirname);
+            dirname = NULL;
         }
 
         moo_temp_dir = dirname;
         moo_install_atexit ();
     }
-
-    G_UNLOCK (moo_temp_dir);
+    MOO_DO_ONCE_END
 
     g_return_val_if_fail (moo_temp_dir != NULL, NULL);
 
     G_LOCK (counter);
 
-    for (i = counter + 1; i < counter + 1000 && !filename; ++i)
+    for (i = counter + 1; i < counter + 1000; ++i)
     {
         char *basename;
 
         basename = g_strdup_printf ("tmpfile-%03d", i);
         filename = g_build_filename (moo_temp_dir, basename, NULL);
-
-        if (g_file_test (filename, G_FILE_TEST_EXISTS))
-        {
-            g_free (filename);
-            filename = NULL;
-        }
-
         g_free (basename);
+
+        if (!g_file_test (filename, G_FILE_TEST_EXISTS))
+            break;
+
+        g_free (filename);
+        filename = NULL;
     }
 
     counter = i;
@@ -1262,10 +1323,10 @@ moo_cleanup (void)
 }
 
 
-static char *
+char *
 moo_get_user_cache_dir (void)
 {
-    return g_strdup_printf ("%s/%s", g_get_user_cache_dir (), moo_get_prgname ());
+    return g_build_filename (g_get_user_cache_dir (), moo_get_prgname (), NULL);
 }
 
 char *
@@ -1276,10 +1337,14 @@ moo_get_user_data_dir (void)
     if (!moo_user_data_dir)
     {
 #ifdef __WIN32__
-        moo_user_data_dir = g_build_filename (g_get_home_dir (), moo_get_prgname (), NULL);
+        const char *basedir = g_get_user_config_dir ();
 #else
-        moo_user_data_dir = g_strdup_printf ("%s/%s", g_get_user_data_dir (), moo_get_prgname ());
+        const char *basedir = g_get_user_data_dir ();
 #endif
+
+        moo_user_data_dir = g_build_filename (basedir,
+                                              moo_get_prgname (),
+                                              NULL);
     }
 
     G_UNLOCK (moo_user_data_dir);
@@ -1288,7 +1353,7 @@ moo_get_user_data_dir (void)
 }
 
 void
-_moo_set_user_data_dir (const char *path)
+moo_set_user_data_dir (const char *path)
 {
     G_LOCK (moo_user_data_dir);
 
@@ -1376,9 +1441,10 @@ add_dir_list_from_env (GPtrArray  *list,
 }
 
 
-char **
-moo_get_data_dirs (MooDataDirType type,
-                   guint         *n_dirs)
+static char **
+moo_get_data_dirs_real (MooDataDirType   type,
+                        char           **add,
+                        guint           *n_dirs)
 {
     static char **moo_data_dirs[2];
     static guint n_data_dirs[2];
@@ -1388,6 +1454,9 @@ moo_get_data_dirs (MooDataDirType type,
 
     G_LOCK (moo_data_dirs);
 
+    if (add && moo_data_dirs[type])
+        g_critical ("moo_add_data_dirs() called too late, ignored");
+
     if (!moo_data_dirs[type])
     {
         const char *env[2];
@@ -1404,6 +1473,9 @@ moo_get_data_dirs (MooDataDirType type,
 
         g_ptr_array_add (all_dirs, moo_get_user_data_dir ());
 
+        while (add && *add)
+            g_ptr_array_add (all_dirs, g_strdup (*add++));
+
         /* environment variables override everything */
         if (env[0] || env[1])
         {
@@ -1477,6 +1549,19 @@ moo_get_data_dirs (MooDataDirType type,
     return g_strdupv (moo_data_dirs[type]);
 }
 
+char **
+moo_get_data_dirs (MooDataDirType type,
+                   guint         *n_dirs)
+{
+    return moo_get_data_dirs_real (type, NULL, n_dirs);
+}
+
+void
+moo_add_data_dirs (char **dirs)
+{
+    g_strfreev (moo_get_data_dirs_real (MOO_DATA_SHARE, dirs, NULL));
+}
+
 
 const char *
 moo_get_locale_dir (void)
@@ -1490,56 +1575,6 @@ moo_get_locale_dir (void)
 }
 
 
-/* For xdgmime: should not contain duplicates */
-const char* const *
-_moo_get_shared_data_dirs (void)
-{
-    static char **data_dirs = NULL;
-    G_LOCK_DEFINE_STATIC(data_dirs);
-
-    G_LOCK (data_dirs);
-
-    if (!data_dirs)
-    {
-        guint i, n_dirs;
-        GSList *dirs = NULL;
-        const char* const *sys_dirs;
-
-        sys_dirs = g_get_system_data_dirs ();
-
-#ifndef __WIN32__
-        dirs = g_slist_prepend (NULL, _moo_normalize_dir_path (g_get_user_data_dir ()));
-#endif
-
-        for (i = 0; sys_dirs[i] && sys_dirs[i][0]; ++i)
-        {
-            char *norm = _moo_normalize_dir_path (sys_dirs[i]);
-
-            if (norm && !g_slist_find_custom (dirs, norm, (GCompareFunc) strcmp))
-                dirs = g_slist_prepend (dirs, norm);
-            else
-                g_free (norm);
-        }
-
-        dirs = g_slist_reverse (dirs);
-        n_dirs = g_slist_length (dirs);
-        data_dirs = g_new (char*, n_dirs + 1);
-
-        for (i = 0; i < n_dirs; ++i)
-        {
-            data_dirs[i] = dirs->data;
-            dirs = g_slist_delete_link (dirs, dirs);
-        }
-
-        data_dirs[n_dirs] = NULL;
-    }
-
-    G_UNLOCK (data_dirs);
-
-    return (const char* const*) data_dirs;
-}
-
-
 char **
 moo_get_data_subdirs (const char    *subdir,
                       MooDataDirType type,
@@ -1621,74 +1656,26 @@ moo_get_user_cache_file (const char *basename)
 
 
 static gboolean
-save_with_backup (const char *filename,
-                  const char *data,
-                  gssize      len,
-                  GError    **error)
+save_config_file (const char     *dir,
+                  const char     *filename,
+                  const char     *content,
+                  gssize          len,
+                  GError        **error)
 {
-    char *tmp_file, *bak_file;
-    gboolean result = FALSE;
-    GError *error_move = NULL;
+    MooFileWriter *writer;
+    gboolean retval;
 
-    tmp_file = g_strdup_printf ("%s.tmp", filename);
-    bak_file = g_strdup_printf ("%s.bak", filename);
+    g_return_val_if_fail (dir != NULL, FALSE);
+    g_return_val_if_fail (filename != NULL, FALSE);
+    g_return_val_if_fail (content != NULL, FALSE);
 
-    if (!_moo_save_file_utf8 (tmp_file, data, len, error))
-        goto out;
+    if (!(writer = moo_config_writer_new (filename, TRUE, error)))
+        return FALSE;
 
-    if (g_file_test (filename, G_FILE_TEST_EXISTS))
-        _moo_rename_file (filename, bak_file, &error_move);
+    moo_file_writer_write (writer, content, len);
+    retval = moo_file_writer_close (writer, error);
 
-    if (!_moo_rename_file (tmp_file, filename, error))
-    {
-        if (error_move)
-        {
-            g_critical ("%s: %s", G_STRLOC, error_move->message);
-            g_error_free (error_move);
-        }
-    }
-    else
-    {
-        if (error_move)
-            g_propagate_error (error, error_move);
-        else
-            result = TRUE;
-    }
-
-out:
-    g_free (tmp_file);
-    g_free (bak_file);
-    return result;
-}
-
-static gboolean
-same_content (const char *filename,
-              const char *data,
-              gsize       len)
-{
-    GMappedFile *file;
-    char *content;
-    gsize i;
-    gboolean equal = FALSE;
-
-    file = g_mapped_file_new (filename, FALSE, NULL);
-
-    if (!file || g_mapped_file_get_length (file) != len)
-        goto out;
-
-    content = g_mapped_file_get_contents (file);
-
-    for (i = 0; i < len; ++i)
-        if (data[i] != content[i])
-            goto out;
-
-    equal = TRUE;
-
-out:
-    if (file)
-        g_mapped_file_free (file);
-
-    return equal;
+    return retval;
 }
 
 static gboolean
@@ -1699,14 +1686,11 @@ save_user_data_file (const char     *basename,
                      GError        **error)
 {
     char *dir, *file;
-    gboolean result = FALSE;
+    gboolean result;
 
     g_return_val_if_fail (basename != NULL, FALSE);
     g_return_val_if_fail (content != NULL, FALSE);
 
-    if (len < 0)
-        len = strlen (content);
-
     if (cache)
     {
         dir = moo_get_user_cache_dir ();
@@ -1718,23 +1702,8 @@ save_user_data_file (const char     *basename,
         file = moo_get_user_data_file (basename);
     }
 
-    g_return_val_if_fail (dir && file, FALSE);
+    result = save_config_file (dir, file, content, len, error);
 
-    if (!_moo_create_dir (dir, error))
-        goto out;
-
-    if (same_content (file, content, len))
-    {
-        result = TRUE;
-        goto out;
-    }
-
-    if (!save_with_backup (file, content, len, error))
-        goto out;
-
-    result = TRUE;
-
-out:
     g_free (dir);
     g_free (file);
     return result;
@@ -1758,24 +1727,24 @@ moo_save_user_cache_file (const char  *basename,
     return save_user_data_file (basename, TRUE, content, len, error);
 }
 
-
-GType
-moo_data_dir_type_get_type (void)
+gboolean
+moo_save_config_file (const char   *filename,
+                      const char   *content,
+                      gssize        len,
+                      GError      **error)
 {
-    static GType type = 0;
+    char *dir;
+    gboolean result;
 
-    if (G_UNLIKELY (!type))
-    {
-        static const GEnumValue values[] = {
-            { MOO_DATA_SHARE, (char*) "MOO_DATA_SHARE", (char*) "share" },
-            { MOO_DATA_LIB, (char*) "MOO_DATA_LIB", (char*) "lib" },
-            { 0, NULL, NULL }
-        };
+    g_return_val_if_fail (filename != NULL, FALSE);
+    g_return_val_if_fail (content != NULL, FALSE);
 
-        type = g_enum_register_static ("MooDataDirType", values);
-    }
+    dir = g_path_get_dirname (filename);
 
-    return type;
+    result = save_config_file (dir, filename, content, len, error);
+
+    g_free (dir);
+    return result;
 }
 
 
@@ -1806,6 +1775,176 @@ _moo_widget_set_tooltip (GtkWidget  *widget,
 }
 
 
+/*******************************************************************************
+ * Asserts
+ */
+
+#if !defined(__WIN32__)
+
+void
+_moo_abort_debug_ignore (MooCodeLoc loc, const char *message)
+{
+    if (moo_code_loc_valid (loc))
+	g_error ("file '%s', function '%s', line %d: %s\n", loc.file, loc.func, loc.line, message);
+    else
+        g_error ("%s\n", message);
+}
+
+#else /* !__WIN32__ */
+
+static void
+break_into_debugger (void)
+{
+    RaiseException(0, EXCEPTION_NONCONTINUABLE, 0, NULL);
+}
+
+void
+_moo_abort_debug_ignore (MooCodeLoc loc, const char *message)
+{
+    gboolean skip = FALSE;
+    static GHashTable *locs_hash;
+    char *loc_id = NULL;
+    gboolean use_location;
+
+    use_location = moo_code_loc_valid (loc);
+
+    if (use_location)
+        loc_id = g_strdup_printf ("%s#%d#%d", loc.file, loc.line, loc.counter);
+    else
+        loc_id = g_strdup (message);
+
+    if (loc_id != NULL)
+    {
+        if (!locs_hash)
+            locs_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+        if (g_hash_table_lookup (locs_hash, loc_id))
+            skip = TRUE;
+    }
+
+    if (!skip)
+    {
+        int ret;
+
+        if (use_location)
+            ret = _moo_win32_message_box (NULL, MB_ABORTRETRYIGNORE, NULL,
+                                          "In file %s, line %d, function %s:\n%s",
+                                          loc.file, loc.line, loc.func, message);
+        else
+            ret = _moo_win32_message_box (NULL, MB_ABORTRETRYIGNORE, NULL,
+                                          "%s", message);
+
+        switch (ret)
+        {
+            case IDABORT:
+                TerminateProcess (GetCurrentProcess (), 1);
+                break;
+
+            case IDRETRY:
+                break_into_debugger ();
+                break;
+
+            case IDIGNORE:
+                if (GetKeyState(VK_CONTROL) < 0)
+                    skip = TRUE;
+                break;
+
+            default:
+                moo_abort ();
+                break;
+        }
+
+        if (skip && loc_id)
+        {
+            g_hash_table_insert (locs_hash, loc_id, GINT_TO_POINTER (1));
+            loc_id = NULL;
+        }
+    }
+
+    g_free (loc_id);
+}
+
+#endif /* !__WIN32__ */
+
+#ifndef MOO_DEV_MODE
+NORETURN
+#endif
+void
+_moo_assert_message (MooCodeLoc loc, const char *message)
+{
+#if defined(MOO_DEV_MODE) && !defined(__WIN32__)
+    _moo_abort_debug_ignore (loc, message);
+#else
+    g_error ("file '%s', function '%s', line %d: %s\n", loc.file, loc.func, loc.line, message);
+#endif
+}
+
+void
+_moo_log (MooCodeLoc loc, GLogLevelFlags flags, const char *format, ...)
+{
+    va_list args;
+    va_start (args, format);
+    _moo_logv (loc, flags, format, args);
+    va_end (args);
+}
+
+static gboolean
+moo_log_debug_enabled (void)
+{
+    static int enabled = -1;
+
+    if (enabled == -1)
+    {
+        enabled = moo_debug_enabled ("misc",
+#ifdef MOO_DEBUG
+                                     TRUE);
+#else
+                                     FALSE);
+#endif
+    }
+
+    return enabled;
+}
+
+void _moo_logv (MooCodeLoc loc, GLogLevelFlags flags, const char *format, va_list args)
+{
+    char *message = g_strdup_vprintf (format, args);
+
+    if (flags >= G_LOG_LEVEL_DEBUG && !moo_log_debug_enabled ())
+        return;
+
+#if defined(MOO_DEV_MODE) && !defined(__WIN32__)
+    if (flags < G_LOG_LEVEL_MESSAGE)
+    {
+        _moo_abort_debug_ignore (loc, message);
+        flags = G_LOG_LEVEL_MESSAGE;
+    }
+#endif
+
+    if (moo_code_loc_valid (loc))
+        g_log (G_LOG_DOMAIN, flags,
+               "In file %s, line %d, function %s: %s",
+               loc.file, loc.line, loc.func, message);
+    else
+        g_log (G_LOG_DOMAIN, flags, "%s", message);
+
+    g_free (message);
+}
+
+void MOO_NORETURN _moo_error (MooCodeLoc loc, const char *format, ...)
+{
+    va_list args;
+    va_start (args, format);
+    _moo_errorv (loc, format, args);
+    va_end (args);
+}
+
+void MOO_NORETURN _moo_errorv (MooCodeLoc loc, const char *format, va_list args)
+{
+    MOO_UNUSED (loc);
+    g_logv (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR, format, args);
+    moo_abort ();
+}
+
 
 /*******************************************************************************
  * Former eggregex stuff
@@ -1866,120 +2005,190 @@ _moo_regex_escape (const char *string,
 }
 
 
-
 char **
 moo_strnsplit_lines (const char *string,
                      gssize      len,
-                     guint      *n_tokens)
+                     guint      *n_lines)
 {
-    GPtrArray *array;
-    gssize i, line;
+    MooLineReader lr;
+    GPtrArray *array = NULL;
+    const char *line;
+    gsize line_len;
 
-    if (!string || !string[0])
+    for (moo_line_reader_init (&lr, string, len);
+         (line = moo_line_reader_get_line (&lr, &line_len, NULL)); )
     {
-        if (n_tokens)
-            *n_tokens = 0;
+        if (!array)
+            array = g_ptr_array_new ();
+        g_ptr_array_add (array, g_strndup (line, line_len));
+    }
+
+    if (array)
+    {
+        if (n_lines)
+            *n_lines = array->len;
+        g_ptr_array_add (array, NULL);
+        return (char**) g_ptr_array_free (array, FALSE);
+    }
+    else
+    {
+        if (n_lines)
+            *n_lines = 0;
         return NULL;
     }
-
-    if (len < 0)
-        len = strlen (string);
-
-    array = g_ptr_array_new ();
-    line = i = 0;
-
-    while (i < len)
-    {
-        switch (string[i])
-        {
-            case '\r':
-                g_ptr_array_add (array, g_strndup (string + line, i - line));
-                if (++i < len && string[i] == '\n')
-                    ++i;
-                line = i;
-                break;
-
-            case '\n':
-                g_ptr_array_add (array, g_strndup (string + line, i - line));
-                ++i;
-                line = i;
-                break;
-
-            case '\xe2': /* Unicode paragraph separator "\xe2\x80\xa9" */
-                if (i+2 < len && string[i+1] == '\x80' && string[i+2] == '\xa9')
-                {
-                    g_ptr_array_add (array, g_strndup (string + line, i - line));
-                    i += 3;
-                    line = i;
-                    break;
-                }
-                /* fallthrough */
-
-            default:
-                ++i;
-        }
-    }
-
-    g_ptr_array_add (array, g_strndup (string + line, i - line));
-
-    if (n_tokens)
-        *n_tokens = array->len;
-
-    g_ptr_array_add (array, NULL);
-    return (char**) g_ptr_array_free (array, FALSE);
 }
 
-
 char **
 moo_splitlines (const char *string)
 {
-    GPtrArray *array;
-    const char *line, *p;
+    GPtrArray *array = NULL;
+    MooLineReader lr;
+    const char *line;
+    gsize line_len;
 
-    if (!string || !string[0])
+    for (moo_line_reader_init (&lr, string, -1);
+         (line = moo_line_reader_get_line (&lr, &line_len, NULL)); )
+    {
+        if (!array)
+            array = g_ptr_array_new ();
+        g_ptr_array_add (array, g_strndup (line, line_len));
+    }
+
+    if (array)
+    {
+        g_ptr_array_add (array, NULL);
+        return (char**) g_ptr_array_free (array, FALSE);
+    }
+    else
+    {
+        return NULL;
+    }
+}
+
+void
+moo_line_reader_init (MooLineReader *lr,
+                      const char    *text,
+                      gssize         len)
+{
+    g_return_if_fail (lr != NULL);
+
+    if (len < 0)
+        len = text ? strlen (text) : 0;
+
+    lr->text = text && len ? text : NULL;
+    lr->len = lr->text ? len : 0;
+}
+
+const char *
+moo_line_reader_get_line (MooLineReader *lr,
+                          gsize         *line_len,
+                          gsize         *lt_len)
+{
+    const char *le, *end, *line;
+    gsize le_len;
+
+    g_return_val_if_fail (lr != NULL, NULL);
+
+    if (!lr->text)
         return NULL;
 
-    array = g_ptr_array_new ();
-
-    p = line = string;
-
-    while (*p)
+    for (le = lr->text, le_len = 0, end = lr->text + lr->len; le < end && le_len == 0; )
     {
-        switch (*p)
+        switch (*le)
         {
             case '\r':
-                g_ptr_array_add (array, g_strndup (line, p - line));
-                if (*++p == '\n')
-                    ++p;
-                line = p;
+                if (le + 1 < end && le[1] == '\n')
+                    le_len = 2;
+                else
+                    le_len = 1;
                 break;
-
             case '\n':
-                g_ptr_array_add (array, g_strndup (line, p - line));
-                line = ++p;
+                le_len = 1;
                 break;
-
             case '\xe2': /* Unicode paragraph separator "\xe2\x80\xa9" */
-                if (p[1] == '\x80' && p[2] == '\xa9')
+                if (le + 2 < end && le[1] == '\x80' && le[2] == '\xa9')
                 {
-                    g_ptr_array_add (array, g_strndup (line, p - line));
-                    p += 3;
-                    line = p;
+                    le_len = 3;
                     break;
                 }
                 /* fallthrough */
-
             default:
-                ++p;
+                le += 1;
+                break;
         }
     }
 
-    g_ptr_array_add (array, g_strndup (line, p - line));
-    g_ptr_array_add (array, NULL);
+    line = lr->text;
 
-    return (char**) g_ptr_array_free (array, FALSE);
+    if (le_len)
+    {
+        lr->text = le + le_len;
+        lr->len -= (le - line) + le_len;
+    }
+    else
+    {
+        lr->text = NULL;
+        lr->len = 0;
+    }
+
+    if (line_len)
+        *line_len = le - line;
+    if (lt_len)
+        *lt_len = le_len;
+
+    return line;
 }
 
+
+gboolean
+moo_find_line_end (const char *string,
+                   gssize      len,
+                   gsize      *line_len,
+                   gsize      *lt_len)
+{
+    const char *le, *end;
+    gsize le_len;
+
+    g_return_val_if_fail (string != NULL || len == 0, FALSE);
+
+    if (len < 0)
+        len = strlen (string);
+
+    for (le = string, le_len = 0, end = string + len; le < end && le_len == 0; )
+    {
+        switch (*le)
+        {
+            case '\r':
+                if (le + 1 < end && le[1] == '\n')
+                    le_len = 2;
+                else
+                    le_len = 1;
+                break;
+            case '\n':
+                le_len = 1;
+                break;
+            case '\xe2': /* Unicode paragraph separator "\xe2\x80\xa9" */
+                if (le + 2 < end && le[1] == '\x80' && le[2] == '\xa9')
+                {
+                    le_len = 3;
+                    break;
+                }
+                /* fallthrough */
+            default:
+                le += 1;
+                break;
+        }
+    }
+
+    if (line_len != NULL)
+        *line_len = le - string;
+    if (lt_len != NULL)
+        *lt_len = le_len;
+
+    return le_len != 0;
+}
+
+
 char **
 _moo_strv_reverse (char **str_array)
 {
@@ -2000,69 +2209,23 @@ _moo_strv_reverse (char **str_array)
     return str_array;
 }
 
-char **
-_moo_ascii_strnsplit (const char *string,
-                      gssize      len,
-                      guint       max_tokens)
+
+gboolean
+_moo_str_equal (const char *s1,
+                const char *s2)
 {
-    GPtrArray *array = NULL;
-    const char *end;
-
-    g_return_val_if_fail (string != NULL, NULL);
-
-    if (len < 0)
-        len = strlen (string);
-    if (max_tokens < 2)
-        max_tokens = G_MAXUINT;
-
-    end = string + len;
-
-    while (string != end)
-    {
-        const char *p;
-
-        while (string != end && g_ascii_isspace (*string))
-            string++;
-
-        if (string == end)
-            break;
-
-        for (p = string; p != end && !g_ascii_isspace (*p); ++p) ;
-
-        if (!array)
-            array = g_ptr_array_new ();
-
-        g_ptr_array_add (array, g_strndup (string, p - string));
-
-        if (array->len + 1 == max_tokens)
-        {
-            g_ptr_array_add (array, g_strndup (p, end - p));
-            break;
-        }
-
-        string = p;
-    }
-
-    if (array)
-    {
-        g_ptr_array_add (array, NULL);
-        return (char**) g_ptr_array_free (array, FALSE);
-    }
-    else
-    {
-        return NULL;
-    }
+    return strcmp (s1 ? s1 : "", s2 ? s2 : "") == 0;
 }
 
 
-#if defined(__WIN32__) && !defined(MOO_DEBUG_ENABLED)
+#if defined(__WIN32__) && !defined(MOO_DEBUG)
 static guint saved_win32_error_mode;
 #endif
 
 void
 moo_disable_win32_error_message (void)
 {
-#if defined(__WIN32__) && !defined(MOO_DEBUG_ENABLED)
+#if defined(__WIN32__) && !defined(MOO_DEBUG)
     saved_win32_error_mode = SetErrorMode (SEM_NOOPENFILEERRORBOX | SEM_FAILCRITICALERRORS);
 #endif
 }
@@ -2070,17 +2233,17 @@ moo_disable_win32_error_message (void)
 void
 moo_enable_win32_error_message (void)
 {
-#if defined(__WIN32__) && !defined(MOO_DEBUG_ENABLED)
+#if defined(__WIN32__) && !defined(MOO_DEBUG)
     SetErrorMode (saved_win32_error_mode);
 #endif
 }
 
 
 #if 0
-#undef _moo_str_equal
+#undef moo_str_equal
 gboolean
-_moo_str_equal (const char *s1,
-                const char *s2)
+moo_str_equal (const char *s1,
+               const char *s2)
 {
     return !strcmp (s1 ? s1 : "", s2 ? s2 : "");
 }
@@ -2120,10 +2283,10 @@ thread_source_func (gpointer data)
 }
 
 guint
-_moo_idle_add_full (gint           priority,
-                    GSourceFunc    function,
-                    gpointer       data,
-                    GDestroyNotify notify)
+moo_idle_add_full (gint           priority,
+                   GSourceFunc    function,
+                   gpointer       data,
+                   GDestroyNotify notify)
 {
     SourceData *sd;
 
@@ -2141,8 +2304,8 @@ guint
 moo_idle_add (GSourceFunc function,
               gpointer    data)
 {
-    return _moo_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
-                               function, data, NULL);
+    return moo_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+                              function, data, NULL);
 }
 
 
@@ -2255,14 +2418,34 @@ _moo_intern_string (const char *string)
 #endif
 }
 
+GdkAtom
+moo_atom_uri_list (void)
+{
+    MOO_DEFINE_ATOM_ (text/uri-list)
+}
+
+
+static char *debug_domains;
+void _moo_set_debug (const char *domains);
+gboolean moo_debug_enabled (const char *domain, gboolean def_enabled);
+
+void
+_moo_set_debug (const char *domains)
+{
+    g_free (debug_domains);
+    debug_domains = g_strdup (domains);
+}
 
 gboolean
 moo_debug_enabled (const char *domain,
                    gboolean    def_enabled)
 {
     const char *val;
+    char **domains, **p;
 
     val = g_getenv ("MOO_DEBUG");
+    if (!val || !val[0])
+        val = debug_domains;
 
     if (!val || !val[0])
         return def_enabled;
@@ -2271,32 +2454,108 @@ moo_debug_enabled (const char *domain,
     if (!strcmp (val, "all"))
         return TRUE;
 
-    return strstr (val, domain) != NULL;
+    domains = g_strsplit_set (val, ",;", 0);
+    for (p = domains; p && *p; ++p)
+    {
+        if (strcmp (domain, *p) == 0)
+        {
+            g_strfreev (domains);
+            return TRUE;
+        }
+    }
+
+    g_strfreev (domains);
+    return FALSE;
 }
 
 
-#undef _moo_message
-void
-_moo_message (const char *format,
-              ...)
+#ifdef MOO_ENABLE_UNIT_TESTS
+
+#include <mooutils/mooutils-tests.h>
+
+static void
+test_strv_one (const char *string,
+               gssize      len,
+               char const *expected[])
 {
-    static int enabled = -1;
+    const char *s;
+    char *freeme = NULL;
+    char **res;
+    guint n_toks;
 
-    if (enabled == -1)
-    {
-        enabled = moo_debug_enabled ("misc",
-#ifdef MOO_DEBUG_ENABLED
-                                     TRUE);
-#else
-                                     FALSE);
-#endif
-    }
+    if (len < 0)
+        s = string;
+    else if (len == 0)
+        s = "";
+    else
+        s = freeme = g_strndup (string, len);
 
-    if (enabled)
-    {
-        va_list args;
-        va_start (args, format);
-        g_logv (G_LOG_DOMAIN "-debug", G_LOG_LEVEL_MESSAGE, format, args);
-        va_end (args);
-    }
+    res = moo_splitlines (s);
+    TEST_ASSERT_STRV_EQ_MSG (res, (char**) expected,
+                             "moo_splitlines(%s)", TEST_FMT_STR (s));
+    g_strfreev (res);
+    g_free (freeme);
+    freeme = NULL;
+
+    res = moo_strnsplit_lines (string, len, &n_toks);
+    TEST_ASSERT_STRV_EQ_MSG (res, (char**) expected,
+                             "moo_strnsplit_lines(%s, %d)",
+                             TEST_FMT_STR (s), (int) len);
+    TEST_ASSERT_INT_EQ (n_toks, res ? g_strv_length (res) : 0);
+    g_strfreev (res);
 }
+
+static void
+test_moo_splitlines (void)
+{
+    guint i;
+
+    struct {
+        const char *s;
+        gssize len;
+        char const *toks[10];
+    } cases[] = {
+        { "abcd", -1, {"abcd", NULL} },
+        { "abcd\n", -1, {"abcd", "", NULL} },
+        { "abcd\r\n", -1, {"abcd", "", NULL} },
+        { "\rabcd\n", -1, {"", "abcd", "", NULL} },
+        { "\r\nabcd\n", -1, {"", "abcd", "", NULL} },
+        { "abcd\nabc", -1, {"abcd", "abc", NULL} },
+        { "abcd\n", 4, {"abcd", NULL} },
+        { "a\nb\rc\r\nd\xe2\x80\xa9""e", -1, {"a", "b", "c", "d", "e", NULL} },
+        { "a\xe2\x80\xa9""b", 1, {"a", NULL} },
+        { "a\xe2\x80\xa9""b", 3, {"a\xe2\x80", NULL} },
+        { "a\xe2\x80\xa9""b", 4, {"a", "", NULL} },
+        { "a\xe2\x80\xa9""b", 5, {"a", "b", NULL} }
+    };
+
+    struct {
+        const char *s;
+        gssize len;
+    } nulls[] = {
+        { NULL, -1 },
+        { "", -1 },
+        { "", 0 },
+        { "abcd", 0 }
+    };
+
+    for (i = 0; i < G_N_ELEMENTS (cases); ++i)
+        test_strv_one (cases[i].s, cases[i].len, cases[i].toks);
+
+    for (i = 0; i < G_N_ELEMENTS (nulls); ++i)
+        test_strv_one (nulls[i].s, nulls[i].len, NULL);
+}
+
+
+void
+moo_test_mooutils_misc (void)
+{
+    MooTestSuite *suite;
+
+    suite = moo_test_suite_new ("mooutils/mooutils-misc.c", NULL, NULL, NULL);
+
+    moo_test_suite_add_test (suite, "test of moo_splitlines()",
+                             (MooTestFunc) test_moo_splitlines, NULL);
+}
+
+#endif
