@@ -1,20 +1,23 @@
 /*
  *   mooutils-misc.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOO_UTILS_MISC_H
 #define MOO_UTILS_MISC_H
 
 #include <gtk/gtk.h>
-#include <string.h>
+#include <mooutils/mooutils-messages.h>
 
 G_BEGIN_DECLS
 
@@ -42,22 +45,19 @@ void        moo_set_log_func_file           (const char     *log_file);
 void        moo_set_log_func_silent         (void);
 void        moo_reset_log_func              (void);
 
-void        moo_segfault                    (void);
+void MOO_NORETURN moo_segfault              (void);
+void MOO_NORETURN moo_abort                 (void);
 
 void        moo_disable_win32_error_message (void);
 void        moo_enable_win32_error_message  (void);
 
-#define MOO_TYPE_DATA_DIR_TYPE              (moo_data_dir_type_get_type ())
-
 typedef enum {
     MOO_DATA_SHARE,
     MOO_DATA_LIB
 } MooDataDirType;
 
-GType       moo_data_dir_type_get_type      (void) G_GNUC_CONST;
-
 void       _moo_set_app_instance_name       (const char     *name);
-void       _moo_set_user_data_dir           (const char     *path);
+void        moo_set_user_data_dir           (const char     *path);
 
 /* ~/.appname */
 gboolean    moo_make_user_data_dir          (const char     *path);
@@ -68,15 +68,21 @@ gboolean    moo_save_user_data_file         (const char     *basename,
                                              const char     *content,
                                              gssize          len,
                                              GError        **error);
+char       *moo_get_user_cache_dir          (void);
 char       *moo_get_user_cache_file         (const char     *basename);
 gboolean    moo_save_user_cache_file        (const char     *basename,
                                              const char     *content,
                                              gssize          len,
                                              GError        **error);
+gboolean    moo_save_config_file            (const char     *filename,
+                                             const char     *content,
+                                             gssize          len,
+                                             GError        **error);
 
 /* user data comes first; MOO_DATA_DIR comes last */
 /* $MOO_APP_DIR:$MOO_DATA_DIRS:$prefix/share/appname or
    $MOO_APP_DIR:$MOO_LIB_DIRS:$prefix/lib/appname */
+void        moo_add_data_dirs               (char          **dirs);
 char      **moo_get_data_dirs               (MooDataDirType  type,
                                              guint          *n_dirs);
 char      **moo_get_data_subdirs            (const char     *subdir,
@@ -88,13 +94,11 @@ char       *moo_tempnam                     (void);
 void        moo_cleanup                     (void);
 
 const char *moo_get_locale_dir              (void);
-const char *const *_moo_get_shared_data_dirs (void);
 
-
-void       _moo_selection_data_set_pointer  (GtkSelectionData *data,
+void        moo_selection_data_set_pointer  (GtkSelectionData *data,
                                              GdkAtom         type,
                                              gpointer        ptr);
-gpointer   _moo_selection_data_get_pointer  (GtkSelectionData *data,
+gpointer    moo_selection_data_get_pointer  (GtkSelectionData *data,
                                              GdkAtom         type);
 
 GdkModifierType _moo_get_modifiers          (GtkWidget      *widget);
@@ -108,24 +112,29 @@ void       _moo_menu_item_set_label         (GtkWidget      *menu_item,
 void       _moo_widget_set_tooltip          (GtkWidget      *widget,
                                              const char     *tip);
 
+typedef struct {
+    const char *text;
+    gsize len;
+} MooLineReader;
+void        moo_line_reader_init            (MooLineReader  *lr,
+                                             const char     *text,
+                                             gssize          len);
+const char *moo_line_reader_get_line        (MooLineReader  *lr,
+                                             gsize          *line_len,
+                                             gsize          *lt_len);
+gboolean    moo_find_line_end               (const char     *string,
+                                             gssize          len,
+                                             gsize          *le_start,
+                                             gsize          *le_len);
 char      **moo_strnsplit_lines             (const char     *string,
                                              gssize          len,
                                              guint          *n_tokens);
 char      **moo_splitlines                  (const char     *string);
-char     **_moo_ascii_strnsplit             (const char     *string,
-                                             gssize          len,
-                                             guint           n_tokens);
+
 char     **_moo_strv_reverse                (char          **str_array);
 
-static inline gboolean
-_moo_str_equal_inline (const char *s1,
-                       const char *s2)
-{
-    return !strcmp (s1 ? s1 : "", s2 ? s2 : "");
-}
-
-#define _moo_str_equal(s1, s2) (_moo_str_equal_inline ((s1), (s2)))
-
+gboolean   _moo_str_equal                   (const char     *s1,
+                                             const char     *s2);
 
 static inline void
 moo_assign_string (char       **where,
@@ -138,11 +147,22 @@ moo_assign_string (char       **where,
 
 #define MOO_ASSIGN_STRING(where, value) moo_assign_string (&(where), (value))
 
+static inline void
+moo_assign_strv (char ***where,
+                 char  **value)
+{
+    char **tmp = *where;
+    *where = g_strdupv (value);
+    g_strfreev (tmp);
+}
+
+#define MOO_ASSIGN_STRV(where, value) moo_assign_strv (&(where), (value))
+
 const char *_moo_get_pid_string             (void);
 
 const char *_moo_intern_string              (const char     *string);
 
-guint       _moo_idle_add_full              (gint            priority,
+guint        moo_idle_add_full              (gint            priority,
                                              GSourceFunc     function,
                                              gpointer        data,
                                              GDestroyNotify  notify);
@@ -174,21 +194,45 @@ gboolean    _moo_regex_escape               (const char *string,
 
 
 #if GLIB_CHECK_VERSION(2,10,0)
-#define _moo_new            g_slice_new
-#define _moo_new0           g_slice_new0
-#define _moo_free           g_slice_free
+# define moo_alloc_block(sz)   g_slice_alloc (sz)
+# define moo_alloc0_block(sz)  g_slice_alloc0 (sz)
+# define moo_free_block(sz,p)  g_slice_free1 (sz, p)
 #else
-#define _moo_new(type)      g_new (type, 1)
-#define _moo_new0(type)     g_new0 (type, 1)
-#define _moo_free(type,mem) g_free (mem)
+# define moo_alloc_block(sz)   g_malloc (sz)
+# define moo_alloc0_block(sz)  g_malloc0 (sz)
+# define moo_free_block(sz,p)  g_free (p)
 #endif
 
+#ifdef g_slice_dup
+# define moo_dup_block(sz,p)   g_slice_copy (sz, p)
+#else
+# define moo_dup_block(sz,p)   g_memdup (p, sz)
+#endif
+
+#define moo_new_n(type, n)      ((type*) moo_alloc_block (sizeof (type) * (n)))
+#define moo_new0_n(type, n)     ((type*) moo_alloc0_block (sizeof (type) * (n)))
+#define moo_free_n(type, n, p)  do { type *p__ = p; moo_free_block (sizeof (type) * n, p__); } while (0)
+
+#define moo_new(type)           moo_new_n (type, 1)
+#define moo_new0(type)          moo_new0_n (type, 1)
+#define moo_free(type,p)        moo_free_n (type, 1, p)
+
+#if defined(MOO_CL_GCC)
+#define moo_obj_dup(type,p)     ({ type const *cp__ = p; type *p__ = moo_dup_block (sizeof (type), cp__); p__; })
+#else
+#define moo_obj_dup(type, p)    ((type*) moo_dup_block (sizeof (type), p))
+#endif
+
+#define MOO_OBJECT_REF_SINK moo_object_ref_sink
+static inline gpointer moo_object_ref_sink (gpointer obj)
+{
 #if GLIB_CHECK_VERSION(2,10,0)
-#define MOO_OBJECT_REF_SINK(obj) g_object_ref_sink (obj)
+    g_object_ref_sink (obj);
 #else
-#define MOO_OBJECT_REF_SINK(obj) gtk_object_sink (g_object_ref (obj))
+    gtk_object_sink (g_object_ref (obj));
 #endif
-
+    return obj;
+}
 
 G_END_DECLS
 
@@ -196,16 +240,12 @@ G_END_DECLS
 #ifdef G_OS_WIN32
 #include <gtk/gtk.h>
 #include <string.h>
-#include <sys/time.h>
 
 G_BEGIN_DECLS
 
 
-#define fnmatch _moo_win32_fnmatch
-#define gettimeofday _moo_win32_gettimeofday
-
-char       *moo_win32_get_app_dir           (void);
-char       *moo_win32_get_dll_dir           (const char     *dll);
+char        *moo_win32_get_app_dir          (void);
+char        *moo_win32_get_dll_dir          (const char     *dll);
 
 void        _moo_win32_add_data_dirs        (GPtrArray      *list,
                                              const char     *prefix);
@@ -216,24 +256,14 @@ gboolean    _moo_win32_open_uri             (const char     *uri);
 void        _moo_win32_show_fatal_error     (const char     *domain,
                                              const char     *logmsg);
 
-int         _moo_win32_fnmatch              (const char     *pattern,
-                                             const char     *string,
-                                             int             flags);
-int         _moo_win32_gettimeofday         (struct timeval *tp,
-                                             gpointer        tzp);
+char      **_moo_win32_lame_parse_cmd_line  (const char     *cmd_line,
+                                             GError        **error);
 
-void       *_moo_win32_mmap                 (gpointer        start,
-                                             guint64         length,
-                                             int             prot,
-                                             int             flags,
-                                             int             fd,
-                                             guint64         offset);
-int         _moo_win32_munmap               (gpointer        start,
-                                             gsize           length);
-
-#define PROT_READ   1
-#define MAP_SHARED  1
-#define MAP_FAILED  ((gpointer) -1)
+int         _moo_win32_message_box          (GtkWidget      *parent,
+                                             guint           type,
+                                             const char     *title,
+                                             const char     *format,
+                                             ...) G_GNUC_PRINTF (4, 5);
 
 
 G_END_DECLS
