@@ -3,18 +3,21 @@
  *
  *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/mooutils-treeview.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-gobject.h"
 #include "mooutils/mootype-macros.h"
-#include "mooutils/moomarshals.h"
+#include "marshals.h"
 #include <string.h>
 #include <gobject/gvaluecollector.h>
 
@@ -380,7 +383,7 @@ _moo_tree_helper_set_modified (MooTreeHelper *helper,
 {
     g_return_if_fail (MOO_IS_TREE_HELPER (helper));
 
-#ifdef MOO_DEBUG_ENABLED
+#ifdef MOO_DEBUG
     if (!helper->modified && modified)
     {
         g_message ("%s: helper modified", G_STRLOC);
@@ -1114,7 +1117,7 @@ expander_data_free (ExpanderData *data)
         if (data->cell)
             g_object_unref (data->cell);
         expander_data_disconnect_model (data);
-        _moo_free (ExpanderData, data);
+        moo_free (ExpanderData, data);
     }
 }
 
@@ -1215,7 +1218,7 @@ _moo_tree_view_setup_expander (GtkTreeView       *tree_view,
                                              (GtkTreeCellDataFunc) expander_cell_data_func,
                                              tree_view, NULL);
 
-    data = _moo_new0 (ExpanderData);
+    data = moo_new0 (ExpanderData);
     data->column = g_object_ref (column);
     data->cell = g_object_ref (cell);
     g_object_set_data_full (G_OBJECT (tree_view), "moo-tree-view-expander-data",
