@@ -1,13 +1,16 @@
 /*
  *   moowindow.h
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef MOOUI_MOOWINDOW_H
@@ -16,7 +19,7 @@
 #include <mooutils/mooutils-gobject.h>
 #include <mooutils/moouixml.h>
 #include <mooutils/mooactioncollection.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
@@ -43,6 +46,8 @@ struct _MooWindow
     GtkWidget           *menubar;
     GtkWidget           *toolbar;
     GtkWidget           *vbox;
+    GtkWidget           *status_area;
+    GtkStatusbar        *statusbar;
 };
 
 struct _MooWindowClass
@@ -50,7 +55,9 @@ struct _MooWindowClass
     GtkWindowClass      parent_class;
 
     /* signals */
-    gboolean (*close)   (MooWindow *window);
+    gboolean (*close)       (MooWindow *window);
+
+    void     (*apply_prefs) (MooWindow *window);
 };
 
 typedef GtkAction *(*MooWindowActionFunc) (MooWindow *window,
@@ -59,8 +66,13 @@ typedef GtkAction *(*MooWindowActionFunc) (MooWindow *window,
 
 GType       moo_window_get_type             (void) G_GNUC_CONST;
 
-gboolean    moo_window_close                (MooWindow          *window);
+gboolean    moo_window_close                (MooWindow  *window);
+void        moo_window_apply_prefs          (MooWindow  *window);
+void        moo_window_message              (MooWindow  *window,
+                                             const char *text);
 
+void        moo_window_set_edit_ops_widget  (MooWindow  *window,
+                                             GtkWidget  *widget);
 
 /*****************************************************************************/
 /* Actions
@@ -103,9 +115,9 @@ gboolean    moo_window_class_find_group     (MooWindowClass     *klass,
 void        moo_window_class_remove_group   (MooWindowClass     *klass,
                                              const char         *name);
 
-MooUIXML   *moo_window_get_ui_xml           (MooWindow          *window);
+MooUiXml   *moo_window_get_ui_xml           (MooWindow          *window);
 void        moo_window_set_ui_xml           (MooWindow          *window,
-                                             MooUIXML           *xml);
+                                             MooUiXml           *xml);
 
 MooActionCollection *moo_window_get_actions (MooWindow          *window);
 GtkAction  *moo_window_get_action           (MooWindow          *window,
