@@ -25,7 +25,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <stdio.h>
@@ -39,18 +39,18 @@
 
 #include <netinet/in.h> /* for ntohl/ntohs */
 
-#ifdef HAVE_MMAP
-#include <sys/mman.h>
-#else
-#warning Building xdgmime without MMAP support. Binary "mime.info" cache files will not be used.
-#endif
-
 #include <sys/stat.h>
 #include <sys/types.h>
 
 #include "xdgmimecache.h"
 #include "xdgmimeint.h"
 
+#ifdef HAVE_MMAP
+#include <sys/mman.h>
+#else
+#warning Building xdgmime without MMAP support. Binary "mime.info" cache files will not be used.
+#endif
+
 #ifndef MAX
 #define MAX(a,b) ((a) > (b) ? (a) : (b))
 #endif
@@ -72,7 +72,7 @@
 #endif
 
 #define MAJOR_VERSION 1
-#define MINOR_VERSION 0
+#define MINOR_VERSION 1
 
 struct _XdgMimeCache
 {
@@ -121,7 +121,7 @@ _xdg_mime_cache_new_from_file (const char *file_name)
 
   if (fd < 0)
     return NULL;
-  
+
   if (XDG_MIME_FSTAT (fd, &st) < 0 || st.st_size < 4)
     goto done;
 
@@ -138,7 +138,7 @@ _xdg_mime_cache_new_from_file (const char *file_name)
 
       goto done;
     }
-  
+
   cache = (XdgMimeCache *) malloc (sizeof (XdgMimeCache));
   cache->ref_count = 1;
   cache->buffer = buffer;
@@ -154,7 +154,7 @@ _xdg_mime_cache_new_from_file (const char *file_name)
 }
 
 static int
-cache_magic_matchlet_compare_to_data (XdgMimeCache *cache, 
+cache_magic_matchlet_compare_to_data (XdgMimeCache *cache,
 				      xdg_uint32_t  offset,
 				      const void   *data,
 				      size_t        len)
@@ -164,13 +164,13 @@ cache_magic_matchlet_compare_to_data (XdgMimeCache *cache,
   xdg_uint32_t data_length = GET_UINT32 (cache->buffer, offset + 12);
   xdg_uint32_t data_offset = GET_UINT32 (cache->buffer, offset + 16);
   xdg_uint32_t mask_offset = GET_UINT32 (cache->buffer, offset + 20);
-  
+
   int i, j;
 
   for (i = range_start; i <= range_start + range_length; i++)
     {
       int valid_matchlet = TRUE;
-      
+
       if (i + data_length > len)
 	return FALSE;
 
@@ -197,16 +197,16 @@ cache_magic_matchlet_compare_to_data (XdgMimeCache *cache,
 		}
 	    }
 	}
-      
+
       if (valid_matchlet)
 	return TRUE;
     }
-  
-  return FALSE;  
+
+  return FALSE;
 }
 
 static int
-cache_magic_matchlet_compare (XdgMimeCache *cache, 
+cache_magic_matchlet_compare (XdgMimeCache *cache,
 			      xdg_uint32_t  offset,
 			      const void   *data,
 			      size_t        len)
@@ -215,12 +215,12 @@ cache_magic_matchlet_compare (XdgMimeCache *cache,
   xdg_uint32_t child_offset = GET_UINT32 (cache->buffer, offset + 28);
 
   int i;
-  
+
   if (cache_magic_matchlet_compare_to_data (cache, offset, data, len))
     {
       if (n_children == 0)
 	return TRUE;
-      
+
       for (i = 0; i < n_children; i++)
 	{
 	  if (cache_magic_matchlet_compare (cache, child_offset + 32 * i,
@@ -228,15 +228,15 @@ cache_magic_matchlet_compare (XdgMimeCache *cache,
 	    return TRUE;
 	}
     }
-  
-  return FALSE;  
+
+  return FALSE;
 }
 
 static const char *
-cache_magic_compare_to_data (XdgMimeCache *cache, 
+cache_magic_compare_to_data (XdgMimeCache *cache,
 			     xdg_uint32_t  offset,
-			     const void   *data, 
-			     size_t        len, 
+			     const void   *data,
+			     size_t        len,
 			     int          *prio)
 {
   xdg_uint32_t priority = GET_UINT32 (cache->buffer, offset);
@@ -248,11 +248,11 @@ cache_magic_compare_to_data (XdgMimeCache *cache,
 
   for (i = 0; i < n_matchlets; i++)
     {
-      if (cache_magic_matchlet_compare (cache, matchlet_offset + i * 32, 
+      if (cache_magic_matchlet_compare (cache, matchlet_offset + i * 32,
 					data, len))
 	{
 	  *prio = priority;
-	  
+
 	  return cache->buffer + mimetype_offset;
 	}
     }
@@ -261,9 +261,9 @@ cache_magic_compare_to_data (XdgMimeCache *cache,
 }
 
 static const char *
-cache_magic_lookup_data (XdgMimeCache *cache, 
-			 const void   *data, 
-			 size_t        len, 
+cache_magic_lookup_data (XdgMimeCache *cache,
+			 const void   *data,
+			 size_t        len,
 			 int          *prio,
 			 const char   *mime_types[],
 			 int           n_mime_types)
@@ -279,12 +279,12 @@ cache_magic_lookup_data (XdgMimeCache *cache,
   list_offset = GET_UINT32 (cache->buffer, 24);
   n_entries = GET_UINT32 (cache->buffer, list_offset);
   offset = GET_UINT32 (cache->buffer, list_offset + 8);
-  
+
   for (j = 0; j < n_entries; j++)
     {
       const char *match;
 
-      match = cache_magic_compare_to_data (cache, offset + 16 * j, 
+      match = cache_magic_compare_to_data (cache, offset + 16 * j,
 					   data, len, prio);
       if (match)
 	return match;
@@ -292,14 +292,14 @@ cache_magic_lookup_data (XdgMimeCache *cache,
 	{
 	  xdg_uint32_t mimetype_offset;
 	  const char *non_match;
-	  
+
 	  mimetype_offset = GET_UINT32 (cache->buffer, offset + 16 * j + 4);
 	  non_match = cache->buffer + mimetype_offset;
 
 	  for (n = 0; n < n_mime_types; n++)
 	    {
-	      if (mime_types[n] && 
-		  xdg_mime_mime_type_equal (mime_types[n], non_match))
+	      if (mime_types[n] &&
+		  _xdg_mime_mime_type_equal (mime_types[n], non_match))
 		mime_types[n] = NULL;
 	    }
 	}
@@ -321,16 +321,16 @@ cache_alias_lookup (const char *alias)
       xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
       xdg_uint32_t offset;
 
-      min = 0; 
+      min = 0;
       max = n_entries - 1;
-      while (max >= min) 
+      while (max >= min)
 	{
 	  mid = (min + max) / 2;
 
 	  offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * mid);
 	  ptr = cache->buffer + offset;
 	  cmp = strcmp (ptr, alias);
-	  
+
 	  if (cmp < 0)
 	    min = mid + 1;
 	  else if (cmp > 0)
@@ -346,9 +346,15 @@ cache_alias_lookup (const char *alias)
   return NULL;
 }
 
+typedef struct {
+  const char *mime;
+  int weight;
+} MimeWeight;
+
 static int
 cache_glob_lookup_literal (const char *file_name,
-			   const char *mime_types[])
+			   const char *mime_types[],
+			   int         n_mime_types)
 {
   const char *ptr;
   int i, min, max, mid, cmp;
@@ -360,25 +366,25 @@ cache_glob_lookup_literal (const char *file_name,
       xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
       xdg_uint32_t offset;
 
-      min = 0; 
+      min = 0;
       max = n_entries - 1;
-      while (max >= min) 
+      while (max >= min)
 	{
 	  mid = (min + max) / 2;
 
-	  offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * mid);
+	  offset = GET_UINT32 (cache->buffer, list_offset + 4 + 12 * mid);
 	  ptr = cache->buffer + offset;
 	  cmp = strcmp (ptr, file_name);
-	  
+
 	  if (cmp < 0)
 	    min = mid + 1;
 	  else if (cmp > 0)
 	    max = mid - 1;
 	  else
 	    {
-	      offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * mid + 4);
+	      offset = GET_UINT32 (cache->buffer, list_offset + 4 + 12 * mid + 4);
 	      mime_types[0] = (const char *)(cache->buffer + offset);
-	      
+
 	      return 1;
 	    }
 	}
@@ -389,7 +395,7 @@ cache_glob_lookup_literal (const char *file_name,
 
 static int
 cache_glob_lookup_fnmatch (const char *file_name,
-			   const char *mime_types[],
+			   MimeWeight  mime_types[],
 			   int         n_mime_types)
 {
   const char *mime_type;
@@ -407,105 +413,109 @@ cache_glob_lookup_fnmatch (const char *file_name,
 
       for (j = 0; j < n_entries && n < n_mime_types; j++)
 	{
-	  xdg_uint32_t offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * j);
-	  xdg_uint32_t mimetype_offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * j + 4);
+	  xdg_uint32_t offset = GET_UINT32 (cache->buffer, list_offset + 4 + 12 * j);
+	  xdg_uint32_t mimetype_offset = GET_UINT32 (cache->buffer, list_offset + 4 + 12 * j + 4);
+	  int weight = GET_UINT32 (cache->buffer, list_offset + 4 + 12 * j + 8);
 	  ptr = cache->buffer + offset;
 	  mime_type = cache->buffer + mimetype_offset;
 
 	  /* FIXME: Not UTF-8 safe */
 	  if (fnmatch (ptr, file_name, 0) == 0)
-	    mime_types[n++] = mime_type;
+	    {
+	      mime_types[n].mime = mime_type;
+	      mime_types[n].weight = weight;
+	      n++;
+	    }
 	}
 
       if (n > 0)
 	return n;
     }
-  
+
   return 0;
 }
 
 static int
-cache_glob_node_lookup_suffix (XdgMimeCache *cache,
-			       xdg_uint32_t  n_entries,
-			       xdg_uint32_t  offset,
-			       const char   *suffix, 
-			       int           ignore_case,
-			       const char   *mime_types[],
-			       int           n_mime_types)
+cache_glob_node_lookup_suffix (XdgMimeCache  *cache,
+			       xdg_uint32_t   n_entries,
+			       xdg_uint32_t   offset,
+			       xdg_unichar_t *file_name,
+			       int            len,
+			       int            ignore_case,
+			       MimeWeight     mime_types[],
+			       int            n_mime_types)
 {
   xdg_unichar_t character;
   xdg_unichar_t match_char;
   xdg_uint32_t mimetype_offset;
   xdg_uint32_t n_children;
-  xdg_uint32_t child_offset; 
+  xdg_uint32_t child_offset;
+  int weight;
 
   int min, max, mid, n, i;
 
-  character = _xdg_utf8_to_ucs4 (suffix);
+  character = file_name[len - 1];
   if (ignore_case)
     character = _xdg_ucs4_to_lower (character);
 
+  assert (character != 0);
+
   min = 0;
   max = n_entries - 1;
   while (max >= min)
     {
       mid = (min + max) /  2;
-
-      match_char = GET_UINT32 (cache->buffer, offset + 16 * mid);
-
+      match_char = GET_UINT32 (cache->buffer, offset + 12 * mid);
       if (match_char < character)
 	min = mid + 1;
       else if (match_char > character)
 	max = mid - 1;
-      else 
+      else
 	{
-	  suffix = _xdg_utf8_next_char (suffix);
-	  if (*suffix == '\0')
-	    {
-	      mimetype_offset = GET_UINT32 (cache->buffer, offset + 16 * mid + 4);
-	      n = 0;
-              if (cache->buffer[mimetype_offset])
-                mime_types[n++] = cache->buffer + mimetype_offset;
+          len--;
+          n = 0;
+          n_children = GET_UINT32 (cache->buffer, offset + 12 * mid + 4);
+          child_offset = GET_UINT32 (cache->buffer, offset + 12 * mid + 8);
 
-	      n_children = GET_UINT32 (cache->buffer, offset + 16 * mid + 8);
-	      child_offset = GET_UINT32 (cache->buffer, offset + 16 * mid + 12);
+          if (len > 0)
+            {
+              n = cache_glob_node_lookup_suffix (cache,
+                                                 n_children, child_offset,
+                                                 file_name, len,
+                                                 ignore_case,
+                                                 mime_types,
+                                                 n_mime_types);
+            }
+          if (n == 0)
+            {
 	      i = 0;
 	      while (n < n_mime_types && i < n_children)
 		{
-		  match_char = GET_UINT32 (cache->buffer, child_offset + 16 * i);
-		  mimetype_offset = GET_UINT32 (cache->buffer, offset + 16 * i + 4);
+		  match_char = GET_UINT32 (cache->buffer, child_offset + 12 * i);
 		  if (match_char != 0)
 		    break;
 
-		  if (cache->buffer[mimetype_offset])
-		    mime_types[n++] = cache->buffer + mimetype_offset;
+		  mimetype_offset = GET_UINT32 (cache->buffer, child_offset + 12 * i + 4);
+		  weight = GET_UINT32 (cache->buffer, child_offset + 12 * i + 8);
+
+		  mime_types[n].mime = cache->buffer + mimetype_offset;
+		  mime_types[n].weight = weight;
+		  n++;
 		  i++;
 		}
-
-	      return n;
-	    }
-	  else
-	    {
-	      n_children = GET_UINT32 (cache->buffer, offset + 16 * mid + 8);
-	      child_offset = GET_UINT32 (cache->buffer, offset + 16 * mid + 12);
-      
-	      return cache_glob_node_lookup_suffix (cache, 
-						    n_children, child_offset,
-						    suffix, ignore_case,
-						    mime_types,
-						    n_mime_types);
 	    }
+	  return n;
 	}
     }
-
   return 0;
 }
 
 static int
-cache_glob_lookup_suffix (const char *suffix, 
-			  int         ignore_case,
-			  const char *mime_types[],
-			  int         n_mime_types)
+cache_glob_lookup_suffix (xdg_unichar_t *file_name,
+			  int            len,
+			  int            ignore_case,
+			  MimeWeight     mime_types[],
+			  int            n_mime_types)
 {
   int i, n;
 
@@ -517,9 +527,10 @@ cache_glob_lookup_suffix (const char *suffix,
       xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
       xdg_uint32_t offset = GET_UINT32 (cache->buffer, list_offset + 4);
 
-      n = cache_glob_node_lookup_suffix (cache, 
-					 n_entries, offset, 
-					 suffix, ignore_case,
+      n = cache_glob_node_lookup_suffix (cache,
+					 n_entries, offset,
+					 file_name, len,
+					 ignore_case,
 					 mime_types,
 					 n_mime_types);
       if (n > 0)
@@ -529,78 +540,53 @@ cache_glob_lookup_suffix (const char *suffix,
   return 0;
 }
 
-static void
-find_stopchars (char *stopchars)
+static int compare_mime_weight (const void *a, const void *b)
 {
-  int i, j, k, l;
- 
-  k = 0;
-  for (i = 0; _xdg_mime_caches[i]; i++)
-    {
-      XdgMimeCache *cache = _xdg_mime_caches[i];
+  const MimeWeight *aa = (const MimeWeight *)a;
+  const MimeWeight *bb = (const MimeWeight *)b;
 
-      xdg_uint32_t list_offset = GET_UINT32 (cache->buffer, 16);
-      xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
-      xdg_uint32_t offset = GET_UINT32 (cache->buffer, list_offset + 4);
-
-      for (j = 0; j < n_entries; j++)
-	{
-	  xdg_uint32_t match_char = GET_UINT32 (cache->buffer, offset);
-	  
-	  if (match_char < 128)
-	    {
-	      for (l = 0; l < k; l++)
-		if (stopchars[l] == match_char)
-		  break;
-	      if (l == k)
-		{
-		  stopchars[k] = (char) match_char;
-		  k++;
-		}
-	    }
-
-	  offset += 16;
-	}
-    }
-
-  stopchars[k] = '\0';
+  return aa->weight - bb->weight;
 }
 
 static int
-cache_glob_lookup_file_name (const char *file_name, 
+cache_glob_lookup_file_name (const char *file_name,
 			     const char *mime_types[],
 			     int         n_mime_types)
 {
-  const char *ptr;
-  char stopchars[128];
   int n;
-  
-  assert (file_name != NULL);
+  MimeWeight mimes[10];
+  int n_mimes = 10;
+  int i;
+  xdg_unichar_t *ucs4;
+  int len;
+
+  assert (file_name != NULL && n_mime_types > 0);
 
   /* First, check the literals */
-  n = cache_glob_lookup_literal (file_name, mime_types);
+  n = cache_glob_lookup_literal (file_name, mime_types, n_mime_types);
   if (n > 0)
     return n;
 
-  find_stopchars (stopchars);
+  ucs4 = _xdg_convert_to_ucs4 (file_name, &len);
+  n = cache_glob_lookup_suffix (ucs4, len, FALSE, mimes, n_mimes);
 
-  /* Next, check suffixes */
-  ptr = strpbrk (file_name, stopchars);
-  while (ptr)
-    {
-      n = cache_glob_lookup_suffix (ptr, FALSE, mime_types, n_mime_types);
-      if (n > 0)
-	return n;
-      
-      n = cache_glob_lookup_suffix (ptr, TRUE, mime_types, n_mime_types);
-      if (n > 0)
-	return n;
+  if (n == 0)
+    n = cache_glob_lookup_suffix (ucs4, len, TRUE, mimes, n_mimes);
+  free(ucs4);
 
-      ptr = strpbrk (ptr + 1, stopchars);
-    }
-  
   /* Last, try fnmatch */
-  return cache_glob_lookup_fnmatch (file_name, mime_types, n_mime_types);
+  if (n == 0)
+    n = cache_glob_lookup_fnmatch (file_name, mimes, n_mimes);
+
+  qsort (mimes, n, sizeof (MimeWeight), compare_mime_weight);
+
+  if (n_mime_types < n)
+    n = n_mime_types;
+
+  for (i = 0; i < n; i++)
+    mime_types[i] = mimes[i].mime;
+
+  return n;
 }
 
 int
@@ -625,6 +611,7 @@ _xdg_mime_cache_get_max_buffer_extents (void)
 static const char *
 cache_get_mime_type_for_data (const void *data,
 			      size_t      len,
+			      int        *result_prio,
 			      const char *mime_types[],
 			      int         n_mime_types)
 {
@@ -640,7 +627,7 @@ cache_get_mime_type_for_data (const void *data,
       int prio;
       const char *match;
 
-      match = cache_magic_lookup_data (cache, data, len, &prio, 
+      match = cache_magic_lookup_data (cache, data, len, &prio,
 				       mime_types, n_mime_types);
       if (prio > priority)
 	{
@@ -649,11 +636,15 @@ cache_get_mime_type_for_data (const void *data,
 	}
     }
 
+  if (result_prio)
+    *result_prio = priority;
+
   if (priority > 0)
     return mime_type;
 
   for (n = 0; n < n_mime_types; n++)
     {
+
       if (mime_types[n])
 	return mime_types[n];
     }
@@ -663,9 +654,10 @@ cache_get_mime_type_for_data (const void *data,
 
 const char *
 _xdg_mime_cache_get_mime_type_for_data (const void *data,
-					size_t      len)
+					size_t      len,
+					int        *result_prio)
 {
-  return cache_get_mime_type_for_data (data, len, NULL, 0);
+  return cache_get_mime_type_for_data (data, len, result_prio, NULL, 0);
 }
 
 const char *
@@ -673,7 +665,7 @@ _xdg_mime_cache_get_mime_type_for_file (const char  *file_name,
 					struct stat *statbuf)
 {
   const char *mime_type;
-  const char *mime_types[2];
+  const char *mime_types[10];
   FILE *file;
   unsigned char *data;
   int max_extent;
@@ -683,13 +675,13 @@ _xdg_mime_cache_get_mime_type_for_file (const char  *file_name,
   int n;
 
   if (file_name == NULL)
-    return XDG_MIME_TYPE_UNKNOWN;
+    return NULL;
 
   if (! _xdg_utf8_validate (file_name))
-    return XDG_MIME_TYPE_UNKNOWN;
+    return NULL;
 
   base_name = _xdg_get_base_name (file_name);
-  n = cache_glob_lookup_file_name (base_name, mime_types, 2);
+  n = cache_glob_lookup_file_name (base_name, mime_types, 10);
 
   if (n == 1)
     return mime_types[0];
@@ -712,7 +704,7 @@ _xdg_mime_cache_get_mime_type_for_file (const char  *file_name,
   data = malloc (max_extent);
   if (data == NULL)
     return XDG_MIME_TYPE_UNKNOWN;
-        
+
   file = fopen (file_name, "rb");
   if (file == NULL)
     {
@@ -728,13 +720,9 @@ _xdg_mime_cache_get_mime_type_for_file (const char  *file_name,
       return XDG_MIME_TYPE_UNKNOWN;
     }
 
-  mime_type = cache_get_mime_type_for_data (data, bytes_read,
+  mime_type = cache_get_mime_type_for_data (data, bytes_read, NULL,
 					    mime_types, n);
 
-  if ((!mime_type || mime_type == XDG_MIME_TYPE_UNKNOWN) &&
-      _xdg_mime_buffer_is_text (data, bytes_read))
-    mime_type = "text/plain";
-
   free (data);
   fclose (file);
 
@@ -744,14 +732,22 @@ _xdg_mime_cache_get_mime_type_for_file (const char  *file_name,
 const char *
 _xdg_mime_cache_get_mime_type_from_file_name (const char *file_name)
 {
-  const char *mime_types[2];
+  const char *mime_type;
 
-  if (cache_glob_lookup_file_name (file_name, mime_types, 2) == 1)
-    return mime_types[0];
+  if (cache_glob_lookup_file_name (file_name, &mime_type, 1))
+    return mime_type;
   else
     return XDG_MIME_TYPE_UNKNOWN;
 }
 
+int
+_xdg_mime_cache_get_mime_types_from_file_name (const char *file_name,
+					       const char  *mime_types[],
+					       int          n_mime_types)
+{
+  return cache_glob_lookup_file_name (file_name, mime_types, n_mime_types);
+}
+
 #if 1
 static int
 is_super_type (const char *mime)
@@ -776,7 +772,7 @@ _xdg_mime_cache_mime_type_subclass (const char *mime,
   const char *umime, *ubase;
 
   int i, j, min, max, med, cmp;
-  
+
   umime = _xdg_mime_cache_unalias_mime_type (mime);
   ubase = _xdg_mime_cache_unalias_mime_type (base);
 
@@ -794,27 +790,27 @@ _xdg_mime_cache_mime_type_subclass (const char *mime,
 #endif
 
   /*  Handle special cases text/plain and application/octet-stream */
-  if (strcmp (ubase, "text/plain") == 0 && 
+  if (strcmp (ubase, "text/plain") == 0 &&
       strncmp (umime, "text/", 5) == 0)
     return 1;
 
   if (strcmp (ubase, "application/octet-stream") == 0)
     return 1;
- 
+
   for (i = 0; _xdg_mime_caches[i]; i++)
     {
       XdgMimeCache *cache = _xdg_mime_caches[i];
-      
+
       xdg_uint32_t list_offset = GET_UINT32 (cache->buffer, 8);
       xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
       xdg_uint32_t offset, n_parents, parent_offset;
 
-      min = 0; 
+      min = 0;
       max = n_entries - 1;
       while (max >= min)
 	{
 	  med = (min + max)/2;
-	  
+
 	  offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * med);
 	  cmp = strcmp (cache->buffer + offset, umime);
 	  if (cmp < 0)
@@ -825,7 +821,7 @@ _xdg_mime_cache_mime_type_subclass (const char *mime,
 	    {
 	      offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * med + 4);
 	      n_parents = GET_UINT32 (cache->buffer, offset);
-	      
+
 	      for (j = 0; j < n_parents; j++)
 		{
 		  parent_offset = GET_UINT32 (cache->buffer, offset + 4 + 4 * j);
@@ -845,20 +841,20 @@ const char *
 _xdg_mime_cache_unalias_mime_type (const char *mime)
 {
   const char *lookup;
-  
+
   lookup = cache_alias_lookup (mime);
-  
+
   if (lookup)
     return lookup;
-  
-  return mime;  
+
+  return mime;
 }
 
 char **
 _xdg_mime_cache_list_mime_parents (const char *mime)
 {
-  int i, j, p;
-  char *all_parents[128]; /* we'll stop at 128 */ 
+  int i, j, k, l, p;
+  char *all_parents[128]; /* we'll stop at 128 */
   char **result;
 
   mime = xdg_mime_unalias_mime_type (mime);
@@ -878,25 +874,139 @@ _xdg_mime_cache_list_mime_parents (const char *mime)
 
 	  if (strcmp (cache->buffer + mimetype_offset, mime) == 0)
 	    {
-	      int k;
 	      xdg_uint32_t parent_mime_offset;
 	      xdg_uint32_t n_parents = GET_UINT32 (cache->buffer, parents_offset);
 
 	      for (k = 0; k < n_parents && p < 127; k++)
 		{
 		  parent_mime_offset = GET_UINT32 (cache->buffer, parents_offset + 4 + 4 * k);
-		  all_parents[p++] = cache->buffer + parent_mime_offset;
+
+		  /* Don't add same parent multiple times.
+		   * This can happen for instance if the same type is listed in multiple directories
+		   */
+		  for (l = 0; l < p; l++)
+		    {
+		      if (strcmp (all_parents[l], cache->buffer + parent_mime_offset) == 0)
+			break;
+		    }
+
+		  if (l == p)
+		    all_parents[p++] = cache->buffer + parent_mime_offset;
 		}
 
 	      break;
 	    }
 	}
     }
-  all_parents[p++] = 0;
-  
+  all_parents[p++] = NULL;
+
   result = (char **) malloc (p * sizeof (char *));
   memcpy (result, all_parents, p * sizeof (char *));
 
   return result;
 }
 
+static const char *
+cache_lookup_icon (const char *mime, int header)
+{
+  const char *ptr;
+  int i, min, max, mid, cmp;
+
+  for (i = 0; _xdg_mime_caches[i]; i++)
+    {
+      XdgMimeCache *cache = _xdg_mime_caches[i];
+      xdg_uint32_t list_offset = GET_UINT32 (cache->buffer, header);
+      xdg_uint32_t n_entries = GET_UINT32 (cache->buffer, list_offset);
+      xdg_uint32_t offset;
+
+      min = 0;
+      max = n_entries - 1;
+      while (max >= min)
+        {
+          mid = (min + max) / 2;
+
+          offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * mid);
+          ptr = cache->buffer + offset;
+          cmp = strcmp (ptr, mime);
+
+          if (cmp < 0)
+            min = mid + 1;
+          else if (cmp > 0)
+            max = mid - 1;
+          else
+            {
+              offset = GET_UINT32 (cache->buffer, list_offset + 4 + 8 * mid + 4);
+              return cache->buffer + offset;
+            }
+        }
+    }
+
+  return NULL;
+}
+
+const char *
+_xdg_mime_cache_get_generic_icon (const char *mime)
+{
+  return cache_lookup_icon (mime, 36);
+}
+
+const char *
+_xdg_mime_cache_get_icon (const char *mime)
+{
+  const char *icon;
+
+  icon = cache_lookup_icon (mime, 32);
+
+  if (icon == NULL)
+    icon = _xdg_mime_cache_get_generic_icon (mime);
+
+  return icon;
+}
+
+static void
+dump_glob_node (XdgMimeCache *cache,
+		xdg_uint32_t  offset,
+		int           depth)
+{
+  xdg_unichar_t character;
+  xdg_uint32_t mime_offset;
+  xdg_uint32_t n_children;
+  xdg_uint32_t child_offset;
+  int i;
+
+  character = GET_UINT32 (cache->buffer, offset);
+  mime_offset = GET_UINT32 (cache->buffer, offset + 4);
+  n_children = GET_UINT32 (cache->buffer, offset + 8);
+  child_offset = GET_UINT32 (cache->buffer, offset + 12);
+  for (i = 0; i < depth; i++)
+    printf (" ");
+  printf ("%c", character);
+  if (mime_offset)
+    printf (" - %s", cache->buffer + mime_offset);
+  printf ("\n");
+  if (child_offset)
+  {
+    for (i = 0; i < n_children; i++)
+      dump_glob_node (cache, child_offset + 20 * i, depth + 1);
+  }
+}
+
+void
+_xdg_mime_cache_glob_dump (void)
+{
+  int i, j;
+  for (i = 0; _xdg_mime_caches[i]; i++)
+  {
+    XdgMimeCache *cache = _xdg_mime_caches[i];
+    xdg_uint32_t list_offset;
+    xdg_uint32_t n_entries;
+    xdg_uint32_t offset;
+    list_offset = GET_UINT32 (cache->buffer, 16);
+    n_entries = GET_UINT32 (cache->buffer, list_offset);
+    offset = GET_UINT32 (cache->buffer, list_offset + 4);
+    for (j = 0; j < n_entries; j++)
+	    dump_glob_node (cache, offset + 20 * j, 0);
+  }
+}
+
+
